//==========================//
//         Mouse3.C         //
//  Windows Mouse Hit Test  //
//==========================//

#include <windows.h>

#define xPos( x ) ( (x) * cxGrid + cxOff )
#define yPos( y ) ( (y) * cyGrid + cyOff )
#define APP_MENU  NULL
// note: no icon or cursor assignments

HANDLE hInst;
char   szAppTitle[] = "Mouse3: Mouse Hit Test";
char   szAppName[]  = "Mouse3";

long APIENTRY WndProc( HWND hwnd,   UINT msg,
                       UINT wParam, LONG lParam )
{
   static BOOL  fValid[7][7];
   static short cxGrid, cyGrid, cxWin, cyWin,
                cxOff, cyOff, iState[7][7];
   HDC          hdc;
   RECT         rect;
   PAINTSTRUCT  ps;
   short        x, y;

   switch( msg )
   {
      case WM_CREATE:
         if( ! GetSystemMetrics( SM_MOUSEPRESENT ) )
         {
            MessageBox( hwnd,
               "Mouse not found! This application can"
               " not operate without a working mouse!",
               "Mouse Hit Test",   MB_ICONSTOP | MB_OK );
            SendMessage( hwnd, WM_DESTROY, 0, 0L );
         }
         for( x=0; x<7; x++ )
            for( y=0; y<7; y++ )
            {  fValid[x][y] = 1;
               iState[x][y] = 0;  }
         for( x=0; x<2; x++ )
            for( y=0; y<2; y++ )
               fValid[ x ][ y ] = fValid[x+5][ y ] =
               fValid[ x ][y+5] = fValid[x+5][y+5] = 0;
         break;

      case WM_SIZE:
         cxWin = LOWORD( lParam );
         cyWin = HIWORD( lParam );
         cxGrid = cyGrid = min( cxWin, cyWin ) / 8;
         cxOff = ( cxWin - ( cxGrid * 7 ) ) / 2;
         cyOff = ( cyWin - ( cyGrid * 7 ) ) / 2;
         InvalidateRect( hwnd, NULL, TRUE );
         break;

      case WM_LBUTTONDOWN:
         x = ( LOWORD( lParam ) - cxOff ) / cxGrid;
         y = ( HIWORD( lParam ) - cyOff ) / cyGrid;
         if( x < 7 && y < 7 )
         {
            switch( iState[x][y] )
            {
               case 0:  iState[x][y] = 1;  break;
               case 1:                     break;
               case 2:  iState[x][y] = 0;  break;
            }
            rect.left   = xPos( x );
            rect.top    = yPos( y );
            rect.right  = xPos(x+1);
            rect.bottom = yPos(y+1);
            InvalidateRect( hwnd, &rect, FALSE );
         }  else  MessageBeep(0);
         break;

      case WM_RBUTTONDOWN:
         x = ( LOWORD( lParam ) - cxOff ) / cxGrid;
         y = ( HIWORD( lParam ) - cyOff ) / cyGrid;
         if( x < 7 && y < 7 )
         {
            switch( iState[x][y] )
            {
               case 0:  iState[x][y] = 2;  break;
               case 1:  iState[x][y] = 0;  break;
               case 2:                     break;
            }
            rect.left   = xPos( x );
            rect.top    = yPos( y );
            rect.right  = xPos(x+1);
            rect.bottom = yPos(y+1);
            InvalidateRect( hwnd, &rect, FALSE );
         }  else  MessageBeep(0);
         break;

      case WM_PAINT:
         hdc = BeginPaint( hwnd, &ps );
         for( x=0; x<7; x++ )
            for( y=0; y<7; y++ )
               if( fValid[x][y] )
               {
                  Rectangle( hdc, xPos(x)+1, yPos(y)+1,
                             xPos(x+1), yPos(y+1) );
                  if( iState[x][y] == 1 )
                  {
                     MoveToEx( hdc, xPos( x ),
                                    yPos( y ), NULL );
                     LineTo( hdc, xPos(x+1),
                                  yPos(y+1) );
                     MoveToEx( hdc, xPos( x ),
                                    yPos(y+1), NULL );
                     LineTo( hdc, xPos(x+1),
                                  yPos( y ) );
                  }
                  else
                  if( iState[x][y] == 2 )
                     Ellipse( hdc,
                              xPos( x )+2, yPos( y )+2,
                              xPos(x+1)-2, yPos(y+1)-2 );
               }  
         EndPaint( hwnd, &ps );
         break;

      case WM_DESTROY:  PostQuitMessage(0);  break;

      default:
         return( DefWindowProc( hwnd,   msg,
                                wParam, lParam ) );
   }
   return 0;
}

#include "template.i"

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
   MSG   msg;

   if( ! hPrevInstance )
      if( ! InitApplication( hInstance ) )
         return( FALSE );
   if( ! InitInstance( hInstance, nCmdShow ) )
      return (FALSE);
   while( GetMessage( &msg, NULL, 0, 0 ) )
   {
      TranslateMessage( &msg );
      DispatchMessage( &msg );
   }
   return( msg.wParam );
   UNREFERENCED_PARAMETER( lpCmdLine );
}
