// MultiThreadvw.cpp : implementation of the CThreadView class
//

#include "stdafx.h"
#include <time.h>
#include "MultiThread.h"
#include "MultiThreaddoc.h"
#include "MultiThreadvw.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CThreadView

IMPLEMENT_DYNCREATE(CThreadView, CView)

BEGIN_MESSAGE_MAP(CThreadView, CView)
	//{{AFX_MSG_MAP(CThreadView)
	ON_WM_DESTROY()
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_COMMAND(ID_CIRCLES_ADD1, OnCirclesAdd1)
	ON_COMMAND(ID_CIRCLES_ADD10, OnCirclesAdd10)
	ON_COMMAND(ID_CIRCLES_ADD25, OnCirclesAdd25)
	ON_COMMAND(ID_KILLTHREADS_FAST, OnKillthreadsFast)
	ON_COMMAND(ID_KILLTHREADS_SLOW, OnKillthreadsSlow)
	ON_COMMAND(ID_LINES_ADD1, OnLinesAdd1)
	ON_COMMAND(ID_LINES_ADD5, OnLinesAdd5)
	ON_COMMAND(ID_LINES_ADD10, OnLinesAdd10)
	ON_COMMAND(ID_RECTANGLES_ADD1, OnRectanglesAdd1)
	ON_COMMAND(ID_RECTANGLES_ADD10, OnRectanglesAdd10)
	ON_COMMAND(ID_RECTANGLES_ADD25, OnRectanglesAdd25)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CThreadView construction/destruction

CThreadView::CThreadView()
{
	m_pDC = NULL;
	//=== use the system clock to reseed the random number generator
	srand( (unsigned)time( NULL ) );
}

CThreadView::~CThreadView()
{
}

void CThreadView::UpdateTitle( int nThreads )
{
	if( nThreads == -1 )
		nThreads = m_threadList.GetCount();

	CString strCaption;
	if( nThreads != 0 )
	{
		CString strFormat; 

		strFormat.LoadString( IDS_CAPTION_FORMAT );
		strCaption.Format( strFormat, nThreads );
	}
	else
 		strCaption.LoadString(IDS_CAPTION_NOTHREADS);

	GetParentFrame()->SetWindowText(strCaption);
}

void CThreadView::StartThread( int nThreadID )
{
	CDemoThread* pThread;
	CPoint		cPos( rand() % 100, rand() % 100 );
	CPoint		cVel1( ( rand() % 10 ) + 2, ( rand() % 10 ) + 2 ); 
	CPoint		cVel2( ( rand() % 10 ) + 2, ( rand() % 10 ) + 2 );
	CSize		cSize( ( ( rand() % 50 ) + 20 ), ( ( rand() % 50 ) + 20 ) );
	COLORREF	rgbColor;

		// calculate a random color
	rgbColor = RGB( (rand()%25)*10, (rand()%25)*10, (rand()%25)*10 );
		// and randomize the direction
	if( rand() % 1 ) cVel1.x *= -1;
	if( rand() % 1 ) cVel1.y *= -1;
	if( rand() % 1 ) cVel2.x *= -1;
	if( rand() % 1 ) cVel2.y *= -1;
	switch( nThreadID )
	{
		case ID_CIRCLES:
			pThread = new CCircleThread( this, m_pDC->GetSafeHdc(),
										 cPos, cVel1, cSize, rgbColor );
			break;

		case ID_RECTANGLES:
			pThread = new CRectThread( this, m_pDC->GetSafeHdc(),
									   cPos, cVel1, cSize, rgbColor );
			break;

		case ID_LINES:
			pThread = new CLineThread( this, m_pDC->GetSafeHdc(),
									   cPos, cVel1, cVel2, cSize, rgbColor );
			break;
	}

	if( pThread == NULL ) return;
	ASSERT_VALID( pThread );
	pThread->m_pThreadParams = NULL;
		//	by calling CreateThread in a suspended state, the thread priority 
		//	can be set before it gets away
	if( ! pThread->CreateThread( CREATE_SUSPENDED ) )
	{
		delete pThread;
		return;
	}
	m_threadList.AddTail( pThread );
		//	creation was successful, add the thread to the list
	VERIFY( pThread->SetThreadPriority( THREAD_PRIORITY_IDLE ) );
		//	idle priority is used here to keep the demo from bogging down 
		//	other applications. A higher priority may be used to speed up
		//	execution.
	pThread->ResumeThread();
		//	now let the thread run
}

void CThreadView::OnDraw(CDC*)
{
}

/////////////////////////////////////////////////////////////////////////////
// CThreadView diagnostics

#ifdef _DEBUG
void CThreadView::AssertValid() const
{
	CView::AssertValid();
}

void CThreadView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CThreadDoc* CThreadView::GetDocument() // non-debug version is inline
{
	return STATIC_DOWNCAST(CThreadDoc, m_pDocument);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CThreadView message handlers

int CThreadView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if( CView::OnCreate(lpCreateStruct) == -1 ) return -1;
		//	the device context is initialized here instead of the constructor
		//	since there is no window handle until Create is called
	m_pDC = new CClientDC(this);
	UpdateTitle();
	return 0;
}

void CThreadView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
		//	the client rect is updated only when a change occurs
		//	this is faster than doing updates on every draw operation
	for( POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
		m_threadList.GetNext(pos)->UpdateBorder();
}

void CThreadView::OnLinesAdd1() 
{
	StartThread( ID_LINES );
	UpdateTitle();
}

void CThreadView::OnLinesAdd5() 
{
	for( int i=0; i<5; i++ ) OnLinesAdd1();
}

void CThreadView::OnLinesAdd10() 
{
	for( int i=0; i<10; i++ ) OnLinesAdd1();
}

void CThreadView::OnCirclesAdd1() 
{
	StartThread( ID_CIRCLES );
	UpdateTitle();
}

void CThreadView::OnCirclesAdd10() 
{
	for( int i=0; i<10; i++ ) OnCirclesAdd1();	
}

void CThreadView::OnCirclesAdd25() 
{
	for( int i=0; i<25; i++ ) OnCirclesAdd1();	
}

void CThreadView::OnRectanglesAdd1() 
{
	StartThread( ID_RECTANGLES );
	UpdateTitle();
}

void CThreadView::OnRectanglesAdd10() 
{
	for( int i=0; i<10; i++ ) OnRectanglesAdd1();	
}

void CThreadView::OnRectanglesAdd25() 
{
	for( int i=0; i<25; i++ ) OnRectanglesAdd1();	
}

void CThreadView::OnDestroy() 
{
	OnKillthreadsFast();
	delete m_pDC;
	m_pDC = NULL;
	CView::OnDestroy();
}

void CThreadView::OnKillthreadsSlow() 
{
		//	OnKillthreadsSlow iterates through the list, killing one thread
		//	at at time. The KillThread routine does not return until the
		//	thread is dead
	while( ! m_threadList.IsEmpty() )
	{
		m_threadList.RemoveHead()->KillThread();
		UpdateTitle();
	}
		//	Invalidate blanks the window and erases any leftovers
	Invalidate();
}

	//=========================================================================
	//	The OnKillthreadsSlow operation is easy to understand since the
	//	process of killing a thread is encapsulated in KillThread.  If there
	//	are more than 50 threads running, the difference between this and the
	//	OnKillthreadsFast methods are quite noticable
	//
	//	The OnKillthreadsFast method tells all of the threads to go away,
	//	then waits for them to be scheduled and to terminate themselves. This
	//	is much faster than killing them individually and waiting for each one
	//	to terminate. Of course, this process is also a bit more complex.
	//=========================================================================

void CThreadView::OnKillthreadsFast() 
{
		// close all threads
	for( POSITION pos = m_threadList.GetHeadPosition(); pos != NULL; )
	{
		CDemoThread* pThread = m_threadList.GetNext(pos);
		VERIFY(SetEvent(pThread->m_hEventKill));
	}
		// wait for all threads to finish shutdown
	for( int nThreadsLeft = m_threadList.GetCount(); nThreadsLeft != 0; )
	{
		WaitForSingleObject( CDemoThread::m_hAnotherDead, INFINITE );
		Sleep( nThreadsLeft * 2 );	// 200ms for every 100 threads
		nThreadsLeft = 0;
		for( pos = m_threadList.GetHeadPosition(); pos != NULL; )
		{
			CDemoThread* pThread = m_threadList.GetNext(pos);
			if( WaitForSingleObject( pThread->m_hEventDead, 0 ) == WAIT_TIMEOUT )
				++nThreadsLeft;
		}
		UpdateTitle( nThreadsLeft );
	}
		//	delete all thread objects
	while( ! m_threadList.IsEmpty() )
	{
		CDemoThread* pThread = m_threadList.RemoveHead();
		VERIFY( WaitForSingleObject( pThread->m_hThread, INFINITE ) == WAIT_OBJECT_0 );
		delete pThread;
	}
	UpdateTitle();
		//	Invalidate blanks the window and erases any leftovers
	Invalidate();
}

