// threads.h : declarations for each GDI thread class
//

#ifndef __THREADS_H__
#define __THREADS_H__

/////////////////////////////////////////////////////////////////////////////
//
// CDemoThread thread
//
/////////////////////////////////////////////////////////////////////////////

class CDemoThread : public CWinThread
{
public:
	DECLARE_DYNAMIC(CDemoThread)
	CDemoThread(CWnd* pWnd, HDC hDC);

// Attributes
public:
	CRect	m_rectClient;
	HDC		m_hDC;
	CDC		m_dc;
	HANDLE	m_hEventKill;
	HANDLE	m_hEventDead;
	static	HANDLE	m_hAnotherDead;
	static	CRITICAL_SECTION	m_csGDILock;

// Operations
public:
	void KillThread();
	void UpdateBorder();
	virtual void SingleStep() = 0;

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDemoThread)
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CDemoThread();
	virtual void Delete();

protected:
	virtual BOOL InitInstance();

	// Generated message map functions
	//{{AFX_MSG(CDemoThread)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
//
// CCircleThread thread
//
/////////////////////////////////////////////////////////////////////////////

#define CIRCLE_COUNT	1	// actually two circles

class CCircleThread : public CDemoThread
{
public:
	DECLARE_DYNAMIC(CCircleThread)
	CCircleThread( CWnd* pWnd, HDC hDC, CPoint ptPos, CPoint ptVel, 
				 CSize Size, COLORREF rgbColor );

// Attributes
public:
	CRect	m_rectPos[CIRCLE_COUNT+1];
	CPoint	m_ptVel;
	HBRUSH	m_hBrush;
	CBrush	m_cBrush;
	int		m_nItems;
	BOOL	m_bErase;

// Operations
public:
	virtual BOOL InitInstance();
	virtual void SingleStep();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCircleThread)
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~CCircleThread() {};

	// Generated message map functions
	//{{AFX_MSG(CCircleThread)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
//
// CLineThread thread
//
/////////////////////////////////////////////////////////////////////////////

#define LINE_COUNT	99

class CLineThread : public CDemoThread
{
public:
	DECLARE_DYNAMIC(CLineThread)
	CLineThread( CWnd* pWnd, HDC hDC, CPoint ptPos1, CPoint ptVel1, 
				 CPoint ptVel2, CSize Size, COLORREF rgbColor );

// Attributes
public:
	CPoint	m_ptPos1[LINE_COUNT+1];
	CPoint	m_ptPos2[LINE_COUNT+1];
	CPoint	m_ptVel1;
	CPoint	m_ptVel2;
	HPEN	m_hPen;
	CPen	m_cPen;
	int		m_nItems;
	BOOL	m_bErase;

// Operations
public:
	virtual BOOL InitInstance();
	virtual void SingleStep();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CLineThread)
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~CLineThread() {};

	// Generated message map functions
	//{{AFX_MSG(CLineThread)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
//
// CRectThread thread
//
/////////////////////////////////////////////////////////////////////////////

#define RECT_COUNT	1	// actually two rectangle

class CRectThread : public CDemoThread
{
public:
	DECLARE_DYNAMIC(CRectThread)
	CRectThread( CWnd* pWnd, HDC hDC, CPoint ptPos, 
				 CPoint ptVel, CSize Size, COLORREF rgbColor );

// Attributes
public:
	CRect	m_rectPos[RECT_COUNT+1];
	CPoint	m_ptVel;
	HBRUSH	m_hBrush;
	CBrush	m_cBrush;
	int		m_nItems;
	BOOL	m_bErase;

// Operations
public:
	virtual BOOL InitInstance();
	virtual void SingleStep();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CRectThread)
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~CRectThread() {};

	// Generated message map functions
	//{{AFX_MSG(CRectThread)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

#endif // __THREADS_H__
