// LevelCtrl.cpp : Implementation of CLevelCtrl
#include "stdafx.h"
#include "KnobCtrl.h"
#include "LevelCtrl.h"

#define PI 3.14159265358979
#define INCREMENTS 20

/////////////////////////////////////////////////////////////////////////////
// CLevelCtrl

STDMETHODIMP CLevelCtrl::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ILevelCtrl,
	};
	for (int i=0;i<sizeof(arr)/sizeof(arr[0]);i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

HRESULT CLevelCtrl::OnDraw( ATL_DRAWINFO& di )
{
	RECT	   &rc = *(RECT*) di.prcBounds;
	HDC			hDC = di.hdcDraw;
	COLORREF	colorPointer;
	HBRUSH		hOldBrush, hBrush;
	HPEN		hOldPen, hPen;
	POINT		ptCenter;
	double		dxRadius, dyRadius, dAngle, dDiff;
	char		szStr;
	long		x, y;

	OleTranslateColor( m_clrFillColor, NULL, &colorPointer );
	hPen = (HPEN)GetStockObject( BLACK_PEN );
	hOldPen = (HPEN)SelectObject( hDC, hPen );
	hBrush = (HBRUSH) GetStockObject( WHITE_BRUSH );
	hOldBrush = (HBRUSH) SelectObject( hDC, hBrush );
	Rectangle( hDC, rc.left, rc.top, rc.right, rc.bottom );

	rc.left	+= 10;	rc.right  -= 10;
	rc.top	+= 10;	rc.bottom -= 10;

	_itoa( m_nLevel, &szStr, 10 );
	DrawText( hDC, &szStr, -1, &rc, DT_RIGHT | DT_BOTTOM | DT_SINGLELINE);
	DrawText( hDC, _T("+ "), -1, &rc, DT_RIGHT | DT_TOP | DT_SINGLELINE);
	DrawText( hDC, _T(" --"), -1, &rc, DT_LEFT| DT_TOP | DT_SINGLELINE);

	m_rectLeft = m_rectRight = rc;
	m_rectLeft.right = m_rectRight.left = rc.left + ( ( rc.right - rc.left ) / 2 );

	dxRadius = ( rc.right - rc.left ) / 2;
	dyRadius = ( rc.bottom - rc.top ) / 2;
	ptCenter.x = ( rc.left + rc.right ) / 2;
	ptCenter.y = ( rc.top + rc.bottom ) / 2;

	dAngle	 = ( 3 * PI / 2 );
	dDiff	 = PI / INCREMENTS;

	hBrush = (HBRUSH) GetStockObject( BLACK_BRUSH );
	for( short i=0; i<INCREMENTS; i++ )
	{
		x = (long)( dxRadius * cos( dAngle ) + ptCenter.x + 0.5 ) - 5;
		y = (long)( dyRadius * sin( dAngle ) + ptCenter.y + 0.5 ) - 5;
		Ellipse( hDC, x, y, x+10, y+10 );
		dAngle += 2 * dDiff;
		if( i == m_nLevel )
			SelectObject( hDC, hBrush );
	}

	dAngle   = ( 3 * PI / 2 ) + ( ( 2 * PI / INCREMENTS ) * m_nLevel );
	m_arrPoint[0].x = (long)( dxRadius * cos( dAngle ) + ptCenter.x + 0.5 );
	m_arrPoint[0].y = (long)( dyRadius * sin( dAngle ) + ptCenter.y + 0.5 );
	dAngle   += ( PI - dDiff );
	m_arrPoint[1].x = (long)( dxRadius * cos( dAngle ) + ptCenter.x + 0.5 );
	m_arrPoint[1].y = (long)( dyRadius * sin( dAngle ) + ptCenter.y + 0.5 );
	dAngle   += dDiff;
	m_arrPoint[2].x = (long)( 3*dxRadius/4 * cos( dAngle ) + ptCenter.x + 0.5 );
	m_arrPoint[2].y = (long)( 3*dyRadius/4 * sin( dAngle ) + ptCenter.y + 0.5 );
	dAngle   += dDiff;
	m_arrPoint[3].x = (long)( dxRadius * cos( dAngle ) + ptCenter.x + 0.5 );
	m_arrPoint[3].y = (long)( dyRadius * sin( dAngle ) + ptCenter.y + 0.5 );

	hBrush = CreateSolidBrush( colorPointer );
	SelectObject( hDC, hBrush );
	Polygon( hDC, m_arrPoint, 4 );

	SelectObject( hDC, hOldPen );
	SelectObject( hDC, hOldBrush );
	DeleteObject( hBrush );

	return S_OK;
}

STDMETHODIMP CLevelCtrl::get_m_nLevel(short * pVal)
{
	*pVal = m_nLevel;
	return S_OK;
}

STDMETHODIMP CLevelCtrl::put_m_nLevel(short newVal)
{
	if( newVal < 0 || newVal >= INCREMENTS )
		return Error( _T("Value must be in the range 0..20") );
	ChangeLevel( newVal );
	return S_OK;
}

/*
LRESULT CLevelCtrl::OnSize( UINT nType, int cx, int cy )
{
	cx = 100;
	cy = 100;
}
*/

LRESULT	CLevelCtrl::OnLButtonDown( UINT uMsg, WPARAM wParam, 
								   LPARAM lParam, BOOL &bHandled )
{
	POINT	pt;

	pt.x = LOWORD(lParam);
	pt.y = HIWORD(lParam);
	if( PtInRect( &m_rectLeft, pt ) )
	{
		Fire_ClickDn( pt.x, pt.y );
		ChangeLevel( --m_nLevel );
	}
	else
	if( PtInRect( &m_rectRight, pt ) )
	{
		Fire_ClickUp( pt.x, pt.y );
		ChangeLevel( ++m_nLevel );
	}
	return FALSE;
}

LRESULT	CLevelCtrl::OnRButtonDown( UINT uMsg, WPARAM wParam,
								   LPARAM lParam, BOOL &bHandled )
{
	POINT	pt;

	pt.x = LOWORD(lParam);
	pt.y = HIWORD(lParam);
	if( PtInRect( &m_rectLeft, pt ) )
	{
		Fire_ClickUp( pt.x, pt.y );
		ChangeLevel( ++m_nLevel );
	}
	else
	if( PtInRect( &m_rectRight, pt ) )
	{
		Fire_ClickDn( pt.x, pt.y );
		ChangeLevel( --m_nLevel );
	}
	return FALSE;
}

void CLevelCtrl::ChangeLevel( short newLevel )
{
	m_nLevel = min( INCREMENTS - 1, max( 0, newLevel ) );
	FireViewChange();
}