//====================================================================
//
//      ERRORS.C
//
//      Basic outline for using structured exception handling and
//      message tables to process program errors.
//
//      FUNCTIONS
//          main                main routine
//          GetName             ask user to enter a name
//          CheckName           see whether user typed "Moriarty"
//          MainFilter          interpret exception signals
//          ShowExceptionMsg    show messages from message table
//
//      from Mastering Windows NT Programming
//      copyright 1993 by Brian Myers & Eric Hamer
//
//		revised 1996, 1997 by Ben Ezzell
//		remove all dependency on MC script (no longer supported)
//

#include <windows.h>
#include <stdio.h>
#include "errors.h"


#define try		 __try
#define except	 __except

//====================================================================
//      FUNCTION PROTOTYPES

void GetName( void );
void CheckName( void );
LONG MainFilter( LPEXCEPTION_POINTERS lpEP );
void ShowExceptionMsg( DWORD dwMsgCode, LPVOID lpvArgs );

//====================================================================
//      GLOBAL VARIABLE

char szName[40];                        // string user enters

//====================================================================
//      MAIN
//      Call two I/O procedures and check for exceptions.

int main()
{
    try
    {
        GetName();                     // get name from user
        CheckName();                   // see what the user typed
    }
    except( MainFilter( GetExceptionInformation() ) )
    {
    }
    return( 0 );
}

//====================================================================
//      GET NAME
//      Ask user to enter a name string.

void GetName()
{
		// get name from user
    printf( "Enter your name:  " );
    gets( szName );
		// check whether user entered any characters
    if( lstrlen((LPSTR)szName) < 1 )
    {
			// user entered nothing; raise an exception
        DWORD adwArgs[2];           // arguments for exception handler
        char szFile[FILENAME_MAX];  // buffer to store source file name
			// store handler arguments in array
        lstrcpy( szFile, __FILE__ );
        adwArgs[0] = (DWORD)szFile;
        adwArgs[1] = __LINE__;
			// call for an exception handler
        RaiseException( ERRORS_ERROR_NO_NAME, 0, 2, adwArgs );
    }
    return;
}

//====================================================================
//      CHECK NAME
//      Raise an exception of the user typed "Moriarty."

void CheckName()
{		// did the user enter the name "Moriarty"?
    if( lstrcmp( szName, "Moriarty" ) == 0 )
    {
        DWORD adwArgs[2];           // arguments for exception handler
        char szFile[FILENAME_MAX];  // buffer to store source file name
			// store handler arguments in array
        lstrcpy( szFile, __FILE__ );
        adwArgs[0] = (DWORD)szFile;
        adwArgs[1] = __LINE__;
			// call for an exception handler
        RaiseException( ERRORS_SUCCESS_MASTERMIND, 0, 2, adwArgs );
    }
    else
    {
        printf( "Thank you, ");
        printf( szName );
    }
    return;
}

//====================================================================
//      MAIN FILTER
//      Interpret and respond to exceptions.

LONG MainFilter( LPEXCEPTION_POINTERS lpEP )
{
    DWORD   dwCode   = lpEP->ExceptionRecord->ExceptionCode;
    LPDWORD lpdwArgs = lpEP->ExceptionRecord->ExceptionInformation;

		// is this a private exception?
    if( ( dwCode & FACILITY_APP_ERRORS ) == FACILITY_APP_ERRORS )
    {
			// show a message from the message table
        ShowExceptionMsg( dwCode, lpdwArgs );
    }
    else	// not a private exception
    {
        char szMsg[40];

			// tell the user something went wrong
        wsprintf( szMsg, "Program received unknown exception #%lu", dwCode );
        MessageBox( NULL, szMsg, "Errors Message", MB_ICONEXCLAMATION | MB_OK );
    }
		// resume execution after the except block
    return( EXCEPTION_EXECUTE_HANDLER );
}

//====================================================================
//      SHOW EXCEPTION MESSAGE
//      Display a message box with a string from the program's
//      message table.

void ShowExceptionMsg( DWORD dwMsgCode, LPVOID lpvArgs )
{
/*
    LPTSTR lpMessage;

    // pull string from message table and insert formatted arguments in it
    FormatMessage(
        FORMAT_MESSAGE_FROM_HMODULE         // take message table string
        | FORMAT_MESSAGE_ALLOCATE_BUFFER    // allocate a buffer for it
        | FORMAT_MESSAGE_ARGUMENT_ARRAY,    // read lpvArgs as DWORD array
        NULL,                               // message table is in this module
        dwMsgCode,                          // code identifying message
        MAKELANGID(LANG_ENGLISH, SUBLANG_ENGLISH_US),   // message's language
        (LPTSTR)&lpMessage,                 // pointer for new buffer
        0,                                  // ignored with _ALLOCATE_BUFFER
        lpvArgs );                          // values to insert in string

    // lpvArgs points to the DWORD array containing the location
    // of the source file name string and a line number.  FormatMessage
    // inserts those values at the format markers (!s! and !ld!) in
    // the message string.

    // display the message in a message box
    MessageBox( NULL, lpMessage, "Errors Message",
                MB_ICONEXCLAMATION | MB_OK );

    // release the buffer allocated by FormatMessage 
    LocalFree( lpMessage );
*/

		// display the appropriate message for the execption error code
	switch( dwMsgCode )
	{
		case ERRORS_ERROR_NO_NAME:
			MessageBox( NULL, "Rats.  Another crook stays nameless.",
						"Errors Message", MB_ICONEXCLAMATION | MB_OK );
			break;

		case ERRORS_SUCCESS_MASTERMIND:
			MessageBox( NULL, "Eureka!  We've found the mastermind.",
						"Errors Message", MB_ICONEXCLAMATION | MB_OK );
			break;
	}
   return;
}
