/*--------------------------------------------------------------------

	DECRYPT.CPP

	Demonstrates Using Window the Crypto API to decrypt a file. 
	Takes an input and output filename from the command line. 
		
    FUNCTIONS USED:

	CryptAcquireContext // Gets handle to CSP.
	CryptGetUserKey		// Gets our default exchange key.
	CryptImportKey		// Imports/decrypts our key blob.
	CryptDecrypt		// Decryptes the data from our input file.
	CryptDestroyKey		// Cleans up our key when were done.
	CryptReleaseContext	// Releases our CSP handle when were done.

	CreateFile			// Opens the input and output files.
	ReadFile			// Reads encrypted data from input file.	
	WriteFile			// Writes "raw" data to output file.
	CloseHandle			// Closes input and output file when done.
		
   		NOTE: This program will work on both Windows 98 and NT4.0
		(or later).

		written/updated by Jim Blaney, 1998
		for Windows 98 Developers Handbook

  -------------------------------------------------------------------*/
#define _WIN32_WINNT 0x0400
#include <windows.h>
#include <wincrypt.h>
#include <stdio.h>

// note: the IN_BUFFER_SIZE used here is purposely an even multiple of the
// various possible encryption block sizes, namely 1, 8, and 64 bytes.
const IN_BUFFER_SIZE    = 8 * 1000; // note: needs to be multiple of block size.
const OUT_BUFFER_SIZE   = IN_BUFFER_SIZE + 64; // extra padding.

int __cdecl main(int argc, char * argv[])
{
	HANDLE      hInFile, hOutFile;
	BYTE        pbBuffer[OUT_BUFFER_SIZE];
	BOOL        finished;
	HCRYPTPROV  hProvider = 0;
	HCRYPTKEY   hKey = 0;
	DWORD       dwByteCount, dwBytesWritten;

	if (argc != 3) {
   	printf("Usage: DECRYPT infile outfile\n");
		exit(0);
	}

	// Get handle for the default provider (use RSA encryption).
	CryptAcquireContext(&hProvider, NULL, NULL, PROV_RSA_FULL, 0);

	// Open infile and create outfile.
	hInFile	 = CreateFile(argv[1], GENERIC_READ, FILE_SHARE_READ, NULL,
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

	hOutFile = CreateFile(argv[2], GENERIC_WRITE, FILE_SHARE_READ, NULL,
		CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

	ReadFile(hInFile, &dwByteCount, sizeof(dwByteCount), &dwBytesWritten, NULL);
	ReadFile(hInFile, pbBuffer, dwByteCount, &dwBytesWritten, NULL);
   // Import Key blob into "CSP" change this!!
   CryptImportKey(hProvider, pbBuffer, dwByteCount, 0, 0, &hKey);

	// Now, read data in, encrypt it, and write encrypted data to output file.
	do {
		ReadFile(hInFile, pbBuffer, IN_BUFFER_SIZE, &dwByteCount, NULL);
		finished = (dwByteCount < IN_BUFFER_SIZE);
        CryptDecrypt(hKey, 0, finished, 0, pbBuffer, &dwByteCount);
		WriteFile(hOutFile, pbBuffer, dwByteCount, &dwBytesWritten, NULL);
	} while (!finished);

	// Cleanup: release handles, close files.
	CryptDestroyKey(hKey);
	CryptReleaseContext(hProvider, 0);
	CloseHandle(hInFile);
	CloseHandle(hOutFile);

	return(0);
}


