/*--------------------------------------------------------------------

	MacSock.CPP

	Demonstrates using the protoocol-specific options support in 
   Winsock 2 to create and use an AppleTalk socket. After successful
   socket open and binding, attempts to register network name on the 
   specified AppleTalk zone.
   
   FUNCTIONS USED:

   WSAStartup     Initializes Winsock API, checks version number.
   socket         Opens a socket (in this case, an AppleTalk socket).
   bind           Binds address to socket. (Using AppleTalk address.)
   setsockopt     Provides protocol-specific additional functionality.
   WSACleanup     Uninitializes Winsock API.
   
   NOTES: 
   - This program runs only on NT 4.0 or later, and requires that the
     AppleTalk protocol be installed (This should add SFMWSHAT.DLL).

		written/updated by Jim Blaney, 1998
		for Windows 98 Developers Handbook

  -------------------------------------------------------------------*/
#include <iostream.h>
#include <winsock2.h>
#include <atalkwsh.h> // Appletalk specific definitions.

const VERSION_MAJOR  =  2;  // Winsock version 2.0 required by this app.
const VERSION_MINOR  =  0;

main (int argc, char *argv[])
{
   int	   iStatus;  // Error result.
   SOCKET	hSocket;  // The socket that gets opened.   
   WSADATA	WSData;   // WinSock initialization structure.  
   
   SOCKADDR_AT	         SockAddr; // AppleTalk-specific address struct.
   WSH_REGISTER_NAME    RegName;  // Name Registration structure.
   
   // Initialize Winsock API. (require version 2.0 or later!)
   if ( WSAStartup(MAKEWORD(VERSION_MAJOR, VERSION_MINOR), &WSData) )
   {
      cout << "Cannot find Winsock version ";
      cout << VERSION_MAJOR << "." << VERSION_MINOR << " or later!\n";
      return(1);
   }

   cout << "Attempting to open AppleTalk socket: ";

   // Open an AppleTalk NBP datagram socket.
   hSocket = socket(PF_APPLETALK, SOCK_DGRAM, DDPPROTO_NBP);
   if ( iStatus == SOCKET_ERROR )
   {
      cout << "Failed - " << WSAGetLastError() << endl;
      goto Cleanup;
   }
   else
      cout << "Success!\n";
   cout << flush;

      // Prepare SockAddr strucure to bind to the socket.
   SockAddr.sat_family  = PF_APPLETALK;   // AppleTalk family.
   SockAddr.sat_net     = 0;              // network number. SET THIS!
   SockAddr.sat_node    = 0;              // the node number.
   SockAddr.sat_socket  = hSocket;        // the AppleTalk socket.

   cout << "Attempting to bind address to socket: ";

   // Attempt to bind address to socket.
   iStatus = bind( hSocket, (LPSOCKADDR)&SockAddr, sizeof(SockAddr));
   if ( iStatus == SOCKET_ERROR )
   {
      cout << "Failed - " << WSAGetLastError() << endl;
      goto Cleanup;
   }
   else
      cout << "Success!\n";
   cout << flush;

   // Prepare RegName structure for registering our name on the network.
   
   // Set name strings.
   strcpy(RegName.ObjectName, "MacSock Server"); // Name on network.
   strcpy(RegName.TypeName,   "Winsock2 Type");  // Name type.
   strcpy(RegName.ZoneName,   "Granny Smith");   // Zone name.    
   
   // Set corresponding string lengths in RegName structure.
   RegName.ObjectNameLen   = strlen(RegName.ObjectName);
   RegName.TypeNameLen     = strlen(RegName.TypeName);
   RegName.ZoneNameLen     = strlen(RegName.ZoneName);
   
   cout << "Attempting to register name: " << flush; 
   
   // Attempt to register name.
   iStatus = setsockopt( 
      hSocket,                   // Appletalk Socket.
	   SOL_APPLETALK,             // Option level.
	   SO_REGISTER_NAME,          // Option value requested.
	   (char *)&RegName,          // Pointer to option buffer.
	   sizeof(WSH_REGISTER_NAME)  // Size of option request buffer.   
   );
   if ( iStatus == SOCKET_ERROR )
      cout << "Failed - " << WSAGetLastError() << endl;
   else
      cout << "Success!\n";
   cout << flush;

Cleanup:

   closesocket( hSocket );
   WSACleanup();

   return 0;
}

