//=======================//
//       PenDraw2.C      //
//  C++ Windows Drawing  //
//=======================//

#include <windows.h>
#include "pendraw2.h"
#define APP_ICON  "PenDraw2"
#define APP_MENU  "PenDraw2"
// note: no cursor assignment

HANDLE hInst;
char   szAppTitle[] = "Pen Draw 2: Figures";
char   szAppName[]  = "PenDraw2";
COLORREF  cColor = RGB( 0, 0, 0 ),
         lpColor[8] =
{	RGB(   0,   0,   0 ),	RGB(   0,   0, 255 ),	// Black,  Blue
	RGB(   0, 255,   0 ),	RGB(   0, 255, 255 ),	// Green,  Cyan
	RGB( 255,   0,   0 ),	RGB( 255,   0, 255 ),	// Red,    Magenta
	RGB( 255, 255,   0 ),	RGB( 255, 255, 255 ) };	// Yellow, White

long APIENTRY WndProc( HWND hwnd,   UINT msg,
					   UINT wParam, LONG lParam )
{
	static  int cxWnd, cyWnd, xUL, yUL, xRB, yRB,
				xp1, yp1, xp2, yp2, xp3, yp3,
				nFigure = IDM_RECT,
				nHatch  = IDM_HS_HORIZONTAL,
				nColor  = IDM_BLACK,
				nPen    = IDM_SOLID;
	HDC			hdc;
	HMENU		hMenu;
	HPEN		hPen, hOldPen;
	HBRUSH		hBrush, hOldBrush;
	PAINTSTRUCT	ps;
	RECT		rect;

	switch( msg )
	{
		case WM_COMMAND:
			hMenu = GetMenu( hwnd );
			switch( LOWORD( wParam ) )
			{
				case IDM_BLACK:			case IDM_BLUE:
				case IDM_GREEN:			case IDM_CYAN:
				case IDM_RED:			case IDM_MAGENTA:
				case IDM_YELLOW:		case IDM_WHITE:
					CheckMenuItem( hMenu, nColor, MF_UNCHECKED );
					nColor = wParam;
					cColor = lpColor[nColor-IDM_BLACK];
					CheckMenuItem( hMenu, nColor, MF_CHECKED );
					break;

				case IDM_ARC:			case IDM_CHORD:
				case IDM_CIRCLE:		case IDM_ELLIPSE:
				case IDM_PIE:			case IDM_RECT:
				case IDM_SQUARE:
					CheckMenuItem( hMenu, nFigure, MF_UNCHECKED );
					nFigure = wParam;
					CheckMenuItem( hMenu, nFigure, MF_CHECKED );
					break;

				case IDM_SOLID:			case IDM_DASH:
				case IDM_DOT:			case IDM_DASHDOT:
				case IDM_DASH2DOT:		case IDM_NULL:
				case IDM_INSIDEFRAME:
					CheckMenuItem( hMenu, nPen, MF_UNCHECKED );
					nPen = wParam;
					CheckMenuItem( hMenu, nPen, MF_CHECKED );
					break;

				case IDM_HS_HORIZONTAL:	case IDM_HS_VERTICAL:
				case IDM_HS_FDIAGONAL:	case IDM_HS_BDIAGONAL:
				case IDM_HS_CROSS:		case IDM_HS_DIAGCROSS:
/*
					// these fill styles are obsolete
					//		and no longer supported
				case IDM_HS_FDIAGONAL1:	case IDM_HS_BDIAGONAL1:
				case IDM_HS_SOLID:
				case IDM_HS_DENSE1:		case IDM_HS_DENSE2:
				case IDM_HS_DENSE3:		case IDM_HS_DENSE4:
				case IDM_HS_DENSE5:		case IDM_HS_DENSE6:
				case IDM_HS_DENSE7:		case IDM_HS_DENSE8:
				case IDM_HS_NOSHADE:	case IDM_HS_HALFTONE:
*/
					CheckMenuItem( hMenu, nHatch, MF_UNCHECKED );
					nHatch = wParam;
					CheckMenuItem( hMenu, nHatch, MF_CHECKED );
					break;

				default:
					return( DefWindowProc( hwnd, msg, wParam, lParam ) );
			}
			InvalidateRect( hwnd, NULL, FALSE );
			break;

		case WM_SIZE:
			cxWnd = LOWORD( lParam );
			cyWnd = HIWORD( lParam );
			xRB = cxWnd - ( xUL = cxWnd / 6 );
			yRB = cyWnd - ( yUL = cyWnd / 6 );
			xp1 = yp2 = 0;
			xp3 = ( xp2 = cxWnd ) / 2;
			xp3 = ( yp1 = cyWnd ) / 2;
			break;

		case WM_PAINT:
			hdc = BeginPaint( hwnd, &ps );
			SetMapMode( hdc, MM_TEXT );
			GetClientRect( hwnd, &rect );
			SetViewportExtEx( hdc, rect.right, rect.bottom, NULL );
			SetWindowExtEx( hdc, rect.right, rect.bottom, NULL );
			hPen = CreatePen( nPen-IDM_SOLID, 1, cColor );
			hOldPen = SelectObject( hdc, hPen );
			hBrush = CreateHatchBrush( nHatch-IDM_HS_HORIZONTAL, cColor );
			hOldBrush = SelectObject( hdc, hBrush );
			switch( nFigure )
			{
				case IDM_ARC:
					Arc( hdc, xUL, yUL, xRB, yRB, xp2, yp1, xp1, yp2 );
					break;

				case IDM_CHORD:
					Chord( hdc, xUL, yUL, xRB, yRB, xp1, yp1, xp2, yp2 );
					break;

				case IDM_PIE:
					Pie( hdc, xUL, yUL, xRB, yRB, xp1, yp1, xp3, yp3 );
					break;

				case IDM_ELLIPSE:
					Ellipse( hdc, xUL, yUL, xRB, yRB );
					break;

				case IDM_CIRCLE:
					Ellipse( hdc, xUL, yUL,
							 xUL + min( xRB-xUL, yRB-yUL ),
							 yUL + min( yRB-yUL, xRB-xUL ) );
					break;

				case IDM_RECT:
					Rectangle( hdc, xUL, yUL, xRB, yRB );
					break;

				case IDM_SQUARE:
					Rectangle( hdc, xUL, yUL,
							   xUL + min( xRB-xUL, yRB-yUL ),
							   yUL + min( yRB-yUL, xRB-xUL ) );
					break;
			}
			DeleteObject( SelectObject( hdc, hOldBrush ) );
			DeleteObject( SelectObject( hdc, hOldPen ) );
			EndPaint( hwnd, &ps );
			break;

		case WM_DESTROY:   PostQuitMessage(0);    break;

		default:	return( DefWindowProc( hwnd, msg, wParam, lParam ) );
   }
   return 0;
}

#include "template.i"

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
	MSG   msg;

	if( ! hPrevInstance )
		if( ! InitApplication( hInstance ) )
			return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) )
		return (FALSE);
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( msg.wParam );
	UNREFERENCED_PARAMETER( lpCmdLine );
}
