//===============//
//  SVGA_Win.C   //
//===============//

#include <windows.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include "svga_win.h"

#define APP_ICON "SVGA_WIN"
#define APP_MENU "SVGA_WIN"
// note: no cursor assignment

#define  PI		3.14159

HANDLE			hInst;            // application hInstance
HPALETTE		hPalOld, hPal = NULL;
LPLOGPALETTE	lpPal;
int				Grid, Sphere;
char  			szAppTitle[] = "SVGA Palette Viewer",
				szAppName[]  = "SVGA_WIN";

BOOL FAR PASCAL About( HWND hDlg,   WORD msg,
                       WORD wParam, LONG lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG: return( TRUE );
		case WM_COMMAND:
			switch( wParam )
			{
				case IDOK:	EndDialog( hDlg, TRUE );
							return( TRUE );

				default:	return( TRUE );
			}
	}
	return( FALSE );
}

void SetGreyPalette()
{
	int i;

	for( i=0; i<256; i++ )
		lpPal->palPalEntry[i].peRed =
		lpPal->palPalEntry[i].peGreen =
		lpPal->palPalEntry[i].peBlue = i;
}

void SetScaledPalette()
{
	int i, j;

	for( i=0,j=0; i<32; i++,j+=8 )        // blues
	{
		lpPal->palPalEntry[i].peBlue = j;
		lpPal->palPalEntry[i].peRed =
		lpPal->palPalEntry[i].peGreen = 0;
	}
	for( i=32,j=0; i<64; i++,j+=8 )       // cyan
	{
		lpPal->palPalEntry[i].peRed = 0;
		lpPal->palPalEntry[i].peGreen =
		lpPal->palPalEntry[i].peBlue = j;
	}
	for( i=64,j=0; i<96; i++,j+=8 )       // magenta
	{
		lpPal->palPalEntry[i].peGreen = 0;
		lpPal->palPalEntry[i].peRed =
		lpPal->palPalEntry[i].peBlue = j;
	}
	for( i=96,j=0; i<128; i++,j+=8 )      // red
	{
		lpPal->palPalEntry[i].peRed = j;
		lpPal->palPalEntry[i].peGreen =
		lpPal->palPalEntry[i].peBlue = 0;
	}
	for( i=128,j=0; i<160; i++,j+=8 )     // yellow
	{
		lpPal->palPalEntry[i].peBlue = 0;
		lpPal->palPalEntry[i].peRed =
		lpPal->palPalEntry[i].peGreen = j;
	}
	for( i=160,j=0; i<192; i++,j+=8 )     // green
	{
		lpPal->palPalEntry[i].peGreen = j;
		lpPal->palPalEntry[i].peRed =
		lpPal->palPalEntry[i].peBlue = 0;
	}
	for( i=192,j=0; i<256; i++,j+=4 )      // grays & whites
		lpPal->palPalEntry[i].peRed =
		lpPal->palPalEntry[i].peGreen =
		lpPal->palPalEntry[i].peBlue = j;
}

void SetRGBPalette()
{
	int i;

	for( i=0; i<24; i++ )
	{
		lpPal->palPalEntry[i].peGreen =
		lpPal->palPalEntry[i].peBlue  = 8*(i+8);
		lpPal->palPalEntry[i].peRed   = 0;         // Cyan
		lpPal->palPalEntry[i+24].peRed =
		lpPal->palPalEntry[i+24].peBlue = 8*(i+8);
		lpPal->palPalEntry[i+24].peGreen = 0;      // Magenta
		lpPal->palPalEntry[i+48].peRed =
		lpPal->palPalEntry[i+48].peGreen = 8*(i+8);
		lpPal->palPalEntry[i+48].peBlue = 0;       // Yellow
	}
	for( i=0; i<45; i++ )
	{
		lpPal->palPalEntry[i+72].peGreen  =
		lpPal->palPalEntry[i+72].peBlue  = 0;
		lpPal->palPalEntry[i+72].peRed  = 4*(i+19);    // Red
		lpPal->palPalEntry[i+116].peRed =
		lpPal->palPalEntry[i+116].peBlue = 0;
		lpPal->palPalEntry[i+116].peGreen = 4*(i+19);  // Green
		lpPal->palPalEntry[i+160].peRed =
		lpPal->palPalEntry[i+160].peGreen = 0;
		lpPal->palPalEntry[i+160].peBlue = 4*(i+19);   // Blue
	}
	for( i=0; i<48; i++ )
		lpPal->palPalEntry[i+204].peRed =
		lpPal->palPalEntry[i+204].peGreen =
		lpPal->palPalEntry[i+204].peBlue = (int)( i * 256 / 48 );
	lpPal->palPalEntry[255].peRed =
	lpPal->palPalEntry[255].peGreen =
	lpPal->palPalEntry[255].peBlue = 0xFF;
}

void Hsi2Rgb( double H, double S, double I, int Index )
{
	double    T, Rv, Gv, Bv;

	T  = (double) ( 2.0 * PI * H );
	Rv = (double) ( 1.0 + S * sin( T - 2.0 * PI / 3.0  ) );
	Gv = (double) ( 1.0 + S * sin( T ) );
	Bv = (double) ( 1.0 + S * sin( T + 2.0 * PI / 3.0 ) );
	T  = (double) ( 63.999 * I / 2.0 );
	lpPal->palPalEntry[Index].peRed   = (int)( Rv * T * 4 );
	lpPal->palPalEntry[Index].peGreen = (int)( Gv * T * 4 );
	lpPal->palPalEntry[Index].peBlue  = (int)( Bv * T * 4 );
}

void SetHSIPalette()
{
	double  H, S, I;
	int     X, Y, Z, Index;

	for( Z=0; Z<=15; Z++ )         // create grey scale
		lpPal->palPalEntry[Z].peRed   =
		lpPal->palPalEntry[Z].peGreen =
		lpPal->palPalEntry[Z].peBlue  = Z * 16;
	Index = 16;                    // create HSI spectrum
	for( X=0; X<4; X++ )           // four intensities
	{
		I = (double)( ( X + 2.0 ) / 5.0 );   //4.0 );
		for( Y=0; Y<3; Y++ )        // three saturations
		{
			S = (double)( ( Y + 2.0 ) / 4.0 );  // 3.0 );
			for( Z=0; Z<20; Z++ )    // twenty hues
			{                        // H,S,I between 0 and 1
				H = (double)( Z / 20.0 );
				Hsi2Rgb( H, S, I, Index++ );
			}                        // cal and store R, G, B
		}
	}
}

void ResetMenu( HWND hwnd, WORD wItem )
{
	int i;
	for( i=IDM_SCALED; i<=IDM_GRAY; i++ )
		CheckMenuItem( GetMenu(hwnd), i, MF_BYCOMMAND | MF_UNCHECKED );
	CheckMenuItem( GetMenu(hwnd), wItem, MF_BYCOMMAND | MF_CHECKED );
}

long FAR PASCAL WndProc( HWND hwnd,     UINT   msg,
                         WPARAM wParam, LPARAM lParam )
{
	HDC           hdc;
	HPEN          hOldPen, hPen;
	HBRUSH        hOldBrush, hBrush;
	RECT          rect;
	PAINTSTRUCT   ps;
	unsigned int  BlockSize, xCen, yCen, i, j;

	switch( msg )
	{
		case WM_CREATE:
			lpPal = (LPLOGPALETTE) LocalAlloc( LHND, 
				sizeof(LOGPALETTE) + sizeof(PALETTEENTRY) * 256 );
			lpPal->palVersion = 0x300;
			lpPal->palNumEntries = 256;
			for( i=0; i<256; i++ )
			{
				lpPal->palPalEntry[i].peRed =
				lpPal->palPalEntry[i].peGreen =
				lpPal->palPalEntry[i].peBlue = 0;
				lpPal->palPalEntry[i].peFlags = PC_RESERVED;
			}
			Grid = Sphere = TRUE;
			SendMessage( hwnd, WM_COMMAND, IDM_SCALED, 0L );
			break;

		case WM_SIZE:
			InvalidateRect( hwnd, NULL, TRUE );
			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDM_SCALED:
					ResetMenu( hwnd, LOWORD( wParam ) );
					SetScaledPalette();
					SetWindowText( hwnd, "Scaled Palette" );
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				case IDM_RGB:
					ResetMenu( hwnd, LOWORD( wParam ) );
					SetRGBPalette();
					SetWindowText( hwnd, "RGB Palette" );
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				case IDM_HSI:
					ResetMenu( hwnd, LOWORD( wParam ) );
					SetHSIPalette();
					SetWindowText( hwnd, "HSI Palette" );
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				case IDM_GRAY:
					ResetMenu( hwnd, LOWORD( wParam ) );
					SetGreyPalette();
					SetWindowText( hwnd, "Grey Palette" );
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				case IDM_GRID:
					if( Grid )
					{
						Grid = FALSE;
						CheckMenuItem( GetMenu(hwnd), IDM_GRID,
									   MF_BYCOMMAND | MF_UNCHECKED );
						Sphere = TRUE;
						CheckMenuItem( GetMenu(hwnd), IDM_SPHERE,
									   MF_BYCOMMAND | MF_CHECKED );
						InvalidateRect( hwnd, NULL, TRUE );
					}
					else
					{
						Grid = TRUE;
						CheckMenuItem( GetMenu(hwnd), IDM_GRID, 
									   MF_BYCOMMAND | MF_CHECKED );
						InvalidateRect( hwnd, NULL, FALSE );
					}
					break;

				case IDM_SPHERE:
					if( Sphere )
					{
						Sphere = FALSE;
						CheckMenuItem( GetMenu(hwnd), IDM_SPHERE,
									   MF_BYCOMMAND | MF_UNCHECKED );
						Grid = TRUE;
						CheckMenuItem( GetMenu(hwnd), IDM_GRID,
									   MF_BYCOMMAND | MF_CHECKED );
						InvalidateRect( hwnd, NULL, TRUE );
					}
					else
					{
						Sphere = TRUE;
						CheckMenuItem( GetMenu( hwnd ), IDM_SPHERE,
									   MF_BYCOMMAND | MF_CHECKED );
						InvalidateRect( hwnd, NULL, FALSE );
					}
					break;

				case IDM_ABOUT:
					DialogBox( hInst, "ABOUT", hwnd, About );
					break;

				default:
					return( DefWindowProc( hwnd, msg, wParam, lParam ) );
			}
			break;

		case WM_PAINT:
			hdc = BeginPaint( hwnd, &ps );
			GetClientRect( hwnd, &rect );
			BlockSize = (int)((rect.bottom-rect.top)/16);
			xCen = max( (int)((rect.right-rect.left)/2),
						(int)( rect.right - 236 ) );
			yCen = max( (int)((rect.bottom-rect.top)/2),
			            (int)( rect.top + 236 ) );
			hPal = CreatePalette( lpPal );
			hPalOld = SelectPalette( hdc, hPal, FALSE );
			RealizePalette( hdc );

			if( Grid )
				for( i=0; i<16; i++ )
					for( j=0; j<16; j++ )
					{
						hBrush = CreateSolidBrush( PALETTEINDEX( j + i * 16 ) );
						hOldBrush = SelectObject( hdc, hBrush );
						Rectangle( hdc, BlockSize*j, BlockSize*i,
								   BlockSize*(j+1)-2, BlockSize*(i+1)-2 );
						SelectObject( hdc, hOldBrush );
						DeleteObject( hBrush );
					}

			if( Sphere )
				for( i=0; i<236; i++ )
				{
					hPen = CreatePen( PS_SOLID, 1, PALETTEINDEX(235-i) );
					hOldPen = SelectObject( hdc, hPen );
					//=== for transparent moire patterns, disable following ==
					if( i < 255 )
					{
						Arc( hdc, xCen-i, yCen-i+1, xCen+i, yCen+i+1, 0, 0, 0, 0 );
						Arc( hdc, xCen-i+1, yCen-i, xCen+i+1, yCen+i, 0, 0, 0, 0 );
					}
					//========================================================
					Arc( hdc, xCen-i, yCen-i, xCen+i, yCen+i, 0, 0, 0, 0 );
					SelectObject( hdc, hOldPen );
					DeleteObject( hPen );
				}
				SelectPalette( hdc, hPalOld, FALSE );
				DeleteObject( hPal );
				EndPaint( hwnd, &ps );
				break;

		case WM_DESTROY:
//			LocalFree( lPal );		// stack error if freed 
			PostQuitMessage(0);
			break;

		default:
			return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return( TRUE );
}

#include "template.i"

#ifdef __BORLANDC__
#pragma argsused
#endif

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
					  LPSTR  lpCmdLine, int    nCmdShow    )
{
	MSG   msg;

	if( ! InitApplication( hInstance ) ) return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) ) return (FALSE);
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );			// translate virtual key codes
		DispatchMessage( &msg );			// dispatch message to window 
	}
	return( msg.wParam );
#ifndef __BORLANDC__
	UNREFERENCED_PARAMETER( lpCmdLine );	// prevents unused para warning
#endif										//    message from compiler
}