// TargetView.cpp : implementation of the CTargetView class
//

#include "stdafx.h"
#include "Target.h"

#include "TargetDoc.h"
#include "TargetView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTargetView

IMPLEMENT_DYNCREATE(CTargetView, CView)

BEGIN_MESSAGE_MAP(CTargetView, CView)
	//{{AFX_MSG_MAP(CTargetView)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTargetView construction/destruction

CTargetView::CTargetView()
{
	m_xRadius = m_yRadius = 30;
	m_hBitmap		= NULL;
	m_pPal			= NULL;
	m_pLogPal		= NULL;
	m_nTrack		= NO_TARGET;
	m_bDrawOverlay	= FALSE;
	m_bCapture		= FALSE;
}

CTargetView::~CTargetView()
{
	FreeGlobalVars();
}

BOOL CTargetView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTargetView drawing

void CTargetView::OnDraw(CDC* pDC)
{
	CTargetDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CDC		 *pDCMem;
	CPalette *pOldPalette;

	if( m_hBitmap )
	{
		pDCMem = new CDC();
		pDCMem->CreateCompatibleDC( pDC );
		pDCMem->SelectObject( m_hBitmap );
		if( m_pPal )
			pOldPalette = pDC->SelectPalette( m_pPal, FALSE );
		pDC->BitBlt( 0, 0, m_bmWidth, m_bmHeight, pDCMem,
					 0, 0, SRCCOPY );
		if( m_pPal )
		{
			pDC->SelectPalette( pOldPalette, FALSE );
			pDC->RealizePalette();
		}
		delete pDCMem;
	}
	DrawOverlay();
}

/////////////////////////////////////////////////////////////////////////////
// CTargetView printing

BOOL CTargetView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CTargetView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CTargetView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CTargetView diagnostics

#ifdef _DEBUG
void CTargetView::AssertValid() const
{
	CView::AssertValid();
}

void CTargetView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTargetDoc* CTargetView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTargetDoc)));
	return (CTargetDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTargetView message handlers

void CTargetView::OnFileOpen() 
{
	CFileDialog cfDlg( TRUE );
	
	if( cfDlg.DoModal() )
	{
		ReadBitmap( cfDlg.GetPathName() );
		Invalidate();
	}
}

void CTargetView::FreeGlobalVars()
{      // if old bitmap, delete same ...
	if( m_hBitmap ) DeleteObject( m_hBitmap );
	if( m_hBitmap ) LocalFree( (HANDLE) m_hBitmap );
	m_hBitmap = NULL;
		// if old palette, delete same
	if( m_pPal ) m_pPal->DeleteObject();
	m_pPal = NULL;
		// if old palette handle, delete same
	if( m_pLogPal ) DeleteObject( m_pLogPal );
	if( m_pLogPal ) LocalFree( m_pLogPal );
	m_pLogPal = NULL;
	m_bmWidth = m_bmHeight = m_bmImgSize = m_bmScanWidth = 0;
}

BOOL CTargetView::ReadBitmap( CString csFName )
{
	BITMAPFILEHEADER   bmFH;
	LPBITMAPINFOHEADER pBmIH;
	LPBITMAPINFO       pBmInfo;
	RGBQUAD   FAR    * pRGB;
	BYTE             * pBits;
	CDC				 * pDC;
	CPalette		 * pNewPal;
	HANDLE			   hBM; 
	HANDLE			   hBmInfo;
	HANDLE			   hBmIH;
	HBITMAP			   hBitmap;
	WORD			   PalSize, bmBitCount;
	LPSTR			   lpBits;
	int				   i, CRes = 256;

		//=== clean up any existing memory objects ===
	FreeGlobalVars();
		//=== open file ===
	TRY
	{
		CFile cFile( csFName, CFile::modeRead | CFile::typeBinary );
		SetCursor( LoadCursor( NULL, IDC_WAIT ) );
			//=== read bitmap header ===
		cFile.Read( (LPSTR) &bmFH, sizeof(BITMAPFILEHEADER) );
		if( bmFH.bfType != 0x4D42 )		   // if type isn't "BF" ...
			return( ErrorMsg( "Not a bitmap image" ) );
			//=== read bitmap info header ===
		hBmIH = LocalAlloc( LMEM_FIXED, sizeof(BITMAPINFOHEADER ) );
		pBmIH = (LPBITMAPINFOHEADER) LocalLock( hBmIH );
		cFile.Read( (LPSTR) pBmIH, sizeof(BITMAPINFOHEADER) );

		if( (WORD) pBmIH->biBitCount != 8 ) // 2^8 colors = 256
			return( ErrorMsg( "Not a 256 color bitmap"));

		if( (DWORD) pBmIH->biCompression != BI_RGB )
			return( ErrorMsg("Compressed images not accepted"));

		PalSize = (WORD)( pBmIH->biClrUsed * sizeof(RGBQUAD) );
		GlobalUnlock( hBmIH );
			//=== read palette info into bitmap info ===
		hBmInfo = LocalAlloc( LPTR, PalSize + sizeof(BITMAPINFOHEADER) );
		pBmInfo = (LPBITMAPINFO) LocalLock( hBmInfo );
		pBmInfo->bmiHeader = *pBmIH;
		LocalFree( hBmIH );          // free old memory handle
			//=== save size data for this image ===
		m_bmWidth     = LOWORD( pBmInfo->bmiHeader.biWidth );
		m_bmHeight    = LOWORD( pBmInfo->bmiHeader.biHeight );
			//=== note width and height saved as WORD, not DWORD ===
		bmBitCount	  = (WORD)  pBmInfo->bmiHeader.biBitCount;
		m_bmImgSize   = (DWORD) pBmInfo->bmiHeader.biSizeImage;
		m_bmScanWidth = (DWORD) m_bmWidth;
		while( m_bmScanWidth % sizeof(WORD) ) 
			m_bmScanWidth++;							  // must be an even WORD size !!!
			//=== if image size not in header, calculate size ===
		if( pBmInfo->bmiHeader.biSizeImage == 0 )
		{
			m_bmImgSize = ( ( ( ( m_bmWidth * bmBitCount ) +
									31 ) / 32 ) * 4 ) * m_bmHeight;
			pBmInfo->bmiHeader.biSizeImage = m_bmImgSize;
		}
			//=== now read the palette information ===
		if( cFile.Read( (LPSTR) pBmInfo->bmiColors, PalSize ) != PalSize )
		{
			GlobalUnlock( hBmInfo );
			LocalFree( hBmInfo );
			pBmInfo = NULL;
			FreeGlobalVars();
			return( ErrorMsg( "Palette read error" ) );
		}
			//=== allocate space for bitmap palette ===
		pRGB = pBmInfo->bmiColors;
			//================================================//
			// check for short palette indicated by biClrUsed //
			//================================================//
		if( pBmInfo->bmiHeader.biClrUsed )
			CRes = pBmInfo->bmiHeader.biClrUsed;
		if( m_pLogPal != NULL ) LocalFree( m_pLogPal );
		m_pLogPal = (tagLOGPALETTE *) LocalAlloc( LPTR, sizeof(LOGPALETTE) +
					( ( CRes - 1 ) * sizeof(PALETTEENTRY) ) );
		m_pLogPal = (LPLOGPALETTE) LocalLock( m_pLogPal );
			//=== allocate and lock memory for palette ===
		m_pLogPal->palNumEntries = CRes;       // fill in size and
		m_pLogPal->palVersion    = 0x0300;     // version (3.0)
		for( i=0; i<CRes; i++ )						// convert colors
		{
			m_pLogPal->palPalEntry[i].peRed   = pRGB[i].rgbRed;
			m_pLogPal->palPalEntry[i].peGreen = pRGB[i].rgbGreen;
			m_pLogPal->palPalEntry[i].peBlue  = pRGB[i].rgbBlue;
			m_pLogPal->palPalEntry[i].peFlags = 0;
		}
		pNewPal = new CPalette();
		pNewPal->CreatePalette( m_pLogPal );
		GlobalUnlock( m_pLogPal );
			//=== now retrive actual image information ===
		hBM = LocalAlloc( LPTR, m_bmImgSize );
		pBits = (BYTE *) LocalLock( hBM );
		cFile.Read( pBits, m_bmImgSize );
		cFile.Close();
			//=== make new palette active ===
		pDC = GetDC();
		CPalette * pOldPal = pDC->SelectPalette( pNewPal, FALSE );
		pDC->RealizePalette();
			//=== create DIB Bitmap ===
		lpBits = (char*) LocalLock( hBM );
		hBitmap = CreateDIBitmap( pDC->m_hDC,
			 (LPBITMAPINFOHEADER) &pBmInfo->bmiHeader,
								  CBM_INIT,
								  lpBits,
				   (LPBITMAPINFO) pBmInfo,
								  DIB_RGB_COLORS );
		while( LOBYTE( LocalFlags( hBM ) ) != 0 )
			 LocalUnlock( hBM );  // remember, this was locked at least twice
		if( hBitmap != NULL )
		{
			if( m_hBitmap )
				DeleteObject( m_hBitmap );				// if exist, delete same ...
			m_hBitmap = hBitmap;							// save global handle
			if( m_pPal )
				m_pPal->DeleteObject();
			m_pPal = pNewPal;								// save global handle
		} else ErrorMsg( "Bitmap failed" );
			//=== discard global memory object ===
		if( pOldPal ) pDC->SelectPalette( pOldPal, FALSE );
		ReleaseDC( pDC );
		GlobalUnlock( hBmInfo );
		LocalFree( hBmInfo );
		delete pNewPal;
		LocalFree( hBM );
		SetCursor( LoadCursor( NULL, IDC_ARROW ) );
		InvalidateRect( NULL, TRUE );
	}
	CATCH( CFileException, e )
	{
		#ifdef _DEBUG
			afxDump << "File access failed: " << e->m_cause << "\n";
		#endif
		return FALSE;
	}
	END_CATCH
	return TRUE;
}

BOOL CTargetView::ErrorMsg( char* Error )
{
	MessageBox( Error, "NOTICE", MB_ICONASTERISK | MB_OK );
	return FALSE;
}

//=== OVERLAY FUNCTIONS =====================================================

void CTargetView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	GetWindowRect( m_cRect );
	m_cPoint.x = ( m_cRect.Width() / 2 );
	m_cPoint.y = ( m_cRect.Height() / 2 );
	DrawOverlay();
	DrawTargets();
}

void CTargetView::DrawTargets()
{	  // draws the mouse target areas using hatched brushes
	CRect		cRect;
	CDC*		pDC = GetDC();
	CPen		cRedPen(	 PS_SOLID, 2, 0x000000FF );
	CBrush*		pBrush;
	CBrush*		pOldBrush;
	LOGBRUSH	logBrush;

	GetClientRect( cRect );
	CPen* pOldPen = pDC->SelectObject( &cRedPen );

	pBrush = new CBrush;
	logBrush.lbStyle = BS_HATCHED;
	logBrush.lbColor = NULL;
	logBrush.lbHatch = HS_FDIAGONAL;	//	A 45-degree downward, left-to-right hatch
	pBrush->CreateBrushIndirect( &logBrush );
	pOldBrush = pDC->SelectObject( pBrush );
		// check TOP_LEFT
	cRect.SetRect( m_cPoint.x - m_xRadius, m_cPoint.y - m_yRadius,
				   m_cPoint.x, m_cPoint.y );
	cRect.NormalizeRect();
	pDC->Rectangle( cRect );
		// check BOTTOM_RIGHT
	cRect.SetRect( m_cPoint.x + m_xRadius, m_cPoint.y + m_yRadius, 
				   m_cPoint.x, m_cPoint.y );
	cRect.NormalizeRect();
	pDC->Rectangle( cRect );
	pDC->SelectObject( pOldBrush );
	delete pBrush;

	pBrush = new CBrush;
	logBrush.lbHatch = HS_BDIAGONAL;
		//	A 45-degree upward, left-to-right hatch
	pBrush->CreateBrushIndirect( &logBrush );
	pOldBrush = pDC->SelectObject( pBrush );
		// check BOTTOM_LEFT
	cRect.SetRect( m_cPoint.x - m_xRadius, m_cPoint.y + m_yRadius,
				   m_cPoint.x, m_cPoint.y );
	cRect.NormalizeRect();
	pDC->Rectangle( cRect );
		// check TOP_RIGHT
	cRect.SetRect( m_cPoint.x + m_xRadius, m_cPoint.y - m_yRadius, 
				   m_cPoint.x, m_cPoint.y );
	cRect.NormalizeRect();
	pDC->Rectangle( cRect );
	pDC->SelectObject( pOldBrush );
	delete pBrush;

	pBrush = new CBrush;
	logBrush.lbHatch = HS_HORIZONTAL;		//	Horizontal hatch
	pBrush->CreateBrushIndirect( &logBrush );
	pOldBrush = pDC->SelectObject( pBrush );
		// LEFT
	cRect.SetRect( m_cPoint.x - m_xRadius - 5, m_cPoint.y - ( m_yRadius / 2 ),
				   m_cPoint.x - ( m_xRadius / 2 ), m_cPoint.y + ( m_yRadius / 2 ) );
	cRect.NormalizeRect();
	pDC->Rectangle( cRect );
		// RIGHT
	cRect.SetRect( m_cPoint.x + ( m_xRadius / 2 ), m_cPoint.y - ( m_yRadius / 2 ),
				   m_cPoint.x + m_xRadius + 5, m_cPoint.y + ( m_yRadius / 2 ) );
	cRect.NormalizeRect();
	pDC->Rectangle( cRect );
	pDC->SelectObject( pOldBrush );
	delete pBrush;

	pBrush = new CBrush;
	logBrush.lbHatch = HS_VERTICAL;			//	Vertical hatch
	pBrush->CreateBrushIndirect( &logBrush );
	pOldBrush = pDC->SelectObject( pBrush );
		// TOP
	cRect.SetRect( m_cPoint.x - ( m_xRadius / 2 ), m_cPoint.y - m_yRadius - 5,
				   m_cPoint.x + ( m_xRadius / 2 ), m_cPoint.y - ( m_yRadius / 2 ) );
	cRect.NormalizeRect();
	pDC->Rectangle( cRect );
		// BOTTOM
	cRect.SetRect( m_cPoint.x - ( m_xRadius / 2 ), m_cPoint.y + ( m_yRadius / 2 ),
				   m_cPoint.x + ( m_xRadius / 2 ), m_cPoint.y + m_yRadius + 5 );
	cRect.NormalizeRect();
	pDC->Rectangle( cRect );
	pDC->SelectObject( pOldBrush );
	delete pBrush;

	pBrush = new CBrush;
	logBrush.lbHatch = HS_CROSS;			//	Horizontal and vertical cross-hatch
	pBrush->CreateBrushIndirect( &logBrush );
	pOldBrush = pDC->SelectObject( pBrush );
		// CENTER
	cRect.SetRect( m_cPoint.x - ( ( m_xRadius / 3 ) * 2 ), m_cPoint.y - ( ( m_yRadius / 3 ) * 2 ), 
				   m_cPoint.x + ( ( m_xRadius / 3 ) * 2 ), m_cPoint.y + ( ( m_yRadius / 3 ) * 2 ) );
	cRect.NormalizeRect();
	pDC->Rectangle( cRect );
	pDC->SelectObject( pOldBrush );
	delete pBrush;
}

void CTargetView::DrawOverlay()
{
	CPen	cRedPen(	 PS_SOLID, 2, 0x000000FF );
	CPen	cCyanPen( PS_SOLID, 2, 0x00FFFF00 );
	CPen	cBluePen( PS_SOLID, 2, 0x00FF0000 );
	CPen*	pOldPen;
	CRect	cRect;
	CDC *	pDC = GetDC();

	pDC->SetROP2( R2_XORPEN );
	GetClientRect( cRect );
	CGdiObject* pOldBrush = pDC->SelectStockObject( HOLLOW_BRUSH );
			// draw crosshairs with empty center
	pOldPen = pDC->SelectObject( &cBluePen );		// select blue pen
	pDC->MoveTo( m_cPoint.x - ( m_xRadius / 2 ), m_cPoint.y );
	pDC->LineTo( cRect.left, m_cPoint.y );
	pDC->MoveTo( m_cPoint.x + ( m_xRadius / 2 ), m_cPoint.y );
	pDC->LineTo( cRect.right, m_cPoint.y );
	pDC->MoveTo( m_cPoint.x, m_cPoint.y - ( m_yRadius / 2 ) );
	pDC->LineTo( m_cPoint.x, cRect.top );
	pDC->MoveTo( m_cPoint.x, m_cPoint.y + ( m_yRadius / 2 ) );
	pDC->LineTo( m_cPoint.x, cRect.bottom );
			// draw ROI as circle
	pDC->SelectObject( &cRedPen );			// select red pen
	cRect.left		= m_cPoint.x - m_xRadius;
	cRect.right		= m_cPoint.x + m_xRadius;
	cRect.top		= m_cPoint.y - m_yRadius;
	cRect.bottom	= m_cPoint.y + m_yRadius;
	pDC->Ellipse( cRect );
	pDC->SelectObject( pOldPen );			// Deselect pen
	pDC->SelectObject( pOldBrush );			// Deselect brush
}

void CTargetView::DrawROITarget()
{
	CPen	cYellowPen( PS_SOLID, 2, 0x0000FFFF );
	CPen*	pOldPen;
	CRect	cRect;
	CDC *	pDC = GetDC();

	pDC->SetROP2( R2_XORPEN );
	CGdiObject* pOldBrush = pDC->SelectStockObject( HOLLOW_BRUSH );
	pOldPen = pDC->SelectObject( &cYellowPen );	// select yellow pen
	cRect.left		= m_cPoint.x - m_xRadius - 1;
	cRect.right		= m_cPoint.x + m_xRadius + 1;
	cRect.top		= m_cPoint.y - m_yRadius - 1;
	cRect.bottom	= m_cPoint.y + m_yRadius + 1;
	pDC->Rectangle( cRect );
	pDC->SelectObject( pOldPen );			// Deselect pen
	pDC->SelectObject( pOldBrush );			// Deselect brush
}

void CTargetView::OnLButtonDown(UINT nFlags, CPoint point) 
{

	m_nTrack = NO_TARGET;
	SetCapture();

	CPoint	cPoint( m_cPoint );
	CRect	cRect( cPoint.x - ( ( m_xRadius / 3 ) * 2 ), 
				   cPoint.y - ( ( m_yRadius / 3 ) * 2 ), 
				   cPoint.x + ( ( m_xRadius / 3 ) * 2 ), 
				   cPoint.y + ( ( m_yRadius / 3 ) * 2 ) );

	cRect.NormalizeRect();
	if( cRect.PtInRect( point ) )	
	{
		m_nTrack = ALL;
		m_hCursor = SetCursor( LoadCursor( theApp.m_hInstance,
										   "HAND_CURSOR" ) );
	}
	else
	{		// check LEFT
		cRect.SetRect( m_cPoint.x - m_xRadius - 5,
					   m_cPoint.y - ( m_yRadius / 2 ),
					   m_cPoint.x - ( m_xRadius / 2 ),
					   m_cPoint.y + ( m_yRadius / 2 ) );
		cRect.NormalizeRect();
		if( cRect.PtInRect( point ) )
		{
			m_nTrack = LEFT;
			m_hCursor = SetCursor( LoadCursor( theApp.m_hInstance,
											   "EW_CURSOR" ) );
			cPoint.x -= m_xRadius;
		}
		else
		{		// check RIGHT
			cRect.SetRect( m_cPoint.x + ( m_xRadius / 2 ), 
						   m_cPoint.y - ( m_yRadius / 2 ),
						   m_cPoint.x + m_xRadius + 5, 
						   m_cPoint.y + ( m_yRadius / 2 ) );
			cRect.NormalizeRect();
			if( cRect.PtInRect( point ) )
			{
				m_nTrack = RIGHT;
				m_hCursor = SetCursor( LoadCursor( theApp.m_hInstance,
												   "EW_CURSOR" ) );
				cPoint.x += m_xRadius;
			}
			else
			{		// check TOP
				cRect.SetRect( m_cPoint.x - ( m_xRadius / 2 ),
							   m_cPoint.y - m_yRadius - 5,
							   m_cPoint.x + ( m_xRadius / 2 ),
							   m_cPoint.y - ( m_yRadius / 2 ) );
				cRect.NormalizeRect();
				if( cRect.PtInRect( point ) )
				{
					m_nTrack = TOP;
					m_hCursor = SetCursor( LoadCursor( theApp.m_hInstance,
													   "NS_CURSOR" ) );
					cPoint.y -= m_yRadius;
				}
				else
				{		// check BOTTOM
					cRect.SetRect( m_cPoint.x - ( m_xRadius / 2 ),
								   m_cPoint.y + ( m_yRadius / 2 ),
								   m_cPoint.x + ( m_xRadius / 2 ), 
								   m_cPoint.y + m_yRadius + 5 );
					cRect.NormalizeRect();
					if( cRect.PtInRect( point ) )
					{
						m_nTrack = BOTTOM;
						m_hCursor = SetCursor( LoadCursor( theApp.m_hInstance,
														   "NS_CURSOR" ) );
						cPoint.y += m_yRadius;
					}
					else
					{		// check TOP_LEFT
						cRect.SetRect( m_cPoint.x - m_xRadius, 
									   m_cPoint.y - m_yRadius,
									   m_cPoint.x,  m_cPoint.y );
						cRect.NormalizeRect();
						if( cRect.PtInRect( point ) ) 
						{
							m_nTrack = TOP_LEFT;
							m_hCursor = SetCursor( LoadCursor( theApp.m_hInstance,
															   "NWSE_CURSOR" ) );
							cPoint.x -= m_xRadius;
							cPoint.y -= m_yRadius;
						}
						else
						{		// check TOP_RIGHT
							cRect.SetRect( m_cPoint.x + m_xRadius,
								           m_cPoint.y - m_yRadius, 
										   m_cPoint.x,  m_cPoint.y );
							cRect.NormalizeRect();
							if( cRect.PtInRect( point ) ) 
							{
								m_nTrack = TOP_RIGHT;
								m_hCursor = SetCursor( LoadCursor( theApp.m_hInstance,
																   "NESW_CURSOR" ) );
								cPoint.x += m_xRadius;
								cPoint.y -= m_yRadius;
							}
							else
							{		// check BOTTOM_LEFT
								cRect.SetRect( m_cPoint.x - m_xRadius,
											   m_cPoint.y + m_yRadius,
											   m_cPoint.x,	m_cPoint.y );
								cRect.NormalizeRect();
								if( cRect.PtInRect( point ) )
								{
									m_nTrack = BOTTOM_LEFT;
									m_hCursor = SetCursor( LoadCursor( theApp.m_hInstance,
																	   "NESW_CURSOR" ) );
									cPoint.x -= m_xRadius;
									cPoint.y += m_yRadius;
								}
								else
								{		// check BOTTOM_RIGHT
									cRect.SetRect( m_cPoint.x + m_xRadius,
												   m_cPoint.y + m_yRadius, 
												   m_cPoint.x,  m_cPoint.y );
									cRect.NormalizeRect();
									if( cRect.PtInRect( point ) )
									{
										m_nTrack = BOTTOM_RIGHT;
										m_hCursor = SetCursor( LoadCursor( theApp.m_hInstance,
																		   "NWSE_CURSOR" ) );
										cPoint.x += m_xRadius;
										cPoint.y += m_yRadius;
									}
								}
							}
						}
					}
				}
			}
		}
	}
	m_bDrawOverlay = TRUE;
	CView::OnLButtonDown(nFlags, point);
}

void CTargetView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CRect		cRect;

	if( m_bDrawOverlay )
	{
		GetClientRect( cRect );
		if( ! cRect.PtInRect( point ) )	// cursor has moved outside of client area
		{
			m_nTrack = NO_TARGET;
			SetCursor( m_hCursor );
			ReleaseCapture();
			return;
		}
		DrawOverlay();					// erase the overlay target
		switch( m_nTrack )
		{
			case NO_TARGET:			/* no action */
				break;

			case ALL:
				m_cPoint.x = max( m_xRadius, 
								  min( point.x, 
									   ( cRect.right - m_xRadius - 2 ) ) );
				m_cPoint.y = max( m_yRadius, 
								  min( point.y, 
									   ( cRect.bottom - m_yRadius - 2 ) ) );
				
				break;

			case TOP:
				m_yRadius = max( 30, m_cPoint.y - point.y );
				break;

			case LEFT:
				m_xRadius = max( 30, m_cPoint.x - point.x );
				break;

			case RIGHT:
				m_xRadius = max( 30, point.x - m_cPoint.x );
				break;

			case BOTTOM:
				m_yRadius = max( 30, point.y - m_cPoint.y );
				break;

  			case TOP_LEFT:
				m_xRadius = max( 30, m_cPoint.x - point.x );
				m_yRadius = max( 30, m_cPoint.y - point.y );
				break;

			case TOP_RIGHT:
				m_xRadius = max( 30, point.x - m_cPoint.x );
				m_yRadius = max( 30, m_cPoint.y - point.y );
				break;

			case BOTTOM_LEFT:
				m_xRadius = max( 30, m_cPoint.x - point.x );
				m_yRadius = max( 30, point.y - m_cPoint.y );
				break;

			case BOTTOM_RIGHT:
				m_xRadius = max( 30, point.x - m_cPoint.x );
				m_yRadius = max( 30, point.y - m_cPoint.y );
				break;
		}
			//=== check results ===
		switch( m_nTrack )
		{
			case TOP:
			case LEFT:
			case RIGHT:
			case BOTTOM:
  			case TOP_LEFT:
			case TOP_RIGHT:
			case BOTTOM_LEFT:
			case BOTTOM_RIGHT:
			{
				while( ( m_cPoint.x - m_xRadius ) <= cRect.left   ) m_xRadius--;
				while( ( m_cPoint.x + m_xRadius ) >= cRect.right  ) m_xRadius--;
				while( ( m_cPoint.y - m_yRadius ) <= cRect.top    ) m_yRadius--;
				while( ( m_cPoint.y + m_yRadius ) >= cRect.bottom ) m_yRadius--;
				if( m_xRadius < 30 )		m_xRadius = 30;
				if( m_yRadius < 30 )		m_xRadius = 30;
			}
		}
		DrawOverlay();					// redraw the overlay target
	}
	CView::OnMouseMove(nFlags, point);
}

void CTargetView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	m_bDrawOverlay = FALSE;
	m_nTrack = NO_TARGET;
	SetCursor( m_hCursor );
	ReleaseCapture();
	CView::OnLButtonUp(nFlags, point);
}

void CTargetView::OnRButtonDown(UINT nFlags, CPoint point) 
{
/*	
	//=== routine to show target areas ===//
	DrawOverlay();
	DrawTargets();
	DrawOverlay();
*/
	CRect	cRect( m_cPoint.x - ( ( m_xRadius / 3 ) * 2 ), 
				   m_cPoint.y - ( ( m_yRadius / 3 ) * 2 ), 
				   m_cPoint.x + ( ( m_xRadius / 3 ) * 2 ), 
				   m_cPoint.y + ( ( m_yRadius / 3 ) * 2 ) );

	if( cRect.PtInRect( point ) )			// is cursor in client area?
	{
		m_bCapture = TRUE;
		DrawOverlay();
		SetCapture();
		DrawROITarget();
	}
	CView::OnRButtonDown(nFlags, point);
}

void CTargetView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	if( m_bCapture )
	{
		m_bCapture = ! m_bCapture;
		ReleaseCapture();
		Invalidate();
		DrawOverlay();
	}
	CView::OnRButtonUp(nFlags, point);
}
