//=========================//
//          Life.C         //
//  Windows NT Modes Demo  //
//=========================//

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include "life.h"

#define  GRID      50
#define  ID_TIMER  1
#define  APP_ICON  "Life"
#define  APP_MENU  "Life"
// note: no cursor assignment

#ifndef __BORLANDC__
#define random(x)  ( rand() % (int)( x ) )
#endif

HANDLE hInst;
char   szAppTitle[] = "Life Application";
char   szAppName[]  = "Life";

long FAR PASCAL WndProc( HWND hwnd,   UINT msg,
                         UINT wParam, LONG lParam )
{
	static  HWND        hMenu;
	static  int         ThisGeneration[GRID][GRID],
						LastGeneration[GRID][GRID],
						MapMode;
	static unsigned int nGenerations = 0, Population;
	int                 Neighbors, i, j, k, l, x, y;
	char                szBuff[64];
	HDC                 hdc;
	RECT                rect;
	PAINTSTRUCT         ps;

	switch( msg )
	{
		case WM_CREATE:
			MapMode = MM_ANISOTROPIC;
			PostMessage( hwnd, WM_COMMAND, IDM_GRIDCLEAR, 0L );
			wsprintf( szBuff, "LIFE     Generation: %d", nGenerations );
			SetWindowText( hwnd, szBuff );
			hMenu = GetMenu( hwnd );
			CheckMenuItem( hMenu, IDM_TIMESTOP,    MF_CHECKED );
			CheckMenuItem( hMenu, IDM_ANISOTROPIC, MF_CHECKED );
			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDM_TIMESTOP:
					CheckMenuItem( hMenu, IDM_TIMESTOP,  MF_CHECKED );
					CheckMenuItem( hMenu, IDM_TIMESTART, MF_UNCHECKED );
					KillTimer( hwnd, ID_TIMER );
					break;

				case IDM_TIMESTART:
					CheckMenuItem( hMenu, IDM_TIMESTOP,  MF_UNCHECKED );
					CheckMenuItem( hMenu, IDM_TIMESTART, MF_CHECKED );
					SetTimer( hwnd, ID_TIMER, 500, NULL );
					break;

				case IDM_GRIDCLEAR:
					for( i=0; i<GRID; i++ )
						for( j=0; j<GRID; j++ )
							LastGeneration[i][j] =
							ThisGeneration[i][j] = FALSE;
					PostMessage( hwnd, WM_COMMAND, IDM_TIMESTOP, 0L );
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				case IDM_RANDOM:
#ifdef __BORLANDC__
					randomize();
#endif
					for( i=0; i<1000; i++ )
						ThisGeneration[random(GRID)][random(GRID)] = TRUE;
					PostMessage( hwnd, WM_COMMAND, IDM_TIMESTART, 0L );
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				case IDM_LAUNCH:
#ifdef __BORLANDC__
					randomize();
#endif
					x = random(45)+1;
					y = random(45)+1;
					ThisGeneration[x+2][y+1] =
					ThisGeneration[x+3][y+1] =
					ThisGeneration[x+4][y+1] =
					ThisGeneration[x+3][y+2] =
					ThisGeneration[x+1][y+3] =
					ThisGeneration[x+2][y+3] =
					ThisGeneration[x+3][y+3] = TRUE;
					PostMessage( hwnd, WM_COMMAND, IDM_TIMESTART, 0L );
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				case IDM_ISOTROPIC:
					MapMode  = MM_ISOTROPIC;
					CheckMenuItem( hMenu, IDM_ISOTROPIC, MF_CHECKED );
					CheckMenuItem( hMenu, IDM_ANISOTROPIC, MF_UNCHECKED );
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				case IDM_ANISOTROPIC:
					MapMode = MM_ANISOTROPIC;
					CheckMenuItem( hMenu, IDM_ANISOTROPIC, MF_CHECKED );
					CheckMenuItem( hMenu, IDM_ISOTROPIC, MF_UNCHECKED );
					InvalidateRect( hwnd, NULL, TRUE );
					break;

				default:	return( DefWindowProc( hwnd, msg, wParam, lParam ) );

			}
			break;

		case WM_PAINT:
			hdc = BeginPaint( hwnd, &ps );
			GetClientRect( hwnd, &rect );
			SetMapMode( hdc, MapMode );
			SetWindowOrgEx( hdc, 0, 0, NULL );
			SetWindowExtEx( hdc, 110000/rect.right, 110000/rect.bottom, NULL );
			SetViewportExtEx( hdc, 2000, 2000, NULL );
			for( i=0; i<GRID; i++ )
				for( j=0; j<GRID; j++ )
					if( ThisGeneration[i][j] )
					{
						SetRect( &rect, i+3, j+3, i+4, j+4 );
						FillRect( hdc, &rect, GetStockObject( BLACK_BRUSH ) );
					}
			EndPaint( hwnd, &ps );
			break;

		case WM_TIMER:
			Population = 0;
			for( i=0; i<GRID; i++ )
				for( j=0; j<GRID; j++ )
				{
					LastGeneration[i][j] = ThisGeneration[i][j];
					if( ThisGeneration[i][j] ) Population++;
				}
			if( !Population )
			{
				PostMessage( hwnd, WM_COMMAND, IDM_TIMESTOP, 0L );
				return(0);
			}
			for( i=0; i<GRID; i++ )
				for( j=0; j<GRID; j++ )
				{
				if( LastGeneration[i][j] ) Neighbors = -1;
									  else Neighbors =  0;
				for( k=-1; k<=1; k++ )
				{
					x = i + k;
					if( x <     0 ) x += GRID;
					if( x >= GRID ) x -= GRID;
					for( l=-1; l<=1; l++ )
					{
						y = j + l;
						if( y <     0 ) y += GRID;
						if( y >= GRID ) y -= GRID;
						if( LastGeneration[x][y] ) Neighbors++;
				}	}
				switch( Neighbors )
				{
					case 2:
						ThisGeneration[i][j] = LastGeneration[i][j];
						break;

					case 3:
						ThisGeneration[i][j] = TRUE;
						break;

					default:
						ThisGeneration[i][j] = FALSE;
			}	}
			wsprintf( szBuff, "LIFE     Generation: %d", nGenerations++ );
			SetWindowText( hwnd, szBuff );
			InvalidateRect( hwnd, NULL, TRUE );
			break;

		case WM_DESTROY:
			KillTimer( hwnd, ID_TIMER );
			PostQuitMessage(0);
			break;

		default:	return( DefWindowProc( hwnd, msg, wParam, lParam ) );
	}
	return( 0 );
}

#include "template.i"

#ifdef __BORLANDC__
#pragma argsused
#endif

int APIENTRY WinMain( HANDLE hInstance, HANDLE hPrevInstance,
                      LPSTR  lpCmdLine, int    nCmdShow    )
{
	MSG   msg;

	if( ! hPrevInstance )
		if( ! InitApplication( hInstance ) )
			return( FALSE );
	if( ! InitInstance( hInstance, nCmdShow ) )
		return (FALSE);
	while( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( msg.wParam );
#ifndef __BORLANDC__
	UNREFERENCED_PARAMETER( lpCmdLine );
#endif
}
