/*--------------------------------------------------------------------

        WINMAIN.C

        Routines from the SHOWWAVE program to register and create
        the main program window.

        PUBLIC FUNCTIONS
            WinMain         create windows and poll for messages
            About_DlgProc   handle messages for the About box

        PRIVATE FUNCTION
            RegisterShowWaveClass   create class for main window

        program by Michael Barnes
        copyright 1992, 1993 by Brian Myers, Chris Doner, & Eric Hamer
        from Mastering Windows NT Programming

  --------------------------------------------------------------------*/

//======================================================================//
//																								//
//				NOTE: For a Microsoft project, include winmm.lib				//
//						under Build / Settings / Link									//
//														-- Ben Ezzell (1996)				//
//																								//
//======================================================================//


#define STRICT
#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>           // multimedia definitions
#include "showwave.h"           // program variables and functions
#include "rsrc.h"               // program resource id definitions

/*--------------------------------------------------------------------
        FUNCTION PROTOTYPE
  --------------------------------------------------------------------*/

int WINAPI WinMain( HINSTANCE hinstThis, HINSTANCE hinstPrev,
    LPSTR lpszCmdLine, int iCmdShow );
BOOL RegisterShowWaveClass( HINSTANCE hInstance );

/*--------------------------------------------------------------------
        GLOBAL VARIABLES
  --------------------------------------------------------------------*/

LPSTR lpWaveData      = NULL;   // huge pointer to sound data
DWORD dwWaveDataSize  = 0;      // size of lpWaveData buffer
DWORD dwCurrentSample = 0;      // position in the current sound
DWORD dwSamplesPerSec = 22050;  // default sampling rate

/*--------------------------------------------------------------------
        STATIC VARIABLE
  --------------------------------------------------------------------*/

static char szShowWaveClass[] = "ShowWaveClass";

/*--------------------------------------------------------------------
        WINMAIN
  --------------------------------------------------------------------*/

int WINAPI WinMain (
    HINSTANCE hinstThis,
    HINSTANCE hinstPrev,
    LPSTR lpszCmdLine,
    int iCmdShow )
{
    if (!RegisterGraphClass( hinstThis ))       // wave graph 
    {
        return( 0 );
    }

    if (!RegisterShowWaveClass( hinstThis ))    // program window
    {
        return( 0 );
    }

    /* create and display the main program window */
    DialogBox( hinstThis,
        MAKEINTRESOURCE(DLG_MAIN),
        NULL,
        (DLGPROC)ShowWave_WndProc );
    
    return( 0 );
    UNREFERENCED_PARAMETER( hinstPrev );
    UNREFERENCED_PARAMETER( iCmdShow );
    UNREFERENCED_PARAMETER( lpszCmdLine );
}

/*--------------------------------------------------------------------
        ABOUT DIALOG PROCEDURE
        Handle messages for the About box.
  --------------------------------------------------------------------*/

BOOL WINAPI About_DlgProc (
    HWND hDlg,
    UINT uMsg,
    WPARAM wParam,
    LPARAM lParam )
{
    switch (uMsg)
    {
        case WM_INITDIALOG:
            return( TRUE );
            
        case WM_COMMAND:
            EndDialog( hDlg, 0 );
            return( TRUE );;
            
        default:
            return( FALSE );
    }
    
    UNREFERENCED_PARAMETER( uMsg );
    UNREFERENCED_PARAMETER( wParam );
    UNREFERENCED_PARAMETER( lParam );
}

/*--------------------------------------------------------------------
        REGISTER SHOWWAVE CLASS
        Register a window class for the main dialog box window.
  --------------------------------------------------------------------*/

BOOL RegisterShowWaveClass ( HINSTANCE hInstance )
{
    WNDCLASS WndClass;

    WndClass.style          = 0;
    WndClass.lpfnWndProc    = ShowWave_WndProc;
    WndClass.cbClsExtra     = 0;
    WndClass.cbWndExtra     = DLGWINDOWEXTRA;
    WndClass.hInstance      = hInstance;
    WndClass.hIcon          = LoadIcon( hInstance, MAKEINTRESOURCE(IDI_WAVE) );
    WndClass.hCursor        = LoadCursor( NULL, IDC_ARROW );
    WndClass.hbrBackground  = (HBRUSH)(COLOR_BTNFACE + 1);
    WndClass.lpszMenuName   = MAKEINTRESOURCE( IDM_MAIN );
    WndClass.lpszClassName  = szShowWaveClass;

    return( RegisterClass(&WndClass) );
}
