; Copyright (C)  Michael Holzrichter 1992,  All rights reserved.

        title   SVADD.ASM:  Scalar-Vector Add
        page    55,132

; Call with:    svadd (float a, float* b, int str_b, float* c, int str_c, int n)
;
; Returns:      c [i*str_b] =  a + b [i*str_b] for i = 0 to n-1
;
; Destroys:     nothing

.286
.287

;.MODEL SMALL
.MODEL LARGE
.CODE

N_ELM     equ     bp+6
C_STR     equ     bp+8
C_OFF     equ     bp+10
C_SEG     equ     bp+12
B_STR     equ     bp+14
B_OFF     equ     bp+16
B_SEG     equ     bp+18
A_ADD     equ     bp+20

          PUBLIC  _svadd__FfPfiPfii

_svadd__FfPfiPfii PROC
          push    bp                      ; Entry sequence - save old BP
          mov     bp,sp                   ; Set stack framepointer

          push    si                      ; Save registers
          push    di
          push    es
          push    ds

          push    cx
          push    bx

          mov     cx, [N_ELM]             ; Get number of elements

          cmp     cx, 0                   ; Check for positive vector length
          jng     error

          mov     si, [B_OFF]             ; Get address of B array
          mov     ds, [B_SEG]

          mov     di, [C_OFF]             ; Get address of C array
          mov     es, [C_SEG]

          mov     ax, 4                   ; Compute stride of C
          mul     byte ptr [C_STR]
          mov     bx, ax

          mov     ax, 4                   ; Compute stride of B
          mul     byte ptr [B_STR]

          fld     dword ptr [A_ADD]       ; Load constant

loop1:    fld     st                      ; Make copy of constant
          fadd    dword ptr ds:[si]       ; Perform addition

          fstp    dword ptr es:[di]       ; Store result

          add     si, ax                  ; Increment pointers
          add     di, bx

          loop    loop1                   ; Repeat until cx is 0

          fstp    st                      ; Pop constant off stack
          fwait                           ; Wait for processor to catch up

          xor     ax, ax                  ; Return code is 0

exit:     pop     bx                      ; Restore registers
          pop     cx

          pop     ds
          pop     es
          pop     di
          pop     si

          pop     bp

          RET     18                      ; Return

error:    mov     ax, 1
          jmp     exit

_svadd__FfPfiPfii ENDP
          END
