; Copyright (C)  Michael Holzrichter 1992,  All rights reserved.

        title   VMNMX.ASM:  Find Sum of Elements of Vector
        page    55,132

; Call with:    vsmnmx (float* a, int str_a, float* b, int n)
;
; Returns:      b [0] = Min      (a [i * str_a])  for i = 0 to n-1
;               b [1] = Max      (a [i * str_a])  for i = 0 to n-1

; Call with:    vamnmx (float* a, int str_a, float* b, int n)
;
; Returns:      b [0] = Min (Abs (a [i * str_a])) for i = 0 to n-1
;               b [1] = Max (Abx (a [i * str_a])) for i = 0 to n-1
;
; Destroys:     nothing

.286
.287

;.MODEL SMALL
.MODEL LARGE
.CODE

N_ELM     equ     bp+6
B_OFF     equ     bp+8
B_SEG     equ     bp+10
A_STR     equ     bp+12
A_OFF     equ     bp+14
A_SEG     equ     bp+16

          PUBLIC  _vsmnmx__FPfiPfi, _vamnmx__FPfiPfi

_vsmnmx__FPfiPfi PROC
          push    bp                      ; Entry sequence - save old BP
          mov     bp,sp                   ; Set stack framepointer

          push    si                      ; Save registers
          push    ds

          push    cx
          push    bx

          mov     cx, [N_ELM]             ; Get number of elements

          cmp     cx, 0                   ; Check for positive vector length
          jng     errors

          mov     si, [A_OFF]             ; Get address of A array
          mov     ds, [A_SEG]

          mov     ax, 4                   ; Compute stride
          mul     word ptr [A_STR]
          mov     bx, ax

          fld     dword ptr ds:[si]       ; Load first value
          fld     st

          add     si, bx
          dec     cx
          jz      getmnmxs

loop1s:   fld     dword ptr ds:[si]       ; Get next element

          fcom    st(1)                   ; Compare with current min
          fstsw   ax
          sahf
          ja      label1s
          fst     st(1)

label1s:  fcom    st(2)                   ; Compare with current max
          fstsw   ax
          sahf
          jb      label2s
          fst     st(2)

label2s:  fstp    st(0)
          
          add     si, bx                  ; Increment pointers

          loop    loop1s                  ; Repeat until cx is 0

getmnmxs: fwait                           ; Wait for processor to catch up

          mov     si, [B_OFF]             ; Save Result
          mov     ds, [B_SEG]
          fstp    dword ptr ds:[si]

          add     si, 4
          fstp    dword ptr ds:[si]

          xor     ax, ax                  ; Return code is 0

exits:    pop     bx                      ; Restore registers
          pop     cx

          pop     ds
          pop     si

          pop     bp

          RET     12                      ; Return

errors:   mov     ax, 1
          jmp     exits

_vsmnmx__FPfiPfi ENDP

_vamnmx__FPfiPfi PROC
          push    bp                      ; Entry sequence - save old BP
          mov     bp,sp                   ; Set stack framepointer

          push    si                      ; Save registers
          push    ds

          push    cx
          push    bx

          mov     cx, [N_ELM]             ; Get number of elements

          cmp     cx, 0                   ; Check for positive vector length
          jng     errora

          mov     si, [A_OFF]             ; Get address of A array
          mov     ds, [A_SEG]

          mov     ax, 4                   ; Compute stride
          mul     word ptr [A_STR]
          mov     bx, ax

          fld     dword ptr ds:[si]       ; Initialize min and max values
          fabs
          fld     st

          add     si, bx
          dec     cx
          jz      getmnmxa

loop1a:   fld     dword ptr ds:[si]       ; Get next element
          fabs                            ; Find magnitude

          fcom    st(1)                   ; Compare with current min
          fstsw   ax
          sahf
          ja      label1a
          fst     st(1)

label1a:  fcom    st(2)                   ; Compare with current max
          fstsw   ax
          sahf
          jb      label2a
          fst     st(2)


label2a:  fstp    st(0)
          
          add     si, bx                  ; Increment pointers

          loop    loop1a                  ; Repeat until cx is 0

getmnmxa: fwait                           ; Wait for processor to catch up

          mov     si, [B_OFF]             ; Save Result
          mov     ds, [B_SEG]
          fstp    dword ptr ds:[si]

          add     si, 4
          fstp    dword ptr ds:[si]

          xor     ax, ax                  ; Return code is 0

exita:    pop     bx                      ; Restore registers
          pop     cx

          pop     ds
          pop     si

          pop     bp

          RET     12                      ; Return

errora:   mov     ax, 1
          jmp     exita

_vamnmx__FPfiPfi ENDP
          END

