; Copyright (C)  Michael Holzrichter 1992,  All rights reserved.

        title   VSC387.ASM:  Take Sine or Cosine of Elements of Vector Using 387
        page    55,132

; Call with:    vsin387 (float* a, int str_a, float* b, int str_b, int n)
;               vcos387 (float* a, int str_a, float* b, int str_b, int n)
;
; Returns:      b [i * str_b] = sin (a [i * str_a]) for i = 0 to n-1
;               b [i * str_b] = cos (a [i * str_b]) for i = 0 to n-1
;
; Destroys:     nothing

.286
.287

;.MODEL SMALL
.MODEL LARGE
.CODE


N_ELM     equ     bp+6
B_STR     equ     bp+8
B_OFF     equ     bp+10
B_SEG     equ     bp+12
A_STR     equ     bp+14
A_OFF     equ     bp+16
A_SEG     equ     bp+18

          PUBLIC  _vsin387__FPfiPfii, _vcos387__FPfiPfii

_vsin387__FPfiPfii PROC
          push    bp                      ; Entry sequence - save old BP
          mov     bp,sp                   ; Set stack framepointer

          push    si                      ; Save registers
          push    di
          push    es
          push    ds

          push    dx
          push    cx
          push    bx

          mov     cx, [N_ELM]             ; Get number of elements

          cmp     cx, 0                   ; Check for positive vector length
          jng     serror

          mov     si, [A_OFF]             ; Get address of A array
          mov     ds, [A_SEG]

          mov     di, [B_OFF]             ; Get address of B array
          mov     es, [B_SEG]

          mov     ax, 4                   ; Compute stride of destination
          mul     word ptr [B_STR]
          mov     bx, ax

          mov     ax, 4                   ; Compute stride of source
          mul     word ptr [A_STR]             

sloop1:   fld     dword ptr ds:[si]       ; Load operand
.386
.387
          fsin                            ; Take square root
.286
.287
          fstp    dword ptr es:[di]       ; Save result

          add     si, ax                  ; Increment pointers
          add     di, bx

          loop    sloop1                  ; Repeat until cx is 0

          fwait                           ; Wait for processor to catch up

          xor     ax, ax                  ; Return code is 0

sexit:    pop     bx                      ; Restore registers
          pop     cx
          pop     dx

          pop     ds
          pop     es
          pop     di
          pop     si

          pop     bp

          RET     14                      ; Return

serror:   mov     ax, 1
          jmp     sexit

_vsin387__FPfiPfii ENDP

_vcos387__FPfiPfii PROC
          push    bp                      ; Entry sequence - save old BP
          mov     bp,sp                   ; Set stack framepointer

          push    si                      ; Save registers
          push    di
          push    es
          push    ds

          push    dx
          push    cx
          push    bx

          mov     cx, [N_ELM]             ; Get number of elements

          cmp     cx, 0                   ; Check for positive vector length
          jng     cerror

          mov     si, [A_OFF]             ; Get address of A array
          mov     ds, [A_SEG]

          mov     di, [B_OFF]             ; Get address of B array
          mov     es, [B_SEG]

          mov     ax, 4                   ; Compute stride of destination
          mul     word ptr [B_STR]
          mov     bx, ax

          mov     ax, 4                   ; Compute stride of source
          mul     word ptr [A_STR]             

cloop1:   fld     dword ptr ds:[si]       ; Load operand
.386
.387
          fcos                            ; Take square root
.286
.287
          fstp    dword ptr es:[di]       ; Save result

          add     si, ax                  ; Increment pointers
          add     di, bx

          loop    cloop1                  ; Repeat until cx is 0

          fwait                           ; Wait for processor to catch up

          xor     ax, ax                  ; Return code is 0

cexit:    pop     bx                      ; Restore registers
          pop     cx
          pop     dx

          pop     ds
          pop     es
          pop     di
          pop     si

          pop     bp

          RET     14                      ; Return

cerror:   mov     ax, 1
          jmp     cexit

_vcos387__FPfiPfii ENDP
          END

