; Copyright (C)  Michael Holzrichter 1992,  All rights reserved.

        title   VSIGN0.ASM:  Apply sign of one vector to magnitude of another
        page    55,132

; Call with:    vsign0 (float* a, int str_a, float* b, int str_b, int n)
;
; Returns:      b [i * str_b] = sign (a [i * str_a]) for i = 0 to n-1
;
; Destroys:     nothing

; Call with:    vsign1 (float* a, int str_a, float* b, int str_b, float* c, int str_c, int n)
;
; Returns:      c [i*str_c] =  sign (a [i*str_a]) * abs (b [i*str_b]) for i = 0 to n-1
;
; Destroys:     nothing

.286
.287

;.MODEL SMALL
.MODEL LARGE
.CODE

N_ELM0    equ     bp+6
B_STR0    equ     bp+8
B_OFF0    equ     bp+10
B_SEG0    equ     bp+12
A_STR0    equ     bp+14
A_OFF0    equ     bp+16
A_SEG0    equ     bp+18

N_ELM1    equ     bp+6
C_STR1    equ     bp+8
C_OFF1    equ     bp+10
C_SEG1    equ     bp+12
B_STR1    equ     bp+14
B_OFF1    equ     bp+16
B_SEG1    equ     bp+18
A_STR1    equ     bp+20
A_OFF1    equ     bp+22
A_SEG1    equ     bp+24

          PUBLIC  _vsign0__FPfiPfii, _vsign1__FPfiPfiPfii

_vsign0__FPfiPfii PROC
          push    bp                      ; Save old stack pointer
          mov     bp, sp

          push    si                      ; Save registers
          push    di
          push    es
          push    ds

          push    dx
          push    cx
          push    bx

          mov     cx, [N_ELM0]            ; Get number of elements

          cmp     cx, 0                   ; Check for positive vector length
          jng     error0

          mov     si, [A_OFF0]            ; Get address of A array
          mov     ds, [A_SEG0]

          mov     di, [B_OFF0]            ; Get address of B array
          mov     es, [B_SEG0]

          mov     ax, word ptr [B_STR0]   ; Compute stride of destination
          dec     ax
          shl     ax, 2
          mov     dx, ax

          mov     ax, word ptr [A_STR0]   ; Compute stride of source
          shl     ax, 2
          sub     ax, 2
          mov     bx, ax

          cld                             ; Initialize direction flag

          lodsw                           ; Align pointers with MSB

loop0:    xor     ax, ax                  ; Store zero as first word
          stosw

          lodsw                           ; Get second word
          and     ax, 8000h               ; Mask off sign bit
          or      ax, 3f80h               ; OR in magnitude of 1.0
          stosw                           ; Store second word

          add     si, bx                  ; Increment pointers
          add     di, dx

          loop    loop0                   ; Repeat until cx is 0

          xor     ax, ax                  ; Return code is 0

exit0:    pop     bx                      ; Restore registers
          pop     cx
          pop     dx

          pop     ds
          pop     es
          pop     di
          pop     si

          mov     sp, bp                  ; Release memory for local variable
          pop     bp

          RET     14                      ; Return

error0:   mov     ax, 1
          jmp     exit0

_vsign0__FPfiPfii ENDP

_vsign1__FPfiPfiPfii PROC
          push    bp                      ; Entry sequence - save old BP
          mov     bp,sp                   ; Set stack framepointer

          push    si                      ; Save registers
          push    di
          push    es
          push    ds

          push    dx
          push    cx
          push    bx
          push    ax

          mov     cx, [N_ELM1]            ; Get number of elements

          cmp     cx, 0                   ; Check for positive vector length
          jng     error1

          mov     si, [A_OFF1]            ; Get address of A array
          mov     ds, [A_SEG1]

          mov     di, [C_OFF1]            ; Get address of C array
          mov     es, [C_SEG1]

          mov     ax, [C_STR1]            ; Compute stride of destination
          shl     ax, 2
          sub     ax, 2
          mov     dx, ax

          mov     ax, [A_STR1]            ; Compute stride of source A
          shl     ax, 2
          sub     ax, 2
          mov     bx, ax

          cld                             ; Clear direction flag

          add     si, 2                   ; Point to second byte
          add     di, 2

loop1:    lodsw                           ; Get second word
          and     ax, 8000h               ; Mask all but sign bit
          stosw                           ; Store in destination

          add     si, bx                  ; Increment pointers
          add     di, dx

          loop    loop1                   ; Repeat until cx is 0


          mov     cx, [N_ELM1]            ; Get number of elements

          mov     si, [B_OFF1]            ; Get address of B array
          mov     ds, [B_SEG1]

          mov     di, [C_OFF1]            ; Get address of C array
          mov     es, [C_SEG1]

          mov     ax, [C_STR1]            ; Compute stride of destination
          shl     ax, 2
          sub     ax, 2
          mov     dx, ax

          mov     ax, [B_STR1]            ; Compute stride of source B
          dec     ax
          shl     ax, 2
          mov     bx, ax

loop2:    movsw                           ; Move first word

          lodsw                           ; Get second word
          and     ax, 7fffh               ; Mask all but sign bit
          or      es:[di], ax             ; OR with sign of 

          add     si, bx                  ; Increment pointers
          add     di, dx

          loop    loop2                   ; Repeat until cx is 0

          xor     ax, ax                  ; Return code is 0

exit1:    pop     ax                      ; Restore registers
          pop     bx
          pop     cx
          pop     dx

          pop     ds
          pop     es
          pop     di
          pop     si

          mov     sp, bp                  ; Release memory for local variable
          pop     bp

          RET     20                      ; Return

error1:   mov     ax, 1                   ; Load error code
          jmp     exit1

_vsign1__FPfiPfiPfii ENDP
          END
          END


