; Copyright (C)  Michael Holzrichter 1992,  All rights reserved.

        title   VVMUL.ASM:  Multiply Two Vectors
        page    55,132

; Call with:    vvmul (float* a, int str_a, float* b, int str_b, float* c, int str_c, int n)
;
; Returns:      c [i*str_b] =  a [i*str_a] * b [i*str_b] for i = 0 to n-1
;
; Destroys:     nothing

.286
.287

;.MODEL SMALL
.MODEL LARGE
.CODE

N_ELM     equ     bp+6
C_STR     equ     bp+8
C_OFF     equ     bp+10
C_SEG     equ     bp+12
B_STR     equ     bp+14
B_OFF     equ     bp+16
B_SEG     equ     bp+18
A_STR     equ     bp+20
A_OFF     equ     bp+22
A_SEG     equ     bp+24

          PUBLIC  _vvmul__FPfiPfiPfii

_vvmul__FPfiPfiPfii PROC
          push    bp                      ; Entry sequence - save old BP
          mov     bp,sp                   ; Set stack framepointer

          sub     sp, 2                   ; Get memory for local variable

          push    si                      ; Save registers
          push    di
          push    es
          push    ds

          push    dx
          push    cx
          push    bx
          push    ax

          mov     cx, [N_ELM]             ; Get number of elements

          cmp     cx, 0                   ; Check for positive vector length
          jng     error

          mov     si, [A_OFF]             ; Get address of A array
          mov     ds, [A_SEG]

          mov     di, [B_OFF]             ; Get address of B array
          mov     es, [B_SEG]

          mov     ax, [C_OFF]             ; Store C address in local variable
          mov     [bp-2], ax

          mov     ax, 4                   ; Compute stride of destination
          mul     byte ptr [C_STR]
          mov     dx, ax

          mov     ax, 4                   ; Compute stride of source B
          mul     byte ptr [B_STR]
          mov     bx, ax

          mov     ax, 4                   ; Compute stride of source A
          mul     byte ptr [A_STR]             

loop1:    fld     dword ptr ds:[si]       ; Load and multiply operands
          fmul    dword ptr es:[di]

          add     si, ax                  ; Increment pointers
          add     di, bx

          xchg    di, [bp-2]              ; Get destination address
          mov     es, [C_SEG]

          fstp    dword ptr es:[di]       ; Store result

          add     di, dx                  ; Increment pointer

          xchg    di, [bp-2]              ; Restore B address
          mov     es, [B_SEG]

          loop    loop1                   ; Repeat until cx is 0

          fwait                           ; Wait for processor to catch up

          xor     ax, ax                  ; Return code is 0

exit:     pop     ax                      ; Restore registers
          pop     bx
          pop     cx
          pop     dx

          pop     ds
          pop     es
          pop     di
          pop     si

          mov     sp, bp                  ; Release memory for local variable
          pop     bp

          RET     20                      ; Return

error:    mov     ax, 1                   ; Load error code
          jmp     exit

_vvmul__FPfiPfiPfii ENDP
          END
