#include <malloc.h>
/*#include <dos.h>*/

#include <windows.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>

#define SWAP(A,B, TEMP) (TEMP=A, A=B, B=TEMP)
#define MIN(A,B) ((A) < (B) ? (A) : B)
#define MAX(A,B) ((A) > (B) ? (A) : B)

#define    DRAG_INTERVAL    100

static  HCURSOR DndGrabCursor;
static  HCURSOR DndGrabbedCursor;
static  HCURSOR DndTextCursor;
static  int DndOffset;

typedef    struct    MESSAGE
    {
    UINT    Message;
    WPARAM    wParam;
    LPARAM    lParam;
    LPARAM    Result;
    }    MESSAGE;

typedef struct  LONGPOINT
    {
    long    X, Y;
    }   LONGPOINT;

typedef struct  LONGRECT
    {
    long    Left, Top, Right, Bottom;
    }   LONGRECT;

typedef struct  DNDEDIT
    {
    HWND        HWindow;        /* handle to edit window                */
    BOOL        Grabbed;        /* TRUE if in grabbed state, else FALSE */
    BOOL        CaretIsHidden;  /* TRUE if caret is hidden              */
    BOOL        CopyOnly;       /* TRUE if performing copy, not move    */
    int         SelectionStart; /* character pos of start of selection  */
    int         SelectionEnd;   /* character pos of end of selection    */
    int         InsertionPos;   /* char pos we will "drop" text at      */
    WORD        CaretBlinkTime; /* so we can restore correct blink speed*/
    int         FontHeight;     /* height of current font == line height*/
    int         FontWidth;      /* width of current font == char scroll */
    long        Drag;           /* time tick of previous drag scroll    */
    int         DragAccelerate; /* interval between drag scrolls        */
    int            DragSize;
    long        VScroll;        /* vert offset in device coordinates    */
    long        HScroll;        /* horiz offset in device coordinates   */
    int         HDrag, VDrag;   /* drag scroll direction, 0 if no scroll*/
    long        ScrollCorrection;   /* hack for horizontal "jump"       */
    LONGRECT    SelRects[3];    /* rectangles containing selected text  */
    RECT        TextRect;       /* encloses text area of edit window    */
    HCURSOR     Cursor;         /* current cursor for WM_SETCURSOR      */
    HFONT        Font;
    }   DNDEDIT;



LRESULT CALLBACK _export   DndEditWinFunc(HWND WindowHandle, UINT MsgNum,
                        WPARAM WordParm, LPARAM LongParm);

static  WNDPROC PreviousEditProc;


/********* Helper Functions *********/

static
int        WindowHeight(DNDEDIT *this)
    {
    return this->TextRect.bottom - this->TextRect.top;
    }

static
int        NWindowLines(DNDEDIT *this)
    {
    return (this->TextRect.bottom - this->TextRect.top) / this->FontHeight;
    }

static
char FAR *MemGetLine(DNDEDIT *DndData, int LineNumber, int LineLength)
    {
    char FAR *  Buffer;
    Buffer  = (char far *)_fmalloc(LineLength + 2 + 1);
    if(Buffer)
        {
        *((int FAR *)Buffer)    = LineLength;
        LineLength  = (int)SendMessage(DndData->HWindow, EM_GETLINE, LineNumber, (LPARAM)Buffer);
        Buffer[LineLength]  = '\0';
        }
    return Buffer;
    }

static
void  MemFreeLine(char FAR *Buffer)
    {
    _ffree(Buffer);
    }

static
int        SecretEditWord(DNDEDIT *this, int Offset)
    {
    unsigned    DataSegment, DataOffset;

    DataSegment = GetWindowWord(this->HWindow, GWW_HINSTANCE) | 0x0001;
    DataOffset  = GetWindowWord(this->HWindow, 0) + Offset;
    return *(int FAR *)(((long)DataSegment<<16)|DataOffset);
    }



/********* Windows Message Interface Functions *********/

int        GetNumLines(DNDEDIT *this)
    {
    return (int)SendMessage(this->HWindow, EM_GETLINECOUNT, 0, 0);
    }

int        GetLineIndex(DNDEDIT *this, int LineNumber)
    {
    return (int)SendMessage(this->HWindow, EM_LINEINDEX, LineNumber, 0);
    }

int        GetLineFromPos(DNDEDIT *this, int Pos)
    {
    return (int)SendMessage(this->HWindow, EM_LINEFROMCHAR, Pos, 0);
    }

int        GetLineLength(DNDEDIT *this, int LineNumber)
    {
    return (int)SendMessage(this->HWindow, EM_LINELENGTH,
            GetLineIndex(this, LineNumber), 0);
    }

void    GetRect(DNDEDIT *this, RECT FAR *Rect)
    {
    SendMessage(this->HWindow, EM_GETRECT, 0, (LPARAM)Rect);
    }

void    GetSubText(DNDEDIT *this, char FAR *Buffer, int StartPos, int EndPos)
    {
    int        TempPos, LineNumber, StartLine, EndLine, StartCopyPos, EndCopyPos, CopyLength;
    char FAR *Line;

    --EndPos;
    if(StartPos > EndPos)
        SWAP(StartPos, EndPos, TempPos);
    StartLine    = GetLineFromPos(this, StartPos);
    EndLine        = GetLineFromPos(this, EndPos);
    for(LineNumber = StartLine; LineNumber <= EndLine; ++LineNumber)
        {
        Line    = MemGetLine(this, LineNumber, GetLineLength(this, LineNumber));
        if(Line != NULL)
            {
            StartCopyPos    = 0;
            EndCopyPos        = _fstrlen(Line);
            if(LineNumber == StartLine)
                StartCopyPos    = StartPos - GetLineIndex(this, LineNumber);
            if(LineNumber == EndLine)
                EndCopyPos        = EndPos - GetLineIndex(this, LineNumber);
            CopyLength    = (EndCopyPos-StartCopyPos)+1;
            _fmemcpy(Buffer, Line+StartCopyPos, CopyLength);
            Buffer    += CopyLength;
            MemFreeLine(Line);            
            }
        }
    *Buffer    = '\0';
    }

void    Scroll(DNDEDIT *this, int HDrag, int VDrag)
    {
    SendMessage(this->HWindow, EM_LINESCROLL, 0, MAKELPARAM(VDrag, HDrag));
    }


static
void    GetSelection(DNDEDIT *this, int FAR *StartPos, int FAR *EndPos)
    {
    long    Positions;
    Positions    = SendMessage(this->HWindow, EM_GETSEL, 0, 0);
    *StartPos    = LOWORD(Positions);
    *EndPos        = HIWORD(Positions);
    }

static
void    SetSelection(DNDEDIT *this, int StartPos, int EndPos)
    {
    SendMessage(this->HWindow, EM_SETSEL, 0, MAKELPARAM(StartPos, EndPos));
    }

static
void    Insert(DNDEDIT *this, char FAR *Buffer)
    {
    SendMessage(this->HWindow, EM_REPLACESEL, 0, (LPARAM)Buffer);
    }

static
void    DeleteSelection(DNDEDIT *this)
    {
    Insert(this, "");
    } 

static
LPARAM    DefWndProc(DNDEDIT *this, MESSAGE FAR *Message)
    {
    return Message->Result = CallWindowProc(PreviousEditProc, this->HWindow,
                Message->Message, Message->wParam, Message->lParam);
    }

int     DndEditRegister(HANDLE Instance)
    {
    WNDCLASS    WindowClass;

    DndGrabCursor       = LoadCursor(0, IDC_ARROW);
    DndGrabbedCursor    = LoadCursor(0, IDC_UPARROW);
    DndTextCursor       = LoadCursor(0, IDC_IBEAM);
    GetClassInfo(0, "Edit", &WindowClass);
    WindowClass.style  |= CS_GLOBALCLASS;
    WindowClass.style  |= CS_DBLCLKS;
    WindowClass.style  |= CS_HREDRAW;
    WindowClass.style  |= CS_VREDRAW;
    PreviousEditProc            = WindowClass.lpfnWndProc;
    WindowClass.hInstance       = Instance;
    WindowClass.lpszClassName   = "DndEdit";
    WindowClass.lpfnWndProc     = (WNDPROC)DndEditWinFunc;
    DndOffset                   = WindowClass.cbWndExtra;
    WindowClass.cbWndExtra     += sizeof(DNDEDIT *);
    if(RegisterClass(&WindowClass))
        return TRUE;
    else
        FatalAppExit(0, "Can't register window class DndEdit");
    }

int DndEditUnregister(HINSTANCE Instance)
    {
    UnregisterClass("DndEdit", Instance);
    return TRUE;
    }


/**************************************/

/* LPtInRect - Long version of PtInRect().  */

static
int LPtInRect(LONGRECT *Rect, LONGPOINT Point)
    {
    int     Result  = TRUE;

    if(Point.Y < Rect->Top || Point.Y > Rect->Bottom
    || Point.X < Rect->Left || Point.X > Rect->Right)
        Result  = FALSE;

    return Result;
    }


static
int     DndShowCaret(DNDEDIT *DndData, int ShowIt)
    {
    int     Changed = FALSE;

    if(ShowIt == DndData->CaretIsHidden)
        {
        DndData->CaretIsHidden  = !DndData->CaretIsHidden;
        Changed                 = TRUE;
        if(ShowIt == TRUE)
            ShowCaret(DndData->HWindow);
        else
            HideCaret(DndData->HWindow);
        }
    return Changed;
    }


static
int        DndClipScroll(DNDEDIT *this, int Vertical, int Horizontal)
    {
    long    NewVScroll, NewHScroll;
    int     Changed;

    Changed    = FALSE;
    NewVScroll    = MAX(this->VScroll+Vertical, 0);
    NewHScroll    = MAX(this->HScroll+Horizontal, 0);
    Changed        = (NewVScroll != this->VScroll) || (NewHScroll != this->HScroll);
    this->HScroll    = NewHScroll;
    this->VScroll    = NewVScroll;
    return Changed;
    }

static
int   DndPlaceCaret(DNDEDIT *this, POINT FAR *CaretPos)
    {
    int         DragIt;
    int         LineNumber, LineLength, HalfWidth;
    size_t      CharPos;
    char FAR   *LineText;
    HDC         DC;
    long        Coord, XCoord, MinXCoord;
    int         X, Y, MaxY, LinesInWindow;
    POINT       CursorPos;
    HGDIOBJ     OldFont;

    GetCursorPos(&CursorPos);
    ScreenToClient(this->HWindow, &CursorPos);

    X   = CursorPos.x;
    Y   = CursorPos.y;
    this->HDrag    = 0;
    this->VDrag    = 0;
    LinesInWindow    = (this->TextRect.bottom
        - this->TextRect.top) / this->FontHeight;
    MaxY    = LinesInWindow * this->FontHeight
            + this->TextRect.top - 1;
    if(X < this->TextRect.left)
        {
        X            = this->TextRect.left;
        this->HDrag    = -1;
        }
    else if(X > this->TextRect.right)
        {
        X            = this->TextRect.right;
        this->HDrag    = 1;
        }
    if(Y < this->TextRect.top)
        {
        Y            = this->TextRect.top;
        this->VDrag    = -1;
        }
    else if(Y > MaxY)
        {
        Y            = MaxY-2;
        this->VDrag    = 1;
        }
    LineNumber    = (int) ((Y - this->TextRect.top + this->VScroll)
                / this->FontHeight);
    LineNumber    = MAX(MIN(LineNumber, GetNumLines(this)-1), 0);
    Y            = (int)((long)LineNumber * this->FontHeight
                - this->VScroll + this->TextRect.top);
    DC            = GetDC(this->HWindow);
    if(this->Font)
        OldFont        = SelectObject(DC, this->Font);
    this->InsertionPos    = GetLineIndex(this, LineNumber);
    LineLength    = GetLineLength(this, LineNumber);
    LineText    = MemGetLine(this, LineNumber, LineLength);
    if(LineText != NULL)
        {
        CharPos        = 0;
        MinXCoord    = MAX(X - this->FontWidth, 0);
        do    {
            Coord        = GetTextExtent(DC, LineText, CharPos);
            XCoord        = LOWORD(Coord) + this->TextRect.left - this->HScroll;
            ++CharPos;
            } while(CharPos <= _fstrlen(LineText) && XCoord < MinXCoord);
        HalfWidth    = (this->TextRect.right - this->TextRect.left) / 2;
        if(XCoord < (this->TextRect.left + this->FontWidth / 2))
            {
            this->ScrollCorrection    = (this->TextRect.left - XCoord)
                    + HalfWidth - this->FontWidth;
            this->HDrag    = -1;
            }
        else
            this->ScrollCorrection    = 0;
        if(this->ScrollCorrection == 0 && this->HDrag == 1 && (XCoord < (this->TextRect.right - HalfWidth)))
            this->HDrag    = 0;
        
        X    = (int)MAX(this->TextRect.left, XCoord);
        this->InsertionPos    += CharPos-1;
        MemFreeLine(LineText);
        }
    if(this->Font)
        SelectObject(DC, OldFont);
    ReleaseDC(this->HWindow, DC);
    CaretPos->x        = X;
    CaretPos->y        = Y;
    DragIt            = this->HDrag || this->VDrag;
    return DragIt;
    }


/* DndZeroSelection - Zero the selection rectangles.    */

static
void    DndZeroSelection(DNDEDIT *DndData)
    {
    int     i;

    for(i = 0; i < 3; ++i)
        {
        DndData->SelRects[i].Left    = 0;
        DndData->SelRects[i].Top     = 0;
        DndData->SelRects[i].Right   = 0;
        DndData->SelRects[i].Bottom  = 0;
        }
    }


/* DndInSelection - TRUE if mouse cursor over selected text.    */

static
int     DndInSelection(DNDEDIT *this, POINT MousePos)
    {
    int     i;
    LONGPOINT   LongPos;

    if(PtInRect(&this->TextRect, MousePos))
        {
        LongPos.X   = MousePos.x + this->HScroll;
        LongPos.Y   = MousePos.y + this->VScroll;
        for(i = 0; i < 3; ++i)
            if(LPtInRect(&this->SelRects[i], LongPos))
                return TRUE;
        }
    return FALSE;
    }


/* DndSetFontSize - Store the current font height and width.    */

static
void    DndSetFontSize(DNDEDIT *this)
    {
    TEXTMETRIC  TextMetric;
    char        *N  = "n";
    HDC         DeviceContext;
    HGDIOBJ        OldFont;
    HFONT        Font;

    DeviceContext   = GetDC(this->HWindow);
    Font            = (HFONT)SendMessage(this->HWindow, WM_GETFONT, 0, 0);
    if(Font)
        OldFont    = SelectObject(DeviceContext, Font);
    GetTextMetrics(DeviceContext, &TextMetric);
    this->FontWidth  = LOWORD(GetTextExtent(DeviceContext, N, 1));
    if(Font)
        SelectObject(DeviceContext, OldFont);
    ReleaseDC(this->HWindow, DeviceContext);
    this->FontHeight = TextMetric.tmHeight;
    this->Font        = Font;
    }



/* CharPosToXCoord - return X coordinate from character position */

static
int     CharPosToXCoord(DNDEDIT *this, HDC Context, int CharPos)
    {
    int     LineNumber, LineLength, XCoord;
    char far *Buffer;
    HGDIOBJ    OldFont;

    if(this->Font)
        OldFont    = SelectObject(Context, this->Font);

    LineNumber  = GetLineFromPos(this, CharPos);
    LineLength  = CharPos - GetLineIndex(this, LineNumber);
    Buffer        = MemGetLine(this, LineNumber, LineLength);
    if(Buffer)
        {
        XCoord  = (int)GetTextExtent(Context, Buffer, LineLength);
        MemFreeLine(Buffer);
        }
    else
        XCoord    = 0;
    if(this->Font)
        SelectObject(Context, OldFont);
    return XCoord;
    }

/*  DndSetSelection - Set selection rectangle
 *
 *  When the user may have changed the selected text, call
 *  this function to update the three rectangles that enclose the
 *  selected characters.
 */
static
void    DndSetSelection(DNDEDIT *this)
    {
    LONGRECT    *SelRects = &this->SelRects[0];
    POINT   CaretPos;
    int     StartLine, EndLine, CaretLine, NLines, Temp;
    long    XStart, XEnd, XTemp;
    HDC     DeviceContext;

    DndZeroSelection(this);
    GetCaretPos(&CaretPos);
    CaretLine   = (CaretPos.y-this->TextRect.top) / this->FontHeight;
    DeviceContext   = GetDC(this->HWindow);
    XStart          = CharPosToXCoord(this, DeviceContext,
                            this->SelectionStart) + this->TextRect.left;
    XEnd            = CharPosToXCoord(this, DeviceContext,
                            this->SelectionEnd)   + this->TextRect.left;
    StartLine   = GetLineFromPos(this, this->SelectionStart);
    EndLine     = GetLineFromPos(this, this->SelectionEnd);
    this->HScroll     = XEnd - CaretPos.x;
    this->VScroll     = (EndLine - CaretLine) * this->FontHeight;
    if(this->SelectionEnd < this->SelectionStart)
        {
        SWAP(XStart, XEnd, XTemp);
        SWAP(StartLine, EndLine, Temp);
        }
    NLines      = 1 + EndLine - StartLine; /* # of lines selected */
    SelRects[0].Top      = StartLine * this->FontHeight + this->TextRect.top;
    SelRects[0].Bottom   = SelRects[0].Top + this->FontHeight;
    SelRects[0].Left     = XStart;
    if(NLines <= 1)
        SelRects[0].Right    = XEnd;
    else
        {
        SelRects[0].Right    = LONG_MAX;
        SelRects[2].Left     = 0;
        SelRects[2].Top      = this->FontHeight * EndLine + this->TextRect.top;
        SelRects[2].Bottom   = SelRects[2].Top + this->FontHeight;
        SelRects[2].Right    = XEnd;
        if(NLines > 2)
            {
            SelRects[1].Left     = 0;
            SelRects[1].Right    = LONG_MAX;
            SelRects[1].Top      = SelRects[0].Bottom;
            SelRects[1].Bottom   = SelRects[2].Top;
            }
        }
    ReleaseDC(this->HWindow, DeviceContext);
    }



static
void    DndStartSelection(DNDEDIT *this)
    {
    int        StartPos, EndPos;

    DndZeroSelection(this);
    GetSelection(this, &StartPos, &EndPos);
    this->SelectionStart    = StartPos;
    if(StartPos == EndPos)
        DndShowCaret(this, TRUE);
    }

/* DndStopSelection - Selected text has changed. */

static
void    DndStopSelection(DNDEDIT *this)
    {
    int     StartPos, EndPos;

    GetRect(this, &this->TextRect);

    GetSelection(this, &StartPos, &EndPos);
    if(StartPos != EndPos)
        {
        GetRect(this, &this->TextRect);
        if(this->SelectionStart == StartPos)
            this->SelectionEnd   = EndPos;
        else
            this->SelectionEnd   = StartPos;
        DndSetSelection(this);
        }
    }




/* DndWinInit - Initialize internal data structure */
static
void    DndWinInit(HWND EditWindow)
    {
    HLOCAL  DataHandle;
    DNDEDIT *DndData;

    DataHandle  = LocalAlloc(LHND, sizeof(DNDEDIT));
    SetWindowWord(EditWindow, DndOffset, (WORD)DataHandle);
    if(DataHandle)
        {
        DndData                 = (DNDEDIT *)LocalLock(DataHandle);
        DndData->Cursor         = DndTextCursor;
        DndData->CaretIsHidden  = FALSE;
        DndData->HWindow        = EditWindow;
        LocalUnlock(DataHandle);
        }
    }

static
int        MouseEventIsPending(DNDEDIT *this)
    {
    MSG        Msg;

    return PeekMessage(&Msg, this->HWindow, WM_LBUTTONUP, WM_LBUTTONUP, PM_NOREMOVE)
        || PeekMessage(&Msg, this->HWindow, WM_MOUSEMOVE, WM_MOUSEMOVE, PM_NOREMOVE);
    }    

static
void    WMDndCreate(DNDEDIT *this, MESSAGE FAR *Message)
    {
    DefWndProc(this, Message);
    DndSetFontSize(this);
    GetRect(this, &this->TextRect);
    }

static
void    WMDndMouseMove(DNDEDIT *this, MESSAGE FAR *Message)
    {
    int     MoveCaret;
    long    NewTick;
    POINT   CaretPos;

    MoveCaret    = FALSE;
    if(!this->Grabbed)
        {
        DefWndProc(this, Message);
        if(DndInSelection(this, MAKEPOINT(Message->lParam)))
            this->Cursor    = DndGrabCursor;
        else
            this->Cursor    = 0;
        }
    else
        {
        if(Message->lParam != -1) /* if "real" mouse move */
            {
            MoveCaret    = TRUE;
            DefWndProc(this, Message);
            if(DndPlaceCaret(this, &CaretPos))
                {
                if(this->Drag == 0)
                    {
                    this->DragAccelerate    = DRAG_INTERVAL;
                    this->DragSize            = 1;
                    this->Drag                = labs(GetTickCount())+1;
                    }
                else
                    {
                    if(this->DragSize > 1)
                        this->DragSize    = MIN(1, this->DragSize / 3);
                    else
                        this->DragAccelerate = MIN(DRAG_INTERVAL, this->DragAccelerate+10);
                    }
                }
            else
                this->Drag    = 0;
            }
        if((this->Drag != 0) && !MouseEventIsPending(this))
            {
            NewTick        = labs(GetTickCount());
            if( labs(NewTick-this->Drag) > this->DragAccelerate)
                {
                this->DragAccelerate    = this->DragAccelerate * 6 / 7;
                this->Drag                = NewTick + 1;
                if(this->ScrollCorrection != 0)
                    this->HDrag    = -(int)(this->ScrollCorrection / this->FontWidth);
                if(this->DragAccelerate < 10)
                       this->DragSize    = MIN(NWindowLines(this), this->DragSize+1);
                Scroll(this, this->HDrag, this->VDrag * this->DragSize);
                MoveCaret    = FALSE; /* because scroll did it */
                }
            PostMessage(this->HWindow, WM_MOUSEMOVE, Message->wParam, -1L);
            }

        }
    if(MoveCaret)
        SetCaretPos(CaretPos.x, CaretPos.y);
    }

static
void    WMDndSetCursor(DNDEDIT *this, MESSAGE FAR *Message)
    {
    if(this->Cursor != 0)
		SetCursor(this->Cursor);
    else
        DefWndProc(this, Message);
    }

static
void    WMDndLButtonDown(DNDEDIT *this, MESSAGE FAR *Message)
    {
    POINT    CaretPos;

    if(DndInSelection(this, MAKEPOINT(Message->lParam))) /* if grabbing selection */
        {
        this->CopyOnly    = ((GetKeyState(VK_CONTROL) & 0x8000) == 0x8000);
        SetCapture(this->HWindow);
        this->Cursor    = DndGrabbedCursor;
        SetCursor(DndGrabbedCursor);
        DndShowCaret(this, FALSE);
        CreateCaret(this->HWindow, (HBITMAP)1, 2*GetSystemMetrics(SM_CXBORDER), this->FontHeight);
        DndPlaceCaret(this, &CaretPos);
        SetCaretPos(CaretPos.x, CaretPos.y);
        this->CaretBlinkTime    = GetCaretBlinkTime();
        SetCaretBlinkTime(0xFFFF);    /* no blinking */
        DndShowCaret(this, TRUE);    /* explicit ShowCaret required after CreateCaret() */
        this->Grabbed    = TRUE;
        }
    else
        {
        DefWndProc(this, Message);
        DndStartSelection(this);
        }
    }

static
void    WMDndLButtonUp(DNDEDIT *this, MESSAGE FAR *Message)
    {
    int            SelectionSize, StartPos, EndPos, TempPos;
    char FAR *     Buffer;

    this->Drag    = 0;
    if(this->Grabbed)	/* if dropping selection */
        {
        DndShowCaret(this, FALSE);
        ReleaseCapture();
        CreateCaret(this->HWindow, (HBITMAP)0, 2*GetSystemMetrics(SM_CXBORDER), this->FontHeight);
        SetCaretBlinkTime(this->CaretBlinkTime);
        DndShowCaret(this, TRUE);
        this->Cursor= 0;

        StartPos    = this->SelectionStart;
        EndPos      = this->SelectionEnd;
        if(StartPos > EndPos)
            SWAP(StartPos, EndPos, TempPos);
        if(this->InsertionPos >= StartPos && this->InsertionPos < EndPos)
            {
            this->Grabbed    = FALSE;
            SetSelection(this, this->InsertionPos, this->InsertionPos);
            }
        else
            {
            SelectionSize   = abs(StartPos-EndPos);
            Buffer          = _fmalloc(SelectionSize+1);
            GetSubText(this, Buffer, StartPos, EndPos);
            if(this->CopyOnly == FALSE)
                {
                DeleteSelection(this);
                if(this->InsertionPos >= StartPos)
                    this->InsertionPos    = this->InsertionPos-SelectionSize;
                }
            SetSelection(this, this->InsertionPos, this->InsertionPos);
            Insert(this, Buffer);
            this->Grabbed    = FALSE;
            SetSelection(this, this->InsertionPos, this->InsertionPos+SelectionSize);
            _ffree(Buffer);
            }
        }
    else
        DndStopSelection(this);
    DefWndProc(this, Message);
    }

static
void    WMDndHScroll(DNDEDIT *this, MESSAGE FAR *Message)
    {
    DefWndProc(this, Message);
    this->HScroll    = SecretEditWord(this, 0x16);
    }

static
void    WMDndVScroll(DNDEDIT *this, MESSAGE FAR *Message)
    {
    DefWndProc(this, Message);
    this->VScroll    = SecretEditWord(this, 0x12) * this->FontHeight;
    }

static
void    WMDndKeyDown(DNDEDIT *this, MESSAGE FAR *Message)
    {
    int        StartPos, EndPos;

    if(this->Grabbed)
		{
		if(Message->wParam == VK_CONTROL)
	        this->CopyOnly    = TRUE;
		}
	else	/* else text selection is probably changing */
        {
        GetSelection(this, &StartPos, &EndPos);
        if(StartPos != EndPos && ((GetKeyState(VK_SHIFT)&0x8000) == 0)
            &&(Message->wParam == VK_LEFT || Message->wParam == VK_RIGHT))
            {
            if(Message->wParam == VK_LEFT)
                SetSelection(this, StartPos, StartPos);
            else if(Message->wParam == VK_RIGHT)
                SetSelection(this, EndPos, EndPos);
            }
        else
            {
            if(StartPos == EndPos)
                DndStartSelection(this);
            DefWndProc(this, Message);
            GetSelection(this, &StartPos, &EndPos);
            if(StartPos == EndPos)
                DndStartSelection(this);
            else
                DndStopSelection(this);
            }
        }        
    }

static
void    WMDndLButtonDblClk(DNDEDIT *this, MESSAGE FAR * Message)
    {
    DefWndProc(this, Message);
    DndStartSelection(this);
    DndStopSelection(this);
    }

static
void    EMDndSetSel(DNDEDIT *this, MESSAGE FAR *Message)
    {
    DefWndProc(this, Message);
    DndStartSelection(this);
    DndStopSelection(this);
    }

static
void    EMDndLineScroll(DNDEDIT *this, MESSAGE FAR *Message)
    {
    int        LineScroll, CharScroll, Changed;
    POINT    CaretPos;

    if(this->Grabbed)
        {
        Changed        = FALSE;
        LineScroll    = LOWORD(Message->lParam);
        if(LineScroll > 0)
            {
            LineScroll    = MAX(0, MIN(LineScroll,(int)
                          (GetNumLines(this)-(this->VScroll/this->FontHeight)
                        - (NWindowLines(this)/2))));
            Message->lParam    = (Message->lParam&0xFFFF0000)|LineScroll;
            }
        CharScroll    = HIWORD(Message->lParam);
        Changed        = DndClipScroll(this, LineScroll*this->FontHeight,
                        CharScroll*this->FontWidth);
        if(Changed)
            {
            DndShowCaret(this, FALSE);
            DefWndProc(this, Message);
            DndPlaceCaret(this, &CaretPos);
            SetCaretPos(CaretPos.x, CaretPos.y);
            DndShowCaret(this, TRUE);
            }
        else
            {
            this->DragAccelerate    = DRAG_INTERVAL;
            this->DragSize            = 1;
            }
        }
    else
        DefWndProc(this, Message);
    }

static
void    WMDndSetFont(DNDEDIT *this, MESSAGE FAR *Message)
    {
    DefWndProc(this, Message);
    DndSetFontSize(this);
    }


typedef    void (*DNDPROC)(DNDEDIT *this, MESSAGE FAR *Message);

typedef    struct    MESSAGEDISPATCH
    {
    UINT    MessageNumber;
    DNDPROC    MessageHandler;
    }    MESSAGEDISPATCH;

MESSAGEDISPATCH    FunctionTable[]    =
    {
        { EM_LINESCROLL,    EMDndLineScroll },
        { EM_SETSEL,        EMDndSetSel  },
        { WM_CREATE,        WMDndCreate },
        { WM_HSCROLL,       WMDndHScroll },
        { WM_KEYDOWN,       WMDndKeyDown },
        { WM_LBUTTONDBLCLK, WMDndLButtonDblClk    },
        { WM_LBUTTONDOWN,   WMDndLButtonDown },
        { WM_LBUTTONUP,     WMDndLButtonUp },
        { WM_MOUSEMOVE,     WMDndMouseMove },
        { WM_SETCURSOR,     WMDndSetCursor },
        { WM_SETFONT,       WMDndSetFont },        
        { WM_VSCROLL,       WMDndVScroll },

    };
#define    NFUNCTIONS    (sizeof(FunctionTable)/sizeof(MESSAGEDISPATCH))

void    HandleMessage(DNDEDIT *this, MESSAGE FAR *Message)
    {
    int        i;

    for(i = 0; i < NFUNCTIONS; ++i)
        if(FunctionTable[i].MessageNumber == Message->Message)
            {
            FunctionTable[i].MessageHandler(this, Message);
            break;
            }

    if(i >= NFUNCTIONS)
        DefWndProc(this, Message);
    }


LRESULT CALLBACK  _export  DndEditWinFunc(HWND EditWindow, UINT MsgNum,
                        WPARAM WordParm, LPARAM LongParm)
    {
    MESSAGE        Message;
    HLOCAL  DataHandle;
    DNDEDIT *DndData;

    Message.Message    = MsgNum;
    Message.lParam    = LongParm;
    Message.wParam    = WordParm;
    Message.Result    = 0;


    if(MsgNum == WM_NCCREATE)
        DndWinInit(EditWindow);
    DataHandle  =(HLOCAL)GetWindowWord(EditWindow, DndOffset);
    if(DataHandle)
        {
        DndData = (DNDEDIT *)LocalLock(DataHandle);
        HandleMessage(DndData, &Message);
           if(MsgNum == WM_DESTROY)
            {
            LocalFree(DataHandle);
            SetWindowWord(EditWindow, DndOffset, 0);
            }
        LocalUnlock(DataHandle);
        }
    if(MsgNum == WM_NCCREATE && Message.Result == 0)
       FatalAppExit(0, "WM_NCCREATE Failed");

    return Message.Result;
    }
