/*   Title:  PRINTDIO.H                              *
 *  Author:  John Reynolds                           *
 * Purpose:  Provide Digital I/O to outside world    *
 *           via parallel port.                      *
 * Copyright 1991 Magnetic Technologies Corporation  *
 *           Rochester, New York  14625              */

#ifndef PRINTDIO_H
#define PRINTDIO_H
/*
   PC Parallel Cable Pin Assignments:

                D I G I T A L   O U T P U T
   Pin  Printer Use  Output Bit    Pin   Printer Use
   ---  -----------  ----------    ---   -----------
     2   Data Bit 0      0          18    Ground
     3   Data Bit 1      1          19    Ground
     4   Data Bit 2      2          20    Ground
     5   Data Bit 3      3          21    Ground
     6   Data Bit 4      4          22    Ground
     7   Data Bit 5      5          23    Ground
     8   Data Bit 6      6          24    Ground
     9   Data Bit 7      7          25    Ground

               D I G I T A L   I N P U T
   Pin  Printer Use   Input Bit      Pin  Printer Use
   ---  -----------   ---------      ---  -----------
     1   Strobe           0*          18   Ground
    14   Auto Feed        1*          19   Ground
    16   Initialize       2           20   Ground
    15   Error            3           21   Ground
    13   Select           4           22   Ground
    12   Paper Out        5           23   Ground
    10   Acknowledge      6           24   Ground
    11   Busy             7*          25   Ground

* Note: Code converts active low inputs to active high
*/

/*----- PARALLEL PORT SELECTION -----*/
 int use_port_LPT(unsigned char number);
 /* Eg. use_port_LPT(2) for LPT2   */
 /* returns 0 if port unavailable  */

/*--- OUTPUT ROUTINES ---*/
 void clear_all_bits();
 void set_all_bits();
 void set_bit(unsigned char bit0_7);
 void clear_bit(unsigned char bit0_7);
 void set_bit_pattern(unsigned char pattern);

/* bits = 1 in mask will be affected,
   others left unchanged             */
 void set_masked_bits(unsigned char mask);
 void clear_masked_bits(unsigned char mask);

 /* return current output settings */
  unsigned char read_output_byte();

 /* return 0 if bit is low, 1 if high */
  unsigned char read_output_bit(unsigned char bit0_7);

/*---- INPUT ROUTINES ----*/
 unsigned char input_byte();
 /* return 0 if bit is low, 1 if high */
 unsigned char input_bit(unsigned char bit0_7);
#endif
