/*
 *  helpapi.c - DLL to provide multi-topic printing.
 *  Copyright (c) Ron Burk 1993
 */

#include <stdlib.h>
#include <windows.h>

#define WM_HELP_INQUIRE    (1)  /* asks what notifications you want */
#define WM_HELP_FUNCPTRS  (10)  /* supplies pointers to functions   */
#define HN_FUNCPTRS   (0x0010)  /* bit to receive WM_HELP_FUNCPTRS  */

typedef LONG (CALLBACK *WINHELPPTR)(LPSTR,WORD,DWORD);
typedef struct  HelpInstance
    {
    struct HelpInstance   *Next;
    HTASK           TaskHandle;
    WINHELPPTR      WinHelp;
    }           HelpInstance;

HelpInstance *HelpList;

HelpInstance    *NewHelpInstance(FARPROC FAR *FuncPtrs)
    {
    HelpInstance    *This;
    This        = (HelpInstance *)malloc(sizeof(HelpInstance));
    if(This != NULL)
        {
        This->Next          = NULL;
        This->TaskHandle    = GetCurrentTask();
        This->WinHelp       = (WINHELPPTR)FuncPtrs[16];
        if(HelpList)
            {
            This->Next  = HelpList;
            HelpList    = This;
            }
        else    /* else first on list */
            HelpList    = This;
        }
    return This;
    }

HelpInstance    *CurrentHelpInstance(void)
    {
    HTASK   ThisTask        = GetCurrentTask();
    HelpInstance    *Rover  = HelpList;

    while(Rover)    /* search linked list for matching task */
        if(Rover->TaskHandle == ThisTask)
            return Rover;
        else
            Rover   = Rover->Next;
    return Rover;
    }


HINSTANCE   ModuleHandle;

/* LibMain - store module handle, register clipboard format */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
int CALLBACK LibMain(HINSTANCE ModuleHandle_,
    WORD DataSegment, WORD HeapSize, LPSTR CommandLine)
    {
    if(HeapSize > 0)
        UnlockData(0);
    ModuleHandle    = ModuleHandle_;
    return 1;
    }

/* WEP - return success */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
int CALLBACK WEP(int ExitCode)
    {
    return 1;
    }


/* HelpYield() - Synchronize winhelp.exe macro execution */
static  void    HelpYield(void)
    {
    MSG     Message;

    while(PeekMessage(&Message, NULL, 0, 0, PM_REMOVE))
        if(Message.message == WM_QUIT)
            {
            PostQuitMessage(0);
            return;
            }
        else
            {
            TranslateMessage(&Message);
            DispatchMessage(&Message);
            }
    }

/* PrintThis() - prints current topic in given help file */
void CALLBACK _export PrintThis(char FAR *HelpFile)
    {
    HelpInstance    *ThisHelp = CurrentHelpInstance();

    ThisHelp->WinHelp(HelpFile, HELP_COMMAND,
            (DWORD)(void FAR *)"Print()");
    HelpYield();
    }


/* LDLLHandler - Special function that WinHelp calls implicitly */
#ifdef __BORLANDC__
    #pragma argsused
#endif        
LONG CALLBACK _export LDLLHandler(WORD Message, LONG Param1, LONG Param2)
    {
    switch(Message)
        {
        case WM_HELP_INQUIRE   :
            return HN_FUNCPTRS;
        case WM_HELP_FUNCPTRS  :
            NewHelpInstance((FARPROC FAR *)Param1);
            return TRUE;
        }
    return FALSE;
    }

