-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=Begin Listing 11-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
/*****************************************************/
/* idletsr.c                                         */
/* -- TSR to detect idle interrupt.                  */
/*****************************************************/

#include <dos.h>
#include <stdlib.h>
#include <malloc.h>
#include "idletsr.h"

extern unsigned end;
int             wIdleCount, wTimerCount;

void (interrupt far * pfn08Old)(); /* Next in chain. */
void (interrupt far * pfn28Old)(); /* Next in chain. */
void (interrupt far * pfn2fOld)(); /* Next in chain. */

void interrupt far
Int28Handler(int es, int ds, int di, int si, int bp,
  int sp, int bx, int dx, int cx, int ax, int ip,
  int cs, int flags)
/*****************************************************/
/* -- Idle interrupt handler.                        */
/*****************************************************/
    {
    wIdleCount++;
    _chain_intr(pfn28Old);
    }

void interrupt far
Int08Handler(int es, int ds, int di, int si, int bp,
  int sp, int bx, int dx, int cx, int ax, int ip,
  int cs, int flags)
/*****************************************************/
/* -- Timer interrupt handler.                       */
/*****************************************************/
    {
    wTimerCount++;
    _chain_intr(pfn08Old);
    }

void interrupt far
Int2fHandler(int es, int ds, int di, int si, int bp,
  int sp, int bx, int dx, int cx, int ax, int ip,
  int cs, int flags)
/*****************************************************/
/* -- Multiplex interrupt handler.                   */
/*****************************************************/
    {
    switch (ax)
        {
    default:
        _chain_intr(pfn2fOld);
        break;

    case cmdIsLoaded:
        ax = 0x00ff;
        break;

    case cmdIdleCheck:
        ax = wIdleCount;
        wIdleCount = 0;
        break;

    case cmdTimerCheck:
        ax = wTimerCount;
        wTimerCount = 0;
        break;
        }
    }

int
main(void)
/*****************************************************/
/* -- Entry point.                                   */
/*****************************************************/
    {
    unsigned far * lpuwUpper;
    unsigned int   cpgf, cpgfMax, cbStack;
    unsigned char  bVal;

    /* Check to see if we have been loaded already. */
    _asm
        {
        mov     ax, cmdIsLoaded;
        int     0x2f
        mov     bVal, al
        }
    if (bVal == 0xff)
        return 0;

    /* Install TSR. */
    lpuwUpper = &end;
    cbStack = ((stackavail() + 2048) / 2048) * 2048;
    cpgf = (FP_SEG(lpuwUpper) - _psp) +
      (FP_OFF(lpuwUpper) + cbStack) / 16 + 1;

    /* Install interrupt handler. */
    pfn2fOld = _dos_getvect(0x2f);
    _dos_setvect(0x2f, Int2fHandler);
    pfn28Old = _dos_getvect(0x28);
    _dos_setvect(0x28, Int28Handler);
    pfn08Old = _dos_getvect(0x08);
    _dos_setvect(0x08, Int08Handler);
    _dos_setblock(cpgf, _psp, &cpgfMax);
    _dos_keep(0, cpgf);
    return -1;
    }


