/* POSIX Agent Program, PSXAGENT.C */

#include <unistd.h>
#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include "win32psx.h"

int  ReadStream(char *buf, int size);
int  WriteStream(char *buf, int size);
int  ReadRequestStream(char *buf, int size);
int  WriteRequestStream(char *buf, int size);
void ServerLoop();

/* Hack until /dev/tty works */
FILE *logfile;
#define stderr logfile

extern int errno;

int NamedPipeIn, NamedPipeOut;
char InNameBuf[256], OutNameBuf[256];

void CreateNamedPipes()
{
  int ret;
  
  InNameBuf[0] = 0;
  OutNameBuf[0] = 0;
  strcat(InNameBuf, NAMED_PIPE_PATH);
  strcat(OutNameBuf, NAMED_PIPE_PATH);
  strcat(InNameBuf, "RPC_REQ");
  strcat(OutNameBuf, "RPC_REP");

  ret = mkfifo(InNameBuf, O_RDWR);
  if(ret == (-1)) {
    if(errno != EEXIST) {
      fprintf(stderr,"mkfifo failed errno %d\n",errno);
      exit(1);
    }
  }
  ret = mkfifo(OutNameBuf, O_RDWR);
  if(ret == (-1)) {
    if(errno != EEXIST) {
      fprintf(stderr,"mkfifo failed errno %d\n",errno);
      exit(1);
    }
  }
}

/* Open the named pipes.
   This routine will block until a POSIX client
   opens the other end. */

void OpenNamedPipes()
{
  NamedPipeIn = open(InNameBuf, O_RDONLY, 0);
  if(NamedPipeIn == (-1)) {
    fprintf(stderr,"Can't open pipe errno %d\n",errno);
    exit(1);
  }
  NamedPipeOut = open(OutNameBuf, O_WRONLY, 0);
  if(NamedPipeOut == (-1)) {
    fprintf(stderr,"Can't open pipe errno %d\n",errno);
    exit(1);
  }
}

/* Close the named pipes */
void CloseNamedPipes()
{
  close(NamedPipeIn);
  close(NamedPipeOut);
}


main()
{
  logfile = fopen("LOGFILE", "w");
  if(logfile == (FILE *)0) {
    fprintf(stderr, "Can't create LOGFILE\n");
    exit(1);
  }

  /* Create the named pipes */
  CreateNamedPipes();

  /* Perform Server Loop */
  ServerLoop();

  return(0);
}

/* Loop getting requests from the request
   pipe and pass on to our stdout.
   Read the replies from stdin and pass on to
   the reply pipe */

void ServerLoop()
{
  struct Request_Header Hd;
  int ret, ret1, numxfer;
  char buf[5120];

 for(;;)
 {
   OpenNamedPipes();
   for(;;)
   {
     ret = read(NamedPipeIn, (char *)&Hd,
                sizeof(struct Request_Header));
     if(ret == (-1)) {
       fprintf(stderr,"REQ pipe errno %d\n",errno);
       exit(1);
     }
     else if(ret == 0) {
       break; /* EOF */
     }
     numxfer = Hd.rh_size;
     WriteStream((char *)&Hd, ret);
     if(numxfer) {
       ret = read(NamedPipeIn, buf, numxfer);
       WriteStream(buf, ret);
     }

     /* Get reply from WIN32 server */
     ret = ReadStream((char *)&Hd,
              sizeof(struct Request_Header));
     numxfer = Hd.rh_size;
     ret1 = write(NamedPipeOut, (char *)&Hd,
                 sizeof(struct Request_Header));
     if(ret1 == (-1)) {
       fprintf(stderr,"REP pipe errno %d\n",errno);
       exit(1);
     }
     if(numxfer) {
       ret = ReadStream(buf, numxfer);
       ret1 = write(NamedPipeOut, buf, ret);
     }
   }
   CloseNamedPipes();
 }
}

/* read data from the communications stream */
int ReadStream(char *buf, int size)
{
  int numxfer;

  if((numxfer = read(0, buf, size)) == (-1)) {
    fprintf(stderr, "Error stdin %d\n", errno);
    exit(1);
  }
  return(numxfer);
}

/* write data to the communications stream */
int WriteStream(char *buf, int size)
{
  int numxfer;

  if((numxfer = write(1, buf, size)) == (-1)) {
    fprintf(stderr, "Error stdout %d\n", errno);
    exit(1);
  }
  return(numxfer);
}
