/* POSIX RPC Client Library, RPCCLT.C */

#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include "win32psx.h"

extern int errno;

int  ReadStream(char *buf, int size);
int  WriteStream(char *buf, int size);
int ServerInputHandle;
int ServerOutputHandle;

void AttachToServer()
{
  char InNameBuf[256], OutNameBuf[256];
  
  InNameBuf[0] = 0;
  OutNameBuf[0] = 0;
  strcat(InNameBuf, NAMED_PIPE_PATH);
  strcat(OutNameBuf, NAMED_PIPE_PATH);
  strcat(InNameBuf, "RPC_REQ");
  strcat(OutNameBuf, "RPC_REP");

  ServerOutputHandle = open(InNameBuf, O_WRONLY, 0);
  if(ServerOutputHandle == (-1)) {
    fprintf(stderr, "Can't open pipe %d\n",errno);
    exit(1);
  }
  ServerInputHandle = open(OutNameBuf, O_RDONLY, 0);
  if(ServerInputHandle == (-1)) {
    fprintf(stderr, "Can't open pipe %d\n",errno);
    exit(1);
  }
}

/* Send a NOOP RPC */
int NoopRPC()
{
  struct Request_Header Hd;
  int numxfer;

  Hd.rh_type = RPC_REQUEST;
  Hd.rh_hdrsize = sizeof(struct Request_Header);
  Hd.rh_size = 0;
  Hd.rh_chan = 1;
  Hd.rh_request = RPC_NOOP;

  WriteStream((char *)&Hd,
      sizeof(struct Request_Header));
  numxfer = ReadStream((char *)&Hd,
           sizeof(struct Request_Header));
  if(Hd.rh_type != RPC_REPLY) {
     fprintf(stderr, "Not an RPC REPLY\n");
     return(0);
  }
 return(1);
}

/* Have WIN32 run a command line */
int SystemRPC(char *cmd)
{
  struct Request_Header Hd;
  int numxfer, exitstatus;

  Hd.rh_type = RPC_REQUEST;
  Hd.rh_hdrsize = sizeof(struct Request_Header);
  Hd.rh_size = strlen(cmd) + 1;
  Hd.rh_chan = 1;
  Hd.rh_request = RPC_RUN_SHELL_CMD_SYNC;
  WriteStream((char *)&Hd,
       sizeof(struct Request_Header));
  WriteStream(cmd, strlen(cmd) + 1);

  /* wait for the reply */
  numxfer = ReadStream((char *)&Hd,
        sizeof(struct Request_Header));
  if(Hd.rh_type != RPC_REPLY) {
     fprintf(stderr, "Not RPC REPLY\n");
     exit(1);
  }
  else {
    if((Hd.rh_size == 0) && (Hd.rh_request == 0)) {
      /* error running command or creating process */
      return(1);
    }
    else if(Hd.rh_size != 4) {
      fprintf(stderr, "Reply not 4 bytes\n");
      exit(1);
    }
    ReadStream((char *)&exitstatus, 4);
  }
  return(exitstatus);
}

/* read data from the communications stream */
int ReadStream(char *buf, int size)
{
  int numxfer;

  if((numxfer = read(ServerInputHandle, buf, size))
           == (-1)) {
    fprintf(stderr,"Error read Child Stdin,%d\n",
        errno);
    exit(1);
  }
  return(numxfer);
}

/* write data to the communications stream */
int WriteStream(char *buf, int size)
{
  int numxfer;

  if((numxfer = write(ServerOutputHandle, buf, size))
       == (-1)){ 
    fprintf(stderr, "Error stdout %d\n", errno);
    exit(1);
  }
  return(numxfer);
}
