/* Listing 1: A Flexible messageBox() Function. */

#define Uses_TDialog
#define Uses_TStaticText
#define Uses_TButton
#define Uses_TProgram
#define Uses_TDeskTop
#include <tv.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>


/*
static TRect getTextDimensions(const char buffer[])

PARAMETER
    buffer        buffer containing message text to display in the
            messageBox function

RETURNS
    Bounding rectangle of the text.  The rectangle always has topleft
    (0,0)
    While computing the bounding rectangle, '\n' will force a new line
    and '\3' will force the line to be centered on the next line
*/
static TRect getTextDimensions(const char buffer[])
    {
    int i=0;
    int rows=1;
    int maxCols=0;
    int cols=0;

    while(buffer[i]!=EOS) {
        if(buffer[i]=='\3' || buffer[i]=='\n') {
            if(cols>maxCols)
                maxCols=cols;
            cols=0;
            ++rows;
            }
        else
            ++cols;
        ++i;
        }

    if(cols>maxCols)
        maxCols=cols;

    TRect r;
    r.a.x=r.a.y=0;
    r.b.x=maxCols;
    r.b.y=rows;
    return r;
    }


/*
ushort messageBox(ushort aOptions,const char title[],const char fmt[],... )

PARAMETERS
    aOptions    message box options, analogous to the TV messageBox
    title        message box title
    fmt        format string (like in printf)
    ...        parameters for format string

RETURNS
    like the TV messageBox
*/
ushort messageBox(ushort aOptions,const char title[],const char fmt[],... )
    {
    // form the text to appear in the message box
    static char buffer[512];
    static const char *buttonName[] = {
        // Message is a static class holding the title text for
        // the buttons
        Message::yesText,
        Message::noText,
        Message::okText,
        Message::cancelText
        };
    static const ushort commands[] = {
        cmYes,
        cmNo,
        cmOK,
        cmCancel
        };

    va_list argptr;

    va_start(argptr,fmt);
    vsprintf(buffer,fmt,argptr);
    va_end(argptr);

    // compute the bounding rectangle of the resulting text
    TRect textDimensions=getTextDimensions(buffer);
    // add some slack space
    textDimensions.b.x+=4;

    // create the button row, x will hold it's length
    short i, x, buttonCount;
    TView* buttonList[5];

    for( i = 0, x = -2, buttonCount = 0; i < 4; i++ ) {
        if( (aOptions & (0x0100 << i)) != 0) {
            buttonList[buttonCount]=new TButton( TRect(0, 0, 12, 2), buttonName[i], commands[i], bfNormal );
            x += buttonList[buttonCount++]->size.x + 2;
            }
        }


    // adjust the width if too small to fit the title
    if(title!=NULL) {
        size_t titleLen=strlen(title)+6;

        if(textDimensions.b.x<titleLen)
            textDimensions.b.x=titleLen;
        }

    // adjust the width if too small to fit the buttons
    if(textDimensions.b.x<x+10)
        textDimensions.b.x=x+10;


    // adjust the height to fit the buttons
    textDimensions.b.y+=4;

    // at this point, textDimensions contains the rectangle of
    // the message box, which will be centered in the deskTop view
    // We should check if the rectangle does not exceed the deskTop
    if(textDimensions.b.y>=deskTop->size.y)
        textDimensions.b.y=deskTop->size.y;
    if(textDimensions.b.x>=deskTop->size.x)
        textDimensions.b.x=deskTop->size.x;

    // create the dialog box
    TDialog *dialog=new TDialog(textDimensions, title);
    if(dialog==NULL)
        return cmCancel;

    // make sure it's centered in the view where we insert it
    dialog->options|=ofCentered;

    // add the static text
    dialog->insert(new TStaticText(TRect(1,1,dialog->size.x-2,dialog->size.y-3),buffer));

    // add the buttons
    x = (dialog->size.x - x) / 2;

    for(i=0;i<buttonCount;++i) {
        dialog->insert(buttonList[i]);
        buttonList[i]->moveTo(x, dialog->size.y - 3);
        x += buttonList[i]->size.x + 2;
        }

    // first button should be selected
    dialog->selectNext(False);

    // hold the dialog
    ushort ccode = TProgram::deskTop->execView(TProgram::application->validView(dialog));

    // remove it
    TObject::destroy( dialog );
    // and return the result code
    return ccode;
    }


