/* Listing 2: A Better TWindow Class */

#define Uses_TProgram
#define Uses_TDeskTop
#define Uses_TWindow
#define Uses_TEvent
#define Uses_TFrame
#include <tv.h>
#include <stdarg.h>
#include <stdio.h>

// new cmXXXX constants
const ushort cmWindowHasNum=9500;        // (int windowNum) for TBaseWindows
const ushort cmReplaceTitle=9600;        // (char *aTitle) for TBaseWindows


class TBaseWindow: public TWindow {
public:
    TBaseWindow(const char aTitle[],short aNumber);

    int replaceTitle(const char format[],...);

protected:
    void handleEvent(TEvent &event);
    void setState(ushort aState, Boolean enable);

private:
    Boolean handleBroadcast(TEvent &event);
    Boolean handleCommand(TEvent &event);
    };


/*
TBaseWindow::TBaseWindow(const char aTitle[],short aNumber)

PARAMETERS
    aTitle        window title
    aNumber        window number (assumed to be !=wnNoNumber)

EFFECT
    constructor
    The window position is determined by its number and the desktop size.
*/
TBaseWindow::TBaseWindow(const char aTitle[],short aNumber)
    :TWindow(TRect((aNumber-1) % 10,(aNumber-1) % 10,TProgram::deskTop->size.x,TProgram::deskTop->size.y),aTitle,aNumber),
     TWindowInit(TBaseWindow::initFrame)
    {
    }



/*
void TBaseWindow::handleEvent(TEvent &event)

PARAMETERS
    event        event to handle

EFFECT
    intercepts the cmWindowHasNum broadcast.  This broadcast is used when
    the application wants to enquire if there's a window with a specific
    number.  This mechanism allows for the reuse of window numbers.
*/
void TBaseWindow::handleEvent(TEvent &event)
    {
    TWindow::handleEvent(event);

    Boolean clearIt=False;    // will be set to True if clearEvent should be called

    switch(event.what) {
        case evBroadcast:
            clearIt=handleBroadcast(event);
            break;
        case evCommand:
            clearIt=handleCommand(event);
            break;
        }

    if(clearIt)
        clearEvent(event);
    }




/*
Boolean TBaseWindow::handleBroadcast(TEvent &event)

PARAMETERS
    event        broadcast event to handle

EFFECT
    handles broadcast events.
    Currently only cmWindowHasNum (respond if this window has the
    requested window number.

RETURNS
    True if event handled
    False if not
*/
Boolean TBaseWindow::handleBroadcast(TEvent &event)
    {
    return (Boolean)(event.message.command==cmWindowHasNum && event.message.infoInt==number && (options & ofSelectable));
    }




/*
Boolean TBaseWindow::handleCommand(TEvent &event)

PARAMETERS
    event        command event to handle

EFFECT
    handles command events.
    Currently only cmReplaceTitle, replacing the window title by whatever
    string infoPtr points to

RETURNS
    True if event handled
    False if not
*/
Boolean TBaseWindow::handleCommand(TEvent &event)
    {
    if(event.message.command==cmReplaceTitle) {
        delete (char *)title;
        title=newStr((char *)event.message.infoPtr);
        // redraw the frame (will also redraw the title)
        frame->drawView();
        return True;
        }
    else
        return False;
    }



/*
void TBaseWindow::setState(ushort aState, Boolean enable)

PARAMETERS
    aState
    enable

EFFECT
    see tv.
    Overridden to enable specific window commands like cascade,tile and
    window list
*/
void TBaseWindow::setState(ushort aState, Boolean enable)
    {
    TWindow::setState(aState,enable);
    if(aState & (sfSelected | sfActive)) {
        TCommandSet documentCommands;

        documentCommands += cmTile;
        documentCommands += cmCascade;
        documentCommands += cmWindowList;

        if(enable)
            enableCommands(documentCommands);
        else
            disableCommands(documentCommands);
        }
    }


/*
int TBaseWindow::replaceTitle(const char format[],...)

PARAMETERS
    format        title format string
    ...            arguments for format string (if any)

EFFECT
    sets the window title to 'format' (formatted like printf).

RETURNS
    number of characters in new title
*/
int TBaseWindow::replaceTitle(const char format[],...)
    {
    va_list argptr;
    char buffer[128];

    va_start(argptr, format);
    int cnt = vsprintf(buffer, format, argptr);
    va_end(argptr);

    delete (char *)title;
    title=newStr(buffer);
    // redraw the frame (will also redraw the title)
    frame->drawView();
    return cnt;
    }


The smallest available window number is then determined using the
following function:

/*
short TMyApp::getWindowNum()

RETURNS
    the next available window number.
    Window numbers of windows that are closed are reused.
*/
short TMyApp::getWindowNum()
    {
    short i=1;

    while(message(deskTop,evBroadcast,cmWindowHasNum,(void *)(unsigned long)i)!=NULL)
        ++i;
    return i;
    }


