;------------------------------------------------------
;   LISTING 3
;
;   Module:     CMD.ASM
;   Summary:    Route driver interrupt call to command
;                   code routine
;   Assembler:  TASM 2.0
;   Author:     T.W. Nelson
;   Version:    1.00
;   Date:       29-Jan-1992
;   Notes:
;
;   Source code Copyright (c) 1992 T.W. Nelson. May
;   be used only with appropriate acknowledgement of
;   copyright.
;
;------------------------------------------------------

INCLUDE sldd.inc
ASSUME  cs:_TEXT,ds:DGROUP,es:NOTHING,ss:NOTHING

_DATA   segment

logon   db  "Self-loading Device Driver,",10,13
        db  "(Template) Version 1.00",10,13,10,13,'$'

;------------------------------------------------------
; Dispatch table for command-code routines
;
cmdTab  equ     $
    dw  init            ;0 = initialize driver
    dw  media_check     ;1
    dw  build_bpb       ;2 = build BIOS param block
    dw  ioctl_read      ;3 = read io control data
    dw  device_read     ;4
    dw  nd_read         ;5 = non-destructive read
    dw  input_status    ;6
    dw  input_flush     ;7 = flush input buffers
    dw  device_write    ;8
    dw  verify_write    ;9 = write with verify
    dw  output_status   ;10
    dw  output_flush    ;11 = flush output buffers
    dw  ioctl_write     ;12 = write io control data
    dw  device_open     ;13  (DOS 3+)
    dw  device_close    ;14  (DOS 3+)
    dw  rem_media       ;15 = removable media (3+)
    dw  output_busy     ;16 = output until busy (3+)
    dw  bad_command     ;17 = not used
    dw  bad_command     ;18 = not used
    dw  generic_ioctl   ;19 = generic io control (3.2+)
    dw  bad_command     ;20 = not used
    dw  bad_command     ;21 = not used
    dw  bad_command     ;22 = not used
    dw  get_logdev      ;23 = get logical device (3.2+)
    dw  set_logdev      ;24 = set logical device (3.2+)

;Dummy BPB for testing block driver setup .....
bpbstruc    equ     $
    dw  512             ;bytes/sector
    db  2               ;sectors/cluster
    dw  1               ;reserved sectors
    db  2               ;#fat tables
    dw  112             ;#directory entries
    dw  720             ;#logical sectors
    db  0fdh            ;media descriptor byte
    dw  2               ;#fat sectors
    dw  9               ;sectors/track
    dw  2               ;#sides
    dw  0               ;hidden sectors

bpbtab  dw  bpbstruc    ;two units, same table
        dw  bpbstruc

_DATA   ends

IFDEF   COMEXEC
_INIT   segment
extrn   driver_end:byte
_INIT   ends
ENDIF   ;COMEXEC

_TEXT   segment

;------------------------------------------------------
; Set up call to command code routine.  On entry,
; es:di points to the request header.  Returns
; result code in request header status word.
;
public  exec_command

exec_command    proc    near
        push    es                      ;save req hdr
        push    di
        xor     bx,bx
        mov     bl,es:[di].rh_cmd       ;load cmd code
        cmp     bx,24                   ;legal?
        jle     exec1                   ;yes, continue
        call    bad_command             ;set error code
        jmp     short exec2             ;bypass call
exec1:
        shl     bx,1                    ;cmdTab index
        call    wptr [bx].cmdTab        ;to cmd routine
exec2:
        pop     di                      ;restore req
        pop     es                      ;header
        or      ax,0100h                ;set 'done' bit
        mov     es:[di].rh_status,ax    ;assign status
        ret
exec_command    endp

;------------------------------------------------------
; Command code routines. On entry, es:di points to the
; caller's request header.  Each routine must return
; status in ax: 0 if successful or (8000h + error code)
; if function ended in error.
;
init:                ;0 = initialize driver

; ----- return break address --------------------------
    mov     es:[di].rh_brkseg,seg _INIT
    IFDEF   COMEXEC
    mov     es:[di].rh_brkofs,offset _INIT:driver_end
    ELSE
    mov     es:[di].rh_brkofs,0
    ENDIF

; ----- return items for block driver -----------------
    mov     es:[di].rh_nunits,2
    mov     es:[di].rh_bufofs,offset _DATA:bpbtab
    mov     es:[di].rh_bufseg,seg _DATA

; ----- display log on message ------------------------
    mov     dx,offset logon
    mov     ah,09h      ;DOS display '$' string
    int     21h
    xor     ax,ax       ;no error
    ret

media_check:         ;1
build_bpb:           ;2 = build BIOS param block
ioctl_read:          ;3 = read io control data
device_read:         ;4
nd_read:             ;5 = non-destructive read
input_status:        ;6
input_flush:         ;7 = flush input buffers
    xor     ax,ax    ;return 'no error' status
    ret
device_write:        ;8
verify_write:        ;9 = write with verify
    mov     cx,es:[di+18]       ;byte count
    les     di,es:[di+14]       ;xfer address
    jcxz    w2       ;check for runaway error
w1:
    mov     al,es:[di]
    mov     ah,0eh          ;BIOS write tty
    xor     bx,bx           ;page 0
    int     10h
    inc     di              ;increment pointer
    loop    w1              ;next byte
w2:

output_status:       ;10
output_flush:        ;11 = flush output buffers
ioctl_write:         ;12 = write io control data
device_open:         ;13  (DOS 3+)
device_close:        ;14  (DOS 3+)
rem_media:           ;15 = removable media (3+)
output_busy:         ;16 = output until busy (3+)
generic_ioctl:       ;19 = generic io control (3.2+)
get_logdev:          ;23 = get logical device (3.2+)
set_logdev:          ;24 = set logical device (3.2+)
    xor     ax,ax    ;return 'no error' status
    ret
bad_command:
    mov     ax,8003h
    ret

_TEXT   ends
        end

;--- End of file --------------------------------------
