/* ----------------------------------------------------
 *  LISTING 5
 *
 *  Filename:           tdrv.c
 *  Summary:            Device driver test bed
 *  Author:             T.W. Nelson
 *  Compile options:    
 *  Version:            1.00
 *  Date:               29-Jan-1992
 *  Notes:
 *
 *  MUST compile with byte-packing enabled
 * ------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>

char    Dname[] = "TEMPLATE";   //name to search for
size_t  DosDataSeg;     //DOS data segment
size_t  ofsLoL;         //offset to DOS LoL
size_t  ofsNULdev;      //offset to NUL device header
size_t  dosMajor;       //DOS version #s
size_t  dosMinor;
void (far *strat)(void);     //-> strategy routine
void (far *intrp)(void);     //-> interrupt routine
union   REGS    regs;
struct  SREGS   sregs;

struct d_hdr    {
    struct d_hdr far *d_next;   //-> next header
    size_t  d_attr;
    size_t  d_strat;
    size_t  d_intr;
    char    d_name[8];
    };

char buf[] =
    "Bothered by unsightly bathtub rings?\r\n"
    ".... Use NEW Duzz-All Cleanser!\r\n\n";

struct reqhdr {
    char    len;
    char    unit;
    char    cmdcode;
    size_t  status;
    char    reserve[8];
    char    media;
    void    far *xfer;
    size_t  count;
    size_t  sector;
    } cmdpack = { sizeof(struct reqhdr), 0, 8, 0, "",
                0, (void far *) buf, sizeof(buf), 0 };

#define GROUND  ((struct d_hdr far *) -1L)

/* ------------------------------------------------- */
void    err_exit( char *errmsg )    {
    printf( "%s\n\n", errmsg );
    exit(1);
}

void   get_DOS_version(void)        {
    regs.x.ax = 0x3000;
    intdos( &regs, &regs );
    dosMajor = (size_t) regs.h.al;
    dosMinor = (size_t) regs.h.ah;

    if( dosMajor < 2 )
        err_exit( "Better get a DOS upgrade.");
}

void   get_NUL_device(void)         {
    int     displacement;

    regs.x.ax = 0x5200;     //DOS get list of lists
    intdosx( &regs, &regs, &sregs );
    DosDataSeg = sregs.es;
    ofsLoL = regs.x.bx;

    switch( dosMajor )  {
        case 2: displacement = 0x17;  break;
        case 3:
                if( dosMinor == 0 )
                        displacement = 0x28;
                else
                        displacement = 0x22;
                break;
        case 4:
        case 5: displacement = 0x22;  break;
        default:    err_exit( "Unknown DOS version" );
    }

    ofsNULdev = ofsLoL + displacement;
}

void    find_device_header(void)    {
    struct d_hdr far *pdrv;
    size_t d_seg, d_ofs;
    register unsigned i;

    for( pdrv = MK_FP( DosDataSeg, ofsNULdev );
        pdrv->d_next != GROUND; pdrv = pdrv->d_next ) {

        for( i = 0; i < 8; i++ )    {
            if( pdrv->d_name[i] != Dname[i] )
                break;      //name doesn't match
        }

        if( i == 8 )    {   //found a match
            d_seg = FP_SEG( pdrv );
            d_ofs = FP_OFF( pdrv );
            strat = MK_FP(d_seg,d_ofs + pdrv->d_strat);
            intrp = MK_FP(d_seg,d_ofs + pdrv->d_intr);
            return;
        }
    }

    printf("Can't find %s device header", Dname);
    err_exit("");
}

main( int argc, char **argv )       {
    regs.x.ax = 0x3d00;     //DOS OPEN file, read only
    regs.x.dx = (unsigned) Dname;
    intdos( &regs, &regs );

    if( regs.x.cflag )  {
            printf( "%s not installed.", Dname );
            err_exit("");
    }

    regs.x.bx = regs.x.ax;      //handle
    regs.x.ax = 0x4400;     //DOS get IOCTL info
    intdos( &regs, &regs );

    if( !(regs.x.dx & 0x0080) )    {
            printf( "%s is a file.", Dname );
            err_exit("");
    }

    regs.x.ax = 0x3e00;     //DOS close handle
    intdos( &regs, &regs );
    get_DOS_version();
    get_NUL_device();
    find_device_header();
    _ES = FP_SEG( (void far *) &cmdpack );
    _BX = FP_OFF( (void far *) &cmdpack );
    (*strat)();     //Call the device driver....
    (*intrp)();     //.... just like DOS does it

    return 0;
}

/* ----- End of File ------------------------------- */
