/*****************************************************/
/* symhook.c                                         */
/* -- Hook functions.                                */
/*****************************************************/

#include <windows.h>
#include <windowsx.h>
#include <toolhelp.h>
#include "symdll.h"
#include "symbase.h"
#define DEFINE_HOOKS
#include "symhook.h"

BOOL WINAPI
HookGetTextMetrics(HDC hdc, LPTEXTMETRIC lptxm)
/*****************************************************/
/* -- GetTextMetrics() hook function.                */
/* -- Return the max of the two font metrics.        */
/*****************************************************/
    {
    LPCDS       lpcds;
    TEXTMETRIC  txm;
    HFONT       hfntSav = NULL;
    BOOL        fVal;

    HookHks(phksGetTextMetrics, FALSE);
    if ((fVal = GetTextMetrics(hdc, lptxm)) &&
      (lpcds = LpcdsGetCur()) &&
      (hfntSav = SelectObject(hdc, lpcds->hfnt)))
        {
        fVal = GetTextMetrics(hdc, &txm);
        lptxm->tmHeight =
          max(lptxm->tmHeight, txm.tmHeight);
        SelectObject(hdc, hfntSav);
        }
    HookHks(phksGetTextMetrics, TRUE);
    return fVal;
    }

BOOL WINAPI
HookGetCharWidth(HDC hdc, UINT chFirst, UINT chLim,
  LPINT lrgdx)
/*****************************************************/
/* -- GetCharWidth() hook function.                  */
/* -- Any characters above 0x80 are mapped to the    */
/*    first 0x80 characters of the symbol font.      */
/*****************************************************/
    {
    BOOL    fVal        = FALSE;
    HFONT   hfntOrig    = NULL;
    LPCDS   lpcds;

    /* Remove hook so we can make use of */
    /* GetCharWidth(). */
    HookHks(phksGetCharWidth, FALSE);

    if (!(lpcds = LpcdsGetCur()))
        {
        fVal =
          GetCharWidth(hdc, chFirst, chLim, lrgdx);
        goto HookGetCharWidthExit;
        }

    /* Do lower half. */
    if (chFirst < 0x80)
        {
        UINT    ch;

        ch = chLim < 0x80 ? chLim : 0x7f;
        if (!(fVal =
          GetCharWidth(hdc, chFirst, ch, lrgdx)))
            goto HookGetCharWidthExit;
        lrgdx += ch - chFirst + 1;
        chFirst = ch + 1;
        }

    /* Do upper half. */
    if (chLim >= 0x80)
        {
        hfntOrig = SelectObject(hdc, lpcds->hfnt);
        fVal = GetCharWidth(hdc, chFirst - 0x80,
          chLim - 0x80, lrgdx);
        }

HookGetCharWidthExit:
    if (hfntOrig)
        SelectObject(hdc, hfntOrig);
    HookHks(phksGetCharWidth, TRUE);
    return fVal;
    }

BOOL WINAPI
HookExtTextOut(HDC hdc, int x, int y, UINT f,
  LPRECT lprect, LPSTR lsz, UINT cch, LPINT lrgw)
/*****************************************************/
/* -- ExtTextOut() hook function.                    */
/* -- Any characters above 0x80 are mapped to the    */
/*    first 0x80 characters of the symbol font.      */
/* -- We call ExtTextOut() for each chunk of like-   */
/*    font characters.  If ETO_OPAQUE was specified, */
/*    we clear it after the first chunk, else the    */
/*    leading chunks will get erased.                */
/*****************************************************/
    {
    BOOL    fVal            = FALSE;
    LPBYTE  lpch, lpchLim;
    HFONT   hfntOrig        = NULL;
    UINT    wTextAlignSav   = 0;
    LPCDS   lpcds;

    /* Remove hook so we can make use of */
    /* ExtTextOut(). */
    HookHks(phksExtTextOut, FALSE);

    /* If no text to output, or task is not */
    /* known, or we can't select the client's */
    /* font, just call the original. */
    if (!lsz || !cch || !(lpcds = LpcdsGetCur()) ||
      !(hfntOrig = SelectObject(hdc, lpcds->hfnt)))
        {
        fVal = ExtTextOut(hdc, x, y, f, lprect, lsz, cch,
          lrgw);
        goto HookExtTextOutExit;
        }

    /* Use TA_UPDATECP so we don't have to track */
    /* where to start the next chunk. */
    MoveToEx(hdc, x, y, NULL);
    wTextAlignSav = SetTextAlign(hdc, TA_UPDATECP);
    for (lpch = lsz, lpchLim = lpch + cch;
      lpch < lpchLim;)
        {
        int     cb;
        LPBYTE  lpchT;
        LPBYTE  lpchScan    = lpch + 1;
        BOOL    fClientFont = *lpch >= 0x80;

        for (; lpchScan < lpchLim; lpchScan++)
            if ((fClientFont && *lpchScan < 0x80) ||
              (!fClientFont && *lpchScan >= 0x80))
                break;

        cb = lpchScan - lpch;
        SelectObject(hdc,
          fClientFont ? lpcds->hfnt : hfntOrig);
        if (fClientFont)   /* Map to lower 0x80. */
            for (lpchT = lpch; lpchT < lpchScan;
              lpchT++)
                *lpchT &= 0x7f;

        if (!ExtTextOut(hdc, 0, 0, f, lprect, lpch, cb,
          lrgw))
            goto HookExtTextOutExit;

        if (fClientFont)    /* Restore string. */
            for (lpchT = lpch; lpchT < lpchScan;
              lpchT++)
                *lpchT |= 0x80;

        f &= ~ETO_OPAQUE;   /* Only erase once. */
        if (lrgw)           /* Update cell spacing. */
            lrgw += cb;

        lpch = lpchScan;
        }
    fVal = TRUE;

HookExtTextOutExit:
    if (hfntOrig)
        SelectObject(hdc, hfntOrig);
    if (wTextAlignSav)
        SetTextAlign(hdc, wTextAlignSav);
    HookHks(phksExtTextOut, TRUE);
    return fVal;
    }
