/*
 * whello.c
 * Basic Windows app, to illustrate usage of find_prev_win() function
 * (adapted by LZ from hellowin.c in Petzold's Programming Windows 3.1)
 */

#include <windows.h>

long FAR PASCAL _export WndProc (HWND, UINT, UINT, LONG);

/******** BEGINNING OF CUSTOM SECTION #1 ********/

/********************************************************************
 * find_prev_wnd():                                                 *
 *  Find the main window of the previous                            *
 *  instance by searching all top-level windows                     *
 ********************************************************************/

HWND            prev_wnd;
WNDENUMPROC     find_prev_wndi;

BOOL CALLBACK _export   find_prev_wnd(HWND this_wnd, LPARAM hPrevInst)
{
    /* Look for a true top-level window having the same instance */
    if (!GetParent(this_wnd) &&
            GetWindowWord(this_wnd, GWW_HINSTANCE) == LOWORD(hPrevInst))
    {
        prev_wnd = this_wnd;
        return FALSE; /* Stop enumerating */
    }
    return TRUE; /* Continue enumerating */
} /* find_prev_wnd() */

/******** END OF CUSTOM SECTION #1 ********/

int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance,
                    LPSTR lpszCmdParam, int nCmdShow)
{
    static char *appid = "HelloWin w/Return";
    HWND        hwnd;
    MSG         msg;
    WNDCLASS    wndclass;

    /***** BEGINNING OF CUSTOM SECTION #2 *****/

    /*******************************************************************
     *  If there is a previous task instance, return to it             *
     *******************************************************************/
    if (hPrevInstance)
    {
        find_prev_wndi =
            (WNDENUMPROC)MakeProcInstance((FARPROC)find_prev_wnd, hInstance);
        EnumWindows(find_prev_wndi, (LPARAM)hPrevInstance);
        FreeProcInstance((FARPROC)find_prev_wndi);

        /* Error: Prev instance has no window */
        if (!prev_wnd)
            return 0; /* Fail silently */

        /* Return to the last active popup window, if any,
           of the previous instance */
        prev_wnd = GetLastActivePopup(prev_wnd);

        /* If the existing window is minimized, restore it */
        if (IsIconic(prev_wnd))
            ShowWindow(prev_wnd, SW_RESTORE);
        SetActiveWindow(prev_wnd);
        return 0;
    } /* There is a previous instance */

    /******** END OF CUSTOM SECTION #2 ********/

    wndclass.style         = CS_HREDRAW | CS_VREDRAW;
    wndclass.lpfnWndProc   = WndProc;
    wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = 0;
    wndclass.hInstance     = hInstance;
    wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
    wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
    wndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
    wndclass.lpszMenuName  = NULL;
    wndclass.lpszClassName = appid;

    RegisterClass (&wndclass);

    hwnd = CreateWindow (appid,         // window class name
                    "Hello w/Return",   // window caption
                    WS_OVERLAPPEDWINDOW | WS_VISIBLE,     // window style
                    0, 0, 200, 200, 
                    NULL,            // parent window handle
                    NULL,            // window menu handle
                    hInstance,       // program instance handle
                    NULL);           // creation parameters

     ShowWindow (hwnd, nCmdShow);
     UpdateWindow (hwnd);

     while (GetMessage (&msg, NULL, 0, 0))
     {
          TranslateMessage (&msg);
          DispatchMessage (&msg);
     }
     return msg.wParam;
}

long FAR PASCAL _export WndProc (HWND hwnd, UINT message,
                        UINT wParam, LONG lParam)
{
     HDC         hdc;
     PAINTSTRUCT ps;
     RECT       rect;

     switch (message)
     {
          case WM_PAINT:
            hdc = BeginPaint (hwnd, &ps) ;
            GetClientRect (hwnd, &rect) ;
            DrawText (hdc, "Hello, Windows!", -1, &rect,
                    DT_SINGLELINE | DT_CENTER | DT_VCENTER) ;
            EndPaint (hwnd, &ps) ;
            return 0 ;

          case WM_DESTROY:
            PostQuitMessage (0);
            return 0;
     }

     return DefWindowProc (hwnd, message, wParam, lParam);
}
