/**************************************************

  Test rig for Shadowed Popup Window

**************************************************/

#include <windows.h>
#include "spopup.h"
#include "testrc.h"

HINSTANCE hInst;		/* Application instance handle */
HMENU hMenu;			/* Handle of menu */
HWND hwndPopup;		/* Handle of popup window */

char * szShort =
		"This is a short message.";
char * szWrapped =
		"This is a much longer message that will be "
		"wrapped over two lines.";
char * szMulti =
		"This message has multiple lines.\nEach line is "
		"terminated by a newline character.\n\nBlank "
		"lines are possible too.";
char * szSPopupInfo =
		"User defined shadowed popup windows allow you "
		"to mix text and graphics.";

LONG FAR PASCAL _export TestRigProc( HWND, UINT,
							WPARAM, LPARAM );

void DoCommand( HWND hwnd, int id );

/***************************************************/
/* WinMain														*/
/***************************************************/

int PASCAL WinMain(
	HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	LPSTR lpszCmdLine,
	int nCmdShow )
{
	WNDCLASS wc;
	HWND hwnd;
	MSG msg;

	hInst  				= hInstance;

	wc.style				= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= TestRigProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInst;
	wc.hIcon				= NULL;
	wc.hCursor			= LoadCursor( NULL, IDC_ARROW );
	wc.hbrBackground	= (HBRUSH)( COLOR_APPWORKSPACE + 1 );
	wc.lpszMenuName	= NULL;
	wc.lpszClassName	= "testrig";
	if( !RegisterClass( &wc ) )
		return( FALSE );
	if( !RegisterSPopupClass( hInst ) )
		return( FALSE );

	hMenu = LoadMenu( hInst, RIGMENU );

	hwnd = CreateWindow( "testrig","Shadowed Popup Test Rig",
		WS_OVERLAPPEDWINDOW, CW_USEDEFAULT,
		CW_USEDEFAULT, CW_USEDEFAULT,
		CW_USEDEFAULT, NULL, hMenu, hInst, NULL );
	ShowWindow( hwnd, nCmdShow );
	UpdateWindow( hwnd );

	while( GetMessage( &msg, NULL, 0, 0 ) ) {
		TranslateMessage( &msg );
		DispatchMessage( &msg );
		}
	return( (int)msg.wParam );
}

/***************************************************/
/* TestRigProc													*/
/***************************************************/

LONG FAR PASCAL _export TestRigProc(
	HWND hwnd,
	UINT message,
	WPARAM wParam,
	LPARAM lParam )
{
	switch( message )
		{
		case WM_DESTROY:
			PostQuitMessage( 0 );
			return( 0L );

		case WM_KEYDOWN:
		case WM_SYSKEYDOWN:
		case WM_LBUTTONDOWN:
		case WM_MBUTTONDOWN:
		case WM_RBUTTONDOWN:
			if( hwndPopup ) {
				ReleaseCapture();
				DestroyWindow( hwndPopup );
				hwndPopup = NULL;
				}
			break;

		case WM_COMMAND:
			DoCommand( hwnd, wParam );
			return( 0L );
		}
	return( DefWindowProc( hwnd, message, wParam, lParam ) );
}

/***************************************************/
/* DoCommand													*/
/***************************************************/

void DoCommand( HWND hwnd, int id )
{
	switch( id )
		{
		case IDM_USRPOPUP: {
			RECT rc;
			HANDLE hIcon;
			HFONT hFontOld;
			HDC hdc;

			hwndPopup = CreateSPopup( 50, 50, 400, 70,
					hwnd, hInst, NULL );
			GetClientRect( hwndPopup, &rc );
			hdc = GetDC( hwndPopup );
			hIcon = LoadIcon( hInst, "TEST" );
			DrawIcon( hdc, 10, 10, hIcon );
			SetRect( &rc, 60, 10, 380, 50 );
			hFontOld = SelectObject( hdc,
				GetStockObject( ANSI_FIXED_FONT ) );
			DrawText( hdc, szSPopupInfo, -1, &rc,
						DT_WORDBREAK|DT_NOPREFIX );
			SelectObject( hdc, hFontOld );
			ReleaseDC( hwndPopup, hdc );
			DestroyIcon( hIcon );
			SetCapture( hwnd );
			SetFocus( hwnd );
			break;
			}

		case IDM_SHORTPOPUP:
			PopupText( hwnd, 50, 50, szShort );
			break;

		case IDM_WRAPPEDPOPUP:
			PopupText( hwnd, 50, 50, szWrapped );
			break;

		case IDM_MULTIPOPUP:
			PopupText( hwnd, 50, 50, szMulti );
			break;

		case IDM_EXIT:
			SendMessage( hwnd, WM_CLOSE, 0, 0L );
			break;
		}
}

