/*****************************************************/
/* notify.c                                          */
/* -- DLL tracks open files by hooking int 21        */
/*    functions                                      */
/* -- To build:                                      */
/* cc -wd -DSTRICT notify.c libentry.obj \           */
/*     toolhelp.lib                                  */
/* implib notify.lib notify.dll                      */
/*****************************************************/
#include <windows.h>
#include <windowsx.h>
#include <toolhelp.h>
#include "notify.h"

typedef struct
    {
    UINT    wES, wDS, wDI, wSI, wBP, wSP, wBX, wDX,
            wCX, wAX, wIPReturn, wCSReturn, wFlagReg;
    } REG;  /* __interrupt routine stack. */

/* Increment this many file info entries at a time. */
#define dcofi   100

/* Get more space when number of free entries hits */
/* this value. */
#define cofiLow 50

/* Internal prototypes. */
void      CopyLpsz(LPSTR, LPSTR);
WORD      PdbGetCur(void);
OFI far * LpofiFind(int);
void      Notify(void);
void      NotifyClose(int);
void      NotifyDuplicate(int, int);
void      NotifyOpen(LPSTR, int);

/* Exported prototypes. */
#ifdef __BORLANDC__
    void __interrupt __export Hook21(REG);
#else
    void __interrupt __export __cdecl Hook21(REG);
#endif

int CALLBACK LibMain(HINSTANCE, WORD, WORD, LPSTR);
int CALLBACK __export WEP(int);

HWND        hwndNotify; /* Notify when list changes. */
OFI huge *  gprgofi;    /* File info list. */
long        cofi;       /* Size of list. */
long        cofiUsed;   /* Number of files in list. */
BOOL        fNotified;  /* Client has been notified. */
BYTE        rgbSav[5];  /* Start of int 21 ISR. */
WORD        wSel;       /* Selector:offset of */
UINT        wOff;       /* original int 21 ISR. */

#ifdef __BORLANDC__
    #pragma argsused
#endif
int CALLBACK
LibMain(HINSTANCE hinsThis, WORD ds, WORD cbHeap,
  LPSTR lpsz)
/*****************************************************/
/* -- Called at load time.                           */
/* -- Install the int 21 ISR hook, save the old      */
/*    contents.                                      */
/*****************************************************/
    {
    BYTE    rgbJump[5]; /* Far jump to our hook. */

    /* Preallocate some space for the file list. */
    if ((gprgofi = GlobalAllocPtr(GMEM_SHARE |
      GMEM_MOVEABLE | GMEM_ZEROINIT,
      dcofi * sizeof(OFI))) == NULL)
        return FALSE;   /* No memory. */

    cofi = dcofi;
    cofiUsed = 0;

    /* Find and read the int 21 ISR. */
    _asm mov     ax, 3521h
    _asm int     21h
    _asm mov     wOff, bx
    _asm mov     wSel, es
    MemoryRead(wSel, wOff, rgbSav, sizeof rgbSav);

    /* Replace first 5 bytes with jump to hook func. */
    rgbJump[0] = 0xea;
    *(DWORD *)(rgbJump + 1) = (DWORD)Hook21;
    MemoryWrite(wSel, wOff, rgbJump, sizeof rgbJump);
    return TRUE;
    }

#ifdef __BORLANDC__
    #pragma argsused
#endif
int CALLBACK __export WEP(int wExitCode)
/*****************************************************/
/* -- Remove the int 21 ISR hook.                    */
/* -- No need to explicitly free file list, since    */
/*    its done for us by Windows.                    */
/*****************************************************/
    {
    MemoryWrite(wSel, wOff, rgbSav, sizeof rgbSav);
    return 1;
    }

#ifdef __BORLANDC__
void __interrupt __export
#else
void __interrupt __export __cdecl
#endif
                Hook21(REG reg)
/*****************************************************/
/* -- Int 21 ISR hook.                               */
/*****************************************************/
    {
    UINT    wFunction   = reg.wAX & 0xff00;
    FARPROC lpfn        = MAKELP(wSel, wOff + 6);
    WORD    wSelPriv;

    /* Hack-o-rama.  I have embedded a routine that */
    /* replicates the results of the first 2 */
    /* instructions of the ISR: */
    /*     push ds */
    /*     mov ds, cs:[0030] */
    /* This enables me to keep a permanent jump */
    /* instruction to Hook21 at the start of the */
    /* ISR.  Then to call the ISR, I replicate the */
    /* effect of executing the above 2 instructions */
    /* and jump to the third instruction in the ISR. */
    _asm push    es
    _asm mov     es, wSel
    _asm mov     ax, es:[0030h]
    _asm mov     wSelPriv, ax
    _asm pop     es
    goto LSkipCallIsr;

LCallIsr:
    _asm push    ds
    _asm mov     ds, wSelPriv
    _asm jmp     DWORD PTR lpfn
    /* We have now transferred control to the ISR */
    /* which will iret, so control is never returned */
    /* here. */

LSkipCallIsr:
    _asm push    ds             /* Save DGROUP.     */
    _asm mov     es, reg.wES    /* Restore original */
    _asm mov     ds, reg.wDS    /* register set.    */
    _asm mov     di, reg.wDI
    _asm mov     si, reg.wSI
    _asm mov     bx, reg.wBX
    _asm mov     dx, reg.wDX
    _asm mov     cx, reg.wCX
    _asm mov     ax, reg.wAX
    _asm push    reg.wFlagReg
    _asm push    cs            /* Make it a far call.    */
    _asm call    LCallIsr      /* Hack-o-rama.           */
    _asm mov     reg.wES, es   /* Save register set for  */
    _asm mov     reg.wDS, ds   /* for return to caller.  */
    _asm mov     reg.wDI, di
    _asm mov     reg.wSI, si
    _asm mov     reg.wBX, bx
    _asm mov     reg.wDX, dx
    _asm mov     reg.wCX, cx
    _asm mov     reg.wAX, ax
    _asm pushf
    _asm pop     reg.wFlagReg
    _asm pop     ds            /* Our DGROUP.  */

    if (!(reg.wFlagReg & 0x0001)) /* Successful? */
        switch (wFunction)
            {
        default:
            break;

        case 0x3c00:    /* Create. */
        case 0x3d00:    /* Open. */
        case 0x5a00:    /* Create temporary. */
        case 0x5b00:    /* Create new. */
            NotifyOpen(MAKELP(reg.wDS, reg.wDX),
              reg.wAX);
            break;

        case 0x3e00:    /* Close. */
            NotifyClose(reg.wBX);
            break;

        case 0x4500:    /* Duplicate. */
            NotifyDuplicate(reg.wBX, reg.wAX);
            break;

        case 0x4600:    /* Force. */
            NotifyDuplicate(reg.wBX, reg.wCX);
            break;

        case 0x6c00:    /* Extended open/create. */
            NotifyOpen(MAKELP(reg.wDS, reg.wSI),
              reg.wAX);
            break;
            }
    }

void
NotifyOpen(LPSTR lpszFile, int wHandle)
/*****************************************************/
/* -- Add the file to the list and notify the client */
/*    the list has changed.                          */
/*****************************************************/
    {
    long        iofi;
    OFI huge *  gpofi;
    WORD        pdb     = PdbGetCur();

    if (cofi == cofiUsed)
        return; /* Out of space. */

    /* Check for duplicates. */
    for (iofi = 0, gpofi = gprgofi; iofi < cofiUsed;
      iofi += (gpofi++)->pdb != 0)
        if (gpofi->pdb == pdb &&
          gpofi->wHandle == wHandle)
            return; /* Duplicate. */

    /* Find a free slot in the list. */
    for (iofi = 0, gpofi = gprgofi; iofi < cofi;
      gpofi++, iofi++)
        if (gpofi->pdb == 0)
            {
            /* Add info to list. */
            gpofi->wHandle = wHandle;
            CopyLpsz(gpofi->szPath, lpszFile);
            gpofi->pdb = pdb;
            cofiUsed++;
            Notify();   /* Inform client. */
            break;
            }
    }

WORD
PdbGetCur(void)
/*****************************************************/
/* -- Return the current PDB.                        */
/*****************************************************/
    {
    WORD    pdb;

    _asm mov     ax, 5100h
    _asm int     21h
    _asm mov     pdb, bx;
    GetCurrentTask();
    return pdb;
    }

void
Notify(void)
/*****************************************************/
/* -- Post notification that file list has changed.  */
/*****************************************************/
    {
    if (IsWindow(hwndNotify))
        {
        if (!fNotified)
            {
            fNotified = TRUE;
            PostMessage(hwndNotify, WM_USER, 0, 0);
            }
        }
    else
        hwndNotify = NULL;
    }

void
CopyLpsz(LPSTR lpszDest, LPSTR lpszSrc)
/*****************************************************/
/* -- Our own version of lstrcpy(), since it needs   */
/*    to be called at interrupt time.                */
/*****************************************************/
    {
    while ((*lpszDest++ = *lpszSrc++) != 0)
        ;
    }

void
NotifyClose(int wHandle)
/*****************************************************/
/* -- Remove the file from the list and notify the   */
/*    client the list has changed.                   */
/*****************************************************/
    {
    OFI far *   lpofi;

    /* See if the file is on the list. */
    if ((lpofi = LpofiFind(wHandle)) != NULL)
        {
        /* Remove from list. */
        lpofi->pdb = 0;
        cofiUsed--;
        Notify();   /* Inform client. */
        }
    }

OFI far *
LpofiFind(int wHandle)
/*****************************************************/
/* -- Find the file handle for the current task.     */
/* -- Return NULL if not found.                      */
/*****************************************************/
    {
    long        iofi;
    OFI huge *  gpofi;
    WORD        pdb     = PdbGetCur();

    for (iofi = 0, gpofi = gprgofi; iofi < cofiUsed;
      iofi += (gpofi++)->pdb != 0)
        if (gpofi->pdb == pdb &&
          gpofi->wHandle == wHandle)
            return gpofi;

    return NULL;
    }

void
NotifyDuplicate(int wHandleOrig, int wHandleNew)
/*****************************************************/
/* -- Add the duplicate handle to the list.          */
/*****************************************************/
    {
    OFI far *   lpofi;

    /* See if the file is on the list. */
    if ((lpofi = LpofiFind(wHandleOrig)) != NULL)
        NotifyOpen(lpofi->szPath, wHandleNew);
    }

void WINAPI __export
SetPostWnd(HWND hwnd)
/*****************************************************/
/* -- Set the window to receive notifications.       */
/*****************************************************/
    {
    hwndNotify = hwnd;
    }

BOOL WINAPI __export
FNextOfi(OFI far * lpofi, long far * lpiofi)
/*****************************************************/
/* -- Client service.                                */
/* -- Return the next file in the list to the        */
/*    client.                                        */
/* -- Call with *lpiofi set to -1 to begin           */
/*    enumeration.                                   */
/* -- Updates lpiofi to next entry each call,        */
/*    returns FALSE once all entries have been       */
/*    enumerated (in which case *lpofi is invalid).  */
/*****************************************************/
    {
    if (fNotified)
        {
        fNotified = FALSE;  /* Ok to notify again. */

        /* Grow list? */
        if (cofi - cofiUsed < cofiLow)
            {
            OFI huge *  gprgofiNew;

            if ((gprgofiNew = GlobalReAllocPtr(gprgofi,
              (cofi + dcofi) * sizeof(OFI),
              GMEM_SHARE | GMEM_MOVEABLE |
              GMEM_ZEROINIT)) != NULL)
                {
                gprgofi = gprgofiNew;
                cofi += dcofi;
                }
            }
        }

    /* Is the list empty? */
    if (cofiUsed == 0)
        return FALSE;

    /* Find the next file. */
    for (++*lpiofi; *lpiofi < cofi; ++*lpiofi)
        if (gprgofi[*lpiofi].pdb != 0)
            {
            *lpofi = gprgofi[*lpiofi];
            return TRUE;
            }

    return FALSE;
    }
