/*****************************************************/
/* ncbtn.c                                           */
/* -- Procedure places a push button on a window's   */
/*    caption.                                       */
/*****************************************************/
#include <windows.h>
#include <windowsx.h>
#include "ncbtn.h"

BOOL CALLBACK __export FEnumWnd(HWND hwnd,
  LPARAM lParam);
LRESULT CALLBACK __export LwPopupWndProc(HWND hwnd,
  UINT wm, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK __export LwSubclassProc(HWND hwnd,
  UINT wm, WPARAM wParam, LPARAM lParam);

void PaintPush(HWND hwnd, HDC hdc);

/* Window word offsets. */
#define bCid   0               /* Control id. */
#define bState sizeof(WORD)    /* State (in or out). */
#define bXPos  (sizeof(WORD) + sizeof(BOOL))

#define fBtnIn   1 /* Button is pushed in. */
#define fBtnOut  0 /* Button is not pushed in. */

char szPopupPushClass[] = "PopupButton";
char szProcOff[]        = "WndProcOff";
char szProcSel[]        = "WndProcSel";

HWND
HwndCreateCaptionPush(HWND hwndOwner, PSTR pszTitle,
  WORD cid, int x, int dx)
/*****************************************************/
/* -- Create a push button on the windows caption.   */
/* -- hwndOwner : Owner window of push button.       */
/* -- pszTitle  : Push button title text.            */
/* -- cid       : Push button control id.            */
/* -- x         : Left of push button (from edge of  */
/*                system menu icon).                 */
/* -- dx        : Width of push button.              */
/*****************************************************/
    {
    WNDCLASS    wcs;
    HINSTANCE   hins    = GetWindowInstance(hwndOwner);
    HWND        hwnd;

    if (!GetClassInfo(hins, szPopupPushClass, &wcs))
        {
        wcs.style = 0;
        wcs.lpfnWndProc = LwPopupWndProc;
        wcs.cbClsExtra = 0;
        wcs.cbWndExtra =
          sizeof(WORD) + sizeof(BOOL) + sizeof(int);
        wcs.hInstance = GetWindowInstance(hwndOwner);
        wcs.hIcon = NULL;
        wcs.hCursor = LoadCursor(NULL, IDC_ARROW);
        wcs.hbrBackground = NULL;
        wcs.lpszMenuName = NULL;
        wcs.lpszClassName = szPopupPushClass;
        if (!RegisterClass(&wcs))
            return NULL;
        }

    if ((hwnd = CreateWindow(szPopupPushClass,
        pszTitle, WS_POPUP, 0, 0,
        dx, GetSystemMetrics(SM_CXSIZE), hwndOwner,
        NULL, hins, (void far *)MAKELONG(cid, x))) !=
      NULL)
        {
        FEnumWnd(hwnd, 0);
        ShowWindow(hwnd, SW_SHOWNOACTIVATE);
        }
    return hwnd;
    }

LRESULT CALLBACK __export
LwPopupWndProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Popup (button owner) window procedure.         */
/*****************************************************/
    {
    RECT    rect;

    switch (wm)
        {
    default:
        break;

    case WM_CREATE:
        {
        HWND    hwndOwner    = GetWindowOwner(hwnd);
        long    lw;

        /* Sub-class owner. */
        if (GetProp(hwndOwner, szProcOff) == NULL &&
          GetProp(hwndOwner, szProcSel) == NULL)
            {
            WNDPROC lpfnOrig;

            lpfnOrig = SubclassWindow(hwndOwner,
              LwSubclassProc);
            SetProp(hwndOwner, szProcOff,
              (HANDLE)OFFSETOF(lpfnOrig));
            SetProp(hwndOwner, szProcSel,
              (HANDLE)SELECTOROF(lpfnOrig));
            }
        lw = (long)((LPCREATESTRUCT)lParam)->
          lpCreateParams;
        SetWindowWord(hwnd, bCid, LOWORD(lw));
        SetWindowWord(hwnd, bXPos, HIWORD(lw));
        SetWindowWord(hwnd, bState, fBtnOut);
        }
        break;

    case WM_PAINT:
        {
        PAINTSTRUCT wps;

        BeginPaint(hwnd, &wps);
        PaintPush(hwnd, wps.hdc);
        EndPaint(hwnd, &wps);
        }
        return 0;

    case WM_ERASEBKGND:
        return TRUE;

    case WM_MOUSEACTIVATE:
        return MA_NOACTIVATE;

    case WM_LBUTTONDOWN:
        SetCapture(hwnd);
        SetWindowWord(hwnd, bState, fBtnIn);
        InvalidateRect(hwnd, NULL, TRUE);
        UpdateWindow(hwnd);
        return 0;

    case WM_MOUSEMOVE:
        if (GetCapture() == hwnd)
            {
            BOOL    fIn;

            GetClientRect(hwnd, &rect);
            fIn = PtInRect(&rect, *(POINT *)&lParam) ?
              fBtnIn : fBtnOut;
            if (fIn ^ GetWindowWord(hwnd, bState))
                {
                SetWindowWord(hwnd, bState, fIn);
                InvalidateRect(hwnd, NULL, TRUE);
                UpdateWindow(hwnd);
                }
            }
        break;

    case WM_LBUTTONUP:
        if (GetCapture() == hwnd)
            {
            ReleaseCapture();
            if (GetWindowWord(hwnd, bState))
                {
                SetWindowWord(hwnd, bState, fBtnOut);
                InvalidateRect(hwnd, NULL, TRUE);
                UpdateWindow(hwnd);
                }
            GetClientRect(hwnd, &rect);
            if (PtInRect(&rect, *(POINT *)&lParam))
                SendMessage(GetWindowOwner(hwnd),
                  WM_COMMAND,
                  GetWindowWord(hwnd, bCid),
                  MAKELONG((WORD)hwnd, BN_CLICKED));
           }
        return 0;
        }

    return DefWindowProc(hwnd, wm, wParam, lParam);
    }

void
PaintPush(HWND hwnd, HDC hdc)
/*****************************************************/
/* -- Paint the push button.                         */
/*****************************************************/
    {
    HBRUSH   hbrs    = NULL;
    HBRUSH   hbrsSav = NULL;
    int      dxBdr   = GetSystemMetrics(SM_CXBORDER);
    int      dyBdr   = GetSystemMetrics(SM_CYBORDER);
    int      fIn     = GetWindowWord(hwnd, bState);
    COLORREF clrTextSav, clrBkSav;
    POINT    dpt;
    RECT     rect, rectT;
    char     szBuf[256];

    GetClientRect(hwnd, &rect);
    dpt.x = rect.right - 2 * dxBdr;
    dpt.y = rect.bottom - 2 * dyBdr;

    /* Draw the button face. */
    if ((hbrs = CreateSolidBrush(GetSysColor(
      COLOR_BTNFACE))) != NULL)
        hbrsSav = SelectObject(hdc, hbrs);
    PatBlt(hdc, dxBdr, dyBdr, dpt.x, dpt.y, PATCOPY);
    if (hbrsSav != NULL)
        SelectObject(hdc, hbrsSav);
    if (hbrs != NULL)
        DeleteObject(hbrs);

    /* Draw the button text. */
    clrTextSav =
      SetTextColor(hdc, GetSysColor(COLOR_BTNTEXT));
    clrBkSav =
      SetBkColor(hdc, GetSysColor(COLOR_BTNFACE));
    GetWindowText(hwnd, szBuf, sizeof szBuf);
    rectT = rect;
    InflateRect(&rectT, -dxBdr, -dyBdr);
    if (fIn)
        {
        rectT.left += dxBdr;
        rectT.top += dyBdr;
        rectT.right += dxBdr;
        rectT.bottom += dyBdr;
        }
    DrawText(hdc, szBuf, -1, &rectT,
      DT_CENTER | DT_SINGLELINE | DT_VCENTER);
    SetTextColor(hdc, clrTextSav);
    SetBkColor(hdc, clrBkSav);

    /* Draw the outside frame, left, right, top, */
    /* bottom. */
    if ((hbrs = CreateSolidBrush(GetSysColor(
      COLOR_WINDOWFRAME))) != NULL)
        hbrsSav = SelectObject(hdc, hbrs);
    PatBlt(hdc, 0, dyBdr, dxBdr, dpt.y, PATCOPY);
    PatBlt(hdc, rect.right - dxBdr, dyBdr, dxBdr,
      dpt.y, PATCOPY);
    PatBlt(hdc, dxBdr, 0, dpt.x, dyBdr, PATCOPY);
    PatBlt(hdc, dxBdr, rect.bottom - dxBdr, dpt.x,
      dyBdr, PATCOPY);
    if (hbrsSav != NULL)
        SelectObject(hdc, hbrsSav);
    if (hbrs != NULL)
        DeleteObject(hbrs);

    /* Draw the four corners in the appropriate */
    /* color. */
    if ((hbrs = CreateSolidBrush(GetSysColor(
      GetActiveWindow() == GetWindowOwner(hwnd) ?
        COLOR_ACTIVECAPTION : COLOR_INACTIVECAPTION)))
      != NULL)
        hbrsSav = SelectObject(hdc, hbrs);
    PatBlt(hdc, 0, 0, dxBdr, dyBdr, PATCOPY);
    PatBlt(hdc, rect.right - dxBdr, 0, dxBdr, dyBdr,
      PATCOPY);
    PatBlt(hdc, 0, rect.bottom - dyBdr, dxBdr, dyBdr,
     PATCOPY);
    PatBlt(hdc, rect.right - dxBdr,
      rect.bottom - dyBdr, dxBdr, dyBdr, PATCOPY);
    if (hbrsSav != NULL)
        SelectObject(hdc, hbrsSav);
    if (hbrs != NULL)
        DeleteObject(hbrs);

    if (!fIn)   /* Highlight? */
        {
        if ((hbrs = CreateSolidBrush(GetSysColor(
          COLOR_BTNHIGHLIGHT))) != NULL)
            hbrsSav = SelectObject(hdc, hbrs);
        PatBlt(hdc, dxBdr, dyBdr, dpt.x - dxBdr,
          2 * dyBdr, PATCOPY);
        PatBlt(hdc, dxBdr, 3 * dyBdr, 2 * dxBdr,
          dpt.y - 3 * dyBdr, PATCOPY);
        if (hbrsSav != NULL)
            SelectObject(hdc, hbrsSav);
        if (hbrs != NULL)
            DeleteObject(hbrs);
        }

    /* Shadow. */
    if ((hbrs = CreateSolidBrush(GetSysColor(
      COLOR_BTNSHADOW))) != NULL)
        hbrsSav = SelectObject(hdc, hbrs);
    if (fIn)
        {
        PatBlt(hdc, dxBdr, dyBdr, dpt.x, dyBdr,
          PATCOPY);
        PatBlt(hdc, dxBdr, 2 * dyBdr, dxBdr,
          dpt.y - dyBdr, PATCOPY);
        }
    else
        {
        PatBlt(hdc, dxBdr, dpt.y, dpt.x, dyBdr,
          PATCOPY);
        PatBlt(hdc, dxBdr * 2, dpt.y - dyBdr,
          dpt.x - dxBdr, dyBdr, PATCOPY);
        PatBlt(hdc, rect.right - 2 * dxBdr, dyBdr,
          dxBdr, dpt.y - 2 * dyBdr, PATCOPY);
        PatBlt(hdc, rect.right - 3 * dxBdr, 2 * dyBdr,
          dxBdr, dpt.y - 3 * dyBdr, PATCOPY);
        }
    if (hbrsSav != NULL)
        SelectObject(hdc, hbrsSav);
    if (hbrs != NULL)
        DeleteObject(hbrs);
    }

LRESULT CALLBACK __export
LwSubclassProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Subclass owner to find out when it moves.      */
/*****************************************************/
    {
    WNDPROC lpfn;

    if ((lpfn = (WNDPROC)MAKELP(
      GetProp(hwnd, szProcSel),
      GetProp(hwnd, szProcOff))) == NULL)
        return 0; /* Can't get original wnd proc! */

    switch (wm)
        {
    default:
        break;

    case WM_MOVE:
        EnumTaskWindows(GetCurrentTask(), FEnumWnd, 0);
        break;

    case WM_DESTROY:
        RemoveProp(hwnd, szProcOff);
        RemoveProp(hwnd, szProcSel);
        SubclassWindow(hwnd, lpfn);
        break;
        }

    return
      CallWindowProc(lpfn, hwnd, wm, wParam, lParam);
    }

BOOL CALLBACK __export
FEnumWnd(HWND hwnd, LPARAM lParam)
/*****************************************************/
/* -- EnumTaskWindows() callback.                    */
/* -- Reposition all popup push buttons.             */
/*****************************************************/
    {
    char szClass[32];

    GetClassName(hwnd, szClass, sizeof szClass);
    if (lstrcmp(szClass, szPopupPushClass) == 0)
        {
        RECT    rect;

        GetWindowRect(GetWindowOwner(hwnd), &rect);
        SetWindowPos(hwnd, NULL, rect.left +
            GetSystemMetrics(SM_CXFRAME) +
            GetSystemMetrics(SM_CXSIZE) +
            GetSystemMetrics(SM_CXBORDER) +
            GetWindowWord(hwnd, bXPos),
          rect.top + GetSystemMetrics(SM_CYFRAME),
          0, 0,
          SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOZORDER);
        }
    return TRUE;
    }
