// regrtest.c - regression testing for WUIMAN classes.

#include <stdio.h>
#include <string.h>
#include "wuistd.h"
#include "wuidbint.h"
#include "wuiobjdb.h"
#include "regrtest.h"

int    Test_TWuiPath()
    {
    char    PathBuffer[256];
    strcpy(PathBuffer, "/Classes/TAttribute");
    TWuiPath    Path(PathBuffer);
    ASSERT2(
        strcmp(Path.AbsolutePath(), "/Classes/TAttribute") == FALSE
        );
    ASSERT2(
        strcmp(Path.RelativePath(), "/Classes/TAttribute") == FALSE
        );
    ASSERT2(
        Path.Descend() == TRUE
        );
    ASSERT2(
        strcmp(Path.AbsolutePath(), "/Classes/TAttribute") == FALSE
        );
    ASSERT2(
        strcmp(Path.RelativePath(), "Classes/TAttribute") == FALSE
        );
    ASSERT2(
        Path.CurrentName() == "Classes"
        );
    ASSERT2(
        Path.Ascend() == TRUE
        );
    ASSERT2(
        strcmp(Path.RelativePath(), "/Classes/TAttribute") == FALSE
        );
    return TRUE;
    }

int     Test_TWuiDatabase()
    {
    TWuiDatabase    Test("killme.wui");
    ASSERT2(
        Test.WriteItem("/", "TMenu", "/,") != FALSE
        );
    char    Buffer[128];
    ASSERT2(
        Test.ReadItem("/", "TMenu", Buffer, 128) != FALSE
        );
    ASSERT2( strcmp("/,", Buffer) == 0 );
    ASSERT2(
        Test.DeleteItem("/", "TMenu") != FALSE
        );
    ASSERT2(
        Test.ReadItem("/", "TMenu", Buffer, 128) == -1
        );
    ASSERT2(
        Test.DeletePath("/") != FALSE
        );
    return TRUE;
    }


void    Test(char *ClassName, HWND ResultWindow)
    {
    int     Result;
    char    Message[128];
    sprintf(Message, "Testing class '%s'", ClassName);
    SetWindowText(ResultWindow, Message);
    if(!strcmp(ClassName, "TWuiDatabase"))
        Result = Test_TWuiDatabase();
    else if(!strcmp(ClassName, "TWuiPath"))
        Result = Test_TWuiPath();
    sprintf(Message, "%s: class '%s'", Result ?
                          "SUCCESS" : "FAILURE", ClassName);
    SetWindowText(ResultWindow, Message);
    }

BOOL CALLBACK _export DlgProc(HWND, UINT, WPARAM, LPARAM);

HINSTANCE   ThisInstance;

#ifdef __BORLANDC__
    #pragma argsused
#endif
int PASCAL WinMain(HINSTANCE Me, HINSTANCE Previous,
		     LPSTR lpszCmdLine, int nCmdShow)
    {
    ThisInstance    = Me;
    DialogBox(Me, "Main", NULL, (DLGPROC)DlgProc);
    return 0;
    }

#define SEND(Control, Message, Param1, Param2) \
SendDlgItemMessage(Dialog, Control, Message, Param1, Param2)

#ifdef __BORLANDC__
    #pragma argsused
#endif
BOOL CALLBACK _export DlgProc(HWND Dialog, UINT Message,
                             WPARAM Param1, LPARAM Param2)
    {

    if(Message == WM_INITDIALOG)
        {
        SEND(ID_CLASS_COMBO, CB_ADDSTRING, 0, (LPARAM)(LPSTR)"<all>");
        SEND(ID_CLASS_COMBO, CB_ADDSTRING, 0, (LPARAM)(LPSTR)"TWuiDatabase");
        SEND(ID_CLASS_COMBO, CB_ADDSTRING, 0, (LPARAM)(LPSTR)"TWuiPath");
        SEND(ID_CLASS_COMBO, CB_SETCURSEL, 0, 0);
        }
    else if(Message == WM_COMMAND)
        if(Param1 == IDOK || Param1 == IDCANCEL) {
            EndDialog(Dialog, 0);
            return TRUE;
            }
        else if(Param1 == ID_TEST_BUTTON)
            {
            char    ClassName[128];
            SEND(ID_CLASS_COMBO, WM_GETTEXT, 127, (LPARAM)(LPSTR)ClassName);
            Test(ClassName, GetDlgItem(Dialog, ID_RESULT_STATIC));
            }
    return FALSE;
    }

