#include <stdlib.h>
#include <string.h>
#include <windows.h>

#include "wuistd.h"
#include "wuiobjdb.h"
#include "wuiman.h"
#include "wuimanui.h"

#include "testrc.h"

BOOL CALLBACK _export WUIMAN_SelectObjectDialog(HWND Dialog,
                 UINT Message, WPARAM Param1, LPARAM Param2);

HBITMAP ObjectClosedBitmap;
HBITMAP ObjectOpenBitmap;


const int CUSTOMHEIGHT = 15;

struct  CustomList
    {
    CustomList  *Parent;
    char    Text[32];
    int     Level;
    int     LastSibling;
    AWuiObject  *Object;
    };


static
CustomList  *NewEntry(HWND Window, AWuiObject *Node,
                              CustomList *Parent, int Level)
    {
    CustomList  *Entry  = new CustomList;
    Node->Name().GetName(Entry->Text, 32);
    Entry->Parent       = Parent;
    Entry->Level        = Level;
    Entry->Object       = Node;
    Entry->LastSibling  = FALSE;
    SendMessage(Window, LB_ADDSTRING, 0, (LPARAM)Entry);
    return Entry;
    }

static
int     LoadLevel(HWND Listbox, CustomList *Parent,
                                 AWuiObject *Node, int Level)
    {
    AWuiObject  *Children    = Node->Child("Children");
    if(!Children)
        return 0;
    int     NChildren = Children->NChildren();
    CustomList  *Entry = 0;
    for(int iChild = 0; iChild < NChildren; ++iChild)
        {
        Entry = NewEntry(Listbox, Children->Child(iChild),
                                             Parent, Level);
        LoadLevel(Listbox, Entry, Children->Child(iChild), Level+1);
        }
    if(Entry)
	Entry->LastSibling  = TRUE;
    return NChildren;
    }

static
int     LoadDatabase(HWND Listbox)
    {
    extern  AWuiObject    *WUIMAN_Root;
    SendMessage(Listbox, LB_RESETCONTENT, 0, 0);
    CustomList  *Root   = NewEntry(Listbox, WUIMAN_Root, 0, 0);
    Root->LastSibling   = TRUE;
    return LoadLevel(Listbox, Root, WUIMAN_Root, 1);
    }

static
CustomList *CurrentEntry(HWND Dialog)
    {
    HWND    Listbox = GetDlgItem(Dialog, ID_OBJDB_LISTBOX);
    DWORD   Index;
    Index   = SendMessage(Listbox, LB_GETCURSEL, 0, 0);
    if(Index != LB_ERR)
        return (CustomList *)SendMessage(Listbox,
                          LB_GETITEMDATA, (WPARAM)Index, 0);
    else
        return 0;
    }
static
void    ObjectFillCombo(HWND Dialog, int Combo, AWuiObject *Object)
    {
    if(Object)
        {
        for(int i=0; i < Object->NChildren(); ++i)
            {
            char    ObjectName[32];
            Object->Child(i)->Name().GetName(ObjectName);
            SendDlgItemMessage(Dialog, Combo, CB_ADDSTRING,
                    0, (LPARAM)(LPCSTR)ObjectName);
            }
        SendDlgItemMessage(Dialog, Combo, CB_SETCURSEL, 0, 0);
        }
    }

static
void    SetPath(CustomList *Entry, char *Path)
    {
    if(Entry->Parent)
        SetPath(Entry->Parent, Path);
    if(strlen(Path) > 1)
        strcat(Path, "/");
    strcat(Path, Entry->Text);
    }

static
void    SelectObject(HWND Dialog)
    {
    CustomList  *Entry = CurrentEntry(Dialog);
    if(!Entry)
        return;
    if(!strcmp(Entry->Text, "/"))
        {
        EnableWindow(GetDlgItem(Dialog, ID_INSERT_BUTTON), 0);
        EnableWindow(GetDlgItem(Dialog, ID_APPEND_BUTTON), 0);
        }
    else
        {
        EnableWindow(GetDlgItem(Dialog, ID_INSERT_BUTTON), 1);
        EnableWindow(GetDlgItem(Dialog, ID_APPEND_BUTTON), 1);
        }
    if(Entry->Object->Child("Children"))
        EnableWindow(GetDlgItem(Dialog, ID_NEWCHILD_BUTTON), 1);
    else
        EnableWindow(GetDlgItem(Dialog, ID_NEWCHILD_BUTTON), 0);
    SendDlgItemMessage(Dialog, ID_PROPERTY_COMBO, CB_RESETCONTENT, 0, 0);
    SendDlgItemMessage(Dialog, ID_EVENT_COMBO, CB_RESETCONTENT, 0, 0);

    char    Path[128];
    Path[0] = '\0';
    SetPath(Entry, Path);
    char    Name[32];
    SetDlgItemText(Dialog, ID_OBJECT_NAME, Path);
    Entry->Object->Class().GetName(Name);
    SetDlgItemText(Dialog, ID_CLASS_NAME, Name);
    ObjectFillCombo(Dialog, ID_PROPERTY_COMBO,
                     Entry->Object->Child("Properties"));
    ObjectFillCombo(Dialog, ID_EVENT_COMBO,
                     Entry->Object->Child("Events"));
    }

#ifdef __BORLANDC__
    #pragma argsused
#endif
BOOL CALLBACK ObjectListbox(HWND Dialog, UINT Message,
                             WPARAM Param1, LPARAM Param2)
    {
    switch(Message)
        {
        case    WM_MEASUREITEM  :
            {
            MEASUREITEMSTRUCT *Measure = (MEASUREITEMSTRUCT *)Param2;
            Measure->itemWidth  = 255;
            Measure->itemHeight = CUSTOMHEIGHT;
            return TRUE;
            }
        case    WM_DRAWITEM :
            {
            const DRAWITEMSTRUCT *Draw = (const DRAWITEMSTRUCT *)Param2;
            CustomList *Item    = (CustomList *)Draw->itemData;
            RECT        TextRect= Draw->rcItem;
            TextRect.left += 5 + (Item->Level * 12);
            if(Draw->itemAction & (ODA_DRAWENTIRE | ODA_SELECT))
                {
                HFONT   Font = (HFONT)GetStockObject(ANSI_FIXED_FONT);
                HPEN    Pen  = CreatePen(PS_SOLID, 2, RGB(0,0,0));
                SelectObject(Draw->hDC, Pen);
                if(Draw->itemState & ODS_SELECTED)
                    {
                    SetBkColor(Draw->hDC, RGB(198,198,198));
                    SetTextColor(Draw->hDC, RGB(255,0,0));
                    }
                else
                    {
                    SetBkColor(Draw->hDC, RGB(255,255,255));
                    SetTextColor(Draw->hDC, RGB(0,0,255));
                    }
                ExtTextOut(Draw->hDC, 0, 0, ETO_OPAQUE,
                              &TextRect, NULL, 0, NULL);
                SetTextAlign(Draw->hDC, TA_UPDATECP);
                int  Vertical   = Draw->rcItem.top;
                int  Horizontal = 0;
                for(int i = 0; i < Item->Level; ++i)
                    {
                    CustomList *Parent = Item;
                    for(int j=i; j < Item->Level; ++j)
                        Parent  = Parent->Parent;
                    if(!Parent->LastSibling)
                        {
                        MoveTo(Draw->hDC, Horizontal, Vertical);
                        LineTo(Draw->hDC, Horizontal, Vertical+CUSTOMHEIGHT);
                        }
                    Horizontal  += 12;
                    }
                if(Item->Level > 0)
                    {
                    MoveTo(Draw->hDC, Horizontal, Vertical);
                    LineTo(Draw->hDC, Horizontal, Vertical
                                        + (CUSTOMHEIGHT/2));
                    LineTo(Draw->hDC, Horizontal+5, Vertical
                                        + (CUSTOMHEIGHT/2));
                    if(!Item->LastSibling)
                        {
                        MoveTo(Draw->hDC, Horizontal, Vertical + (CUSTOMHEIGHT/2));
                        LineTo(Draw->hDC, Horizontal, Vertical
                                        + CUSTOMHEIGHT);
                        }
                    }
                MoveTo(Draw->hDC, Horizontal+6, Vertical);
                SelectObject(Draw->hDC, Font);
                TextOut(Draw->hDC, 0, 0, Item->Text, strlen(Item->Text));
                /////
                if(Draw->itemState & ODS_FOCUS)
                    DrawFocusRect(Draw->hDC, &TextRect);
                DeleteObject(SelectObject(Draw->hDC, GetStockObject(BLACK_PEN)));
                }
            else if(Draw->itemAction & ODA_FOCUS)
                DrawFocusRect(Draw->hDC, &TextRect);
            return TRUE;
            }
        case    WM_DELETEITEM   :
            {
            const DELETEITEMSTRUCT *Delete = (DELETEITEMSTRUCT *)Param2;
            CustomList  *Entry = (CustomList *)Delete->itemData;
            delete Entry;
            return TRUE;
            }
        }
    return FALSE;
    }

#ifdef __BORLANDC__
    #pragma argsused
#endif
BOOL CALLBACK _export WUIMAN_MainDialog(HWND Dialog, UINT Message,
                             WPARAM Param1, LPARAM Param2)
    {
    extern  HINSTANCE   WUIMAN_Instance;

    if(Message == WM_INITDIALOG)
        {
        extern char WUIMAN_IniFile[];
        char    Buffer[128];
        wsprintf(Buffer, "WUIMAN [%s]", WUIMAN_IniFile);
        SetWindowText(Dialog, Buffer);
        LoadDatabase(GetDlgItem(Dialog, ID_OBJDB_LISTBOX));
#if 0
        ObjectClosedBitmap  = LoadBitmap(WUIMAN_Instance,
                                "WUIMAN_OBJECT_CLOSED_BMP");
        ObjectOpenBitmap    = LoadBitmap(WUIMAN_Instance,
                                  "WUIMAN_OBJECT_OPEN_BMP");
#endif
        return TRUE;
        }
    else if(Message == WM_MEASUREITEM && Param1 == ID_OBJDB_LISTBOX)
        return ObjectListbox(Dialog, Message, Param1, Param2);
    else if(Message == WM_DRAWITEM && Param1 == ID_OBJDB_LISTBOX)
        return ObjectListbox(Dialog, Message, Param1, Param2);
    else if(Message == WM_DELETEITEM && Param1 == ID_OBJDB_LISTBOX)
        return ObjectListbox(Dialog, Message, Param1, Param2);
    else if(Message == WM_CHARTOITEM/* && Param1 == ID_OBJDB_LISTBOX*/)
        {
        switch(Param1)
            {
            case    VK_INSERT   :
                MessageBox(NULL, "VK_INSERT", "", MB_OK);
                break;
            case    VK_DELETE   :
                MessageBox(NULL, "VK_DELETE", "", MB_OK);
                break;
            default :
                MessageBox(NULL, "Other key!", "", MB_OK);
                break;
            }
        }
    else if(Message == WM_PAINT && IsIconic(Dialog))
        {
        }
    else if(Message == WM_COMMAND)
        {
        switch(Param1)
            {
            case    IDOK        :
            case    IDCANCEL    :
                EndDialog(Dialog, 0);
                return TRUE;
            case    ID_OBJDB_LISTBOX    :
                switch(HIWORD(Param2))
                    {
                    case    LBN_SELCHANGE   :
                        SelectObject(Dialog);
                        return TRUE;
                    }
                return FALSE;
            case    ID_DELETE_BUTTON    :
                {
                CustomList *Current = CurrentEntry(Dialog);
                if(Current == 0)
                    {
                    MessageBox(Dialog, "Select an object first.",
                                                "WUIMAN", MB_OK);
                    return TRUE;
                    }
                char    Path[128];
                Path[0] = '\0';
                SetPath(Current, Path);
                WUIMAN_Send(Path, "Delete", 0, 0);
                LoadDatabase(GetDlgItem(Dialog, ID_OBJDB_LISTBOX));
                return TRUE;
                }
            case    ID_INSERT_BUTTON    :
            case    ID_APPEND_BUTTON    :
            case    ID_NEWCHILD_BUTTON  :
                {
                CustomList *Current = CurrentEntry(Dialog);
                if(Current == 0)
                    {
                    MessageBox(Dialog, "Select an object first.",
                                                "WUIMAN", MB_OK);
                    return TRUE;
                    }
                AWuiObject   *Cloned = 0;
                DialogBoxParam(WUIMAN_Instance,
                             "WUIMAN_SELECT_OBJECT", Dialog,
                         (DLGPROC)WUIMAN_SelectObjectDialog,
                                        (LPARAM)&Cloned);
                if(Cloned == 0)   // if user made no selection
                    return TRUE;
                if(Param1 == ID_NEWCHILD_BUTTON)
                    {
                    AWuiObject   *Children = Current->Object->Child("Children");
                    ASSERT(Children != 0);
                    Children->Insert(Cloned);
                    }
                LoadDatabase(GetDlgItem(Dialog, ID_OBJDB_LISTBOX));
                char    Path[128];
                Path[0] = '\0';
                SetPath(Current, Path);
                WUIMAN_Send(Path, "Redraw", 0, 0);
                return TRUE;
                }
            default :
                return FALSE;
            }
        }
    else if(Message == WM_DESTROY)
        {
        }
    return FALSE;
    }



BOOL CALLBACK _export WUIMAN_SelectObjectDialog(HWND Dialog,
                 UINT Message, WPARAM Param1, LPARAM Param2)
    {
    switch(Message)
        {
        case    WM_INITDIALOG   :
            {
            SetWindowLong(Dialog, DWL_USER, Param2);
            LoadDatabase(GetDlgItem(Dialog, ID_OBJDB_LISTBOX));
            SendDlgItemMessage(Dialog, ID_CLONE_DEPTH,
                EM_LIMITTEXT, 3, 0);
            return TRUE;
            }
        case    WM_MEASUREITEM  :
            return ObjectListbox(Dialog, Message, Param1, Param2);
        case    WM_DRAWITEM     :
            return ObjectListbox(Dialog, Message, Param1, Param2);
        case    WM_DELETEITEM   :
            return ObjectListbox(Dialog, Message, Param1, Param2);
        case    WM_COMMAND      :
            switch(Param1)
                {
                case    IDOK        :
                    {
                    // get selected object, clone to desired
                    // depth, and return via DWL_USER pointer

                    CustomList  *Entry = CurrentEntry(Dialog);
                    if(Entry == 0)
                        EndDialog(Dialog, -1);
                    char    DepthString[5];
                    GetDlgItemText(Dialog, ID_CLONE_DEPTH, DepthString, 4);
                    int     Depth   = atoi(DepthString);
                    AWuiObject   **Result = (AWuiObject **)
                            GetWindowLong(Dialog, DWL_USER);
                    char    NewName[64];
                    GetDlgItemText(Dialog, ID_NEWNAME_TEXT, NewName, 64);
                    *Result = Entry->Object->Clone(Depth);
                    (*Result)->Rename(NewName);
                    EndDialog(Dialog, 0);
                    }
                case    IDCANCEL    :
                    EndDialog(Dialog, -1);
                    return TRUE;
                case    ID_OBJDB_LISTBOX    :
                    switch(HIWORD(Param2))
                        {
                        case    LBN_SELCHANGE   :
                            CustomList  *Entry = CurrentEntry(Dialog);
                            SetDlgItemText(Dialog, ID_NEWNAME_TEXT, Entry->Text);
                            return TRUE;
                        }
                    default             :
                    return FALSE;
                }
        default :
            return FALSE;
        }
    }

