// wuiobjdb.h - Definitions for WUIman OBJect DataBase.
// Copyright (c) 1993 by Ron Burk

#ifndef WUIOBJDB_H
#define WUIOBJDB_H

#if !(defined(_INC_WINDOWS) || defined(__WINDOWS_H))
    #include <windows.h>
#endif
#include "wuistd.h"
#include "wuiname.h"
#include "wuiargs.h"
#include "wuilist.h"

class   TWuiDatabase;
class   AWuiObject;

// TWuiObjects - a list of type TWuiObject
class   TWuiObjects : public TWuiGenericList
    {
public:
    TWuiObjects(int Size=0);
    void       *Insert(AWuiObject *NewObject, const TWuiName To);
    void       *Insert(AWuiObject *NewObject, int Position=-1);
    AWuiObject *Get(const TWuiName Which);
    AWuiObject *Get(int Position);
    AWuiObject *Remove(const TWuiName Which);
    AWuiObject *Remove(int Position = -1);
    int         NObjects();
private:
    int         Find(const TWuiName Which);
    };

inline
AWuiObject *TWuiObjects::Remove(int Position)
    {   MEMBERASSERT();
    return (AWuiObject *)TWuiGenericList::Remove(Position);
    }
inline
AWuiObject *TWuiObjects::Get(int Position)
    {   MEMBERASSERT();
    return (AWuiObject *)TWuiGenericList::Get(Position);
    }
inline
int     TWuiObjects::NObjects()
    {   MEMBERASSERT();
    return TWuiGenericList::NElements();
    }

// TWuiPath - Class to handle absolute and relative paths.
// Note: this version is unsafe -- Push() may write past
// end of string.
class   TWuiPath
    {
public:
    TWuiPath(char *TotalPath);
    int         Descend();
    int         Ascend();
    int         Push(const char *SubPath);
    int         Pop();
    TWuiName    CurrentName();
    const char *AbsolutePath();
    const char *RelativePath();
private:
    char *AbsolutePath_;
    char *RelativePath_;
    };

inline
const char *TWuiPath::AbsolutePath()
    {   MEMBERASSERT();
    return AbsolutePath_;
    }
inline
const char *TWuiPath::RelativePath()
    {   MEMBERASSERT();
    return RelativePath_;
    }

class   AWuiObject
    {
public:
    AWuiObject(const TWuiName ObjectName,
                                     const char *Value = 0);
    virtual              ~AWuiObject();
    virtual AWuiObject   *Clone() = 0;
    virtual AWuiObject   *New(const TWuiName Name,
                                     const char *Value) = 0;
    virtual TWuiObjects  *Children();
    virtual TWuiObjects  *Attributes();
    virtual long          Get(TWuiPath Path, TWuiArgs *Args,
                                      AWuiObject *Attribute,
                            char *Buffer, size_t MaxLength);
    virtual long          Set(TWuiPath Path, TWuiArgs *Args,
                                      AWuiObject *Attribute,
                                         const char *Buffer);
    void                  Trace(const char *Function,
                              TWuiPath Path, TWuiArgs *Args);

    TWuiName              Name();
    static  AWuiObject   *Root;
    static  TWuiDatabase *Database;
    static  int           TraceFlag;
private:
    long                  InitialLoad(char *Path);
    TWuiName              Name_;
    };

inline
TWuiName AWuiObject::Name()
    {   
    return Name_;
    }

class   TWuiRegister
    {
public:
    TWuiRegister(AWuiObject *MasterCopy);
    };

////////////
// 11/06/93
// TWuiContainer - An AWuiObject that supports children and attributes.
class   TWuiContainer : public AWuiObject
    {
public:
    TWuiContainer(const TWuiName ObjectName, const char *Value=0);
    virtual AWuiObject   *Clone();
    virtual AWuiObject   *New(const TWuiName Name, const char *Value);
    virtual TWuiObjects  *Children();
    virtual TWuiObjects  *Attributes();
private:
    TWuiObjects          *Children_;
    TWuiObjects          *Attributes_;
    };

// TWuiAttribute - Base class for attributes.
class   TWuiAttribute : public AWuiObject
    {
public:
    TWuiAttribute(const TWuiName ObjectName,
                                       const char *Value=0);
    virtual AWuiObject   *Clone();
    virtual AWuiObject   *New(const TWuiName Name,
                                         const char *Value);
    virtual long          Get(TWuiPath Path, TWuiArgs *Args,
                                      AWuiObject *Attribute,
                            char *Buffer, size_t MaxLength);
    virtual long          Set(TWuiPath Path, TWuiArgs *Args,
                                      AWuiObject *Attribute,
                                        const char *Buffer);
protected:
    const char           *Value();
private:
    char                 *Value_;
    };

inline
const char *TWuiAttribute::Value()
    {   MEMBERASSERT();
    return Value_;
    }

class   TWuiPersistantAttribute : public TWuiAttribute
    {
public:
    TWuiPersistantAttribute(const TWuiName ObjectName,
                                         const char *Value);
    virtual AWuiObject   *Clone();
    virtual AWuiObject   *New(const TWuiName Name,
                                       const char *Value=0);
    virtual long          Set(TWuiPath Path, TWuiArgs *Args,
                                      AWuiObject *Attribute,
                                        const char *Buffer);
    };

#endif
