/*-----------------------------------------------------
 *  LISTING 3
 *
 *  Filename:           DISPLAY.C
 *  Summary:            Driver module for memory-
 *                      mapped display hardware in
 *                      text mode
 *  Author:             T.W. Nelson
 *  Compiler:           BORLAND
 *  Compile options:    large model only (-ml)
 *  Version:            1.00
 *  Date:               22-Feb-1993
 *  Notes:
 *
 *  Must be linked with drive header "drvhdr.obj"
 *  1st in the link order.
 *-------------------------------------------------- */

#include <dos.h>
#include "drv.h"

extern  unsigned _heapseg;      //from drvhdr.asm
extern  unsigned   _loadseg;

int errno = 0;      //needed by std library code
int major = 1;
int minor = 0;
char id[] = "IBMPCDISPLAY";

#define monoSeg 0xB000
#define colorSeg 0xB800
#define UP   0
#define DOWN 1

unsigned dispSeg = monoSeg;     //assume mono display
int nrows = 25;                 //with 80x25 screen
int ncols = 80;
int curs_x, curs_y;     //current cursor position
int s_scan, e_scan; //current cursor type (size)
int v_mode;     //video mode

#pragma inline

/* ------------------------------------------------- */
DRVFUNC setCursorPos( int col, int row, int page )
{
    union REGS reg;
    reg.x.ax = 0x0200;       //BIOS set cursor pos
    reg.h.bh = (char) page;
    reg.h.dh = (char) row;
    reg.h.dl = (char) col;
    int86( 0x10, &reg, &reg );

    return TRUE;
}

DRVFUNC getCursorPos( int *col, int *row, int page )
{
    union REGS reg;
    reg.x.ax = 0x0300;      //BIOS get cursor pos
    reg.h.bh = (char) page;
    int86( 0x10, &reg, &reg );
    *col = (int) reg.h.dl;
    *row = (int) reg.h.dh;

    return TRUE;
}

DRVFUNC setCursorType( int startln, int endln )
{
    union REGS reg;
    reg.x.ax = 0x0100;      //BIOS set cursor type
    reg.h.ch = (char) startln;
    reg.h.cl = (char) endln;
    int86( 0x10, &reg, &reg );

    return TRUE;
}

DRVFUNC getCursorType( int *startln,
                       int *endln, int page )
{
    union REGS reg;
    reg.x.ax = 0x0300;      //BIOS get cursor pos
    reg.h.bh = (char) page;
    int86( 0x10, &reg, &reg );
    *startln = (int) reg.h.ch;
    *endln = (int) reg.h.cl;

    return TRUE;
}

DRVFUNC setVideoMode( int mode )
{
    union REGS reg;
    reg.h.ah = 0x00;      //BIOS set video mode
    reg.h.al = (char) mode;
    int86( 0x10, &reg, &reg );

    return TRUE;
}

DRVFUNC getVideoMode( int *mode )
{
    union REGS reg;
    reg.h.ah = 0x0f;      //BIOS get video mode
    int86( 0x10, &reg, &reg );
    *mode = (int) reg.h.al;

    return TRUE;
}

DRVFUNC setVideoPage( int page )
{
    union REGS reg;
    reg.h.ah = 0x05;      //BIOS set video page
    reg.h.al = (char) page;
    int86( 0x10, &reg, &reg );
    return TRUE;
}

DRVFUNC getVideoPage( int *page )
{
    union REGS reg;
    reg.h.ah = 0x0f;      //BIOS get video mode
    int86( 0x10, &reg, &reg );
    *page = (int) reg.h.bh;

    return TRUE;
}

DRVFUNC scrollWindow(int direction, //0=up, !0=down
                     int nlines,    //#lines to scroll
                     int attr,      //blanking attr
                     int uplf_y,    //upper left row
                     int uplf_x,    //upper left col
                     int lwrt_y,    //lower right row
                     int lwrt_x )   //lower right col
{
    union REGS reg;
    reg.h.ah = (direction == UP ? 0x06 : 0x07);
    reg.h.al = (char) nlines;
    reg.h.bh = (char) attr;
    reg.h.ch = (char) uplf_y;
    reg.h.cl = (char) uplf_x;
    reg.h.dh = (char) lwrt_y;
    reg.h.dl = (char) lwrt_x;
    int86( 0x10, &reg, &reg );

    return TRUE;
}

DRVFUNC hideCursor(void)
{
    getCursorType( &s_scan, &e_scan, 0 );
    setCursorType( 0x20, e_scan );
    return TRUE;
}

DRVFUNC revealCursor(void)
{
    setCursorType( s_scan, e_scan );
    return TRUE;
}

DRVFUNC displayStr( char *string, int col, int row )
{
   /* Displays asciiz string at given screen
    * coordinates and existing video attribute. This
    * code has no workaround for CGA snow glitch.
    */

    int disoff = (row * ncols * 2) + (col * 2);

    asm mov es,dispSeg
    asm mov di,disoff       //dest
    asm lds si,string       //source
again:
    asm lodsb               //load source byte
    asm cmp al,0            //at end?
    asm je quit             //yes
    asm stosb               //no, write it
    asm inc di              //skip attrib byte
    asm jmp again           //get next
quit:
    return TRUE;
}

DRVFUNC clearScrn( int attrib )
{
   /* Clears entire video display using space chars
    * and the desired attribute.
    */

    scrollWindow( UP, 0, attrib, 0, 0,
                         nrows - 1, ncols - 1 );
    return TRUE;
}

DRVFUNC shutdown(void)
{
    //Function called automatically by Driver Manager
    //during module unload.  This need not be listed
    //in fTable[] unless you need access to it from
    //your base application ........

    return TRUE;
}

//Driver function export table .........
struct expTab fTable[] = {
    "", driver_init,    //initializer always 1st
    "setCursorPos", setCursorPos,
    "getCursorPos", getCursorPos,
    "setCursorType", setCursorType,
    "getCursorType", getCursorType,
    "setVideoMode", setVideoMode,
    "getVideoMode", getVideoMode,
    "setVideoPage", setVideoPage,
    "getVideoPage", getVideoPage,
    "scrollWindow", scrollWindow,
    "hideCursor", hideCursor,
    "revealCursor", revealCursor,
    "displayStr", displayStr,
    "clearScrn",  clearScrn,
    0,0                 //ending nulls *required*
    };

DRVFUNC driver_init( struct ModuleDescriptor *mDesc )
{
   /* Compute how much memory is to be used for the
    * module (in paragraphs), set module version and
    * ID certification, set a pointer to the function
    * export table, and perform other initialization
    * tasks such as hardware detection. User may
    * allocate memory as needed after _heapseg for a
    * heap or other workspace.
    */

    union REGS reg;
    unsigned as_needed = 0;

    mDesc->modsize =
            (_heapseg + as_needed) - _loadseg;
    mDesc->v_major = major;
    mDesc->v_minor = minor;
    mDesc->id = id;
    mDesc->fTab = fTable;
    mDesc->exit_f = shutdown;
    reg.h.ah = 0x0f;      //BIOS get video mode
    int86( 0x10, &reg, &reg );
    ncols = (int) reg.h.ah;     //#screen columns
    getCursorPos( &curs_x, &curs_y, 0 );
    getVideoMode( &v_mode );

    if( v_mode != 7 )
        dispSeg = colorSeg;

    return TRUE;
}

/*------ End of File ------------------------------- */
