/*****************************************************/
/* childmnu.c                                        */
/* -- Application demonstrates multiple menued child */
/*    windows.                                       */
/* -- To build: cc childmnu.c childmnu.rc            */
/*****************************************************/
#include <windows.h>
#include <windowsx.h>
#include "childmnu.h"

#define cbBufMax    256 /* Generic string size. */

/* Globals. */
char    szChildClass[]  = "ChildClass";
char    szMainClass[]   = "MainClass";
char    szMainTitle[]   = "Menued Child Window Demo";
HWND    hwndMain;   /* Main window. */
HWND    hwndActive; /* Active child window. */

/* Prototypes. */
void    CreateHybrid(HWND hwndParent);
BOOL CALLBACK _export
  FDestroyHybrid(HWND hwnd, LPARAM lParam);
BOOL    FIsHybrid(HWND hwnd);
HWND    HwndNextHybrid(HWND hwndChild);
LRESULT CALLBACK _export
  LwHybridWndProc(HWND hwnd, UINT wm, WPARAM wParam,
    LPARAM lParam);
LRESULT CALLBACK _export
  LwMainWndProc(HWND hwnd, UINT wm, WPARAM wParam,
    LPARAM lParam);

#ifdef __BORLANDC__
    #pragma argsused
#endif
int PASCAL WinMain(HINSTANCE hins, HINSTANCE hinsPrev,
  LPSTR lpsz, int wShow)
/*****************************************************/
/* -- Entry point.                                   */
/*****************************************************/
    {
    MSG         msg;
    WNDCLASS    wcs;

    if (!hinsPrev)
        {
        /* Register the main class. */
        wcs.style = 0;
        wcs.lpfnWndProc = LwMainWndProc;
        wcs.cbClsExtra = 0;
        wcs.cbWndExtra = 0;
        wcs.hInstance = hins;
        wcs.hIcon = LoadIcon(NULL, IDI_APPLICATION);
        wcs.hCursor = LoadCursor(NULL, IDC_ARROW);
        wcs.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
        wcs.lpszMenuName = MAKEINTRESOURCE(idmMain);
        wcs.lpszClassName = szMainClass;
        if (!RegisterClass(&wcs))
            return FALSE;

        /* Register the child class. */
        wcs.lpfnWndProc = LwHybridWndProc;
        wcs.lpszMenuName = MAKEINTRESOURCE(idmChild);
        wcs.lpszClassName = szChildClass;
        if (!RegisterClass(&wcs))
            return FALSE;
        }

    msg.wParam = 0;
    if ((hwndMain = CreateWindow(szMainClass,
      szMainTitle, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT,
      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
      NULL, NULL, hins, NULL)) != NULL)
        {
        ShowWindow(hwndMain, wShow);
        while (GetMessage(&msg, NULL, 0, 0))
            {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
            }
        }

    return msg.wParam;
    }

LRESULT CALLBACK _export LwMainWndProc(HWND hwnd,
  UINT wm, WPARAM wParam, LPARAM lParam)
/*****************************************************/
/* -- Main window procedure.                         */
/*****************************************************/
    {
    switch (wm)
        {
    default:
        break;

    case WM_COMMAND:
        if (wParam == idmFileNew)
            {
            CreateHybrid(hwnd);
            return 0;
            }
        break;

    case WM_DESTROY:
        /* Explicitly destroy hybrids. */
        EnumChildWindows(hwnd, FDestroyHybrid, 0);
        PostQuitMessage(0);
        break;

    case WM_NCACTIVATE:
        if (wParam)
            {
            if (hwndActive != NULL)
                DefWindowProc(hwndActive, wm, TRUE, 0);
            }
        else
            {
            if (hwndActive != (HWND)(UINT)lParam &&
              hwndActive != NULL)
                DefWindowProc(hwndActive, wm, FALSE,
                  0);
            if (FIsHybrid((HWND)(UINT)lParam))
                return TRUE;
            }
        break;  /* End case WM_NCACTIVATE. */

    case WM_USER:
        SetActiveWindow(hwnd);
        return 0;
        }       /* End switch wm. */

    return DefWindowProc(hwnd, wm, wParam, lParam);
    }

void CreateHybrid(HWND hwndParent)
/*****************************************************/
/* -- Create a new hybrid.                           */
/*****************************************************/
    {
    RECT    rect;
    HWND    hwndChild;
    char    szBuf[cbBufMax];

    GetClientRect(hwndParent, &rect);
    hwndChild = CreateWindow(szChildClass, NULL,
      WS_SYSMENU | WS_CAPTION | WS_CLIPSIBLINGS |
        WS_MAXIMIZEBOX | WS_MINIMIZEBOX |
        WS_THICKFRAME | WS_POPUP,
      rect.right / 4, rect.bottom / 4, rect.right / 2,
      rect.bottom / 2, hwndParent, NULL,
      GetWindowInstance(hwndParent), NULL);
    if (hwndChild != NULL)
        {
        SetParent(hwndChild, hwndParent);
        wsprintf(szBuf, "Child %x", hwndChild);
        SetWindowText(hwndChild, szBuf);
        ShowWindow(hwndChild, SW_SHOWNORMAL);
        }
    }

#ifdef __BORLANDC__
    #pragma argsused
#endif
BOOL CALLBACK _export FDestroyHybrid(HWND hwnd,
  LPARAM lParam)
/*****************************************************/
/* -- EnumChildWindows() callback destroys all       */
/*    hybrids.                                       */
/*****************************************************/
    {
    if (FIsHybrid(hwnd))
        DestroyWindow(hwnd);
    return TRUE;
    }

BOOL FIsHybrid(HWND hwnd)
/*****************************************************/
/* -- Return whether the window is a hybrid.         */
/*****************************************************/
    {
    char    szBuf[cbBufMax];

    if (GetWindowTask(hwnd) != GetCurrentTask())
        return FALSE;
    GetClassName(hwnd, szBuf, sizeof(szBuf) - 1);
    return !lstrcmp(szBuf, szChildClass);
    }

LRESULT CALLBACK _export LwHybridWndProc(HWND hwnd,
  UINT wm, WPARAM wParam, LPARAM lParam)
/*****************************************************/
/* -- Hybrid window procedure.                       */
/*****************************************************/
    {
    DWORD   lwStyle;
    HWND    hwndSibling;
    char    szBuf[cbBufMax];

    switch (wm)
        {
    default:
        break;

    case WM_NCACTIVATE:
        if (wParam)
            {
            hwndActive = hwnd;
            DefWindowProc(hwndMain, wm, TRUE, 0);
            }
        else
            {
            if ((HWND)(UINT)lParam == hwndMain)
                return TRUE;
            if (!FIsHybrid((HWND)(UINT)lParam))
                DefWindowProc(hwndMain, wm, FALSE, 0);
            }
        break;

    case WM_SYSKEYDOWN:
        if (wParam == VK_MENU)
            {
            SetActiveWindow(hwndMain);
            PostMessage(hwndMain, wm, wParam, lParam);
            return 0;
            }
        break;

    case WM_DESTROY:
        /* Activate another hybrid if there is one. */
        if (hwnd == hwndActive)
            {
            hwndActive = NULL;
            if ((hwndSibling = HwndNextHybrid(hwnd)) !=
              NULL)
                SetActiveWindow(hwndSibling);
            }
        break;

    case WM_COMMAND:
        /* Prove that the hybrid's menu works. */
        wsprintf(szBuf,
          "Menu %d of popup %d was picked",
          wParam % 10, (wParam % 100) / 10);
        MessageBox(hwndMain, szBuf, szMainTitle,
          MB_OK);
        break;

    case WM_WINDOWPOSCHANGING:
        /* Use child style when iconic to prevent */
        /* desktop from poking around sides of icon. */
        lwStyle = GetWindowLong(hwnd, GWL_STYLE);
        if ((lwStyle & (WS_POPUP | WS_CHILD)) ==
          (IsIconic(hwnd) ? WS_POPUP : WS_CHILD))
            SetWindowLong(hwnd, GWL_STYLE,
              lwStyle ^ (WS_CHILD | WS_POPUP));
        break;
        }

    return DefWindowProc(hwnd, wm, wParam, lParam);
    }

HWND HwndNextHybrid(HWND hwndChild)
/*****************************************************/
/* -- Return next hybrid sibling.                    */
/*****************************************************/
    {
    HWND    hwnd;

    for (hwnd = GetWindow(hwndMain, GW_CHILD);
      hwnd != NULL;
      hwnd = GetWindow(hwnd, GW_HWNDNEXT))
        if (hwnd != hwndChild && FIsHybrid(hwnd))
            return hwnd;
    return NULL;
    }
