/*****************************************************
Name:
   BINDCTLS.C, Listing 1
File Type:
   Library
Description:
   Library of functions for supporting data binding
   to controls.
Function List:
   BindCtlData
   CtlProcBindButton
   CtlProcBindDialog
   CtlProcBindEdit
   RegisterBindButton
   RegisterBindDialog
   RegisterBindEdit
Portability:
   MS Windows
*****************************************************/

/* MS Windows */
#include <windows.h>

/* Own */
#include "bindctls.h"

static WNDPROC _CtlProcButton;
static WNDPROC _CtlProcEdit;

LRESULT BindCtlData( HWND hWnd, LPARAM lParam )
   {

   int i;
   LPBINDCTLDATA lpData = (LPBINDCTLDATA)lParam;

   for ( i = 0; lpData[i].lpData; i++ )
      {
      SendDlgItemMessage( hWnd, lpData[i].iCtlId,
            WM_USER_BINDDATA,
            0, lpData[i].lpData );
      }

   return ( 0L );

   }   /* function BindCtlData */


LRESULT CALLBACK __export CtlProcBindButton( HWND hWnd,
      UINT uiMsg, WPARAM wParam, LPARAM lParam )
   {

   if ( uiMsg == WM_USER_BINDDATA )
      {
      SetWindowLong( hWnd, GWL_BINDBUTTON, lParam );
      if ( lParam )
         {
         SendMessage( hWnd, BM_SETCHECK, (WPARAM)
               *( (int FAR *)lParam ), 0L );
         }
      return ( 0L );
      }   /* if ( uiMsg == WM_USER_BINDDATA ) */

   if ( uiMsg == WM_KILLFOCUS )
      {

      int FAR *p = (int FAR *)GetWindowLong( hWnd,
            GWL_BINDBUTTON );
      if ( p )
         {
         *p = (int)SendMessage( hWnd,
               BM_GETCHECK, 0, 0L );
         }
      }   /* if ( uiMsg == WM_KILLFOCUS ) */

   else if ( uiMsg == BM_SETCHECK )
      {
      
      int FAR *p = (int FAR *)GetWindowLong( hWnd,
            GWL_BINDBUTTON );
      if ( p )
         {
         *p = (int)wParam;
         }
      }   /* else if ( uiMsg == BM_SETCHECK ) */

   return ( CallWindowProc( _CtlProcButton, hWnd,
         uiMsg, wParam, lParam ));

   }  /* function CtlProcBindButton */


LRESULT CALLBACK __export CtlProcBindEdit( HWND hWnd,
      UINT uiMsg, WPARAM wParam, LPARAM lParam )
   {

   if ( uiMsg == WM_USER_BINDDATA )
      {
      SetWindowLong( hWnd, GWL_BINDEDIT, lParam );
      if ( lParam )
         {
         SetWindowText( hWnd, (LPSTR)lParam );
         }

      return ( 0L );

      }   /* if ( uiMsg == WM_USER_BINDDATA ) */

   if ( uiMsg == WM_KILLFOCUS )
      {

      LPSTR lpstr = (LPSTR)GetWindowLong( hWnd,
            GWL_BINDEDIT );

      if ( lpstr )
         {
         GetWindowText( hWnd, lpstr, -1 );
         }
      }   /* if ( uiMsg == WM_KILLFOCUS ) */

   else if ( uiMsg == WM_SETTEXT )
      {

      LRESULT lResult =
            CallWindowProc(_CtlProcEdit, hWnd,
            uiMsg, wParam, lParam );
      LPSTR lpstr = (LPSTR)GetWindowLong( hWnd,
            GWL_BINDEDIT );

      if ( lpstr )
         {
         GetWindowText( hWnd, lpstr, -1 );
         }
      
      return ( lResult );

      }   /* else if ( uiMsg == WM_SETTEXT ) */


   return ( CallWindowProc( _CtlProcEdit, hWnd,
         uiMsg, wParam, lParam ));

   }  /* function CtlProcBindEdit */


BOOL RegisterBindButton( HINSTANCE hInstance )
   {

   WNDCLASS wc;

   GetClassInfo( NULL, "BUTTON", &wc );
   _CtlProcButton = wc.lpfnWndProc;
   wc.lpfnWndProc = CtlProcBindButton;
   wc.hInstance = hInstance;
   wc.lpszClassName = "BINDBUTTON";
   wc.cbWndExtra += 1 + sizeof ( void FAR *);

   return ( RegisterClass( &wc ));

   }   /* RegisterBindEdit */


BOOL RegisterBindDialog( HINSTANCE hInstance )
   {

   WNDCLASS wc;

   wc.style         = 0;
   wc.lpfnWndProc   = WndProcBindDialog;
   wc.cbClsExtra    = 0;
   wc.cbWndExtra    = DLGWINDOWEXTRA;
   wc.hInstance     = hInstance;
   wc.hIcon         = NULL;
   wc.hCursor       = LoadCursor( NULL, IDC_ARROW );
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
   wc.lpszMenuName  = NULL;
   wc.lpszClassName = "BINDDIALOG";

   return ( RegisterClass( &wc ));

   }   /* RegisterBindDialog */


BOOL RegisterBindEdit( HINSTANCE hInstance )
   {

   WNDCLASS wc;

   GetClassInfo( NULL, "EDIT", &wc );
   _CtlProcEdit = wc.lpfnWndProc;
   wc.lpfnWndProc = CtlProcBindEdit;
   wc.hInstance = hInstance;
   wc.lpszClassName = "BINDEDIT";
   wc.cbWndExtra += sizeof ( void FAR *);

   return ( RegisterClass( &wc ));

   }   /* RegisterBindEdit */


LRESULT CALLBACK __export WndProcBindDialog( HWND hWnd,
      UINT uiMsg, WPARAM wParam, LPARAM lParam )
   {

   LRESULT lResult;

   if ( uiMsg == WM_INITDIALOG && lParam )
      {
      BindCtlData( hWnd, lParam );
      }

   if ( GetWindowLong( hWnd, DWL_DLGPROC ) ==
         (LONG)WndProcBindDialog )
      {
      SetWindowLong( hWnd, DWL_DLGPROC, 0L );
      }

   lResult = DefDlgProc( hWnd, uiMsg, wParam, lParam );

   if ( uiMsg == WM_COMMAND && IsWindow( hWnd ) &&
         ( wParam == IDOK || wParam == IDCANCEL ))
      {
      if ( GetFocus() != GetDlgItem( hWnd, wParam ))
         {
         SetFocus( GetDlgItem( hWnd, wParam ));
         }
      EndDialog( hWnd,
            ( wParam == IDOK ? TRUE : FALSE ));
      }

   return ( lResult );

   }   /* function WndProcBindDialog */

/* End of BINDCTLS.C */
