///////////////////////////////////////////////////////
// demoactv.c                                        //
// -- Program uses FRegisterActiveProc() to display  //
//    a message the active window changes.           //
// -- Written by Paul R. Bonneau, 11/26/94.          //
///////////////////////////////////////////////////////
#include <windows.h>
#include <windowsx.h>
#include "activtrk.h"

char szClass[] = "ActiveWindowDisplay";

void FormatInfoWnd(HWND hwnd, char *pszOut,
  BOOL fNew);

LRESULT CALLBACK __export LwWndProc(HWND hwnd,
  UINT wm, WPARAM wParam, LPARAM lParam);

#ifdef __BORLANDC__
    #pragma argsused
#endif
int PASCAL WinMain(HINSTANCE hins, HINSTANCE hinsPrev,
  LPSTR lpsz, int wShow)
    {
    MSG  msg;
    HWND hwnd;

    if (NULL == hinsPrev)
        {
        WNDCLASS wcs;

        wcs.style       = CS_HREDRAW | CS_VREDRAW;
        wcs.lpfnWndProc = LwWndProc;
        wcs.cbClsExtra  = 0;
        wcs.cbWndExtra  = 0;
        wcs.hInstance   = hins;
        wcs.hIcon       = LoadIcon((HINSTANCE)NULL,
                                           IDI_APPLICATION);
        wcs.hCursor     = LoadCursor((HINSTANCE)NULL,
                                                 IDC_ARROW);
        wcs.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
        wcs.lpszMenuName = NULL;
        wcs.lpszClassName = szClass;
        if (!RegisterClass(&wcs))
            return FALSE;
        }

    msg.wParam = 0;
    if (NULL != (hwnd = CreateWindowEx(WS_EX_TOPMOST,
      szClass, "Active Window", WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
      CW_USEDEFAULT, (HWND)NULL, (HMENU)NULL, hins, NULL)))
        {
        ShowWindow(hwnd, wShow);
        while (GetMessage(&msg, (HWND)NULL, 0, 0))
            {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
            }
        }

    return msg.wParam;
    }

LRESULT CALLBACK __export
LwWndProc(HWND hwnd, UINT wm, WPARAM wParam, LPARAM lParam)
    {
    PAINTSTRUCT wps;
    char        szBuf[512];
    static HWND hwndNew;
    static HWND hwndOld;
    RECT        rect;

    switch (wm)
        {
    default:
        break;

    case WM_CREATE:
        if (!FRegisterWnd(hwnd))
            return -1;
        break;

    case WM_DESTROY:
        UnregisterWnd(hwnd);
        PostQuitMessage(0);
        break;

	case wmActive:
        hwndNew = (HWND)wParam;
        hwndOld = (HWND)LOWORD(lParam);
        InvalidateRect(hwnd, NULL, TRUE);
        UpdateWindow(hwnd);
        return 0;

    case WM_PAINT:
        BeginPaint(hwnd, &wps);
        szBuf[0] = 0;
        FormatInfoWnd(hwndNew, szBuf, TRUE);
        FormatInfoWnd(hwndOld, szBuf, FALSE);
        GetClientRect(hwnd, &rect);
        DrawText(wps.hdc, szBuf, -1, &rect,
          DT_LEFT | DT_WORDBREAK | DT_EXPANDTABS);
        EndPaint(hwnd, &wps);
        return 0;
        }

    return DefWindowProc(hwnd, wm, wParam, lParam);
    }

void FormatInfoWnd(HWND hwnd, char *pszOut, BOOL fNew)
///////////////////////////////////////////////////////
// -- Format window information into a text buffer.  //
// -- hwnd      : Window to report on.               //
// -- pszOut    : Output buffer.                     //
// -- fNew      : Input parameter, set for new       //
//                active window msg, clear for       //
//                previous.                          //
///////////////////////////////////////////////////////
    {
    char szTitle[256];
    char *pszType = fNew ? "New" : "\r\nPrevious";
    char *pszTitle;

    if (IsWindow(hwnd))
        {
        GetWindowText(hwnd, szTitle, sizeof szTitle);
        pszTitle = szTitle;
        }
    else
        pszTitle = "Invalid";
    wsprintf(pszOut + lstrlen(pszOut),
      "%s active window %x\r\n\tTitle: %s",
      (LPSTR)pszType, (UINT)hwnd, (LPSTR)pszTitle);
    }
