///////////////////////////////////////////////////////
// wait.c                                            //
// -- Routine waits for another task to terminate.   //
// -- Written by Paul R. Bonneau 11/25/94.           //
///////////////////////////////////////////////////////
#include <windows.h>
#include <windowsx.h>
#include <toolhelp.h>
#include "wait.h"

BOOL CALLBACK __export FNotifyProc(WORD nfy, DWORD lw);

HTASK htskCaller, htskSpawned;
UINT  wmPrivate;

BOOL CALLBACK __export FWaitHins(HINSTANCE hins,
                                              BOOL *pfError)
///////////////////////////////////////////////////////
// -- Wait for the given instance to terminate.      //
// -- Returns TRUE if the calling task should        //
//    terminate.                                     //
// -- hins    : Instance of task to wait for.        //
// -- pfError : Set on return if an error occurred.  //
///////////////////////////////////////////////////////
    {
    TASKENTRY   tke;
    MSG         msg;
    BOOL        fResult     = FALSE;
    BOOL        fRegistered = FALSE;
    FARPROC     lpfn        = NULL;
    static BOOL fBusy;

    *pfError = TRUE;

    if (fBusy)
        goto LCleanup; // Already servicing a request.
    fBusy = TRUE;

    if (0 == (wmPrivate =
      RegisterWindowMessage("SpawnedTaskTerminated")))
        goto LCleanup;

    // Get an instance thunk for notification proc.
    htskCaller = GetCurrentTask();
    tke.dwSize = sizeof tke;
    TaskFindHandle(&tke, htskCaller);
    if (NULL == (lpfn = MakeProcInstance(
      (FARPROC)FNotifyProc, tke.hInst)))
        goto LCleanup;

    // Find task handle for the given instance.
    htskSpawned = NULL;
    TaskFirst(&tke);
    do
        if (tke.hInst == hins)
            {
            htskSpawned = tke.hTask;
            break;
            }
    while (TaskNext(&tke));

    // If we couldn't find the task, it has terminated
    // already.
    if (NULL == htskSpawned)
        {
        *pfError = FALSE;
        goto LCleanup;
        }

    // Install callback that looks for terminations.
    if (!NotifyRegister(htskCaller,
      (LPFNNOTIFYCALLBACK)lpfn, NF_NORMAL))
        goto LCleanup;
    fRegistered = TRUE;

    // Now, just lounge around until we receive the private
    // message from the callback
    *pfError = FALSE;
    for (;;)
        {
        if (!GetMessage(&msg, NULL, 0, 0))
            {
            fResult = TRUE;
            goto LCleanup;
            }

        if (msg.message == wmPrivate)
            break;

        TranslateMessage(&msg);
        DispatchMessage(&msg);
        }

LCleanup:
    if (fRegistered)
        NotifyUnRegister(htskCaller);
    if (NULL != lpfn)
        FreeProcInstance(lpfn);
    fBusy = FALSE;
    return fResult;
    }

#ifdef __BORLANDC__
    #pragma argsused
#endif
BOOL CALLBACK __export FNotifyProc(WORD nfy, DWORD lw)
///////////////////////////////////////////////////////
// -- NotifyRegister() callback.                     //
///////////////////////////////////////////////////////
    {
    if (NFY_EXITTASK == nfy &&
      GetCurrentTask() == htskSpawned)
        PostAppMessage(htskCaller, wmPrivate, 0, 0);

    return 0;
    }

#ifdef __BORLANDC__
    #pragma argsused
#endif //__BORLANDC__
int CALLBACK LibMain(HINSTANCE hins, WORD ds,
                                     WORD cbHeap, LPSTR lsz)
    {
    return 1;
    }

// Symantec doesn't supply a default WEP
#ifdef __BORLANDC__
    #pragma argsused
#endif
int CALLBACK __export WEP(int wExit)
    {
    return 1;
    }


