// wincovw.cpp : implementation of the CWincolorView class
//

#include "stdafx.h"
#include "wincolor.h"

#include "wincodoc.h"
#include "wincovw.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWincolorView

IMPLEMENT_DYNCREATE(CWincolorView, CFormView)

BEGIN_MESSAGE_MAP(CWincolorView, CFormView)
	//{{AFX_MSG_MAP(CWincolorView)
	ON_WM_CTLCOLOR()
	ON_WM_VSCROLL()
	ON_WM_SHOWWINDOW()
	ON_BN_CLICKED(IDC_RESET, OnReset)
	ON_WM_HSCROLL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWincolorView construction/destruction

CWincolorView::CWincolorView()
	: CFormView(CWincolorView::IDD)
{
	//{{AFX_DATA_INIT(CWincolorView)
	m_EditRed = "";
	m_EditBlue = "";
	m_EditGreen = "";
	//}}AFX_DATA_INIT
	
	// Make some brushes for the ScrollBars.
	m_hRedBrush =   ::CreateSolidBrush( RGB( 255, 0, 0)); 
	m_hGreenBrush = ::CreateSolidBrush( RGB( 0, 255, 0)); 
	m_hBlueBrush =  ::CreateSolidBrush( RGB( 0, 0, 255)); 
	
	m_hCompBrush = m_hAverageBrush = NULL;
}

CWincolorView::~CWincolorView()
{  
	// Delete any brush resources we used  .
	BOOL z = ::DeleteObject( m_hRedBrush);
	z = ::DeleteObject( m_hGreenBrush);
	z = ::DeleteObject( m_hBlueBrush);
	z = ::DeleteObject( m_hCompBrush);
	z = ::DeleteObject( m_hAverageBrush);
}

void CWincolorView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWincolorView)
	DDX_Control(pDX, IDC_SCROLL_AVERAGE, m_ScrollAverage);
	DDX_Control(pDX, IDC_SCROLL_GREEN, m_ScrollGreen);
	DDX_Control(pDX, IDC_SCROLL_BLUE, m_ScrollBlue);
	DDX_Control(pDX, IDC_SCROLL_RED, m_ScrollRed);
	DDX_Text(pDX, IDC_EDIT_RED, m_EditRed);
	DDX_Text(pDX, IDC_EDIT_BLUE, m_EditBlue);
	DDX_Text(pDX, IDC_EDIT_GREEN, m_EditGreen);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
// CWincolorView diagnostics

#ifdef _DEBUG
void CWincolorView::AssertValid() const
{
	CFormView::AssertValid();
}

void CWincolorView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CWincolorDoc* CWincolorView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CWincolorDoc)));
	return (CWincolorDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CWincolorView message handlers

// color the controls
HBRUSH CWincolorView::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{                    
	// Get the control ID we are now working on.
	int	CtrlID = pWnd->GetDlgCtrlID();
	                                                 
	// Fill in the colors for each spin-button ScrollBar.
	switch( CtrlID)
   {     
   		case IDC_SCROLL_RED: 	return( m_hRedBrush);
   		case IDC_SCROLL_GREEN: 	return( m_hGreenBrush);
   		case IDC_SCROLL_BLUE: 	return( m_hBlueBrush);
   		case IDC_SCROLL_AVERAGE:			   
   			if( m_hAverageBrush!= NULL)
   				DeleteObject( m_hAverageBrush); 
   			
   			// create a brush the opposite of dialog background
   			m_hAverageBrush = ::CreateSolidBrush( 0x00ffffff - RGB( m_ScrollRed.GetScrollPos(), m_ScrollGreen.GetScrollPos(), m_ScrollBlue.GetScrollPos()));	 
				return m_hAverageBrush;   					
   			
	}
	
	// Update the combination of colors displayed in the client area.
	if( nCtlColor != CTLCOLOR_EDIT)
	{                            
		if( m_hCompBrush != NULL)
			DeleteObject( m_hCompBrush);	// release the old resource
		
		// Create a brush according to the combination of ALL spin-buttons.
		m_hCompBrush = ::CreateSolidBrush( RGB( m_ScrollRed.GetScrollPos(), m_ScrollGreen.GetScrollPos(), m_ScrollBlue.GetScrollPos()));
		return( m_hCompBrush);
	}		
	
	return CFormView::OnCtlColor(pDC, pWnd, nCtlColor);
}
 
// Deal with scrolling.
void CWincolorView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{  
	int	nCtrlID;
	char	str[4];
	int	nMove;
	int	total;
	
	// We only care about SB_LINEUP and SB_LINEDOWN.
	if( nSBCode == SB_LINEUP)
		nMove = 1;
	else if( nSBCode == SB_LINEDOWN)
		nMove = -1;
	else nMove = 0;		
	  
	nCtrlID = pScrollBar->GetDlgCtrlID();
	
	// If the id is not one of our colors, pass it on.
	if( (nCtrlID == IDC_SCROLL_RED ||
		 nCtrlID == IDC_SCROLL_GREEN ||
		 nCtrlID == IDC_SCROLL_BLUE) &&
		 nMove != 0)
	{  
		// Change the CScrollBar position.
		pScrollBar->SetScrollPos( pScrollBar->GetScrollPos() + nMove);
		
		// set the average of the RGB CScrollBars
		total = m_ScrollRed.GetScrollPos() + m_ScrollGreen.GetScrollPos() + m_ScrollBlue.GetScrollPos();
		m_ScrollAverage.SetScrollPos( total/3);
		
		// Display the CScrollBar position in correct edit box.
		wsprintf( str, "%3d", pScrollBar->GetScrollPos());
		switch( nCtrlID)
		{
			case IDC_SCROLL_RED:
				m_EditRed = str;	
				break;
			case IDC_SCROLL_GREEN:
				m_EditGreen = str;
				break;
			case IDC_SCROLL_BLUE:
				m_EditBlue = str;
				break;
		}
		
		// Update display.
		UpdateData( FALSE);	                                      
		
		// Force the client view to take the new composite color.
		CWnd::Invalidate();
		return;	
	}						 

	// Now go on and do the default stuff.
	CWnd::OnVScroll(nSBCode, nPos, pScrollBar);		
}

// This is not crucial for this demonstration, but these simple steps
// make the appearance more attractive.
VOID CWincolorView::OnInitialUpdate()
{
	// Make the window the size of the main dialog box.
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit(); 
	
	// Update the window title.
	GetDocument()->SetTitle( "W/DDJ");
}

// This is the last function before the window is show- sort of.
// If we post a message, we can set the values for the static
// ScrollBars.
void CWincolorView::OnShowWindow(BOOL bShow, UINT nStatus)
{    
	
	CFormView::OnShowWindow(bShow, nStatus);
    
	// Do some maintenance on the ScrollBars.
	PostMessage( WM_COMMAND, IDC_RESET);
	 
	// Show the new color values.
	UpdateData( FALSE);	
}

void CWincolorView::OnReset()
{  
	// Set the range for the ScrollBars.
	m_ScrollRed.SetScrollRange( 0, 255, TRUE);
	m_ScrollGreen.SetScrollRange( 0, 255, TRUE);
	m_ScrollBlue.SetScrollRange( 0, 255, TRUE);
	m_ScrollAverage.SetScrollRange( 0, 255, TRUE);
	   
   // Show the current status of the ScrollBars.
   m_ScrollRed.ShowScrollBar();
	m_ScrollGreen.ShowScrollBar();
	m_ScrollBlue.ShowScrollBar();
	m_ScrollAverage.ShowScrollBar();
	
	// Set the current position of the ScrollBars.
	m_ScrollRed.SetScrollPos( 192);
	m_ScrollGreen.SetScrollPos( 192);
	m_ScrollBlue.SetScrollPos( 192);
	m_ScrollAverage.SetScrollPos( 192);
   
   // Display the initial values of the ScrollBars.
   m_EditRed = "192";
   m_EditGreen = "192";
   m_EditBlue = "192";
                                                  
	// Make a brush for the background.
	m_hCompBrush = ::CreateSolidBrush( RGB( 192, 192, 192));	
	
	// create a brush the opposite of dialog background
	m_hAverageBrush = ::CreateSolidBrush( 0x00ffffff - RGB( 192, 192, 192));	
	
	UpdateData( FALSE);	                                      
	
	// Force the client view to take the new composite color.
	CWnd::Invalidate();	
}

void CWincolorView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{   
	// the average bar can't be directly controlled
   if( pScrollBar->GetDlgCtrlID() == IDC_SCROLL_AVERAGE)
   	return;
   	
	CFormView::OnHScroll(nSBCode, nPos, pScrollBar);
}
