#include "malloc.h"
#include <toolhelp.h>
#include "dlg3d.h"
extern "C" {
    #include "lmalloc.h"
    };

int IsFixed(HGLOBAL Handle)
    {
    UINT Flags = GlobalFlags(Handle);
    if(((UINT)Handle & 1) && !(Flags&GMEM_DISCARDABLE))
        return TRUE;    // it's FIXED
    else if((Flags&GMEM_LOCKCOUNT) != 0)
        return TRUE;    // it's fixed
    else
        return FALSE;
    }  

static void MemDebug(HGLOBAL Target, char *Buffer)
    {
    GLOBALENTRY Info;
    BOOL    Status;
    *Buffer = '\0';
    Info.dwSize = sizeof(Info);
    Status  = GlobalEntryHandle(&Info, Target);
    if(Status)
        wsprintf(Buffer,
            "handle[0x%04X] selector[0x%04X] fixed[%c] "
            "linear address [0x%08lX] "
            "wcLock=0x%04X, wcPageLock=0x%04X, "
            "wFlags=0x%04X, GlobalFlags=0x%04X",
            (WORD)Target, GlobalHandleToSel(Target),
            IsFixed(Target)?'Y':'N',
            Info.dwAddress, Info.wcLock, Info.
              wcPageLock, Info.wFlags, GlobalFlags(Target));
    }

const DWORD MEMSIZE = (499);

struct TDllExe
    {
    HGLOBAL (PASCAL *Alloc)(UINT, DWORD);
    HGLOBAL (PASCAL *ReAlloc)(HGLOBAL, DWORD, UINT);
    HGLOBAL (PASCAL *Free)(HGLOBAL);
    void    (PASCAL *Fix)(HGLOBAL);
    UINT    (PASCAL *PageLock)(HGLOBAL);
    void FAR *(PASCAL *Lock)(HGLOBAL);
    void    Set(HWND Dialog)
        {
        if((int) SendDlgItemMessage(Dialog,
                                 ID_EXE, BM_GETCHECK, 0, 0))
            {
            Alloc   = GlobalAlloc;
            ReAlloc = GlobalReAlloc;
            Free    = GlobalFree;
            Fix     = GlobalFix;
            PageLock= GlobalPageLock;
            Lock    = GlobalLock;
            }
        else
            {
            Alloc   = DllGlobalAlloc;
            ReAlloc = DllGlobalReAlloc;
            Free    = DllGlobalFree;
            Fix     = DllGlobalFix;
            PageLock= DllGlobalPageLock;
            Lock    = DllGlobalLock;
            }
        }
    };

struct TFlagInfo
    {
    char       *FlagName;
    UINT        FlagBits;
    };

static TFlagInfo FlagInfo[] =
        {
        { "GHND",               GHND                },
        { "GMEM_DDESHARE",      GMEM_DDESHARE       },
        { "GMEM_DISCARDABLE",   GMEM_DISCARDABLE    },
        { "GMEM_FIXED",         GMEM_FIXED          },
        { "GMEM_LOWER",         GMEM_LOWER          },
        { "GMEM_MODIFY",        GMEM_MODIFY         },
        { "GMEM_MOVEABLE",      GMEM_MOVEABLE       },
        { "GMEM_NOCOMPACT",     GMEM_NOCOMPACT      },
        { "GMEM_NODISCARD",     GMEM_NODISCARD      },
        { "GMEM_NOT_BANKED",    GMEM_NOT_BANKED     },
        { "GMEM_NOTIFY",        GMEM_NOTIFY         },
        { "GMEM_SHARE",         GMEM_SHARE          },
        { "GMEM_ZEROINIT",      GMEM_ZEROINIT       },
        { "GPTR",               GPTR                },
        };
#define NFLAGS (sizeof(FlagInfo)/sizeof(TFlagInfo))
#define LBM(m, a, b) SendMessage(FlagBox, m, a, b)

static UINT GetFlags(HWND Dialog)
    {
    HWND FlagBox = GetDlgItem(Dialog, ID_ALLOCFLAGS);
    int  FlagIds[100];
    UINT Flags = 0;

    int NFlags = (int) LBM(LB_GETSELCOUNT, 0, 0);
    LBM(LB_GETSELITEMS, NFlags, (LPARAM)FlagIds);
    for(int i=0; i < NFlags; ++i)
        {
        TFlagInfo *Info = (TFlagInfo *)
        LBM(LB_GETITEMDATA, FlagIds[i], 0);
        Flags   |= Info->FlagBits;
        }
    return Flags;
    }

static void InitDialog(HWND Dialog)
    {
    HWND FlagBox = GetDlgItem(Dialog, ID_ALLOCFLAGS);
    for(int i = 0; i < NFLAGS; ++i)
        {
        LBM(LB_ADDSTRING, 0, (LPARAM)FlagInfo[i].FlagName);
        LBM(LB_SETITEMDATA, i, (LPARAM)(void *)&FlagInfo[i]);
        }
    SendDlgItemMessage(Dialog, ID_EXE, BM_SETCHECK, TRUE, 0);
    }

BOOL CALLBACK __export DialogProc(HWND Dialog, UINT Message,
                               WPARAM Param1, LPARAM Param2)
    {
    static  HGLOBAL Handle;
    static  UINT    Flags;
    static  TDllExe DllExe;
    if(Message == WM_INITDIALOG)
        {
        InitDialog(Dialog);
        Handle      = NULL;
        DllExe.Set(Dialog);
        }
    else if(Message == WM_COMMAND)
        {
        if(Param1 == ID_EXE || Param1 == ID_DLL)
            DllExe.Set(Dialog);
        else if(Param1 == ID_ALLOCFLAGS)
            Flags   = GetFlags(Dialog);
        else if(Param1 == IDOK || Param1 == IDCANCEL)
            EndDialog(Dialog, TRUE);
        else if(HIWORD(Param2) == BN_CLICKED)
            {
            if(Param1 == ID_GLOBALALLOC)
                Handle  = DllExe.Alloc(Flags, MEMSIZE);
            else if(Param1 == ID_GLOBALREALLOC)
                Handle  = DllExe.ReAlloc(Handle, MEMSIZE, Flags);
            else if(Param1 == ID_GLOBALFREE)
                Handle  = DllExe.Free(Handle);
            else if(Param1 == ID_GLOBALFIX)
                DllExe.Fix(Handle);
            else if(Param1 == ID_GLOBALPAGELOCK)
                DllExe.PageLock(Handle);
            else if(Param1 == ID_GLOBALLOCK)
                DllExe.Lock(Handle);
            char Buffer[256];
            MemDebug(Handle, Buffer);
            SetDlgItemText(Dialog, ID_HANDLE, Buffer);
            }
        }
    return FALSE;
    }

int PASCAL WinMain(HINSTANCE Me, HINSTANCE, LPSTR, int)
    {
    Register3d(Me);
    DialogBox(Me, "MALLOC", NULL, DialogProc);
    Register3d(NULL);
    return TRUE;
    }
