#include "clipbase.h"
#include <stdio.h>
#include <string.h>
#pragma hdrstop
#define STOP_PRECOMPILATION

static char DataPath[256];  // root directory for data
static char*Fragment = 0;   // text for current selection

#define MSG(s) MessageBox(NULL, s, "ClipBase", MB_OK);

// StrClone - allocate a copy of a string
char *StrClone(const char *S)
    {
    size_t  Length = strlen(S);
    char *Result = new char[Length+1];
    memcpy(Result, S, Length+1);
    return Result;
    };

// TNode - Store data for each node in tree
struct  TNode
    {
    TNode(const char *Path_, int IsDir);
    char   *Path;
    char   *Text;
    };

TNode::TNode(const char *Path_, int IsDir)
        : Path(StrClone(Path_)), Text(0)
    {
    char FileName[256];
    strcpy(FileName, Path_);
    if(IsDir)
        strcat(FileName, "\\.txt");
    // read in text of corresponding file
    FILE *File = fopen(FileName, "r");
    if(File != NULL)
        {
        char *Result = (char *)malloc(16 * 1024);
        *Result = '\0';
        size_t N = fread(Result, 1, 16*1024,  File);
        Result[N] = '\0';
        fclose(File);
        Text    = StrClone(Result);
        free(Result);
        }
    }

// Init - use .ini file to establish base directory.
static int Init()
    {
    GetPrivateProfileString("ClipBase", "DataPath",
        "c:\\clipbase", DataPath, sizeof(DataPath),
        "clipbase.ini");
    DWORD Status = GetFileAttributes(DataPath);
    if(Status == 0xFFFFFFFF ||
      (Status & FILE_ATTRIBUTE_DIRECTORY) == 0)
        {
        char Msg[256];
        wsprintf(Msg, "'%s': is not a directory!", DataPath);
        MSG(Msg);
        return FALSE;
        }
    else
        return TRUE;
    }

// ExpandLevel - Fill in children of a tree node (directory)
int  ExpandLevel(HWND W, HTREEITEM Parent)
    {
    WIN32_FIND_DATA     Data;
    HANDLE              Handle;
    char                Path[256];
    TV_INSERTSTRUCT Entry;
    Entry.hParent           = Parent;
    Entry.hInsertAfter      = TVI_SORT;
    Entry.item.mask         = TVIF_TEXT | TVIF_CHILDREN
                            | TVIF_PARAM;
    if(Parent == NULL)
        strcpy(Path, DataPath);
    else
        {
        TV_ITEM     Info;
        Info.mask     = TVIF_PARAM | TVIF_HANDLE;
        Info.hItem    = Parent;
        TreeView_GetItem(W, &Info);
        TNode *Node = (TNode *)Info.lParam;
        // if this level was already expanded once
        if(Info.state & TVIS_EXPANDEDONCE)
            return TRUE;    // then we are done!
        strcpy(Path, Node->Path);
        }
    char    Pattern[256];
    strcpy(Pattern, Path);
    strcat(Pattern, "\\*.*");
    Handle = FindFirstFile(Pattern, &Data);
    for(int i=0; Handle != NULL; ++i)
        {
        // don't display . or ..
        if(Data.cFileName[0] != '.')
            {
            int     IsDir = (Data.dwFileAttributes &
                            FILE_ATTRIBUTE_DIRECTORY) != 0;
            char    Path2[256];
            wsprintf(Path2, "%s\\%s", Path, Data.cFileName);
            TNode *Node    = new TNode(Path2, IsDir);
            Entry.item.lParam   = (LPARAM)Node;
            Entry.item.pszText  = Data.cFileName;
            Entry.item.cchTextMax = strlen(Data.cFileName);
            if(IsDir)
                Entry.item.cChildren    = 1;
            else
                Entry.item.cChildren    = 0;
            HTREEITEM Item  = TreeView_InsertItem(W, &Entry);
            }
        if(!FindNextFile(Handle, &Data))
            {
            FindClose(Handle);
            Handle  = NULL;
            }
        }
    return TRUE;
    }

BOOL CALLBACK ClipBase(HWND Dialog, UINT Message,
                       WPARAM Param1, LPARAM Param2)
    {
    if(Message == WM_NOTIFY && (int)Param1 == ID_CLIPTREE)
        {
        NM_TREEVIEW *Data = (NM_TREEVIEW *)Param2;
        if(Data->hdr.code == TVN_ITEMEXPANDING
          && Data->action == TVE_EXPAND)
            ExpandLevel(GetDlgItem(Dialog, ID_CLIPTREE),
                Data->itemNew.hItem);
        else if(Data->hdr.code == TVN_SELCHANGED)
            SetDlgItemText(Dialog, ID_CLIPTEXT, Fragment =
                ((TNode *)Data->itemNew.lParam)->Text);
        }
    else if(Message == WM_INITDIALOG)
        {
        HWND    Tree;
        RECT    Client;
        Tree    = GetDlgItem(Dialog, ID_CLIPTREE);
        ExpandLevel(Tree, NULL);
        SetFocus(Tree);
        return FALSE;
        }
    else if(Message == WM_COMMAND)
        {
        UINT Id = LOWORD(Param1);
        if(Id == IDOK && Fragment != 0)
            {
            if(OpenClipboard(Dialog))
                {
                HGLOBAL Glob = GlobalAlloc(GMEM_MOVEABLE
                    |GMEM_DDESHARE, strlen(Fragment)+1);
                void *Data = GlobalLock(Glob);
                strcpy((char*)Data, Fragment);
                GlobalUnlock(Glob);
                SetClipboardData(CF_TEXT, Glob);
                CloseClipboard();
                }
            else
                MSG("Can't open clipboard!");
            }
        else if(Id == IDCANCEL)
            EndDialog(Dialog, FALSE);
        }
    return FALSE;
    }

int PASCAL WinMain(HINSTANCE Current,
    HINSTANCE Previous, LPSTR CommandLine,
    int ShowCommand)
    {
    InitCommonControls();
    if(Init())
        DialogBox(Current, "ClipBase", NULL, ClipBase);
    return 1;
    }
