/*
	File: ZDialog.h
	Author: Raja
	Date: May 12, 1995.

	The ZDialog class: Making CTL3D.DLL and OnCtlColor work together. 

	Usage: Derive your own dialog classes from ZDialog instead of CDialog,
	and you will automatically make CTL3D.DLL and OnCtlColor work together.

	Note that this problem does not exist with the 32-bit version of
	CTL3D.DLL.
*/

#include "stdafx.h"
#include "zdialog.h"

#if (_MFC_VER == 0x0250)
#include "e:\msvcdist\mfc\src\auxdata.h"
#endif // _MFC_VER

IMPLEMENT_DYNAMIC (ZDialog, CDialog)

BEGIN_MESSAGE_MAP(ZDialog, CDialog)
	//{{AFX_MSG_MAP(ZDialog)
#ifndef WIN32
	ON_WM_CTLCOLOR()
#endif // WIN32
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/*****************************************************************************/
// Raja 12/05/1995.
// Constructors and destructor.
// The constructors resemble the CDialog constructors.
/*****************************************************************************/

ZDialog::ZDialog ()
	:	CDialog (), 
		m_oldDlgProc (NULL)
{
}

ZDialog::ZDialog (LPCSTR lpszTemplateName, CWnd *pParent)
	:	CDialog (lpszTemplateName, pParent), 
		m_oldDlgProc (NULL)
{
}

ZDialog::ZDialog (UINT nID, CWnd *pParent)
	:	CDialog (nID, pParent), 
		m_oldDlgProc (NULL)
{
}

/*****************************************************************************/
// Raja 12/05/1995.
// OnCtlColor - Handler for WM_CTLCOLOR.
// Copied from the CDialog version of OnCtlColor (dlgcore.cpp). The only 
// change is to return NULL instead of calling Default. Calling Default will
// call the dialog window procedure, which will call the dialog procedure
// which will call this function again, to loop forever !
/*****************************************************************************/

#ifndef WIN32

HBRUSH ZDialog::OnCtlColor (CDC *pDC, CWnd *pWnd, UINT nCtlColor)
{
	LRESULT lResult;
	
	if (pWnd->SendChildNotifyLastMsg(&lResult))
		return (HBRUSH)lResult;     // eat it

	if (!GrayCtlColor(pDC->m_hDC, pWnd->GetSafeHwnd(), nCtlColor,
  			afxDlgBkBrush, afxDlgTextClr))
		return (HBRUSH) NULL; // Don't call Default here. Changed !!!
	return afxDlgBkBrush;
}

#endif // WIN32

/*****************************************************************************/
// Raja 12/05/1995.
// ZDialogProcedure - static function.
// Parameters and return value: Same as any Windows dialog procedure.
// Just handles the WM_CTLCOLOR message. 
// Note: This code has been based on the MSDN article: Q109264.
//
// However, there are some changes from the MSDN article:
// 1. Store the message in _afxLastMsg for the OnChildNotify functions of
// child controls in the dialog to be called correctly.
// 2. Use _AfxCallWndProc to send the WM_CTLCOLOR message to a CWnd, 
// rather than calling the OnCtlColor function directly. Otherwise the
// ZDialog::OnCtlColor will always be called rather than the derived class's
// OnCtlColor. Calling _AfxCallWndProc will route OnCtlColor messages through
// the MESSAGE_MAP mechanism.
/*****************************************************************************/

#ifndef WIN32

LRESULT FAR PASCAL _export ZDialog::ZDialogProcedure (HWND hWnd, 
					UINT message, WPARAM wParam, LPARAM lParam)
{
	LRESULT lResult;
	ZDialog *pWnd = (ZDialog *)CWnd::FromHandlePermanent (hWnd);
	ASSERT (pWnd);
	ASSERT (pWnd->IsKindOf (RUNTIME_CLASS (ZDialog)));

	if (message == WM_CTLCOLOR) {

		MSG oldState = _afxLastMsg;    // save for nesting
		_afxLastMsg.hwnd = hWnd;
		_afxLastMsg.message = message;
		_afxLastMsg.wParam = wParam;
		_afxLastMsg.lParam = lParam;

		lResult = _AfxCallWndProc (pWnd, hWnd, message, wParam, lParam);
		_afxLastMsg = oldState;				// Restore old message.
	}
	else
		lResult = ::CallWindowProc (pWnd->m_oldDlgProc, hWnd, message, 
								wParam, lParam);
	return lResult;
}
#endif // WIN32

/*****************************************************************************/
// Raja 04/01/1994.
// OnInitDialog - MFC virtual function handler for WM_INITDIALOG.
// Returns: TRUE to indicate that the first control in the dialog should get
// the focus.
// Subclass the dialog procedure. Note that this uses DWL_DLGPROC instead
//	of GWL_WNDPROC which MFC uses internally whenever it subclasses.
/******************************************************************************/

BOOL ZDialog::OnInitDialog ()
{
	CDialog::OnInitDialog ();
#ifndef WIN32
	m_oldDlgProc = (WNDPROC)::SetWindowLong (GetSafeHwnd (), DWL_DLGPROC,
							(LONG) &ZDialog::ZDialogProcedure);
#endif // WIN32
	return TRUE;
}

