// File:   ccache.h
// Author: Pete Rail November 1994 
// Copyright (c) EDS, 1994.
// Readers of Windows/DOS Developer's Journal are hereby
// granted a nonexclusive, royalty-free license to use, modify 
// and distribute this program.

#ifndef CCACHE_H
#define CCACHE_H

// all objects placed in cache must be derived from CCacheObject
class CCacheObject : public CObject
{            
public:   
    virtual CCacheObject* Duplicate(void) = 0; 
    virtual const char* GetKey( CString& keyValue ) const = 0;
};

class CCache : public CMapStringToOb
{
public: 
    CCache( const char* name, int numEntries=1 );
    ~CCache();   
    BOOL IsFull() const;
    CCacheObject* Lookup( const char* key );
    void RemoveAll();
    BOOL RemoveKey( const char* key );
    void SetAt( const char* key, CCacheObject* newValue );
    
// Get/Set for private variables
    void SetEntryLimit( int numEntries ); 
    int GetEntryLimit() const { return m_EntryLimit; }
    void SetName( const char* name ) { m_Name = name; }
    const char* GetName() const { return (const char*)m_Name; }
    void GetStats( int& hits, int& misses, int& inserts, int& removes ) 
const;
    
private:
    void InitStats();
        
protected:
    CObList m_LRU;         // least-recently used list
    
private:        
    CString m_Name;        // caller supplied name of cache
    int m_EntryLimit;      // max entries allowed in cache
    int m_Hits;            // # successful...
    int m_Misses;          // ...and unsuccessful searches
    int m_Inserts;         // # adds to cache
    int m_Removes;         // # explict/implict removes
};

#endif


