/*************************************************************************

	This program is designed to be launched from a setup pgm to in turn
	set up the video drivers.  It cannot launch the VFW setup until the
	current setup closes.  It tests if VFW is present and it's version.
	If VFW doesn't exist or it's an older version it offers to launch the
	VFW setup pgm. VidSetup pgm is passed a string in the command line
	containing a test file. It waits until the test file no longer exists
	and it is safe to launch the VFW setup pgm.

	To modify
		- modify the various messages (incl CD_TITLE & STR_VERSION)
		- define szSetupPath
		- define szCaption
		- recompile

	To operate:
		- put the video setup program in a directory off the cwd as
			defined by szSetupPath
		- launch this program from the first setup program
		- pass a test file name in the command string of this pgm

*************************************************************************/

#include <windows.h>
#include <ver.h>
				// forward references & defines
long FAR PASCAL _export WndProc(HWND, WORD, WORD, LONG);
#define MAX_BUF			260		// pathname buffer size
#define TIME_OUT		5000	// ms wait for test file to be destroyed
#define ID_STARTTEST	100		// WM_COMMAND msg id
#define MAJOR_VER		1		// minimum major version
#define MINOR_VER		10		// minor version (for min. major version)
#define	STR_VERSION		"1.1"	// string of VFW version
#define CD_TITLE		"Casino Master CD"	// CD-ROM TITLE

PSTR szAppName = "VIDSETUP";	// class name
PSTR szCaption = "Video Setup";	// msg window captions
HINSTANCE hInst;
PSTR szTestFile;			// prev setup file, test for existance
							// setup path off cwd
PSTR szSetupPath = "VFW\\SETUP";
							// default msg - error, no test string
PSTR szDefMsg =
		"To install Video for Windows v "STR_VERSION
		"\nrun SETUP in the \\VFW Directory"
		"\nof the "CD_TITLE".";
							// ask user if want to install VFW
PSTR szAskMsg =
		"This program requires"
		"\nMS Video for Windows v "STR_VERSION" or later."
		"\nA minimal amount of space needed on"
		"\nyour hard drive. Do you want to"
		"\ninstall it on your system now?";
							// close after user decided not to install VFW
PSTR szCloseMsg =
		"To install Video for Windows v "STR_VERSION" at"
		"\na later time, run SETUP in the"
		"\n\\VFW Directory of the "CD_TITLE".";
							// problem installing VFW
PSTR szErrMsg =
		"Couldn't launch Video Setup!\n"
		"You can run Setup manually from the CD-ROM\n"
		"\\VFW directory.";

int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance,
	LPSTR lpszCmdLine, int nCmdShow)
{
  HWND			hWnd;
  MSG           msg;
  WNDCLASS      wndclass;
						// if no test file in cmd line, abort (user
						// probably tried to run the program directly)
  if(!lpszCmdLine || !lpszCmdLine[0])
  {						// default message and close
	MessageBox(NULL,szDefMsg,
				szCaption,MB_OK|MB_ICONEXCLAMATION);
	return 0;
  }
  	
  if(!hPrevInstance) {
	wndclass.style          = CS_HREDRAW | CS_VREDRAW ;
	wndclass.lpfnWndProc	= (WNDPROC)WndProc ;
	wndclass.cbClsExtra     = 0 ;
	wndclass.cbWndExtra     = 0 ;
	wndclass.hInstance      = hInstance ;
	wndclass.hIcon			= NULL ;
	wndclass.hCursor		= LoadCursor(NULL, IDC_ARROW) ;
	wndclass.hbrBackground  = COLOR_WINDOW + 1 ;
	wndclass.lpszMenuName   = NULL ;
	wndclass.lpszClassName	= szAppName ;

	RegisterClass(&wndclass) ;
	}
				 // set global variables
	hInst = hInstance;
	szTestFile = (PSTR)LocalAlloc(LPTR,lstrlen(lpszCmdLine)+1);
	if(szTestFile) lstrcpy(szTestFile,lpszCmdLine);
				 // create main window, but don't show
	hWnd = CreateWindow(szAppName,NULL,WS_OVERLAPPED,
		0,CW_USEDEFAULT,0,CW_USEDEFAULT,
		HWND_DESKTOP,(HMENU)NULL,hInst,NULL);

  while(GetMessage(&msg, NULL, 0, 0))
   {
	   TranslateMessage(&msg);
	   DispatchMessage(&msg);
   }
  return msg.wParam;
}

// returns TRUE if passed filename exists and version info
// has been returned in lpdwVersion

static BOOL GetFileVersion(LPCSTR lpFileName, LPDWORD lpdwVersion)
{
	DWORD dwhVersion, dwBufSize;
	BOOL bRet = FALSE;
							// get size of buffer needed to hold info	
	dwBufSize = GetFileVersionInfoSize(lpFileName,&dwhVersion);
	if(dwBufSize != NULL)	// if info available
	{
		HGLOBAL hg;			// alloc and lock buffer
		LPSTR lp;
		if(hg = GlobalAlloc(GHND,dwBufSize))
		{
			if(lp = (LPSTR)GlobalLock(hg))
			{				// fill buffer with version info
				if(GetFileVersionInfo(lpFileName,dwhVersion,dwBufSize,lp))
				{			// get ptr to structure
					VS_FIXEDFILEINFO FAR* lpVer;
					WORD wSize;
					if(VerQueryValue(lp,"\\",&lpVer,&wSize))
					{		// if successful, copy data and set flag TRUE
						if(wSize != 0)
						{
							*lpdwVersion = lpVer->dwProductVersionMS;
							bRet = TRUE; 
						}
					}
				}			// clean up
				GlobalUnlock(hg);
			}
			GlobalFree(hg);
		}
	}
	return bRet;
}

// returns TRUE if VFW >= version 1.1 (MAJOR_VER.MINOR_VER)

static BOOL IsCorrectVersionVFW(void)
{
	char str[MAX_BUF];
	int nlen;
	DWORD dwVersion;
	BOOL bRet = FALSE;
							// get pathname for video library
	nlen = GetSystemDirectory(str,sizeof(str));
	if(nlen == 0 || nlen > sizeof(str)) return FALSE;
	if(*(str + nlen) != '\\') lstrcat(str,"\\");
	lstrcat(str,"MSVIDEO.DLL");
							// get version, return TRUE if >= vfw 1.1	
	if(GetFileVersion(str,&dwVersion))
		bRet = (HIWORD(dwVersion) > MAJOR_VER) ||
				((HIWORD(dwVersion) == MAJOR_VER) &&
					(LOWORD(dwVersion) >= MINOR_VER));

#ifdef _DEBUG				// show video library pathname and version
	MessageBox((HWND)NULL,str,"DEBUG",MB_OK);
	wsprintf(str,"VFW version %u.%u",HIWORD(dwVersion),LOWORD(dwVersion));
	MessageBox((HWND)NULL,str,"DEBUG",MB_OK);
#endif

	return bRet;
}

// if test file no longer exists returns TRUE 
// wait in loop TIME_OUT ms for condition to be satisfied
	
static BOOL WaitUntilClose(HWND hWnd)
{
	OFSTRUCT ofs;
	DWORD dwTimeOut = TIME_OUT;	// timeout setting
								// test if previous setup has closed
	dwTimeOut += GetTickCount();		// terminate after times up
	while(dwTimeOut > GetTickCount())
	{
		MSG msg;
										// give time to other setup
		PeekMessage(&msg,hWnd,0,0,PM_REMOVE);
		if(IsWindow(msg.hwnd))			// got invalid hwnd msgs ??
		{								// so put in wnd test here
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
										// if test file doesn't exist
		if(!szTestFile || OpenFile(szTestFile,&ofs,OF_EXIST) == HFILE_ERROR)
			return TRUE;		

	}
	return FALSE;
}

// returns FALSE if test file hasn't closed, or user agreed to install
// VFW and the VFW setup program wasn't properly launched

static BOOL LaunchSetup(HWND hWnd)
{
	char szCmd[MAX_BUF];
	LPSTR lpstr;
	WORD wRet;
								// if original setup didn't close return
	if(!WaitUntilClose(hWnd)) return FALSE;
	else						// ok to setup vfw, ask user if ok
	{
		if(MessageBox(hWnd,szAskMsg,
			szCaption,MB_YESNO|MB_ICONQUESTION) == IDNO)
		{						// user said no, tell how to do it later
			MessageBox(hWnd,szCloseMsg,
				szCaption,MB_OK|MB_ICONEXCLAMATION);
			return TRUE;		// terminate aok
		}						// user said install
	 							// get win vid setup path
								// get cwd path
		GetModuleFileName(hInst,szCmd,sizeof(szCmd));
		lpstr = szCmd + lstrlen(szCmd);
		while(lpstr != szCmd && *lpstr != '\\' && *lpstr != ':') lpstr--;
		if(*lpstr != '\\') *(++lpstr) = '\\';
		*(++lpstr) = '\0';		// now tack on path to setup pgm
		lstrcat(szCmd,szSetupPath);
		
#ifdef _DEBUG					// show VFW setup pathname, don't launch
		MessageBox(hWnd,szCmd,"DEBUG",MB_OK);
		return TRUE;
#endif
								// launch and return result
		wRet = WinExec(szCmd,SW_SHOW);
		return (wRet >= 32);
	}
}

static void InstallVFW(HWND hWnd)
{
	if(!LaunchSetup(hWnd))				// launch vfw setup
		MessageBox(hWnd,szErrMsg,		// msg on error
			szCaption,MB_OK|MB_ICONSTOP);
}

// the main window process

long FAR PASCAL _export WndProc(HWND hWnd, WORD message, WORD wParam, LONG lParam)
{
	switch(message) {
	  	case WM_CREATE:		// after main created, start test
			PostMessage(hWnd,WM_COMMAND,ID_STARTTEST,0L);
	  	return 0L;

		case WM_COMMAND:	// if not correct version VFW launch VFW setup
			if(wParam == ID_STARTTEST) {
			    if(!IsCorrectVersionVFW()) InstallVFW(hWnd);
			    PostMessage(hWnd,WM_CLOSE,0,0L);	// close when done
				return 0L;
			}
		break;
	
		case WM_DESTROY:
			PostQuitMessage(0);
		return 0L;
    }
  	return DefWindowProc(hWnd,message,wParam,lParam);
}
