// mvwmdi.cpp : implementation file
//

#include "issstd.h"
#include "multivie.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////
// CMDIMultiViewChildWnd

IMPLEMENT_DYNCREATE(CMDIMultiViewChildWnd, CMDIChildWnd)

CMDIMultiViewChildWnd::CMDIMultiViewChildWnd()
{
    nViewId = AFX_IDW_PANE_FIRST ;
}

CMDIMultiViewChildWnd::~CMDIMultiViewChildWnd()
{
}

afx_msg void
CMDIMultiViewChildWnd::OnSysCommand(UINT nID,LPARAM lParam)
{
    if (nID != SC_CLOSE)
        CMDIChildWnd::OnSysCommand(nID, lParam) ;
    else {
        // get currently active view for this frame
        CWnd  *pView =
              GetDescendantWindow(AFX_IDW_PANE_FIRST) ;
        CView *pOView ;
        ASSERT(pView->IsKindOf(RUNTIME_CLASS(CView))) ;
        CDocument* pDoc = ((CView *)pView)->GetDocument() ;
        POSITION rView = pDoc->GetFirstViewPosition() ;
        // is there at least one other visible
        // view for this doc
        while (rView) {
            pOView = pDoc->GetNextView(rView) ;
            ASSERT_VALID(pOView) ;
            if (pView != NULL &&
                pView != pOView &&
                pView->IsWindowVisible()) { // yes
                ActivateFrame(SW_HIDE) ;
                pDoc->UpdateFrameCounts() ;
                return ;
            }
        }             

        // The following code is exactly what is in
        // CDocument::OnFileClose which is a protected
        // method.  CFrameClass is friend of CDocument
        // but CMDIMultiViewChildWnd is not
        if (!pDoc->SaveModified())
            return;
        // shut it down
        pDoc->OnCloseDocument();
    }

}

BOOL
CMDIMultiViewChildWnd::OnCreateClient(LPCREATESTRUCT,
                                 CCreateContext* pContext)
{
    // default create client will create a
    // view if asked for it
    if (pContext != NULL)
    {
        // try to create view object from RuntimeClass
        if (pContext->m_pNewViewClass != NULL)
        {
            // ensure that the view that is created is in
            // an empty frame or that all the views for
            // this frame belong to the same document
            if (nViewId > AFX_IDW_PANE_FIRST) {
                CView *pView =
                   (CView *)GetDlgItem(AFX_IDW_PANE_FIRST);
                ASSERT(pView->IsKindOf(
                                    RUNTIME_CLASS(CView)));
                if (pView->GetDocument() !=
                            pContext->m_pCurrentDoc) {
                    TRACE0("Warning: Attempt to create"
                           " a view in a frame that was"
                          " in use by another document\n");
                    return FALSE ;
                }
            }            

            CWnd* pView = (CWnd*)pContext->
                           m_pNewViewClass->CreateObject();
                // NOTE: can be a CWnd with
                // PostNcDestroy self cleanup

            if (pView == NULL)
            {
                TRACE1("Warning: Dynamic create of view "
                       "type %Fs failed\n",
                       pContext->
                         m_pNewViewClass->m_lpszClassName);
                return FALSE;
            }
            ASSERT(pView->IsKindOf(RUNTIME_CLASS(CWnd)));
            // Views are always created with a border !

            if (!pView->Create(NULL, NULL,
                               AFX_WS_DEFAULT_VIEW |
                                    WS_CLIPSIBLINGS,
                               CRect(0,0,0,0),
                               this,
                               nViewId,
                               pContext))
            {
                TRACE0("Warning: couldn't create view "
                       "for frame\n");
                return FALSE; // can't go without a view
            }

            // WS_CLIPSIBLINGS must be on - 
            // some Create functions do not
            // use the requested style (i.e. CFormView)
            DWORD dwStyle = pView->GetStyle();
            if (!(dwStyle & WS_CLIPSIBLINGS))
            {
                TRACE1("Warning: Setting WS_CLIPSIBLINGS "
                       "for view %Fs\n",
                       pContext->
                       m_pNewViewClass->m_lpszClassName);
                // toggle WS_CLIPSIBLINGS
                ::SetWindowLong(pView->GetSafeHwnd(),
                                GWL_STYLE,
                                dwStyle ^ WS_CLIPSIBLINGS);
            }

            nViewId++ ;

            return (BOOL)pView ;

        }
    }

    return TRUE;

}


void CMDIMultiViewChildWnd::SetActiveView(CView *pViewNew,
                                          BOOL bNotify)
{
    CMDIFrameWnd *pMDIFrame ;
    CFrameWnd    *pFrameOld ;
    CWnd         *pViewOld ;
    
    // view deactivation only
    if (pViewNew == NULL) {
        pViewOld = (CWnd *)GetActiveView() ;
        pViewOld->ShowWindow(SW_HIDE);
        CMDIChildWnd::SetActiveView(pViewNew, bNotify) ;
        return ;
    }

    pMDIFrame = ((CMDIMultiViewChildWnd *)
                    (pViewNew->GetParentFrame()))
                        ->GetMDIFrame() ;
    ASSERT(pMDIFrame != NULL) ;
    ASSERT(pMDIFrame->
                IsKindOf(RUNTIME_CLASS(CMDIFrameWnd)));

    pFrameOld = pMDIFrame->GetActiveFrame() ;
    ASSERT(pFrameOld != NULL);
    ASSERT(pFrameOld->
                IsKindOf(RUNTIME_CLASS(CMDIChildWnd))) ;

    int nViewNewId = pViewNew->GetDlgCtrlID() ;
    pViewOld = GetDescendantWindow(AFX_IDW_PANE_FIRST) ;
    
    if (nViewNewId != AFX_IDW_PANE_FIRST) {
        ::SetWindowWord(pViewOld->m_hWnd,
                        GWW_ID, (WORD)nViewNewId) ;
        ::SetWindowWord(pViewNew->m_hWnd,
                        GWW_ID, AFX_IDW_PANE_FIRST) ;
    
        pViewOld->ShowWindow(SW_HIDE);
        CMDIChildWnd::SetActiveView(pViewNew, bNotify) ;
        RecalcLayout() ;
        pViewNew->ShowWindow(SW_SHOW) ;
        pViewNew->BringWindowToTop() ;
    }

    ActivateFrame(SW_SHOW) ;
    pViewNew->GetDocument()->UpdateFrameCounts() ;

}

BEGIN_MESSAGE_MAP(CMDIMultiViewChildWnd, CMDIChildWnd)
    ON_WM_SYSCOMMAND()
    //{{AFX_MSG_MAP(CMDIMultiViewChildWnd)
        // NOTE - the ClassWizard will add and remove mapping macros here.
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()


///////////////////////////////////////////////////////////
// CMDIMultiViewChildWnd message handlers

