
/* ----------------------------------------------------
 *  Filename:           ctorfail.cpp
 *  Summary:            Demo: assert valid object
 *  Author:             T.W. Nelson
 *  Compiler:           BCC++ 4.0
 *  Compile options:
 *  Start date:         23-Dec-1995
 *  Last update:        23-Jan-1996
 *  Version:            1.00
 *
 * ------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <new.h>

class BufferedFile
{
  private:
    FILE *_fp;
    char *_buf;
    int _isvalid() const  //assert class invariant
        {   return ((_fp != 0) && (_buf != 0));
        }   //== 0 if object not valid

  public:
    BufferedFile() : _fp(0), _buf(0) {}  //default
    BufferedFile( const char *name,
                  const char * mode,
                  size_t bufsz = 1024 )
    {   new_handler oldh = set_new_handler(0);
        _buf = new char [bufsz]; //ignore bufsz == 0
        _fp = fopen( name, mode );
        if( _isvalid() )
            setvbuf( _fp, _buf, _IOFBF, bufsz );
        set_new_handler(oldh);
    }
    ~BufferedFile()
        {   fclose(_fp);
            if( _buf )
                delete _buf;
        }
    int operator ! ()  //returns !0 on failure
        {   return _isvalid() ? 0 : 1;
        }
    operator int () //demonstrates (int) typecast and
        {   return 5; //possible operator ambiguity
        }
    operator void * () //returns 0 on failure
        {   return (void *) (_isvalid() ? this : 0);
        }
};

int main( int argc, char **argv )
{
        if (argc != 3)
                exit(printf("Usage: %s infile outfile\n", argv[0]));
                
    BufferedFile input( argv[1], "r", 2048 );
    BufferedFile *output =
            new BufferedFile( argv[2], "w+", 2048 );

    if( !input )   //calls operator !()
    {   printf("Can't open %s\n", argv[1] );
        // 'input' deleted automatically.....
        return -1;
    }
    else if( !*output ) //calls operator !()
    {   printf("Can't create %s\n", argv[2] );
        delete output;  //explicit delete
        return -1;
    }

    printf( "Both files opened successfully\n" );

    //........ other program content .......

    //This statement forces use of operator int() for
    //type conversion.  You must explicity cast 'input'
    //to an int value to resolve ambiguity with
    //operator void *() ........

    int b = 6 + int(input);     //b == 11
    delete output;

    return 0;
}

/* ----- EOF --------------------------------------- */
