unit Main;

interface

uses
  WinProcs, Classes, Controls, Forms, Menus,
  Buttons, ExtCtrls, StdCtrls, SysUtils;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
      mmFile: TMenuItem;
        smFileExit: TMenuItem;
      mmTools: TMenuItem;
        smToolsNone: TMenuItem;
        smToolsLine: TMenuItem;
        smToolsRect: TMenuItem;
        smToolsCircle: TMenuItem;
        smToolsFilledRect: TMenuItem;
        smToolsFilledCircle: TMenuItem;
    ToolBarPanel: TPanel;
      SBtnNone: TSpeedButton;
      SBtnLine: TSpeedButton;
      SBtnRect: TSpeedButton;
      SBtnCircle: TSpeedButton;
      SBtnFilledRect: TSpeedButton;
      SBtnFilledCircle: TSpeedButton;
    MainImage: TImage;
    PtrLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure smFileExitClick(Sender: TObject);
    procedure ChooseTool(Sender: TObject);
    procedure StopDrawing;
    procedure DrawLine;
    procedure DrawRect(Filled : boolean);
    procedure DrawCircle(Filled : boolean);
  end;

var
  MainForm: TMainForm;

implementation

const
  tsNone = 249; {menu item and speed button tags}
  tsLine = 250;
  tsRect = 251;
  tsCircle = 252;
  tsFilledRect = 253;
  tsFilledCircle = 254;
  ToolSelected : integer = tsNone; {default}
  crDrawLine = 1;  {cursors}
  crRect = 2;
  crFRect =  3;
  crCircle = 4;
  crFCircle = 5;

{$R *.DFM}
{$R TAGS2.RES}  {cursors in this res file}

procedure TMainForm.FormCreate(Sender: TObject);
const
  CursorNames : array[crDrawLine..crFCircle]  of pChar =
    ('crLine', 'crRect', 'crFRect', 'crCircle', 'crFCircle');
var
  x : integer;
begin
  for x := crDrawLine to crFCircle do
    Screen.Cursors[x]  :=  LoadCursor(hInstance, CursorNames[x]);
end;


procedure TMainForm.smFileExitClick(Sender: TObject);
begin
  Close;
end;


procedure TMainForm.ChooseTool(Sender: TObject);
var
  x : integer;
  OldTool : integer;
begin
  OldTool := ToolSelected;
  ToolSelected := (Sender as tComponent).Tag;
  if OldTool <> ToolSelected then
  begin
    for x := 0 to ComponentCount -1 do
      if Components[x] is tMenuItem then
      begin  {as long as all component tags < 255 can use IN}
        if (tMenuItem(Components[x]).Tag in [OldTool, ToolSelected])then
          tMenuItem(Components[x]).Checked :=
            not tMenuItem(Components[x]).Checked;
      end
      else
      if Components[x] is tSpeedButton then
      begin {old button will pop up automatically}
        if (tSpeedButton(Components[x]).Tag = ToolSelected) then
          tSpeedButton(Components[x]).Down := true;
      end;

    case ToolSelected of
      tsNone : StopDrawing;
      tsLine : DrawLine;
      tsRect : DrawRect(false);
      tsFilledRect : DrawRect(true);
      tsCircle : DrawCircle(false);
      tsFilledCircle : DrawCircle(true);
    end;
    PtrLabel.Tag :=  longint(@ToolSelected);
    PtrLabel.Caption := IntToStr(integer(pointer(PtrLabel.Tag)^));
  end;
end;


procedure TMainForm.StopDrawing;
begin
  MainImage.Cursor := crDefault;
  {do whatever is necessary to stop drawing}
end;

procedure TMainForm.DrawLine;
begin
  MainImage.Cursor := crDrawLine;
  {draw a line}
end;


procedure TMainForm.DrawRect(Filled : boolean);
begin
  if Filled then
  begin
    MainImage.Cursor := crFRect;
    {draw filled rectangle}
  end
  else
  begin
    MainImage.Cursor := crRect;
    {draw empty rectangle}
  end;
end;


procedure TMainForm.DrawCircle(Filled : boolean);
begin
  if Filled then
  begin
    MainImage.Cursor := crFCircle;
    {draw filled circle}
  end
  else
  begin
    MainImage.Cursor := crCircle;
    {draw empty circle}
  end;
end;

end.
