// serv.c

#include <windows.h>
#include <malloc.h>
#include <tchar.h>
#include "wdj_iocp.h"

#define MAX_CLIENTS  20
#define TEMP_FILE    TEXT("~wdjplt.tmp")

typedef struct MYCONTEXT_s {
   OVERLAPPED Ov;
   TCHAR      ReadBuffer[MAX_PATH];
   TCHAR      WriteBuffer[MAX_PATH];
   HANDLE     hPipe;
} MYCONTEXT, *PMYCONTEXT;

DWORD WorkerThread(HANDLE hPort);

void main(void)
{
   SYSTEM_INFO SysInfo;
   HANDLE      hPipe, hPort, hTemp, hThread;
   DWORD       i, Size, dwMaxThreads, ThreadID;
   PMYCONTEXT  pContext;

   // calculate number of worker threads to create
   GetSystemInfo(&SysInfo);
   dwMaxThreads = SysInfo.dwNumberOfProcessors * 2;

   // create temp file just to create the io comp port
   hTemp = CreateFile(TEMP_FILE, GENERIC_READ | GENERIC_WRITE,
         0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL |
         FILE_FLAG_OVERLAPPED, NULL);

   if ((hPort = CreateIoCompletionPort(hTemp, NULL, 0, 0)) == NULL) return;

   // create the worker threads
   for (i = 0; i < dwMaxThreads; i++) {
       if ((hThread = CreateThread(NULL, 0, WorkerThread,
             (LPVOID)hPort, 0, &ThreadID)) != NULL) {
           CloseHandle(hThread);
       }
   }

   while(TRUE) {
      // create an instance of the pipe for the next client
      hPipe = CreateNamedPipe(PIPE_NAME,
            PIPE_ACCESS_DUPLEX | FILE_FLAG_OVERLAPPED,
            PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE |
            PIPE_WAIT, MAX_CLIENTS, WRITE_BUFFER,
            READ_BUFFER, 5000, NULL);

      // create context structure for next client
      pContext = malloc(sizeof(MYCONTEXT));
      pContext->hPipe = hPipe;
      pContext->Ov.Offset = pContext->Ov.OffsetHigh = 0;
      pContext->Ov.hEvent = NULL;

      // associate the new pipe instance with the port
      hPort = CreateIoCompletionPort(hPipe, hPort,
            (DWORD)pContext, dwMaxThreads);

   AcceptNewClient:

      // wait for the next client to connect
      if (ConnectNamedPipe(hPipe, NULL) ||
          GetLastError() == ERROR_PIPE_CONNECTED) {

         // issue a read to the port to start listening
         if ((!ReadFile(hPipe, pContext->ReadBuffer,
               READ_BUFFER, &Size, &pContext->Ov))
               && (GetLastError() != ERROR_IO_PENDING)) {
           if (GetLastError() == ERROR_BROKEN_PIPE) {
               DisconnectNamedPipe(hPipe);
               goto AcceptNewClient;
           } else {
               DisconnectNamedPipe(hPipe);
               CloseHandle(hPipe);
               free(pContext);
           }
        }
      } else {
          CloseHandle(hPipe);
          CloseHandle(hTemp);
          DeleteFile(TEMP_FILE);
      }
   }
}

DWORD WorkerThread(HANDLE hPort)
{
   DWORD        Bytes;
   PMYCONTEXT   pContext;
   LPOVERLAPPED pOv;
   OVERLAPPED   Ov;

   Ov.Offset = Ov.OffsetHigh = 0;
   Ov.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
   Ov.hEvent = (HANDLE)((DWORD)Ov.hEvent | 0x00000001);

   while (TRUE) {
      if (!GetQueuedCompletionStatus(hPort, &Bytes,
            (LPDWORD)&pContext, &pOv, (DWORD)-1)) {
         DisconnectNamedPipe(pContext->hPipe);
         // reuse the pipe and context structure to
         // wait for a new client (but not here) ...
         continue;
      }

      _tprintf(TEXT("Server: %s, %d\n"),
            pContext->ReadBuffer, Bytes);
      CharUpper(pContext->ReadBuffer);

      if((!WriteFile(pContext->hPipe, pContext->ReadBuffer,
            Bytes, &Bytes, &Ov))
            && (GetLastError() != ERROR_IO_PENDING)) {
         DisconnectNamedPipe(pContext->hPipe);
         // reuse the pipe and context structure to
         // wait for a new client (but not here) ...
         continue;
      }

      if ((!ReadFile(pContext->hPipe, pContext->ReadBuffer,
            READ_BUFFER, &Bytes, &pContext->Ov))
            && (GetLastError() != ERROR_IO_PENDING)) {
         DisconnectNamedPipe(pContext->hPipe);
         // reuse the pipe and context structure to
         // wait for a new client (but not here) ...
        continue;
      }
   }
   return TRUE;
}

