//
// BUG0596.CPP
//
// This bug demonstrates a very bad problem
// in VC++ 2.2.  When explicity calling
// a virtual destructor, VC++ forgets to push
// an integer flag parameter on the stack.  The
// compiler generated scalar destructor routine
// expects it.  Two things happen because of
// this.  First, the scalar destructor routine might
// call ::delete for memory not on the heap, depending
// on the random memory contents on the stack.  Second,
// when the scalar destructor routine exits, it will
// have adjusted the stack by an extra four bytes, since
// it expects to have a flag parameter.  Thus, ESP is no
// longer valid upon return, which is catastrophic!
//
// Built using VC++ 2.2: cl bug0596.cpp
//
//    - or -             cl /Ox bug0596.cpp
//

#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>

//
// Class foo has a virtual destructor
//
class CFoo
{
    public:
        CFoo(){ cout << "ctor @"
                     << (void * ) this
                     << "\n"; }
        virtual ~CFoo(){ cout << "dtor @"
                              << (void *) this
                              << "\n"; }
};

//
// This operator new is necessary to support
// the placement syntax.  When you call the
// new operator using placement syntax, it
// will call this routine to get the address
// it should use to construct the new object.
//
void *operator new( unsigned int, char *p )
{
    return p;
}

//
// Later in the example I create a new object
// in this memory buffer using the new
// placement syntax.
//

char buffer[ sizeof( CFoo ) ];

//
// I create my own version of
// the global delete operator so I can
// catch an attempted delete of the
// above buffer.
//
void operator delete( void *p )
{
    cout << "Deleting " << p;
    if ( p == (void*) buffer )
        cout << " - Error!\n";
    else
        cout << " - Ok!\n";
}

void foo()
{
    int entry_sp;
    _asm mov entry_sp,esp
    cout << "Stack pointer entering foo() = "
         << hex << (void*) entry_sp << "\n";
//
// The standard new/delete combination works properly.
// If you compile with /Fc, you will see that the
// destructor call pushes a '1' on the stack before
// calling CFoo::`scalar deleting destructor'
//
    CFoo *pf = new CFoo;
    delete pf;
//
// When I create an object using the placement syntax,
// the ctor correctly constructs an object in the buffer
// I pass to new.  However, when calling the destructor,
// the compiler forgets to generate a 'push 0' instruction,
// which is expected by CFoo::`scalar deleting destructor'.
// Since the destructor uses a pascal-like calling
// convention, it will clean up the stack even though no
// arguments were passed on it, leading to an invalid
// stack pointer upon return.
//
    pf = new ( buffer ) CFoo;
    pf->~CFoo();
    int exit_sp;
    _asm mov exit_sp, esp
    cout << "Stack pointer exiting foo()  = "
         << hex << (void*) exit_sp << "\n";
    if ( entry_sp != exit_sp ) {
        cout << "Aborting!\n";
        cout.flush();
        abort();
    }
}

//
// The main routine has one tricky bit.
// When CFoo::`scalar deleting destructor' is
// called with no argument, it pulls a random
// value off the stack.  As it happens, that random
// value will be the saved value of edi or esi
// as passed to foo().  I can force that 'random'
// value to be a 1 by setting both registers to
// that value in main().
//
main()
{
    cout << "Calling foo()\n";
    _asm mov esi, 1
    _asm mov edi, 1
    foo();
    cout << "Returning from foo()\n";
    return 1;
}
//End of file
