/*
**  PALSPY.C
**	Displays the current system palette along with the current logical
**	palette mapping.
**
**	Chris R. Branch
*/

#include "palspy.h"

HPALETTE	ghPalette;
HINSTANCE	ghInstance;

static char	appWndClass[] = "PalSpy_ColorWin";
static char	appWndTitle1[] = "Palette Spy";
static char	appWndTitle2[] = "Palette Spy (Identity Mapping)";
static char	appWndTitle3[] = "Palette Spy (Default Palette)";

struct DibHeaderTag {
    BITMAPINFOHEADER    bih;
    WORD                color[MAX_PALETTE_ENTRIES];
} dibHeader;

/* undocumented GDI function (see "Undocumented Windows") */
BOOL FAR PASCAL IsDCCurrentPalette(HDC hdc);

LRESULT CALLBACK _export WndProc(HWND hwnd, UINT iMessage, WPARAM wParam, LPARAM lParam);
static void Cls_OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags);
void		UpdateScreen(HWND hwnd, HDC hdc);
COLORREF	MapTextColor(BYTE red, BYTE green, BYTE blue);
HPALETTE	MakeSysPalette(void);
WORD 		GetPaletteMapping(HPALETTE hPalette, BYTE mapTable[]);
HPALETTE	GetCurrentPalette(void);

#pragma argsused
int PASCAL _export WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
			   LPSTR lpszCmdLine, int nCmdShow)
{
	WNDCLASS	wndclass;
	MSG		msg;
	HWND		hwnd;
	int		windowWidth;
	int		windowHeight;

	/*
	**  Register window class and create window for the app.
	*/
	wndclass.style		= CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc	= (WNDPROC) WndProc;
	wndclass.cbClsExtra	= 0;
	wndclass.cbWndExtra	= 0;
	wndclass.hInstance	= hInstance;
	wndclass.hIcon		= LoadIcon(hInstance, MAKEINTRESOURCE(1));
	wndclass.hCursor	= LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground	= GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName	= NULL;
	wndclass.lpszClassName	= appWndClass;

	if (!RegisterClass(&wndclass))
		return(FALSE);

	windowWidth	= (INITIAL_BOX_WIDTH + BOX_SPACING) * NUM_COLUMNS;
	windowHeight	= (INITIAL_BOX_HEIGHT + BOX_SPACING) * NUM_ROWS;
	hwnd = CreateWindow(appWndClass,		/* class name	*/
			    appWndTitle1,		/* window name  */
			    WS_OVERLAPPEDWINDOW, 	/* window style */
			    0,				/* x position	*/
			    0,				/* y position	*/
			    windowWidth,		/* width	*/
			    windowHeight,		/* height	*/
			    NULL,			/* parent window*/
			    NULL,			/* menu handle	*/
			    hInstance,			/* instance	*/
			    NULL);			/* not used	*/
	if (hwnd == NULL)
		return(FALSE);

	ghInstance = hInstance;
	ghPalette = MakeSysPalette();

	ShowWindow(hwnd, SW_SHOWNORMAL);
	UpdateWindow(hwnd);

	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	if (ghPalette != NULL) {
		DeleteObject(ghPalette);
		ghPalette = NULL;
	}

	return(0);
}


/*
**  WndProc()
**	Main window procedure for color window.
*/
LONG CALLBACK _export WndProc(HWND hwnd, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	HPALETTE	hOldPalette;
	HMENU		hMenu;
	PAINTSTRUCT	ps;
	HDC		hdc;
	BOOL		repaint;
	BOOL		alwaysOnTop;
	HWND		hInsertAfter;
	UINT		itemState;

	switch(iMessage) {
		case WM_CREATE:
			hMenu = GetSystemMenu(hwnd, FALSE);
			if (hMenu != NULL) {
				AppendMenu(hMenu, MF_SEPARATOR, 0, 0);
				AppendMenu(hMenu, MF_STRING, IDM_ALWAYSONTOP,
					"Always on &top");
			}
			break;
		case WM_ACTIVATEAPP:
			InvalidateRect(hwnd, NULL, TRUE);
			break;
		case WM_PALETTECHANGED:
			if ((HWND) wParam == hwnd)
				break;
		case WM_QUERYNEWPALETTE:		/* fall through */
			hdc = GetDC(hwnd);
			if (ghPalette != NULL)
				SelectPalette(hdc, ghPalette, TRUE);
			repaint = RealizePalette(hdc);
			ReleaseDC(hwnd, hdc);
			InvalidateRect(hwnd, NULL, FALSE);
			return(repaint);
		case WM_PAINT:
			hdc = BeginPaint(hwnd, &ps);
			if (ghPalette != NULL) {
				hOldPalette = SelectPalette(hdc, ghPalette, TRUE);
				RealizePalette(hdc);
				UpdateScreen(hwnd, hdc);
				SelectPalette(hdc, hOldPalette, TRUE);
			}
			EndPaint(hwnd, &ps);
			return(0);
		case WM_MOUSEACTIVATE:
			if (HIWORD(lParam) == WM_RBUTTONDOWN) {
				InvalidateRect(hwnd, NULL, TRUE);
				return(MA_NOACTIVATEANDEAT);
			}
			break;
		case WM_LBUTTONDOWN:
			(void)HANDLE_WM_LBUTTONDOWN(hwnd, wParam, lParam, Cls_OnLButtonDown);
			return(0L);
		case WM_LBUTTONDBLCLK:
			(void)HANDLE_WM_LBUTTONDBLCLK(hwnd, wParam, lParam, Cls_OnLButtonDown);
			return(0L);
		case WM_SYSCOMMAND:
			hMenu = GetSystemMenu(hwnd, FALSE);
			if ((wParam & 0xFFF0) == IDM_ALWAYSONTOP && hMenu != NULL) {
				itemState = GetMenuState(hMenu, IDM_ALWAYSONTOP, MF_BYCOMMAND);
				alwaysOnTop = (itemState & MF_CHECKED) ? FALSE : TRUE;
				hInsertAfter = (alwaysOnTop) ? HWND_TOPMOST : HWND_NOTOPMOST;
				CheckMenuItem(hMenu, IDM_ALWAYSONTOP,
					(alwaysOnTop) ? MF_CHECKED : MF_UNCHECKED);
				SetWindowPos(hwnd, hInsertAfter, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
				return(0L);
			}
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
	}

	return(DefWindowProc(hwnd, iMessage, wParam, lParam));
}


/*
**  Cls_OnLButtonDown()
**	Called for WM_LBUTTONDOWN and WM_LBUTTONDBLCLK.
*/
#pragma argsused
static void Cls_OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags)
{
	RECT	clientRect;
	short	entryHit;
	short	row, col;
	short	boxWidth;
	short	boxHeight;

	GetClientRect(hwnd, &clientRect);
	boxWidth = max(2, (clientRect.right / NUM_COLUMNS) - BOX_SPACING);
	boxHeight = max(2, (clientRect.bottom / NUM_ROWS) - BOX_SPACING);

	if (x < (boxWidth + BOX_SPACING) * NUM_COLUMNS &&
	    y < (boxHeight + BOX_SPACING) * NUM_ROWS) {
		col = x / (boxWidth + BOX_SPACING);
		row = y / (boxHeight + BOX_SPACING);

		entryHit = row * NUM_COLUMNS + col;
		if (fDoubleClick)
			ShowDetails(hwnd, entryHit);
	}
}


/*
**  UpdateScreen()
**	Display all colors in the system palette.
*/
void UpdateScreen(HWND hwnd, HDC hdc)
{
	BYTE		transTable[MAX_PALETTE_ENTRIES];
	WORD		numEntries;
	LOGFONT		lf;
	HPALETTE	hPalette;
	PALETTEENTRY	pe;
	HBRUSH		hBrush;
	HFONT		hFont;
	HFONT		hOldFont;
	RECT		clientRect;
	RECT		r;
	COLORREF	textColor;
	BOOL		isIdentity;
	char		string[15];
	char		*formatStr;
	char		*windowTitle;
	int		row;
	int		col;
	int		boxWidth;
	int		boxHeight;
	int		oldMode;
	int		numThatMap;
	int		firstMaps;
	int		x, y;
	int		i, j;

	/*
	**  Get mapping for the foreground palette unless it's PALSPY's
	**  special palette.
	*/
	hPalette = GetCurrentPalette();
	if (hPalette != ghPalette)
		numEntries = GetPaletteMapping(hPalette, transTable);
	else
		numEntries = 0;

	/*
	**  Check for identity mapping and change the window title
	**  appropriately.
	*/
	if (numEntries > 0) {
		isIdentity = TRUE;
		for (i=0; i < numEntries; ++i) {
			if (transTable[i] != i) {
				isIdentity = FALSE;
				break;
			}
		}
	} else
		isIdentity = FALSE;

	if (hPalette == GetStockObject(DEFAULT_PALETTE))
		windowTitle = appWndTitle3;
	else if (isIdentity)
		windowTitle = appWndTitle2;
	else
		windowTitle = appWndTitle1;
	SetWindowText(hwnd, windowTitle);

	GetClientRect(hwnd, &clientRect);
	boxWidth = max(2, (clientRect.right / NUM_COLUMNS) - BOX_SPACING);
	boxHeight = max(2, (clientRect.bottom / NUM_ROWS) - BOX_SPACING);

	memset(&lf, 0, sizeof(LOGFONT));
	if (boxWidth < boxHeight)
		lf.lfWidth = boxWidth / 3;
	else
		lf.lfHeight = boxHeight;
	lf.lfCharSet = ANSI_CHARSET;
	lf.lfWeight = FW_NORMAL;
	lstrcpy(lf.lfFaceName, "Arial");
	hFont = CreateFontIndirect(&lf);
	if (hFont != NULL)
		hOldFont = SelectObject(hdc, hFont);
	else
		hOldFont = NULL;

	oldMode = SetBkMode(hdc, TRANSPARENT);
	for (i=0; i < MAX_PALETTE_ENTRIES; ++i) {
		hBrush = CreateSolidBrush(PALETTEINDEX(i));
		GetSystemPaletteEntries(hdc, i, 1, &pe);
		textColor = MapTextColor(pe.peRed, pe.peGreen, pe.peBlue);
		SetTextColor(hdc, textColor);

		row = i / NUM_COLUMNS;
		col = i % NUM_COLUMNS;

		x = col * (boxWidth + BOX_SPACING);
		y = row * (boxHeight + BOX_SPACING);
		SetRect(&r, x, y, x + boxWidth, y + boxHeight);
		FillRect(hdc, &r, hBrush);

		numThatMap = 0;
		firstMaps = -1;
		for (j=0; j < numEntries; ++j) {
			if (transTable[j] == i) {
				++numThatMap;
				if (firstMaps < 0)
					firstMaps = j;
			}
		}
		if (numThatMap > 0) {
			formatStr = (numThatMap == 1) ? "%d" : "(%d)";
			wsprintf(string, formatStr, firstMaps);
			DrawText(hdc, string, lstrlen(string), &r, DT_CENTER);
		}

		DeleteObject(hBrush);
	}
	SetBkMode(hdc, oldMode);

	if (hOldFont != NULL) {
		SelectObject(hdc, hOldFont);
		DeleteObject(hFont);
	}
}


/*
**  MapTextColor()
**	Takes an RGB value to be used as a background color.  Chooses
**	between black and white for the most readable text color on this
**	background color.  This determination is based on the color
**	luminance.  See Microsoft Knowledge Base article Q29240 for more
**	information on translating RGB <--> HLS.
*/
COLORREF MapTextColor(BYTE red, BYTE green, BYTE blue)
{
	int		lum;
	int		maxRGB;
	int		minRGB;
	COLORREF	textColor;

	maxRGB = max(red, max(green, blue));
	minRGB = min(red, min(green, blue));

	/*
	**  L = (((maxRGB + minRGB) * HLSMAX) + RGBMAX) / (2 * RGBMAX)
	*/
	lum = (int) (((maxRGB + minRGB) * 240L + 255) / 510);
	if (lum < 120)
		textColor = PALETTEINDEX(255);
	else
		textColor = PALETTEINDEX(0);

	return(textColor);
}


/*
**  MakeSysPalette()
**	Creates a logical palette where all entries are marked PC_EXPLICIT.
**	PC_EXPLICIT tells the palette manager we want the system palette
**	color at that index.  This logical palette is used to monitor
**	the current state of the system palette.
*/
HPALETTE MakeSysPalette(void)
{
	HPALETTE	hPalette;
	LOGPALETTE	*pLogPal;
	int		colorTableSize;
	int		i;

	hPalette = NULL;
	colorTableSize = sizeof(PALETTEENTRY) * (MAX_PALETTE_ENTRIES - 1);
	pLogPal = (LOGPALETTE *) LocalAlloc(LPTR, sizeof(LOGPALETTE) + colorTableSize);
	if (pLogPal != NULL) {
		pLogPal->palVersion = 0x300;
		pLogPal->palNumEntries = MAX_PALETTE_ENTRIES;
		for (i=0; i < MAX_PALETTE_ENTRIES; ++i) {
			pLogPal->palPalEntry[i].peFlags = PC_EXPLICIT;
			pLogPal->palPalEntry[i].peRed = i;
		}

		hPalette = CreatePalette(pLogPal);
		LocalFree(pLogPal);
	}

	return(hPalette);
}


/*
**  GetPaletteMapping()
**	Retrieves the current logical to system mapping and stores it for
**	later use.  The caller must supply adequate space (256 entries)
**	for the mapping table.  Returns the number of entries actually
**	copied to the table, or 0 if the mapping is not available.
*/
WORD GetPaletteMapping(HPALETTE hPalette, BYTE mapTable[])
{
	LPBITMAPINFO	lpbi;
	BYTE		theBits[MAX_PALETTE_ENTRIES];
	WORD		numEntries;
	WORD		tableSize;
	HBITMAP		hBitmap;
	HDC		hdc;
	HPALETTE	hOldPalette;
	WORD		i;

	/* set up DIB header, color table, and bits */
	memset(&dibHeader, 0, sizeof(struct DibHeaderTag));
	dibHeader.bih.biSize	 = sizeof(BITMAPINFOHEADER);
	dibHeader.bih.biWidth	 = MAX_PALETTE_ENTRIES;
	dibHeader.bih.biHeight	 = 1;
	dibHeader.bih.biPlanes	 = 1;
	dibHeader.bih.biBitCount = 8;

	for (i=0; i < MAX_PALETTE_ENTRIES; ++i) {
		dibHeader.color[i] = i;
		theBits[i] = i;
	}

	/*
	**  Create a device-dependent bitmap based on the DIB we just
	**  set up.  Each pixel is automatically translated to it's
	**  foreground mapping by CreateDIBitmap().  We get the mapping
	**  by simply retrieving the bits using GetBitmapBits() and then
	**  clean-up.
	*/
	numEntries = 0;
	if (hPalette != NULL) {
		lpbi = (LPBITMAPINFO) &dibHeader;
		hdc = GetDC(NULL);
		hOldPalette = SelectPalette(hdc, hPalette, TRUE);
		hBitmap = CreateDIBitmap(hdc, (LPBITMAPINFOHEADER) lpbi,
				CBM_INIT, theBits, lpbi, DIB_PAL_COLORS);
		if (hBitmap != NULL) {
			tableSize = sizeof(BYTE) * MAX_PALETTE_ENTRIES;
			GetBitmapBits(hBitmap, tableSize, mapTable);
			DeleteObject(hBitmap);
			GetObject(hPalette, sizeof(WORD), &numEntries);
		}
		if (hOldPalette)
			SelectPalette(hdc, hOldPalette, TRUE);
		ReleaseDC(NULL, hdc);
	}

	return(numEntries);
}


/*
**  GetCurrentPalette()
**	Walks the GDI local heap searching for all palettes.  Each palette
**	is selected into a DC, then checked with the undocumented
**	ISDCCURRENTPALETTE function to see if it's the current foreground
**	palette.
*/
HPALETTE GetCurrentPalette(void)
{
	BOOL		isCurrent;
	HDC		hdc;
	HPALETTE	hPalette;
	HPALETTE	hOldPalette;
	LOCALENTRY	le;
	SYSHEAPINFO	sysHeap;
	BOOL		validObject;

	isCurrent = FALSE;
	hdc = GetDC(NULL);
	if (hdc != NULL) {
		sysHeap.dwSize = sizeof(SYSHEAPINFO);
		SystemHeapInfo(&sysHeap);

		le.dwSize = sizeof(LOCALENTRY);
		validObject = LocalFirst(&le, sysHeap.hGDISegment);
		while (!isCurrent && validObject) {
			if (le.wType == LT_GDI_PALETTE) {
				hPalette = le.hHandle;
				hOldPalette = SelectPalette(hdc, hPalette, TRUE);
				isCurrent = IsDCCurrentPalette(hdc);
				if (hOldPalette != NULL)
					SelectPalette(hdc, hPalette, TRUE);
			}
			validObject = LocalNext(&le);
		}
		ReleaseDC(NULL, hdc);
	}

	return((isCurrent) ? hPalette : NULL);
}

