// MTrackerView.cpp : implementation of the CMTrackerView class
//

#include "stdafx.h"
#include "MTracker.h"

#include "MTrckDoc.h"
#include "MTrackVw.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMTrackerView

IMPLEMENT_DYNCREATE(CMTrackerView, CView)

BEGIN_MESSAGE_MAP(CMTrackerView, CView)
	//{{AFX_MSG_MAP(CMTrackerView)
	ON_WM_LBUTTONDOWN()
	ON_WM_SETCURSOR()
	ON_COMMAND(ID_OBJECTS_ADD, OnObjectsAdd)
	ON_COMMAND(ID_OBJECTS_DELETE, OnObjectsDelete)
	ON_UPDATE_COMMAND_UI(ID_OBJECTS_DELETE, OnUpdateObjectsDelete)
	ON_WM_RBUTTONDOWN()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMTrackerView construction/destruction

CMTrackerView::CMTrackerView()
{
   m_hAddCursor = AfxGetApp()->LoadCursor(IDC_ADDOBJECT);
   m_bAdding = FALSE;
}

CMTrackerView::~CMTrackerView()
{
   for(int i = 0; i < m_mrt.GetMaxID(); i++)
   {
      CRectTracker *pTracker = m_mrt.GetTrackerFromID(i);
      if(pTracker != NULL)
         delete pTracker;
   }
}

BOOL CMTrackerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMTrackerView drawing

void CMTrackerView::OnDraw(CDC* pDC)
{
	CMTrackerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

   for(int i = 0; i < m_mrt.GetMaxID(); i++)
   {
      CRectTracker *pTracker = m_mrt.GetTrackerFromID(i);
      if(pTracker != NULL)
      {
         pDC->Rectangle(pTracker->m_rect);
      }
   }

   m_mrt.Draw(pDC);
}

/////////////////////////////////////////////////////////////////////////////
// CMTrackerView diagnostics

#ifdef _DEBUG
void CMTrackerView::AssertValid() const
{
	CView::AssertValid();
}

void CMTrackerView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMTrackerDoc* CMTrackerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMTrackerDoc)));
	return (CMTrackerDoc*)m_pDocument;
}
#endif //_DEBUG

void CDerivedMRT::OnChangingRects(int &dx, int &dy)
{
   CRect rectUnion;
   
   // perform very primitive "snap-to-grid"
   dx = dx - (dx % 20);
   dy = dy - (dy % 20);


   // keep trackers within bounds of window
   rectUnion.SetRectEmpty();
   for(int i = 0; i < GetMaxID(); i++)
   {
      if(IsSelected(i))
      {
         CRectTracker *pTracker = GetTrackerFromID(i);
         if(rectUnion.IsRectNull())
            rectUnion = pTracker->m_rect;
         else
            rectUnion.UnionRect(rectUnion, pTracker->m_rect);
      }
   }
   
   if(rectUnion.left + dx < 0)
      dx = -rectUnion.left;
   if(rectUnion.right + dx > m_rectMax.right)
      dx = m_rectMax.right - rectUnion.right;
   if(rectUnion.top + dy < 0)
      dy = -rectUnion.top;
   if(rectUnion.bottom + dy > m_rectMax.bottom)
      dy = m_rectMax.bottom - rectUnion.bottom;
}

/////////////////////////////////////////////////////////////////////////////
// CMTrackerView message handlers

void CMTrackerView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if(m_bAdding)
   {
      CRectTracker *pTracker = new CRectTracker;
      pTracker->m_rect = CRect(point.x, point.y, point.x + 100, point.y + 100);
      pTracker->m_nStyle = CRectTracker::dottedLine | CRectTracker::resizeOutside;
      m_mrt.UnselectAll();
      m_mrt.Select(m_mrt.AddTracker(pTracker), TRUE);
      Invalidate();
      m_bAdding = FALSE;
   }
   else
   {
      m_mrt.HandleLButtonDown(this, nFlags, point);
   }
	CView::OnLButtonDown(nFlags, point);
}

BOOL CMTrackerView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
   if(m_bAdding)
   {
      SetCursor(m_hAddCursor);
      return TRUE;
   }

	if(m_mrt.HandleSetCursor(pWnd, nHitTest))
      return TRUE;
	
	return CView::OnSetCursor(pWnd, nHitTest, message);
}

void CMTrackerView::OnObjectsAdd() 
{
	m_bAdding = TRUE;
	
}

void CMTrackerView::OnObjectsDelete() 
{
	for(int i = 0; i < m_mrt.GetMaxID(); i++)
   {
      CRectTracker *pTracker = m_mrt.GetTrackerFromID(i);
      if(m_mrt.IsSelected(i))
      {
         m_mrt.RemoveTracker(i);
         delete pTracker;
      }
   }
   Invalidate();	
}

void CMTrackerView::OnUpdateObjectsDelete(CCmdUI* pCmdUI) 
{
   pCmdUI->Enable(m_mrt.NumSelected() > 0);	
}

void CMTrackerView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	m_bAdding = FALSE;
	
	CView::OnRButtonDown(nFlags, point);
}

void CMTrackerView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	CRect rect(0, 0, cx, cy);
   m_mrt.m_rectMax = rect;
}
