#include "stdafx.h"
#include "multrack.h"

static HCURSOR rghCursor;
#define GetNthTag(i) ((CRectTrackerTag *)m_arrayTrackers[i])

CMultipleRectTracker::CMultipleRectTracker()
{
   m_bMultiTracking = FALSE;
   m_pWndClipTo = NULL;
   static bInitialized = FALSE;
   if(!bInitialized){  // load our only cursor
      HINSTANCE hInst = AfxFindResourceHandle(
         MAKEINTRESOURCE(AFX_IDC_TRACK4WAY), RT_GROUP_CURSOR);
      rghCursor = ::LoadCursor(hInst, 
                     MAKEINTRESOURCE(AFX_IDC_MOVE4WAY));
      bInitialized = TRUE;
   }
}

CMultipleRectTracker::~CMultipleRectTracker()
{
   for(int i = 0; i < GetMaxID(); i++)
      RemoveTracker(i);
}

int CMultipleRectTracker::AddTracker(CRectTracker *pTracker)
{
   CRectTrackerTag *pNewTag = new CRectTrackerTag;

   pNewTag->m_pTracker = pTracker;
   pNewTag->m_bSelected = FALSE;
   pNewTag->m_rectLast.SetRectEmpty();

   for(int i = 0; i < GetMaxID(); i++){
      if(m_arrayTrackers[i] == NULL){ // look for empty space first
         m_arrayTrackers[i] = pNewTag;
         return i;
      }
   }
   m_arrayTrackers.Add(pNewTag);
   return i;
}

void CMultipleRectTracker::RemoveTracker(int nID)
{
   ASSERT(nID >= 0 && nID < GetMaxID());
   if(GetNthTag(nID) != NULL){
      delete GetNthTag(nID);
      m_arrayTrackers[nID] = NULL;
   }
}

void CMultipleRectTracker::Draw(CDC *pDC)
{
   for(int i = 0; i < GetMaxID(); i++)
      if(IsSelected(i))
         GetTrackerFromID(i)->Draw(pDC);
}

CRectTracker *CMultipleRectTracker::GetTrackerFromID(int nID) const
{
   ASSERT(nID >= 0 && nID < GetMaxID());
   if(GetNthTag(nID) == NULL)
      return NULL;
   return GetNthTag(nID)->m_pTracker;
}

BOOL CMultipleRectTracker::IsSelected(int nID) const
{
   ASSERT(nID >= 0 && nID < GetMaxID());
   if(GetNthTag(nID) == NULL)
      return FALSE;
   return GetNthTag(nID)->m_bSelected;
}

int CMultipleRectTracker::NumSelected() const
{
   for(int i = 0, nCount = 0; i < GetMaxID(); i++)
      nCount += IsSelected(i) ? 1 : 0;
   return nCount;
}

int CMultipleRectTracker::GetCount() const
{
   for(int i = 0, nCount = 0; i < GetMaxID(); i++)
      nCount += (GetNthTag(i) != NULL) ? 1 : 0;
   return nCount;
}

int CMultipleRectTracker::GetTrackerIDAtPoint(CPoint point) const
{
   int nSave = -1;
   for(int i = 0; i < GetMaxID(); i++){
      if(GetNthTag(i) != NULL && GetNthTag(i)->m_pTracker != NULL
         && GetNthTag(i)->m_pTracker->HitTest(point) >= 0){
      if(IsSelected(i)) 
         return i; // tries to return a selected tracker
      nSave = i; // but returns an unselected one if necessary
      }
   }
   return nSave;
} 

void CMultipleRectTracker::Select(int nID, BOOL bSelected)
{
   ASSERT(nID >= 0 && nID < GetMaxID());
   if(GetNthTag(nID) == NULL)
      return;
   GetNthTag(nID)->m_bSelected = bSelected;
}

void CMultipleRectTracker::UnselectAll()
{
   for(int i = 0; i < GetMaxID(); i++)
      Select(i, FALSE);
}

void CMultipleRectTracker::HandleLButtonDown(CWnd *pWnd, 
                                       UINT nFlags, CPoint point)
{
   BOOL bAdding = (nFlags & MK_SHIFT || nFlags & MK_CONTROL);
   int nTrackerID = GetTrackerIDAtPoint(point);

   if(nTrackerID >= 0){ // found at least one tracker
      CRectTracker *pTracker = GetTrackerFromID(nTrackerID);
      if(IsSelected(nTrackerID)){ 
         if(bAdding) // unselect this tracker
            Select(nTrackerID, FALSE);
         else { // track
            if(NumSelected() > 1 || 
               pTracker->HitTest(point) == CRectTracker::hitMiddle){
               if(!MultiTrack(pWnd, point, m_pWndClipTo)){
                  UnselectAll(); // leave only this one selected
                  Select(nTrackerID, TRUE);
               }
            }
            else // let CRectTracker do the work
               pTracker->Track(pWnd, point, FALSE, m_pWndClipTo);
         }
         pWnd->Invalidate();
      }
      else {
         if(!bAdding)
            UnselectAll(); // cancel previous selection
         Select(nTrackerID, TRUE);
         pWnd->Invalidate();
         if(!bAdding) { // do click and immediate move
            pWnd->UpdateWindow();
            HandleSetCursor(pWnd, HTCLIENT);
            if(pTracker->HitTest(point) == CRectTracker::hitMiddle)
               MultiTrack(pWnd, point, m_pWndClipTo);
            else
               pTracker->Track(pWnd, point, FALSE); 
            pWnd->Invalidate();
         }
      }
   }
   else {	// rubber band
      CRectTracker tracker;
      if (tracker.TrackRubberBand(pWnd, point, TRUE))	{
         // see if rubber band intersects with any trackers
         CRect rectT;
         tracker.m_rect.NormalizeRect(); // for intersectrect

         if(!bAdding)
            UnselectAll(); // cancel previous selection
         for(int i = 0; i < GetMaxID(); i++) {
            CRectTracker *pTracker = GetTrackerFromID(i);
            if(pTracker == NULL)
               continue;
            if(rectT.IntersectRect(pTracker->m_rect, 
                                       tracker.m_rect))
               Select(i); // this tracker is in rubber band
	      }
         pWnd->Invalidate();
      }
      else { // rubber band was canceled
         if(!bAdding) {
            UnselectAll();
            pWnd->Invalidate();
         }
      }
   }
}

BOOL CMultipleRectTracker::HandleSetCursor(CWnd* pWnd, 
                                          UINT nHitTest) const
{
   if (nHitTest != HTCLIENT)
      return FALSE;

   CPoint point;
   GetCursorPos(&point);
   pWnd->ScreenToClient(&point);

   int nID = GetTrackerIDAtPoint(point);
   if(nID >= 0) {
      if(IsSelected(nID))
         if(NumSelected() == 1) // let CRectTracker handle it
            return GetTrackerFromID(nID)->SetCursor(pWnd, nHitTest);
         else { // set the cursor and return TRUE
            SetCursor(rghCursor);
            return TRUE;
         }
   }
   return FALSE;
}

void CMultipleRectTracker::DrawTrackerRect(LPCRECT lpRect, 
      CWnd* pWndClipTo, CDC* pDC, CWnd* pWnd, CRect &rectLast)
{
   CRect rect = *lpRect;
   rect.NormalizeRect();

   if (pWndClipTo != NULL) {
      pWnd->ClientToScreen(&rect);
      pWndClipTo->ScreenToClient(&rect);
   }

   CSize size(0, 0);
   if (!m_bFinalErase)
      size.cx = size.cy = 1;
   if (m_bFinalErase || !m_bErase)
      pDC->DrawDragRect(rect, size, rectLast, size);
   rectLast = rect;
}

void CMultipleRectTracker::OnChangingRects(int &/*x*/, int &/*y*/)
{
}

BOOL CMultipleRectTracker::MultiTrack(CWnd* pWnd, CPoint point, 
                                       CWnd* pWndClipTo)
{
   int nOldX, nOldY, dx, dy, i;

   if (::GetCapture() != NULL)
      return FALSE;

   m_bFinalErase = m_bErase = FALSE;
   nOldX = point.x;
   nOldY = point.y;

   pWnd->SetCapture();
   pWnd->UpdateWindow();
   if (pWndClipTo != NULL)
      pWndClipTo->UpdateWindow();

   CDC* pDrawDC;
   if (pWndClipTo != NULL) {
      // clip to arbitrary window by using adjusted Window DC
      pDrawDC = pWndClipTo->GetDCEx(NULL, DCX_CACHE);
   }
   else {
      // otherwise, just use normal DC
      pDrawDC = pWnd->GetDC();
   }
   CRect rectOld;
   BOOL bMoved = FALSE;

   AfxLockTempMaps();  // protect maps while looping

   for(i = 0; i < GetMaxID(); i++) 
      if(IsSelected(i)) // set up tracking variables
         GetNthTag(i)->m_rectLast = 
               GetNthTag(i)->m_rectSave = 
                  GetNthTag(i)->m_pTracker->m_rect;

   // get messages until capture lost or cancelled/accepted
   m_bMultiTracking = TRUE;
   for(BOOL bKeepLooping = TRUE; bKeepLooping;) {
      MSG msg;
      VERIFY(::GetMessage(&msg, NULL, 0, 0));

      if (CWnd::GetCapture() != pWnd)
	      break;
      switch (msg.message) { // handle movement/accept messages
         case WM_LBUTTONUP:
         case WM_MOUSEMOVE:
            dx = (short)LOWORD(msg.lParam) - nOldX; // get delta-x
            dy = (short)HIWORD(msg.lParam) - nOldY; // get delta-y

            OnChangingRects(dx, dy); // allow derived class to edit
            nOldX += dx;
            nOldY += dy;

            for(i = 0; i < GetMaxID(); i++) {
               if(!IsSelected(i)) 
                  continue; // move only selected trackers
               CRectTracker *pTracker = GetTrackerFromID(i);

               rectOld = pTracker->m_rect;
               pTracker->m_rect.OffsetRect(dx, dy); // move tracker

               m_bFinalErase = (msg.message == WM_LBUTTONUP);
               if (bMoved) {
                  m_bErase = TRUE;
                  DrawTrackerRect(&rectOld, pWndClipTo, 
                                pDrawDC, pWnd, 
                                GetNthTag(i)->m_rectLast);
               }
               pTracker->OnChangedRect(rectOld); // derived tracker

               if (msg.message != WM_LBUTTONUP)
                  bMoved = TRUE; // eventual return value
               if (m_bFinalErase)
                  bKeepLooping = FALSE; // get out

               if (dx != 0 || dy != 0) {
                  m_bErase = FALSE; // draw in new place
                  DrawTrackerRect(&pTracker->m_rect,
                                 pWndClipTo, pDrawDC, pWnd, 
                                 GetNthTag(i)->m_rectLast);
               }
            }
            break;
         case WM_KEYDOWN: // if VK_ESCAPE, cancel all moves
            if (msg.wParam != VK_ESCAPE) // else pass through
               break;
         case WM_RBUTTONDOWN: // cancel all moves
            for(i = 0; i < GetMaxID(); i++) {
               if(IsSelected(i)){
                  CRectTracker *pTracker = GetTrackerFromID(i);
                  if(bMoved) {
                     m_bErase = m_bFinalErase = TRUE;
                     DrawTrackerRect(&pTracker->m_rect, 
								pWndClipTo, pDrawDC, pWnd, 
						      GetNthTag(i)->m_rectLast);
                  }
                  pTracker->m_rect = GetNthTag(i)->m_rectSave;
               }
            }
            bKeepLooping = FALSE; // get out
            break;
         default: // all other messages  
            DispatchMessage(&msg); 
            break;
      }
   }

   if (pWndClipTo != NULL)
      pWndClipTo->ReleaseDC(pDrawDC);
   else
      pWnd->ReleaseDC(pDrawDC);
   ReleaseCapture();

   AfxUnlockTempMaps();//FALSE);

   m_bFinalErase = FALSE;
   m_bErase = FALSE;
   m_bMultiTracking = FALSE;
   return bMoved;
}
