#include "cdlink.h"

#include <windowsx.h>
#include <objbase.h>
#include <shlobj.h>
#include <stdio.h>
#include <initguid.h>
#include <string.h>

void    Error(const char* Str)
    {
    MessageBox(NULL, Str, "cdlink.exe", MB_OK);
    }

HRESULT SetLinkCwd(LPCSTR Link, const char* NewDir)
    {
    HRESULT         Result;
    IShellLink*     ShellLink;
    IPersistFile*   Persist;
    char szGotPath[MAX_PATH];
    char szDescription[MAX_PATH];
    WIN32_FIND_DATA FileInfo;

    /* get IShellLink ptr */
    Result = CoCreateInstance(&CLSID_ShellLink, NULL,
            CLSCTX_INPROC_SERVER, &IID_IShellLink, &ShellLink);
    if(!SUCCEEDED(Result))
        {
        Error("CoCreateInstance failed!");
        return Result;
        }

    /* get IPersistFile interface for shell objects */
    Result = ShellLink->lpVtbl->QueryInterface(ShellLink,
        &IID_IPersistFile, &Persist);
    if (SUCCEEDED(Result))
        {
        WORD UniLink[MAX_PATH];
        MultiByteToWideChar(CP_ACP, 0, Link, -1, UniLink, MAX_PATH);
        /* Load shortcut file */
        Result = Persist->lpVtbl->Load(Persist, UniLink,
            STGM_READWRITE);
        if (SUCCEEDED(Result))
            {
            char Cwd[MAX_PATH];
            Result = ShellLink->lpVtbl->GetWorkingDirectory(
                ShellLink, Cwd, MAX_PATH);
            if(SUCCEEDED(Result) && Cwd[0])
                {
                Result = ShellLink->lpVtbl->SetWorkingDirectory(
                    ShellLink, NewDir);
                /* save our changes */
                Persist->lpVtbl->Save(Persist, 0, 0);
                }
            else
                {
                /* else, it's the cdlink.exe shortcut, we think! */
                char    OldDir[512], NewName[512];
                const char* File=strrchr(NewDir, '\\');
                if(!File)
                    File    = strrchr(NewDir, '/');
                if(!File)
                    File    = strrchr(NewDir, ':');
                if(!File)
                    File    = NewDir;
                strcpy(OldDir, Link);
                if(strrchr(OldDir, '\\'))
                    *strrchr(OldDir, '\\') = '\0';
                else if(strrchr(OldDir, '/'))
                    *strrchr(OldDir, '/') = '\0';
                wsprintf(NewName, "%s\\CD %s.lnk", OldDir, File?(File+1):"");
                MoveFile(Link, NewName);
                }
            }
        else
            Error("Load failed for link.");
        /* free up IPersistFile object */
        Persist->lpVtbl->Release(Persist);
        }
    else
        Error("Couldn't get IPersistFile interface pointer.");

    ShellLink->lpVtbl->Release(ShellLink);
    return Result;
    }



/* name of main dialog resource */
#define MAIN_DLG   "MainDialog"

/* global instance handle for this program */
HINSTANCE CurrentInstance;


/* CdLink - the function that does the "real" work.
 *
 * Given a directory, this scans it for .lnk files.
 * For each .lnk file, we change the "start directory"
 * to be the specified path.
 */
void    CdLink(const char* Path, const char* NewDir)
    {
    WIN32_FIND_DATA     Data;
    HANDLE              Handle;
    TCHAR               Pattern[MAX_PATH+1];
  
    lstrcpy(Pattern, Path);
    lstrcat(Pattern, TEXT("\\*.*"));
    Handle = FindFirstFile(Pattern, &Data);
    while(Handle != NULL)
        {
        char    Link[512];

        wsprintf(Link, "%s\\%s", Path, Data.cFileName);
        if(strstr(Data.cFileName, ".lnk")
        || strstr(Data.cFileName, ".pif"))
            {
            SetLinkCwd(Link, NewDir);
            }

        if(!FindNextFile(Handle, &Data))
            {
            FindClose(Handle);
            Handle  = NULL;
            }
        }
    }


#ifdef __BORLANDC__
    #pragma argsused
#endif
UINT APIENTRY FileHook(HWND h, UINT m, WPARAM w, LPARAM l)
{    return 0;  }


void    Browse(HWND Dialog)
    {
    OPENFILENAME    FileInfo;
    char Path[512], Name[512], Save[512];;

    GetCurrentDirectory(sizeof(Save), Save);
    strcpy(Path, Save);
    strcpy(Name, "");
    memset(&FileInfo, 0, sizeof(FileInfo));
    FileInfo.lStructSize    = sizeof(FileInfo);
    FileInfo.hwndOwner      = Dialog;
    FileInfo.hInstance      = CurrentInstance;
    FileInfo.lpstrFilter    = "(directory)\0*.*\0\0";
    FileInfo.nFilterIndex   = 1;
    FileInfo.lpstrInitialDir= Path;
    FileInfo.lpstrFile      = Name;
    FileInfo.nMaxFile       = sizeof(Name);
    FileInfo.lpstrTitle     = "Select directory to CD to";
    FileInfo.lpfnHook       = FileHook;
    FileInfo.Flags          = OFN_NOTESTFILECREATE
        | OFN_NOVALIDATE | OFN_ENABLEHOOK;
    if(GetOpenFileName(&FileInfo))
        {
        GetCurrentDirectory(sizeof(Path), Path);
        SetDlgItemText(Dialog, ID_PATH_COMBO, Path);
        }

    SetCurrentDirectory(Save);
    }

/* LoadCombo - load most recently used paths into combobox */
void    LoadCombo(HWND Combo)
    {
    int i;
    for(i = 0; ; ++i)
        {
        char Path[512], StringId[16];
        wsprintf(StringId, "MRU_%02d", i);
        GetPrivateProfileString(INI_MRU, StringId, "", Path,
            sizeof(Path), INI_FILE);
        if(Path[0])
            SendMessage(Combo, CB_ADDSTRING, 0, (LPARAM)Path);
        else    /* else, no more paths */
            break;
        }
    SendMessage(Combo, CB_SETCURSEL, 0, 0);
    }

/* StoreCombo - store current state of path list in .ini file */
void    StoreCombo(HWND Combo, const char* Mru)
    {
    int iList, iOutput;
    char    Path[512], StringId[16];
    if(Mru[0])
        WritePrivateProfileString(INI_MRU, "MRU_00", Mru, INI_FILE);
    else
        return;

    iOutput = 1;
    for(iList = 0; iOutput < MAX_MRU_PATH; ++iList)
        {
        if(CB_ERR == SendMessage(Combo, CB_GETLBTEXT,
                iList, (LPARAM)Path))
            break;
        if(lstrcmpi(Path, Mru))
            {
            wsprintf(StringId, "MRU_%02d", iOutput++);
            WritePrivateProfileString(INI_MRU, StringId,
                Path, INI_FILE);
            }
        }
    }

#ifdef __BORLANDC__
    #pragma argsused
#endif
static BOOL MainDialogCommand(HWND Dialog, int ControlId,
                              HWND Control, UINT Notify)
    {
    if((ControlId == IDCANCEL || ControlId == IDOK)
        && Notify == BN_CLICKED)
        {
        /* if user wants to change directories */
        if(ControlId == IDOK)
            {
            char    LinkPath[512], Path[512];
            HWND    Combo = GetDlgItem(Dialog, ID_PATH_COMBO);
            GetCurrentDirectory(sizeof(LinkPath), LinkPath);
            GetWindowText(Combo, Path, sizeof(Path));
            StoreCombo(Combo, Path);
            CdLink(LinkPath, Path);
            }
        EndDialog(Dialog, ControlId);
        return TRUE;
        }
    else if(ControlId == ID_BROWSE && Notify == BN_CLICKED)
        Browse(Dialog);
      
    return FALSE;
    }

BOOL CALLBACK MainDialog(HWND Dialog, UINT Message,
                         WPARAM wParam, LPARAM lParam)
    {
    switch(Message)
        {
        case    WM_INITDIALOG   :
            LoadCombo(GetDlgItem(Dialog, ID_PATH_COMBO));
            return TRUE; /* FALSE if you called SetFocus() */

        HANDLE_MSG(Dialog, WM_COMMAND, MainDialogCommand);
        }

    return FALSE; /* FALSE == we did not process msg */
    }

#ifdef __BORLANDC__
    #pragma argsused
#endif
int WINAPI WinMain(HINSTANCE Instance, HINSTANCE Previous,
        LPSTR CommandLine, int ShowCommand)
    {
    char    Path[512];
    CurrentInstance    = Instance;
    CoInitialize(0);
    DialogBox(CurrentInstance, MAIN_DLG, NULL, MainDialog);
    return 0;
    }
