#include "stdafx.h"
#include "csockobj.h"

//////////////////////////////////////////////////
// CSocketArchive
void CSocketArchive::ClearIndex(){
	if (IsStoring()){
		if (m_nMapCount > 1){
			m_pStoreMap->RemoveAll();
			m_pStoreMap->SetAt(NULL,(void*)(DWORD)0);
			m_nMapCount = 1;
		}
	}
	else{
		while (m_nMapCount > 1){
			m_nMapCount--;
			m_pLoadArray->RemoveAt(m_nMapCount);
		}
	}
}

//////////////////////////////////////////////////
// CSocketSlave
void CSocketSlave::OnReceive(int nErrorCode){
	ASSERT(m_pMaster);
	if (nErrorCode == 0){
		DWORD dwBytes;
		IOCtl(FIONREAD, &dwBytes);
		//Don't read if nothing is there
		if (dwBytes > 0){
			m_pMaster->ReadPending();
		}
		return;
	}
	m_pMaster->ErrorSocket(nErrorCode);
}

CSocketSlave::CSocketSlave(CObjectSocket * pOS){
	ASSERT(pOS);
	m_pMaster = pOS;
}

void CSocketSlave::OnClose(int nErrorCode){
	ASSERT(m_pMaster);
	m_pMaster->ConnectionClosed();
}

//////////////////////////////////////////////
// CSocketObjects
CObjectSocket::CObjectSocket(){
	m_pSocket = NULL;
	m_pSocketFile = NULL;
	m_pArchiveIn = NULL;
	m_pArchiveOut = NULL;
}

CObjectSocket::~CObjectSocket(){
	Disconnect();
}

BOOL CObjectSocket::Connect(LPCTSTR lpszHostAddress,
							UINT nHostPort){
	ASSERT(lpszHostAddress);
	try{
		m_pSocket = new CSocketSlave(this);

		if (!m_pSocket->Create()){
			delete m_pSocket;
			m_pSocket = NULL;
			return FALSE;
		}
		if (!m_pSocket->Connect(lpszHostAddress,nHostPort)){
			delete m_pSocket;
			m_pSocket = NULL;
			return FALSE;
		}
		m_pSocketFile = new CSocketFile(m_pSocket);
		m_pArchiveIn = 
			new CSocketArchive(m_pSocketFile,CArchive::load);
		m_pArchiveOut = 
			new CSocketArchive(m_pSocketFile,CArchive::store);
		return TRUE;
	}

	catch (CMemoryException * me){
		me->Delete();
		ClearNewPointers();
		ErrorMemory();
		return FALSE;
	}
	catch (CArchiveException * ae){
		ae->Delete();
		ClearNewPointers();
		return FALSE;
	}
	catch (CFileException * fe){
		fe->Delete();
		ClearNewPointers();
		return FALSE;
	}
}

void CObjectSocket::ClearNewPointers(){
	if (m_pSocket){
		delete m_pSocket;
		m_pSocket = NULL;
	}
	if (m_pSocketFile){
		delete m_pSocketFile;
		m_pSocketFile = NULL;
	}
	if (m_pArchiveIn){
		delete m_pArchiveIn;
		m_pArchiveIn = NULL;
	}
	if (m_pArchiveOut){
		delete m_pArchiveOut;
		m_pArchiveOut = NULL;
	}
}

BOOL CObjectSocket::Connect(CSocket * pSocket){
	ASSERT(pSocket);
	try{
		m_pSocket = new CSocketSlave(this);
		if (!pSocket->Accept(*m_pSocket)){
			delete m_pSocket;
			m_pSocket = NULL;
			return FALSE;
		}
		m_pSocketFile = new CSocketFile(m_pSocket);
		m_pArchiveIn = 
			new CSocketArchive(m_pSocketFile,CArchive::load);
		m_pArchiveOut = 
			new CSocketArchive(m_pSocketFile,CArchive::store);
		return TRUE;
	}
	catch (CMemoryException * me){
		me->Delete();
		ClearNewPointers();
		ErrorMemory();
		return FALSE;
	}
	catch (CArchiveException * ae){
		ae->Delete();
		ClearNewPointers();
		return FALSE;
	}
	catch (CFileException * fe){
		fe->Delete();
		ClearNewPointers();
		return FALSE;
	}
}


void CObjectSocket::Disconnect(){
	if (IsConnected()){
		m_pArchiveOut->Abort();
		delete m_pArchiveOut;
		m_pArchiveOut = NULL;
		m_pArchiveIn->Abort();
		delete m_pArchiveIn;
		m_pArchiveIn = NULL;
		delete m_pSocketFile;
		m_pSocketFile = NULL;
		m_pSocket->ShutDown();
		BYTE Buffer[50];
		while (m_pSocket->Receive(Buffer,50) > 0);
		delete m_pSocket;
		m_pSocket = NULL;
	}
}

void CObjectSocket::ReadPending(){
	ASSERT(IsConnected());
	CNetObject * pNObj = NULL;
	try{
		do{
			*m_pArchiveIn >> pNObj;
			m_pArchiveIn->ClearIndex();
			pNObj->DoAction(this);
		}
		while (!m_pArchiveIn->IsBufferEmpty());
	}
	catch (CArchiveException * ae){
		ErrorArchive(ae->m_cause);
		ae->Delete();
	}
	catch (CFileException * fe){
		ErrorFile(fe->m_cause);
		fe->Delete();
	}
	catch (CMemoryException * me){
		ErrorMemory();
		me->Delete();
	}
}

BOOL CObjectSocket::SendObject(CNetObject * pNObj){
	ASSERT(pNObj);
	ASSERT(IsConnected());
	try{
		if (!IsConnected()) return FALSE;
		*m_pArchiveOut << pNObj;
		m_pArchiveOut->Flush();
		m_pArchiveOut->ClearIndex();
		return TRUE;
	}
	catch (CArchiveException * ae){
		ErrorArchive(ae->m_cause);
		ae->Delete();
		return FALSE;
	}
	catch (CFileException * fe){
		ErrorFile(fe->m_cause);
		fe->Delete();
		return FALSE;
	}
}

void CObjectSocket::OnForcedDrop(){
	//Do nothing - programmer defined
}

#define OSERR_DROP _T("Remote Disconnect Detected! Disconnecting")
#define OSERR_MEMORY _T("Out of Memory! Disconnecting")
#define OSERR_FILE   _T("Socket File Error! Disconnecting")
#define OSERR_SOCKET _T("Socket Error Detected! Disconnecting")
#define OSERR_OBJECT \
		_T("Bad Object Structure Transfered! Disconnecting")

void CObjectSocket::ErrorMemory(){
	Disconnect();
	OnForcedDrop();
	AfxMessageBox(OSERR_MEMORY);
}

void CObjectSocket::ErrorArchive(int nErrorCode){
	Disconnect();
	OnForcedDrop();
	switch (nErrorCode){
		case CArchiveException::badIndex:
		case CArchiveException::badClass:
		case CArchiveException::badSchema:
			AfxMessageBox(OSERR_OBJECT);
			break;
		default:
			AfxMessageBox(OSERR_FILE);
	}
}

void CObjectSocket::ErrorFile(int nErrorCode){
	Disconnect();
	OnForcedDrop();
	AfxMessageBox(OSERR_FILE);
}

void CObjectSocket::ErrorSocket(int nErrorCode){
	Disconnect();
	OnForcedDrop();
	AfxMessageBox(OSERR_SOCKET);
}

void CObjectSocket::ConnectionClosed(){
	Disconnect();
	OnForcedDrop();
	AfxMessageBox(OSERR_DROP);
}

////////////////////////////////////////////////////
// CNetObject
IMPLEMENT_SERIAL(CNetObject, CObject, 1)

void CNetObject::Serialize(CArchive& ar){
	CObject::Serialize(ar);
}

void CNetObject::DoAction(CObjectSocket * pOS){
	ASSERT(FALSE);
	// This member should be overridden in the 
	//	  programmer defined CNetObject
}






