
#include "stdafx.h"
#include "csockobj.h"
#include "resource.h"
#include "nettttdlg.h"
#include "tranobj.h" 



//////////////////////////////////////
// CSendName
IMPLEMENT_SERIAL(CSendName, CNetObject, 1);

CSendName::CSendName(LPCTSTR name)
{
	ASSERT(name);
	m_Name = name;
}

void CSendName::Serialize(CArchive& ar)
{
	CNetObject::Serialize(ar);

	if (ar.IsLoading())
	{
		ar >> m_Name;
	}
	else
	{
		ar << m_Name;
	}
}

void CSendName::DoAction(CObjectSocket * pOS)
{
	pMainDlg->m_RemoteName = m_Name;
	pMainDlg->UpdateStatus();
	delete this;
}

///////////////////////////////////
// CSendMessage

IMPLEMENT_SERIAL(CSendMessage, CNetObject, 1)

CSendMessage::CSendMessage(CNettttDlg * pnd)
{
	CString tmp;
	pnd->m_NewMessage.GetWindowText(m_Message);
	pnd->m_NewMessage.SetWindowText("");
	pnd->m_SendBut.EnableWindow(FALSE);
	
	tmp.Format("%s: %s\r\n",(LPCTSTR)pnd->m_LocalName,(LPCTSTR)m_Message);
	pnd->m_MsgBoard += tmp;
	pnd->UpdateStatus();
}

void CSendMessage::Serialize(CArchive& ar)
{
	CNetObject::Serialize(ar);
	if (ar.IsStoring())
	{
		ar << m_Message;
	}
	else
	{
		ar >> m_Message;
	}
}

void CSendMessage::DoAction(CObjectSocket * pOS)
{
	CString tmp;
	tmp.Format("%s: %s\r\n",(LPCTSTR)pMainDlg->m_RemoteName,(LPCTSTR)m_Message);
	pMainDlg->m_MsgBoard += (LPCTSTR)tmp;
	pMainDlg->UpdateStatus();
	delete this;
}

////////////////////////////////////////////
// CTurn

IMPLEMENT_SERIAL(CTurn, CNetObject, 1)

CTurn::CTurn(BOOL LocalFirst)
{
	for (int i=0; i < 9;i++)
	{
		m_Board[i] = NOMARK;
	}
	if (LocalFirst)
	{
		m_LocalTurn = TRUE;
		m_IsX = TRUE;
	}
	else
	{
		m_LocalTurn = FALSE;
		m_IsX = FALSE;
	}

}

CTurn* CTurn::NextTurn(int MarkedBox)
{
	if (m_IsX)
	{
		m_Board[MarkedBox] = XMARK;
	}
	else
	{
		m_Board[MarkedBox] = YMARK;
	}
	m_LocalTurn = FALSE;
	
	CTurn * tmp = new CTurn;
	ASSERT(tmp);
	for (int i=0; i < 9; i++)
	{
		tmp->m_Board[i] = m_Board[i];
	}
	tmp->m_IsX = !m_IsX;
	tmp->m_LocalTurn = TRUE;
	return tmp;
}

BOOL CTurn::IsDone()
{
	if (HasWon())
		return TRUE;
	for (int i=0;i<9;i++)
	{
		if (m_Board[i] == NOMARK)
		{
			return FALSE;
		}
	}
	return TRUE;
}

BOOL CTurn::HasWon(int * BoxStart,int *  BoxEnd)
{
	BOOL hasBox = ((BoxStart != NULL)&&(BoxEnd != NULL));
	if (IsUsedSpace(0))
	{
		if ((m_Board[0] == m_Board[1]) && (m_Board[0] == m_Board[2]))
		{
			if (hasBox)
			{
				*BoxStart = 0;
				*BoxEnd = 2;
			}
			return TRUE;
		}
		if ((m_Board[0] == m_Board[4]) && (m_Board[0] == m_Board[8]))
		{
			if (hasBox)
			{
				*BoxStart = 0;
				*BoxEnd = 8;
			}
			return TRUE;
		}
		if ((m_Board[0] == m_Board[3]) && (m_Board[0] == m_Board[6]))
		{
			if (hasBox)
			{
				*BoxStart = 0;
				*BoxEnd = 6;
			}
			return TRUE;
		}
	}
	if (IsUsedSpace(1))
	{
		if ((m_Board[1] == m_Board[4]) && (m_Board[1] == m_Board[7]))
		{
			if (hasBox)
			{
				*BoxStart = 1;
				*BoxEnd = 7;
			}
			return TRUE;
		}
	}
	if (IsUsedSpace(2))
	{
		if ((m_Board[2] == m_Board[5]) && (m_Board[2] == m_Board[8]))
		{
			if (hasBox)
			{
				*BoxStart = 2;
				*BoxEnd = 8;
			}
			return TRUE;
		}
		if ((m_Board[2] == m_Board[4]) && (m_Board[2] == m_Board[6]))
		{
			if (hasBox)
			{
				*BoxStart = 2;
				*BoxEnd = 6;
			}
			return TRUE;
		}
	}
	if (IsUsedSpace(3))
	{
		if ((m_Board[3] == m_Board[4]) && (m_Board[3] == m_Board[5]))
		{
			if (hasBox)
			{
				*BoxStart = 3;
				*BoxEnd = 5;
			}
			return TRUE;
		}
	}
	if (IsUsedSpace(6))
	{
		if ((m_Board[6] == m_Board[7]) && (m_Board[6] == m_Board[8]))
		{
			if (hasBox)
			{
				*BoxStart = 6;
				*BoxEnd = 8;
			}
			return TRUE;
		}
	}
	return FALSE;	
}

void CTurn::DoAction(CObjectSocket * pOS)
{
	if (pMainDlg->pGame == NULL)
	{
		pMainDlg->CreateGameBoard();
	}
	pMainDlg->pGame->AddTurn(this);
}

void CTurn::Serialize(CArchive& ar)
{
	CNetObject::Serialize(ar);
	int i;
	if (ar.IsStoring())
	{
		for (i = 0; i < 9; i++)
		{
			ar << m_Board[i];
		}
		ar << m_LocalTurn;
		ar << m_IsX;
	}
	else
	{
		for (i = 0; i < 9; i++)
		{
			ar >> m_Board[i];
		}
		ar >> m_LocalTurn;
		ar >> m_IsX;
	}
}

BOOL CTurn::IsUsedSpace(int space)
{
	return (m_Board[space] != NOMARK);
}

BOOL CTurn::IsXMark(int space)
{
	return (m_Board[space] == XMARK);
}
