# include <windows.h>
# include <eh.h>
# include <float.h>
# include "transex.h"

static _se_translator_function next;

void Win32Exception::SetTranslator()
    {
    if (!installed)
        {
        _se_translator_function next =
            _set_se_translator(Translator);
        installed = TRUE;
        }
    }

unsigned Win32Exception::Translate(unsigned ex,    unsigned mask)
    {
    unsigned prev = translated;
    translated = (translated & ~mask) | (ex & mask);
    return prev;
    }

Win32Exception::~Win32Exception()
    {
    }

void Win32Exception::Translator(unsigned code,
    _EXCEPTION_POINTERS* p)
    {
    switch (code) // parse exception
        {
        case EXCEPTION_ACCESS_VIOLATION:
            if (translated & tr_AccessViolation)
                throw AccessViolation();
            break;
        case EXCEPTION_BREAKPOINT:
            if (translated & tr_Breakpoint)
                throw Breakpoint();
            break;
        case EXCEPTION_DATATYPE_MISALIGNMENT:
            if (translated & tr_DatatypeMisalignment)
                throw DatatypeMisalignment();
            break;
        case EXCEPTION_SINGLE_STEP:
            if (translated & tr_SingleStep)
                throw SingleStep();
            break;
        case EXCEPTION_ARRAY_BOUNDS_EXCEEDED:
            if (translated & tr_ArrayBoundsExceeded)
                throw ArrayBoundsExceeded();
            break;
        case EXCEPTION_FLT_DENORMAL_OPERAND:
            if (translated & tr_FltDenormalOperand)
                throw FltDenormalOperand();
            break;
        case EXCEPTION_FLT_DIVIDE_BY_ZERO:
            if (translated & tr_FltDivideByZero)
                throw FltDivideByZero();
            break;
        case EXCEPTION_FLT_INEXACT_RESULT:
            if (translated & tr_FltInexactResult)
                throw FltInexactResult();
            break;
        case EXCEPTION_FLT_INVALID_OPERATION:
            if (translated & tr_FltInvalidOperation)
                throw FltInvalidOperation();
            break;
        case EXCEPTION_FLT_OVERFLOW:
            if (translated & tr_FltOverflow)
                throw FltOverflow();
            break;
        case EXCEPTION_FLT_STACK_CHECK:
            if (translated & tr_FltStackCheck)
                throw FltStackCheck();
            break;
        case EXCEPTION_FLT_UNDERFLOW:
            if (translated & tr_FltUnderflow)
                throw FltUnderflow();
            break;
        case EXCEPTION_INT_DIVIDE_BY_ZERO:
            if (translated & tr_IntDivideByZero)
                throw IntDivideByZero();
            break;
        case EXCEPTION_INT_OVERFLOW:
            if (translated & tr_IntOverflow)
                throw IntOverflow();
            break;
        case EXCEPTION_PRIV_INSTRUCTION:
            if (translated & tr_PrivInstruction)
                throw PrivInstruction();
            break;
        case EXCEPTION_NONCONTINUABLE_EXCEPTION:
            if (translated & tr_NoncontinuableException)
                throw NoncontinuableException();
            break;
        default:
            if (next)
                next(code, p); // chain
            if (translated & tr_UserException)
                throw Win32UserException(code);
        }
    if (next)
        next(code, p); // chain
    }

int Win32Exception::installed;
unsigned Win32Exception::translated;

FltException::FltException()
    {
    _clearfp(); // turn off exception for _controlfp to succeed
    unsigned cw = _controlfp(0, 0); // save current control word
    _fpreset(); // reset everything
    _controlfp(cw, unsigned(~0)); // restore control word
    }
