#define  STRICT
#include <windows.h>
#include <stdio.h>
#include <commctrl.h>
#include "tvdemo2.h"

/*****************************************
  Treeview Demo Program by Eric Heimburg

  In variable and function names, the
  abbreviation "TV" stands for "Treeview"
*****************************************/

/***** Reusable Utility Functions */
void ClientToWindow(HWND hWnd, POINT *Pt) {
  RECT wRect, cRect;
  GetWindowRect(hWnd, &wRect);
  GetClientRect(hWnd, &cRect);
  MapWindowPoints(hWnd, HWND_DESKTOP, (LPPOINT)&cRect, 2);
  Pt->x -= (wRect.left - cRect.left);
  Pt->y -= (wRect.top - cRect.top);
}

WNDPROC SubclassWnd(HWND hWnd, WNDPROC NewProc) {
  WNDPROC Old = (WNDPROC)GetWindowLong(hWnd, GWL_WNDPROC);
  SetWindowLong(hWnd, GWL_WNDPROC, (LONG)NewProc);
  return Old;
}

char *TV_GetItemText(HWND hTV, HTREEITEM hItem, char *Buf,
                     size_t BufSize) {
  TV_ITEM tvitem;
  tvitem.hItem = hItem;
  tvitem.mask = TVIF_TEXT;
  tvitem.pszText = Buf;
  tvitem.cchTextMax = BufSize;
  TreeView_GetItem(hTV, &tvitem);
  return Buf;
}

BOOL TV_SetItemText(HWND hTV, HTREEITEM hItem, char *Text) {
  TV_ITEM tvitem;
  tvitem.hItem = hItem;
  tvitem.mask = TVIF_TEXT;
  tvitem.pszText = Text;
  return (TreeView_SetItem(hTV, &tvitem) != -1);
}

#define TV_CHECK_OFF    (INDEXTOSTATEIMAGEMASK(1))
#define TV_CHECK_ON     (INDEXTOSTATEIMAGEMASK(2))
BOOL TV_ToggleCheckMark(HWND hTV, HTREEITEM hItem) {
  BOOL bWasOn;
  TV_ITEM tvitem;
  tvitem.hItem = hItem;
  tvitem.mask = TVIF_STATE;
  if (TreeView_GetItem(hTV, &tvitem)) {
    bWasOn= ((tvitem.state & 0xF000) == 0x2000);
    tvitem.state = (bWasOn ? 0x1000 : 0x2000);
    tvitem.stateMask = TVIS_STATEIMAGEMASK;
    TreeView_SetItem(hTV, &tvitem);
    return bWasOn;
  }
  else
    return FALSE;
}

/***** Internal Structure, and function to show the struct*/
typedef struct __TVINTERNALSTRUCT {
  HTREEITEM Parent;
  HTREEITEM Sibling;
  HTREEITEM FirstChild;
  LPCTSTR ItemText;
  DWORD StateFlags;
  WORD ImageID;
  WORD DontKnow;
  WORD ViewableIndexFromRoot;
  WORD IndentationLevel;
  DWORD Reserved;   /* always 0 */
} TVINTERNALSTRUCT;

void ShowInternalStructure(HWND hTV, HTREEITEM hItem) {
  char text[512];
  TVINTERNALSTRUCT *p = (TVINTERNALSTRUCT*)hItem;

  if (!hItem) return;
  sprintf(text, "Pointer:\t\t\t%08Xh\n"
                "Parent:\t\t\t%08Xh\n"
                "Sibling:\t\t\t%08Xh\n"
                "FirstChild:\t\t\t%08Xh\n"
                "ItemText:\t\t\t%s\n"
                "StateFlags:\t\t%08Xh\n"
                "ImageID:\t\t\t%hd\n"
                "DontKnow:\t\t\t%04hXh\n"
                "ViewableIndexFromRoot:\t%hd\n"
                "IndentationLevel:\t\t%hd\n"
                "Reserved:\t\t\t%08Xh\n",
    hItem,
    p->Parent,
    p->Sibling,
    p->FirstChild,
    p->ItemText,
    p->StateFlags,
    p->ImageID,
    p->DontKnow,
    p->ViewableIndexFromRoot,
    p->IndentationLevel,
    p->Reserved );

  MessageBox(hTV, text, p->ItemText, MB_OK);
}

/***** Subclassed Edit Control WndProc */
WNDPROC OrgEditProc;

LRESULT CALLBACK EditProc(HWND hWnd, UINT uMsg,
                          WPARAM wParam, LPARAM lParam) {
  switch (uMsg) {
    case WM_GETDLGCODE:
      return DLGC_WANTALLKEYS;
    case WM_CHAR:
      if ((wParam == VK_ESCAPE) || (wParam == VK_RETURN)) {
        SendMessage(GetParent(hWnd), TVM_ENDEDITLABELNOW,
              ( (wParam == VK_ESCAPE) ? TRUE : FALSE ), 0L);
        return 0;
      }
  }
  return CallWindowProc(OrgEditProc, hWnd, uMsg,
                        wParam, lParam);
}

/***** Global variables */
BOOL bDragging = FALSE;   /* used by both Treeviews */
HTREEITEM hStartingItem;  /* used by both Treeviews */
WNDPROC OrgDragDropTVProc, OrgListBoxStyleTVProc;
HINSTANCE hInstance;

/***** Drag-and-drop Treeview Message Handlers */
void ScrollDragDropTV(HWND hTV, BOOL bUp) {
  HTREEITEM hNext, hFirst = TreeView_GetFirstVisible(hTV);

  ImageList_DragShowNolock(FALSE);
  if (hFirst == NULL) return; /* no items in tree */
  hNext = TreeView_GetNextItem(hTV, hFirst,
    bUp ? TVGN_PREVIOUSVISIBLE : TVGN_NEXTVISIBLE);
  if (hNext)
    SendMessage(hTV, TVM_SELECTITEM, TVGN_FIRSTVISIBLE,
               (LPARAM)hNext);
  UpdateWindow(hTV);
  ImageList_DragShowNolock(TRUE);
}

#pragma argsused
LRESULT CALLBACK DragDropTVProc(HWND hTV, UINT uMsg,
                             WPARAM wParam, LPARAM lParam) {
  HTREEITEM hNewTarget, hOldTarget;
  TV_HITTESTINFO ht;
  RECT r;
  char text[512], s1[256], s2[256];

  switch (uMsg) {
    case WM_MOUSEMOVE:
    case WM_TIMER:
      if (bDragging) {
        GetCursorPos(&ht.pt);
        ScreenToClient(hTV, &ht.pt);
        hNewTarget = TreeView_HitTest(hTV, &ht);
        hOldTarget = TreeView_GetDropHilight(hTV);

        ImageList_DragShowNolock(FALSE);
        SendMessage(hTV, TVM_SELECTITEM, TVGN_DROPHILITE,
                   (LPARAM)hNewTarget);
        if ((hOldTarget != hNewTarget) && (hOldTarget)) {
          TreeView_GetItemRect(hTV, hOldTarget, &r, FALSE);
          InvalidateRect(hTV, &r, TRUE);
        }
        ImageList_DragShowNolock(TRUE);
        MapWindowPoints(hTV, GetParent(hTV), &ht.pt, 1);
        ClientToWindow(GetParent(hTV), &ht.pt);
        ImageList_DragMove(ht.pt.x, ht.pt.y);

        if (ht.flags & TVHT_ABOVE)
          ScrollDragDropTV(hTV, TRUE);
        else if (ht.flags & TVHT_BELOW)
          ScrollDragDropTV(hTV, FALSE);
        return 0;
      }
      break;
    case WM_LBUTTONUP:
    case WM_CAPTURECHANGED:
      if (bDragging) {
        bDragging = FALSE;
        GetCursorPos(&ht.pt);
        ScreenToClient(hTV, &ht.pt);
        hNewTarget = TreeView_HitTest(hTV, &ht);
        ReleaseCapture();
        ImageList_DragLeave(GetParent(hTV));
        ImageList_EndDrag();
        ShowCursor(TRUE);
        SendMessage(hTV, TVM_SELECTITEM, TVGN_DROPHILITE,0);
        KillTimer(hTV, 1);
        if (hNewTarget && (uMsg != WM_CAPTURECHANGED)) {
          TV_GetItemText(hTV, hStartingItem, s1,sizeof(s1));
          TV_GetItemText(hTV, hNewTarget, s2, sizeof(s2));
          sprintf(text, "Dropped %s onto %s", s1, s2);
          MessageBox(hTV, text, "", MB_OK);
        }
        return 0;
      }
    case WM_GETDLGCODE:
      if (bDragging) return DLGC_WANTALLKEYS;
      break;
    case WM_CHAR:
      if (wParam == ' ')
      {
        hNewTarget = TreeView_GetSelection(hTV);
        if (hNewTarget)
          ShowInternalStructure(hTV, hNewTarget);
        return 0;
      }
      if (wParam == VK_ESCAPE) {
        if (bDragging)
          ReleaseCapture();
        else
          SendMessage(GetParent(hTV),WM_CHAR,wParam,lParam);
        return 0;
      }
      break;
  }
  return CallWindowProc(OrgDragDropTVProc, hTV,
                        uMsg, wParam, lParam);
}

LRESULT OnDragDropTVNotify(HWND hTV, NM_TREEVIEW *pHdr) {
  HWND hEdit;
  POINT pt;
  RECT r;
  HIMAGELIST hIml;
  TV_ITEM *pItem;

  switch (pHdr->hdr.code) {
    case TVN_BEGINLABELEDIT:
      hEdit = TreeView_GetEditControl(hTV);
      OrgEditProc = SubclassWnd(hEdit, EditProc);
      return TRUE;
    case TVN_ENDLABELEDIT:
      pItem = &((TV_DISPINFO*)pHdr)->item;
      if ( pItem->pszText != NULL)
        TV_SetItemText(hTV, pItem->hItem, pItem->pszText);
      return TRUE;
    case TVN_BEGINDRAG:
      bDragging = TRUE;
      SetFocus(hTV);
      ShowCursor(FALSE);
      SetCapture(hTV);
      pt = pHdr->ptDrag;
      hStartingItem = pHdr->itemNew.hItem;
      /* create imagelist with dragging-image in it */
      hIml = TreeView_CreateDragImage(hTV, hStartingItem);
      if (!hIml) return FALSE; /* couldn't make image! */
      /* begin dragging item */
      TreeView_GetItemRect(hTV, hStartingItem, &r, 1);
      ImageList_BeginDrag(hIml, 0, pt.x-r.left, pt.y-r.top);
      /* call ImageList_DragEnter. This function requires
      window coords, not client coords, so must convert. */
      MapWindowPoints(hTV, GetParent(hTV), &pt, 1);
      ClientToWindow(GetParent(hTV), &pt);
      ImageList_DragEnter(GetParent(hTV), pt.x, pt.y);
      SetTimer(hTV, 1, 50, NULL);
      return TRUE;
  }
  return 0;
}

/***** Listbox-Style-Scrolling Treeview Message Handlers */
void ScrollListBoxStyleTV(HWND hTV, BOOL bUp) {
  UINT vKey = (bUp ? VK_UP : VK_DOWN);
  SendMessage(hTV, WM_KEYDOWN, vKey, 0);
  SendMessage(hTV, WM_KEYUP, vKey, 0);
}

#pragma argsused
LRESULT CALLBACK ListBoxStyleTVProc(HWND hTV, UINT uMsg,
                             WPARAM wParam, LPARAM lParam) {
  HTREEITEM hNewTarget;
  TV_HITTESTINFO ht;

  switch (uMsg) {
    case WM_LBUTTONDOWN:
    case WM_LBUTTONDBLCLK:
      ht.pt.x = LOWORD(lParam);
      ht.pt.y = HIWORD(lParam);
      hNewTarget = TreeView_HitTest(hTV, &ht);
      if (hNewTarget && (ht.flags & TVHT_ONITEMSTATEICON)) {
        TV_ToggleCheckMark(hTV, hNewTarget);
        return 0;
      }
      break;
    case WM_MOUSEMOVE:
    case WM_TIMER:
      if (bDragging) {
        GetCursorPos(&ht.pt);
        ScreenToClient(hTV, &ht.pt);
        hNewTarget = TreeView_HitTest(hTV, &ht);
        if (hNewTarget != NULL)
          SendMessage(hTV, TVM_SELECTITEM, TVGN_CARET,
                      (LPARAM)hNewTarget);
        else if (ht.flags & TVHT_ABOVE)
          ScrollListBoxStyleTV(hTV, TRUE);
        else if (ht.flags & TVHT_BELOW)
          ScrollListBoxStyleTV(hTV, FALSE);
        return 0;
      }
      break;
    case WM_LBUTTONUP:
    case WM_CAPTURECHANGED:
      if (bDragging) {
        bDragging = FALSE;
        SendMessage(hTV, TVM_HITTEST, 0, (LPARAM)&ht);
        ReleaseCapture();
        ImageList_DragLeave(GetParent(hTV));
        ImageList_EndDrag();
        KillTimer(hTV, 1);
        return 0;
      }
      break;
    case WM_GETDLGCODE:
      if (bDragging) return DLGC_WANTALLKEYS;
      break;
    case WM_CHAR:
      if (wParam == ' ')
      {
        hNewTarget = TreeView_GetSelection(hTV);
        if (hNewTarget)
          ShowInternalStructure(hTV, hNewTarget);
        return 0;
      }
      if (wParam == VK_ESCAPE) {
        if (bDragging)
          ReleaseCapture();
        else
          SendMessage(GetParent(hTV),WM_CHAR,wParam,lParam);
        return 0;
      }
      break;
  }
  return CallWindowProc(OrgListBoxStyleTVProc, hTV,
                        uMsg, wParam, lParam);
}

LRESULT OnListBoxStyleTVNotify(HWND hTV, NM_TREEVIEW *pHdr){
  HWND hEdit;
  TV_ITEM *pItem;

  switch (pHdr->hdr.code) {
    case TVN_BEGINLABELEDIT:
      hEdit = TreeView_GetEditControl(hTV);
      OrgEditProc = SubclassWnd(hEdit, EditProc);
      return TRUE;
    case TVN_ENDLABELEDIT:
      pItem = &((TV_DISPINFO*)pHdr)->item;
      if ( pItem->pszText != NULL)
        TV_SetItemText(hTV, pItem->hItem, pItem->pszText);
      return TRUE;
    case TVN_BEGINDRAG:
      bDragging = TRUE;
      SetFocus(hTV);
      SetCapture(hTV);
      hStartingItem = pHdr->itemNew.hItem;
      SetTimer(hTV, 1, 50, NULL);
      return TRUE;
  }
  return 0;
}

/***** Func. to fill a treeview with 70 dummy items */
#define NUM_NORMALPICS  4
#define NUM_SPECIALPICS 4

void FillTreeview(HWND hTV) {
  TV_INSERTSTRUCT tvins;
  TV_ITEM tvi;
  HTREEITEM hItem;
  int loop;
  char text[256];
  HTREEITEM hItem2;

  memset(&tvi, 0, sizeof(tvi));
  memset(&tvins,0,sizeof(tvins));

  tvi.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE |
             TVIF_STATE;
  tvi.stateMask = TVIS_STATEIMAGEMASK;
  tvi.state = TV_CHECK_OFF;
  tvins.hParent = NULL;
  tvins.hInsertAfter = TVI_LAST;

  for (loop = 0; loop < 50; loop++) {
    if (loop == 8) hItem2 = tvins.hParent;
    sprintf(text, "Item #%d", loop);
    tvi.pszText = text;
    tvi.cchTextMax = lstrlen(text);
    tvi.iImage = (loop % NUM_NORMALPICS);
    tvi.iSelectedImage = (loop % NUM_NORMALPICS);
    tvins.item = tvi;
    hItem = TreeView_InsertItem(hTV, &tvins);
    if ((loop % 10) == 0)
      tvins.hParent = hItem;
  }
  tvins.hParent = hItem2;
  for (loop = 0; loop < 20; loop++) {
    sprintf(text, "Extra Item #%d", loop);
    tvi.pszText = text;
    tvi.cchTextMax = lstrlen(text);
    tvi.iImage = (loop % NUM_NORMALPICS);
    tvi.iSelectedImage = (loop % NUM_NORMALPICS);
    tvins.item = tvi;
    hItem = TreeView_InsertItem(hTV, &tvins);
    if ((loop % 10) == 0)
      tvins.hParent = hItem;
  }
}

/***** Main Dialog Proc */
#pragma argsused
BOOL CALLBACK DlgProc(HWND hWnd, UINT uMsg,
                      WPARAM wParam, LPARAM lParam) {
  HIMAGELIST hNormIml, hSpecialIml;
  HBITMAP hNormBmp, hSpecialBmp;

  switch (uMsg) {
    case WM_INITDIALOG:
      /* create image lists, fill, attach to Treeviews */
      hNormIml = ImageList_Create(16, 16,
               ILC_COLORDDB, NUM_NORMALPICS, 1);
      hSpecialIml = ImageList_Create(16, 16,
               ILC_COLORDDB, NUM_SPECIALPICS, 1);
      hNormBmp = LoadBitmap(hInstance, "TVBITMAP");
      hSpecialBmp = LoadBitmap(hInstance, "TVSPECIAL");
      ImageList_Add(hNormIml, hNormBmp, NULL);
      ImageList_Add(hSpecialIml, hSpecialBmp, NULL);
      DeleteObject(hNormBmp);
      DeleteObject(hSpecialBmp);

      SendMessage(GetDlgItem(hWnd, ID_TV1),
          TVM_SETIMAGELIST, TVSIL_NORMAL, (LPARAM)hNormIml);
      SendMessage(GetDlgItem(hWnd, ID_TV2),
          TVM_SETIMAGELIST, TVSIL_NORMAL, (LPARAM)hNormIml);
      if (!TreeView_GetImageList(GetDlgItem(hWnd, ID_TV2),
                                 TVSIL_STATE))
        SendMessage(GetDlgItem(hWnd, ID_TV2),
                    TVM_SETIMAGELIST,
                    TVSIL_STATE,
                    (LPARAM)hSpecialIml);

      FillTreeview(GetDlgItem(hWnd, ID_TV1));
      FillTreeview(GetDlgItem(hWnd, ID_TV2));

      OrgDragDropTVProc = SubclassWnd(
          GetDlgItem(hWnd, ID_TV1), DragDropTVProc);
      OrgListBoxStyleTVProc = SubclassWnd(
          GetDlgItem(hWnd, ID_TV2), ListBoxStyleTVProc);
      return TRUE;
    case WM_CLOSE:
      EndDialog(hWnd, IDCANCEL);
      break;
    case WM_COMMAND:
      if ( (wParam == MAKELPARAM(IDOK, BN_CLICKED)) ||
           (wParam == MAKELPARAM(IDCANCEL, BN_CLICKED)) )
        EndDialog(hWnd, LOWORD(wParam));
      break;
    case WM_NOTIFY:
      if (wParam == ID_TV1)
        return OnDragDropTVNotify(
           GetDlgItem(hWnd, ID_TV1), (NM_TREEVIEW*)lParam);
      else if (wParam == ID_TV2)
        return OnListBoxStyleTVNotify(
           GetDlgItem(hWnd, ID_TV2), (NM_TREEVIEW*)lParam);
      break;
    default:
      return FALSE;
  }
  return TRUE;
}

/***** WinMain function */
#pragma argsused
int PASCAL WinMain( HINSTANCE hInst, HINSTANCE hPrevInst,
                    LPSTR CmdLine, int nCmdShow ) {
  InitCommonControls();
  hInstance = hInst; /* initialize global variable */

  if (DialogBox(hInst, "MAINDLG", NULL, DlgProc) == -1)
    MessageBox(0, "Couldn't create main dialog", 0, MB_OK);
  return 1;
}
