/***********************************************\
*
* IRQHOOK.c - main module for VxD IRQHOOK
*
* 03/17/95 SSS Created
*
* PURPOSE : Installs interrupt handlers to 
* monitor interrupt activity.
*
* TOOLS : You will need VtoolsD 2.02 for 
* Windows 95 to build this VxD 
*
\***********************************************/

/* Disable "modifiers on data are ignored"
 warning from DDK include files */
#pragma warning( disable : 4229 ) 

#define DEVICE_MAIN 
#include "irqhook.h"
#include "ioctl.h"
#undef DEVICE_MAIN

#include LOCKED_CODE_SEGMENT 
#include LOCKED_DATA_SEGMENT 


Declare_Virtual_Device(IRQHOOK)

DefineControlHandler(SYS_DYNAMIC_DEVICE_INIT, 
 OnSysDynamicDeviceInit);
DefineControlHandler(SYS_DYNAMIC_DEVICE_EXIT, 
 OnSysDynamicDeviceExit);
DefineControlHandler(W32_DEVICEIOCONTROL, 
 OnDeviceIoctl);


BOOL __cdecl ControlDispatcher(
 DWORD dwControlMessage,
 DWORD EBX,
 DWORD EDX,
 DWORD ESI,
 DWORD EDI,
 DWORD ECX)
{
 START_CONTROL_DISPATCH

 ON_SYS_DYNAMIC_DEVICE_INIT(
 OnSysDynamicDeviceInit);
 ON_SYS_DYNAMIC_DEVICE_EXIT(
 OnSysDynamicDeviceExit);
 ON_W32_DEVICEIOCONTROL(
 OnDeviceIoctl);

 END_CONTROL_DISPATCH

 return TRUE;
}


/* Static Data : */
/* BitMask of IRQs to monitor; 
 to be filled in by the App */
DWORD IRQ_Log_Mask; 
/* Hookable irqs (assume nothing) */
DWORD Hookable_IRQs = 0; 
/* Output buffer index */
static WORD wIndex = 0; 
/* Last gotten by the GUI */
static WORD wGUIIndex = 0; 
/* Pointer to the allocated buffer */
static DWORD *pdwBuffer; 

/* TimeStamp structure */
struct TIMESTAMP TimeStamp;

/* Thunks required by VToolsD */
DeviceService_THUNK Thunk_VPICD_Virtualize_Irq;
DeviceService_THUNK Thunk_VPICD_Force_Default_Behavior;

/* HW Interrupt thunk code is placed here */
VPICD_HWInt_THUNK My_VPICD_HWInt_Thunk; 
 
/* An array of all our interrupt handles 
 (indexed by the IRQ number) */
IRQHANDLE My_IRQHandle[16]; 

/* Reusable VID (Interrupt Descriptor) */
VID My_VPICD_IRQ_Descriptor =
 { 0, /* WORD VID_IRQ_Number */
 /* WORD VID_Options */
 VPICD_OPT_CAN_SHARE | VPICD_OPT_REF_DATA, 
 0, /* DWORD VID_Hw_Int_Proc */
 0, /* DWORD VID_Virt_Int_Proc */
 0, /* DWORD VID_EOI_Proc */
 0, /* DWORD VID_Mask_Change_Proc */
 0, /* DWORD VID_IRET_Proc */
 500, /* DWORD VID_IRET_Time_Out */
 (PVOID)0 /* PVOID VID_Hw_Int_Ref */
 };

/* Common handler to all the IRQs we monitor. 
 VPICD calls it to notify about interrupts */
BOOL __stdcall My_VPICD_HWInt_Handler(VMHANDLE hVM,
 IRQHANDLE hIRQ)
{
DWORD dwWhatIRQ;
DWORD dwHiTime, dwLowTime; 
 
/* Flaw in VToolsD : parameter list was not 
 upgraded for 4.0 so just take reference data 
 from edx
*/
  _asm mov dwWhatIRQ,edx;

 /* Log IRQ event */
  if (IRQ_Log_Mask & (1<<dwWhatIRQ))
   {
/* This gives us time stamp with 0.8 us accuracy */
   VTD_Get_Real_Time(&dwHiTime, &dwLowTime);
/* Log the interrupt in circular buffer */
/* 28 bit counter will overflow in about 4.5 minutes */
   pdwBuffer[wIndex++] = (dwWhatIRQ<<28) | 
    (dwLowTime & 0x0FFFFFFF);
/* Wrap around the index */
   wIndex %= STORE_SIZE_DWORDS; 
   }

 return FALSE; /* We do not "process" interrupt(s) */
}


/* VPICD_Virtualize_IRQ hook ... */
VOID __stdcall Hook_VPICD_Virtualize_IRQ(
 PDSFRAME pDSFrame)
{
PVID Client_VID_Ptr;
WORD Client_IRQ;
IRQHANDLE Client_IRQHandle;

/* Get client's VID pointer and IRQ number */
 Client_VID_Ptr = (PVID)pDSFrame->REDI;
 Client_IRQ = Client_VID_Ptr->VID_IRQ_Number;

/* Uninstall our callback if there is one first ... */
 if (My_IRQHandle[Client_IRQ])
  VPICD_Force_Default_Behavior(My_IRQHandle[Client_IRQ]);

/* Install client's callback */
 Call_Previous_Hook_Proc(pDSFrame, 
  &Thunk_VPICD_Virtualize_Irq);
/* Save client's handle */
 Client_IRQHandle = pDSFrame->REAX; 

/* Adjust the reusable descriptor for our new handler */
 My_VPICD_IRQ_Descriptor.VID_IRQ_Number = Client_IRQ;
/* Reference data is set to the IRQ number,*/
/* so that we can tell what IRQ occured in 
 the common handler */ 
 My_VPICD_IRQ_Descriptor.VID_Hw_Int_Ref = 
  (PVOID)Client_IRQ; 
/* Reinstall our handler */
 pDSFrame->REDI = (DWORD)&My_VPICD_IRQ_Descriptor;
 Call_Previous_Hook_Proc(pDSFrame, 
  &Thunk_VPICD_Virtualize_Irq);
 /* Save the handle */
 My_IRQHandle[Client_IRQ] = pDSFrame->REAX; 

/* Restore trashed client regs for clean return
 from the hook */
 pDSFrame->REDI = (DWORD)Client_VID_Ptr; 
 pDSFrame->REAX = Client_IRQHandle;
} 
 

/* VPICD_Force_Default_Behavior hook ... */
VOID __stdcall Hook_VPICD_Force_Default_Behavior(
 PDSFRAME pDSFrame)
{
IRQHANDLE Client_IRQHandle;
DWORD Client_IRQNumber;

/* Get client's IRQ handle and number */
 Client_IRQHandle = pDSFrame->REAX; 
 Client_IRQNumber = VPICD_Convert_Handle_To_IRQ(
 Client_IRQHandle);
 
/* Remove the client's handler first ... */
 Call_Previous_Hook_Proc(pDSFrame, 
  &Thunk_VPICD_Force_Default_Behavior);

/* Are we the only one left ? */
 if (My_IRQHandle[Client_IRQNumber] && 
  (VPICD_Get_Virtualization_Count(
  Client_IRQNumber) == 1)) 
  {
/* If so, remove our own handler */
  pDSFrame->REAX = 
   My_IRQHandle[Client_IRQNumber]; 
  Call_Previous_Hook_Proc(pDSFrame, 
   &Thunk_VPICD_Force_Default_Behavior);
/* Mark empty spot */
  My_IRQHandle[Client_IRQNumber] = 0; 
  }
/* Restore client's handle before returning from 
 the hook. This is in case some client 
 inadvertendly relies on client handler being 
 preserved by VPICD_Force_Default_Behavior() */
 pDSFrame->REAX = Client_IRQHandle; 
} 
 
BOOL OnSysDynamicDeviceInit()
{
int iIRQ;
 
/* Allocate some logging space on non-pageable heap */
 pdwBuffer = 
  (DWORD *)HeapAllocate(STORE_SIZE, 0); 
 dprintf("Allocated buffer size %d DWORDs at %lx\n", 
  STORE_SIZE_DWORDS, pdwBuffer);

/* Create reusable VPICD callback thunk */
 My_VPICD_IRQ_Descriptor.VID_Hw_Int_Proc =
  (DWORD)VPICD_Thunk_HWInt(My_VPICD_HWInt_Handler, 
  &My_VPICD_HWInt_Thunk);

/* Install our handlers on all 16 IRQs */
 dprintf("Hooked IRQs : "); 
 for (iIRQ=0; iIRQ<16; iIRQ++)
  {
/* Fill in reusable IRQ descriptor : 
 IRQ to be monitored */
  My_VPICD_IRQ_Descriptor.VID_IRQ_Number = iIRQ; 
/* Also passed as a reference data */
  My_VPICD_IRQ_Descriptor.VID_Hw_Int_Ref = (PVOID)iIRQ;
/* Should never fail if Helper.VxD is there */
  if (My_IRQHandle[iIRQ] = 
   VPICD_Virtualize_IRQ(&My_VPICD_IRQ_Descriptor)) 
   { 
   Hookable_IRQs |= 1<<iIRQ;
   dprintf("%d ", iIRQ); 
   }
  }
 dprintf("\n");

/* Watch out for future VPICD client registrations ... */
 Assert(Hook_Device_Service((VPICD_DEVICE_ID << 16) | 
  (__VPICD_Virtualize_IRQ),
  Hook_VPICD_Virtualize_IRQ,
  &Thunk_VPICD_Virtualize_Irq)); 

 /* ... and deregistrations */ 
 Assert(Hook_Device_Service((VPICD_DEVICE_ID << 16) | 
  (__VPICD_Force_Default_Behavior),
  Hook_VPICD_Force_Default_Behavior,
  &Thunk_VPICD_Force_Default_Behavior));
 
 dprintf("Successfully hooked VPICD_Virtualize_IRQ and \
  VPICD_Force_Default_Behavior\n"); 
 
 return TRUE;
}

BOOL OnSysDynamicDeviceExit()
{
int iIRQ;

/* Unhook VPICD_Force_Default_Behavior service*/
 Assert(Unhook_Device_Service((VPICD_DEVICE_ID << 16) |
  (__VPICD_Force_Default_Behavior),
  Hook_VPICD_Force_Default_Behavior,
  &Thunk_VPICD_Force_Default_Behavior)); 

/* Unhook VPICD_Virtualize_IRQ service */
 Assert(Unhook_Device_Service( (VPICD_DEVICE_ID << 16) | 
  (__VPICD_Virtualize_IRQ),
  Hook_VPICD_Virtualize_IRQ,
  &Thunk_VPICD_Virtualize_Irq)); 
 
 dprintf("Successfully UNhooked \
  VPICD_Force_Default_Behavior and \
  VPICD_Virtualize_IRQ\n"); 
 
/* Remove all out handles */
 dprintf("Unhooked IRQs : ");
 for (iIRQ=0; iIRQ<16; iIRQ++) 
  if (My_IRQHandle[iIRQ])
   {
   dprintf("%d ", iIRQ);
   VPICD_Force_Default_Behavior(My_IRQHandle[iIRQ]); 
   }
 dprintf("\n");

/* Free the log buffer */
 HeapFree((PVOID)pdwBuffer,0); 
 dprintf("Freed the buffer\n");

 return TRUE;
}

/* IOCTL interface to the App */
DWORD OnDeviceIoctl(PIOCTLPARAMS pIOC)
{
 switch (pIOC->dioc_IOCtlCode)
 { 
 /* VxD is loaded ... */
 case DIOC_OPEN:
  dprintf("DIOC_OPEN\n");
  return DEVIOCTL_NOERROR;
 
 /* VxD is unloaded ... */
 case DIOC_CLOSEHANDLE:
  dprintf("DIOC_CLOSEHANDLE\n");
  return DEVIOCTL_NOERROR;

 /* Reset all */
 case CMD_reset:
  wIndex = wGUIIndex = 0;
  return DEVIOCTL_NOERROR;

 /* Get interrupt statistics */
 case CMD_getstat:
  BEGIN_CRITICAL()

 /* Copy the buffer (size in bytes) */
  if (wGUIIndex<=wIndex) {
  memcpy( pIOC->dioc_OutBuf, 
   &pdwBuffer[wGUIIndex], 
   (wIndex-wGUIIndex)*sizeof(DWORD) );
 /* Return length of copied info (in DWORDs) */
  *pIOC->dioc_bytesret = (wIndex-wGUIIndex);
  } 
 else { /* Wrap around */
  memcpy( pIOC->dioc_OutBuf, 
   &pdwBuffer[wGUIIndex], 
   (STORE_SIZE_DWORDS-wGUIIndex)*sizeof(DWORD));
  *pIOC->dioc_bytesret = 
   STORE_SIZE_DWORDS-wGUIIndex;
  memcpy( &(((DWORD*)pIOC->dioc_OutBuf)
   [STORE_SIZE_DWORDS-wGUIIndex]), 
   pdwBuffer, wIndex*sizeof(DWORD) );
  *pIOC->dioc_bytesret += wIndex;
 }

 /* Syncronize indeces */
 wGUIIndex = wIndex;

 END_CRITICAL()
 return DEVIOCTL_NOERROR;

 /* Get hookable IRQ bitmask */
 case CMD_get_hookable_irq_bitmask:
  *(DWORD *)(pIOC->dioc_OutBuf) = Hookable_IRQs;
  *pIOC->dioc_bytesret = sizeof Hookable_IRQs;
  dprintf("CMD_get_hookable_irq_bitmask : \
   Hookable_IRQs = 0x%lx\n", Hookable_IRQs);
  return DEVIOCTL_NOERROR;

 /* Set bitmask of IRQs of interest */
 case CMD_setirqs:
  IRQ_Log_Mask = *(DWORD *)(pIOC->dioc_InBuf);
  dprintf("CMD_setirqs : IRQ_Log_Mask := 0x%lx\n",
   IRQ_Log_Mask);
  return DEVIOCTL_NOERROR;

 /* Get the timestamp for timestamp translation : 
 ticks -> physical time */
 case CMD_gettime:
  BEGIN_CRITICAL()

  TimeStamp.Time = 
   VTD_Get_Date_And_Time(&TimeStamp.Date);
  VTD_Get_Real_Time(&TimeStamp.HiWord, 
   &TimeStamp.LoWord);

  END_CRITICAL()

  memcpy( pIOC->dioc_OutBuf, &TimeStamp, 
   sizeof TimeStamp );
  return DEVIOCTL_NOERROR;

 default:
  return DEVIOCTL_NOERROR;
 }
}
