// NETMFEX.CPP

#ifdef _UNICODE
#define UNICODE
#endif

// make CopyFileEx function declaration appear
#define _WIN32_WINNT    0x0400 

#include <windows.h>
#include <winnt.h>
#include <tchar.h>
#include <winbase.h>
#include <lm.h>

// function prototype, exactly the same as MoveFileEx

NetMoveFileEx(
LPCTSTR lpExistingFileName,    // address of name of the existing file
LPCTSTR lpNewFileName,        // address of new name for the file
DWORD dwFlags );            // flag to determine how to move file

// support function prototypes

BOOL ConvertNetSpecToLocalValid( LPCTSTR lpFilespec
                        , LPTSTR lpLocalFilespec, DWORD dwLen );
BOOL DelFile( LPCTSTR lpExistingFileName );
BOOL ExtractServerNameFromUNC( LPCTSTR lpUNCName, LPTSTR lpBuffer,
                              DWORD dwLen );
BOOL ExtractShareNameFromUNC( LPCTSTR lpUNCName, LPTSTR lpBuffer,
                             DWORD dwLen );
BOOL GetServerNameFromFileSpec( LPCTSTR lpFileSpec,
                               LPTSTR lpBuffer, DWORD dwSize  );
BOOL GetSharePath( LPCTSTR lpServer, LPCTSTR lpNetname,
                  LPTSTR lpPath, DWORD dwLen );
BOOL GetSharePath2( LPCTSTR lpUNCName, LPTSTR lpPath, 
                   DWORD dwLen );
BOOL GetUNCFromNetworkDriveLetter( LPCTSTR lpFilespec, 
                                  LPTSTR lpBuffer, DWORD dwLen );
LPCWSTR UnicodeString( LPCTSTR lpString, LPTSTR lpBuffer, 
                      DWORD dwSize /*in bytes*/ );
BOOL UsesNetworkResource( LPCTSTR lpFileName );


BOOL GetServerNameFromFileSpec( LPCTSTR lpFileSpec, 
                               LPTSTR lpBuffer, DWORD dwSize  )
{
    // if this is a UNC, call another function
    if( 0 == _tcsncmp( lpFileSpec, _T( "\\\\" ), 2 ) )
        return ExtractServerNameFromUNC( lpFileSpec, lpBuffer, 
        dwSize );

    TCHAR szUNCName[MAX_PATH];

    // convert the network drive letter to UNC format share
    if( !GetUNCFromNetworkDriveLetter( lpFileSpec, szUNCName, 
        sizeof( szUNCName ) ) )
        return FALSE;

    // call the extract server function
    return ExtractServerNameFromUNC( szUNCName, lpBuffer, dwSize );
}

// converts from \\server\share\directory\directory\file.ext 
//                          to \directory\directory\file.ext
BOOL GetSharePath2( LPCTSTR lpUNCName, LPTSTR lpPath, DWORD dwLen )
{
    // if the string does not contain a leading double-backslash,
    // this is not an UNC; get out
    if( 0 != _tcsncmp( lpUNCName, _T( "\\\\" ), 2 ) )
    {
        SetLastError( ERROR_INVALID_NAME  );
        return FALSE;
    }

    // skip over the "\\"
    lpUNCName += 2;

    // find the next backslash
    TCHAR* pNextSlash = _tcschr( lpUNCName, _T( '\\' ) );

    // if there is no next backslash, this is not an UNC; get out
    if( !pNextSlash )
    {
        SetLastError( ERROR_INVALID_NAME  );
        return FALSE;
    }

    // skip over the next backslash
    pNextSlash++;

    // find the next backslash
    pNextSlash = _tcschr( pNextSlash, _T( '\\' ) );

    // if there is no next backslash, the UNC is incomplete
    if( !pNextSlash )
    {
        SetLastError( ERROR_INVALID_NAME  );
        return FALSE;
    }

    // copy the remainer of the string
    _tcscpy( lpPath, pNextSlash );

    return TRUE;
}

#ifndef _UNICODE

// return a wide (unicode) string given a string
// the lpBuffer argument itself is the return value
LPCWSTR UnicodeString( LPCTSTR lpString, LPTSTR lpBuffer, 
                      DWORD dwSize /*in bytes*/ )
{
    ZeroMemory( lpBuffer, dwSize );

    // convert the string to Unicode
    int chars = MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, 
        lpString, _tcslen( lpString )
        , (LPWSTR) lpBuffer, dwSize / 2 );

    // return the user buffer
    return (LPCWSTR) lpBuffer;
}
#endif

// get a share's path
// converts from SHARE to
// F:\Dir\subdir 
// e.g. c$ = C:\ 
// e.g. admin$ = C:\WINNT
BOOL GetSharePath( LPCTSTR lpServer, LPCTSTR lpNetname, LPTSTR 
                  lpPath, DWORD dwLen )
{
    LPBYTE *bufptr = NULL;
    NET_API_STATUS rv;

    TCHAR szBuffer[MAX_PATH] = {0}, szBuffer2[MAX_PATH] = {0};

#ifndef _UNICODE
    // get information on the share
    rv = NetShareGetInfo( (LPTSTR) 
        UnicodeString( lpServer, szBuffer, sizeof( szBuffer ) )
        ,(LPTSTR) UnicodeString( lpNetname, szBuffer2, 
        sizeof( szBuffer2 ) ), 2, (LPBYTE*) &bufptr );

    if( NERR_Success != rv )
    {
        SetLastError( rv );
        return FALSE;
    }
#else
    // get information on the share
    if( NERR_Success != ( rv = NetShareGetInfo( (LPTSTR) lpServer,
        (LPTSTR) lpNetname, 2, (LPBYTE*) &bufptr ) ) )
    {
        SetLastError( rv );
        return FALSE;
    }
#endif

    LPSHARE_INFO_2 pInfo = (LPSHARE_INFO_2) bufptr;

#ifdef _UNICODE
    // copy the path to the buffer argument
    _tcscpy( lpPath, pInfo->shi2_path );
#else
    // convert the Unicode string to a string
    int chars = WideCharToMultiByte( CP_ACP, 0, 
        (LPCWSTR) pInfo->shi2_path, -1, lpPath, dwLen, 
        NULL, NULL );
#endif

    // free NETAPI memory
    NetApiBufferFree( bufptr );

    return TRUE;
}

// extract the share portion of a UNC format file specification
// \\server\share\dir\subdir\filename.typ
//         |-----|
BOOL ExtractShareNameFromUNC( LPCTSTR lpUNCName, LPTSTR lpBuffer, 
                             DWORD dwLen )
{
    *lpBuffer = _T( '\0' );

    // if the string does not contain a leading double-backslash, 
    // this is not an UNC; get out
    if( 0 != _tcsncmp( _T( "\\\\" ), lpUNCName, 2 ) )
    {
        SetLastError( ERROR_INVALID_NAME  );
        return FALSE;
    }

    // skip over the "\\"
    lpUNCName += 2;

    // find the next backslash
    TCHAR* pNextSlash = _tcschr( lpUNCName, _T( '\\' ) );

    // if there is no next backslash, this is not an UNC; get out
    if( !pNextSlash )
    {
        SetLastError( ERROR_INVALID_NAME  );
        return FALSE;
    }

    pNextSlash++;

    // find the next backslash
    TCHAR* pNextSlash2 = _tcschr( pNextSlash, _T( '\\' ) );

    // if there is no next backslash, the rest of the string is 
    // the share name, so search for the zero-terminate character
    if( !pNextSlash2 )
        pNextSlash2 = _tcschr( pNextSlash, _T( '\0' ) );

    // if there is no next backslash or trailing null, get out
    if( !pNextSlash2 )
    {
        SetLastError( ERROR_INVALID_NAME  );
        return FALSE;
    }

    // determine the length of the share name
    DWORD dwShareLen = pNextSlash2 - pNextSlash;

    // copy the share name to the buffer argument
    _tcsncpy( lpBuffer, pNextSlash, dwShareLen );

    // skip over the share
    lpBuffer += dwShareLen;

    // zero terminate the buffer
    *lpBuffer = _T( '\0' );

    return TRUE;
}

// extract the server portion of a UNC format file specification
// \\server\share\dir\subdir\filename.typ
//  |------|
BOOL ExtractServerNameFromUNC( LPCTSTR lpUNCName, LPTSTR lpBuffer, 
                              DWORD dwLen )
{
    *lpBuffer = _T( '\0' );

    // if the string does not contain a leading double-backslash, 
    // this is not an UNC; get out
    if( 0 != _tcsncmp( _T( "\\\\" ), lpUNCName, 2 ) )
    {
        SetLastError( ERROR_INVALID_NAME  );
        return FALSE;
    }

    // skip over the "\\"
    lpUNCName += 2;

    // find the next backslash
    TCHAR* pNextSlash = _tcschr( lpUNCName, _T( '\\' ) );

    if( !pNextSlash )
    {
        SetLastError( ERROR_INVALID_NAME  );
        return FALSE;
    }

    // determine the length of the server name
    DWORD dwServerLen = pNextSlash - lpUNCName;

    // copy the server name to the buffer argument
    _tcsncpy( lpBuffer, lpUNCName, dwServerLen );

    // skip over the server
    lpBuffer += dwServerLen;

    // zero terminate the buffer
    *lpBuffer = _T( '\0' );

    return TRUE;
}

// converts from G: to \\server\share
BOOL GetUNCFromNetworkDriveLetter( LPCTSTR lpFilespec, 
                                  LPTSTR lpBuffer, DWORD dwLen )
{
    *lpBuffer = _T( '\0' );

    TCHAR szDevName[3] = {0};

    // copy the first two characters of the file specification, 
    // which should be 
    // X:
    _tcsncpy( szDevName, lpFilespec, 2 );

    // translate the drive letter to an UNC
    if( NO_ERROR != WNetGetConnection( szDevName, lpBuffer, 
        &dwLen ) )
        return FALSE;

    return TRUE;
}


BOOL ConvertNetSpecToLocalValid( LPCTSTR lpFilespec, 
                                LPTSTR lpLocalFilespec, 
                                DWORD dwLen )
{
    TCHAR szUNC[MAX_PATH], szPath[MAX_PATH];
    TCHAR szTempPath[MAX_PATH], szServer[32], szShare[MAX_PATH];

    // X:\ format
    if( lpFilespec[1] == ':' )
    {
        // convert the D:\ format to \\unc\share
        if( !GetUNCFromNetworkDriveLetter( lpFilespec, szUNC, 
            sizeof( szUNC ) ) )
            return FALSE;
        // extract the server name
        if( !ExtractServerNameFromUNC( szUNC, szServer, 
            sizeof( szServer ) ) )
            return FALSE;
        // extract the share name
        if( !ExtractShareNameFromUNC( szUNC, szShare, 
            sizeof( szShare ) ) )
            return FALSE;
        // get the share's actual path on the remote system
        if( !GetSharePath( szServer, szShare, szPath, 
            sizeof( szPath ) ) )
            return FALSE;
        // if the last character is a backslash, get rid of it
        if( szPath[_tcslen(szPath)-1] == '\\' )
            szPath[_tcslen(szPath)-1] = '\0';
        // copy the share portion of the file spec
        _tcscpy( szTempPath, szPath );
        // concatenate the subdir and file portion of the file spec
        _tcscat( szTempPath, &lpFilespec[2] );

        // if the buffer size is too small, get out
        if( dwLen < _tcslen( szTempPath ) )
        {
            SetLastError( ERROR_INSUFFICIENT_BUFFER );
            return FALSE;
        }

        // copy the resultant filespec to the buffer argument
        _tcscpy( lpLocalFilespec, szTempPath );
    }
    // \\SERVER\SHARE\DIR\SUBDIR\FILE.TYP FORMAT
    else
    {
        TCHAR szFilePath[MAX_PATH];
        // extract the server name
        if( !ExtractServerNameFromUNC( lpFilespec, szServer, 
            sizeof( szServer ) ) )
            return FALSE;
        // extract the share name
        if( !ExtractShareNameFromUNC( lpFilespec, szShare, 
            sizeof( szShare ) ) )
            return FALSE;
        // get the share's actual path on the remote system
        if( !GetSharePath( szServer, szShare, szPath, 
            sizeof( szPath ) ) )
            return FALSE;
        // if the last character is a backslash, get rid of it
        if( szPath[_tcslen(szPath)-1] == '\\' )
            szPath[_tcslen(szPath)-1] = '\0';
        // isolate the share's directory and file specification
        if( !GetSharePath2( lpFilespec, szFilePath, 
            sizeof( szFilePath ) ) )
            return FALSE;
        // copy the share portion of the file spec
        _tcscpy( szTempPath, szPath );
        // concatenate the subdir and file portion of the file spec
        _tcscat( szTempPath, szFilePath );

        // if the buffer size is too small, get out
        if( dwLen < _tcslen( szTempPath ) )
        {
            SetLastError( ERROR_INSUFFICIENT_BUFFER );
            return FALSE;
        }

        // copy the resultant filespec to the buffer argument
        _tcscpy( lpLocalFilespec, szTempPath );
    }

    return TRUE;
}

// determine if the file specification refers to a network resource
BOOL UsesNetworkResource( LPCTSTR lpFileName )
{
    // if there is no file name, it can't be a network resource
    if( NULL == lpFileName )
        return FALSE;

    // build a string to can contain a drive letter
    TCHAR szDrive[4] = { 0, 0, '\\', 0};

    // if the first two characters are backslashes, this is a UNC
    if( 0 == _tcsncmp( _T( "\\\\" ), lpFileName, 2 ) )
        return TRUE;

    // copy the first two letters of the file spec
    _tcsncpy( szDrive, lpFileName, 2 );

    // if the drive type is remote, the drive letter is mapped to
    // a network resource
    if( DRIVE_REMOTE == GetDriveType( szDrive ) )
        return TRUE;

    return FALSE;
}

// delete a file at startup.  Implements:
// NetMoveFileEx( "file", NULL, MOVEFILE_DELAY_UNTIL_REBOOT );
// *** do not call this function directly! Use NetMoveFileEx() ***
BOOL DelFile( LPCTSTR lpExistingFileName )
{
    // file name preamble and size
    static TCHAR szQs[] = _T( "\\??\\" );
    static DWORD dwQsize = _tcslen( szQs );

    PTCHAR pChar = NULL;
    LONG rv;

    TCHAR  szLocalFromFilespec[MAX_PATH];

    // convert the temporary source file spec to a local 
    // valid name
    if( !ConvertNetSpecToLocalValid( lpExistingFileName, 
        szLocalFromFilespec, sizeof( szLocalFromFilespec ) ) )
        return FALSE;

    TCHAR szServer[32];

    // extract the server name from the filename
    if( !GetServerNameFromFileSpec( lpExistingFileName, szServer, 
        sizeof( szServer ) ) )
        return FALSE;

    HKEY hKey = NULL, hSubKey = NULL;

    // if it is not possible to connect to the remote system's 
    // registry return failure
    rv = RegConnectRegistry( szServer, HKEY_LOCAL_MACHINE, &hKey );
    if( ERROR_SUCCESS != rv )
    {
        SetLastError( rv );
        return FALSE;
    }

    // open the Session Manager key
    rv = RegOpenKeyEx( hKey, 
        _T( "SYSTEM\\CurrentControlSet\\Control\\Session Manager" )
        , 0, KEY_READ | KEY_WRITE, &hSubKey );
    if( ERROR_SUCCESS != rv )
    {
        RegCloseKey( hKey );
        SetLastError( rv );
        return FALSE;
    }

    // this key is no longer needed since we have a key to the 
    // Session Manager
    RegCloseKey( hKey );
    hKey = NULL;

    DWORD dwBufferSize = 0, dwCurrentLength = 0;
    DWORD dwType;
    TCHAR* lpBuffer = NULL;
    TCHAR* lpInsertionPt = NULL;

    // determine the size of the buffer required by specifying 
    // NULL as the destination buffer
    rv = RegQueryValueEx( hSubKey, 
        _T( "PendingFileRenameOperations" ), 0, &dwType
        , NULL, &dwCurrentLength );

    if( ERROR_SUCCESS == rv )
    {
        // calculate a buffer size large enough to include one 
        // file, and three nulls
        dwBufferSize = dwCurrentLength + MAX_PATH + 3;
        lpBuffer = new TCHAR[dwBufferSize];

        // get the current value
        rv = RegQueryValueEx( hSubKey, 
            _T( "PendingFileRenameOperations" ), 0, &dwType
                , (LPBYTE) lpBuffer, &dwCurrentLength );

        // calculate the actual number of characters (whether 
        // ANSI or UNICODE)
        dwCurrentLength /= sizeof( TCHAR );
        // subtract one since the last null will be replaced by a 
        // file spec
        dwCurrentLength--;
        // set the next file spec insertion point in the 
        // value buffer
        lpInsertionPt = lpBuffer + dwCurrentLength;
    }
    else
    // no value currently exists
    if( ERROR_FILE_NOT_FOUND == rv )
    {
        // allocate a buffer large enough to include one 
        // file, and three nulls
        dwBufferSize = MAX_PATH + 3;
        lpBuffer = new TCHAR[dwBufferSize];

        // set the registry value type
        dwType = REG_MULTI_SZ;
        // set the next file spec insertion point to the 
        // beginning of the buffer
        lpInsertionPt = lpBuffer;
    }

    // if an error other than success or no value occurred, 
    // return failure
    if( ERROR_FILE_NOT_FOUND != rv && ERROR_SUCCESS != rv )
    {
        delete [] lpBuffer;
        RegCloseKey( hSubKey );
        SetLastError( rv );
        return FALSE;
    }

    // copy \??\ preamble
    CopyMemory( lpInsertionPt, szQs, sizeof( TCHAR ) * dwQsize );
    lpInsertionPt += dwQsize;

    // effectively append the delete file spec
    CopyMemory( lpInsertionPt, szLocalFromFilespec, 
        sizeof( TCHAR ) * ( _tcslen( szLocalFromFilespec ) + 1 ) );
    lpInsertionPt += _tcslen( szLocalFromFilespec ) + 1;

    // add the final trailing null (second of two)
    *lpInsertionPt = _T( '\0' );
    *++lpInsertionPt = _T( '\0' );

    // increment buffer length by delete file spec length
    // plus a total of three nulls 
    dwCurrentLength += dwQsize + 
        ( _tcslen( szLocalFromFilespec ) + 1 ) + 2;

    // set the new value
    rv = RegSetValueEx( hSubKey, 
        _T( "PendingFileRenameOperations" ), 0, dwType
            , (CONST BYTE *) lpBuffer, dwCurrentLength * 
            sizeof( TCHAR ) );

    delete [] lpBuffer;

    RegCloseKey( hSubKey );

    // if an error occurred setting the value, return failure
    if( rv != ERROR_SUCCESS )
    {
        SetLastError( rv );
        return FALSE;
    }

    return TRUE;
}

// Implement MoveFileEx boot-time move functionality for network 
// source and network destination file specifications, including
// boot-time deletes
BOOL NetMoveFileEx( LPCTSTR lpExistingFileName, 
                   LPCTSTR lpNewFileName, DWORD dwFlags )
{
    // if there is no source file specification, return failure
    if( !lpExistingFileName )
    {
        SetLastError( ERROR_PATH_NOT_FOUND );
        return FALSE;
    }

    // file name preamble and size
    static TCHAR szQs[] = _T( "\\??\\" );
    static DWORD dwQsize = _tcslen( szQs );

    BOOL bNetworkSource =UsesNetworkResource( lpExistingFileName );
    BOOL bNetworkDestination =UsesNetworkResource( lpNewFileName );

    // if dwFlags does not specify the move file bit or neither 
    // file specification uses a network resource, call the 
    // WIN32 function
    if( !( dwFlags & MOVEFILE_DELAY_UNTIL_REBOOT ) || 
        ( !bNetworkSource && !bNetworkDestination ) )
        return( MoveFileEx( lpExistingFileName, lpNewFileName, 
                dwFlags ) );

    // if the destination file name is NULL, this is actually 
    // a deferred delete, call modified function
    if( NULL == lpNewFileName )
        return DelFile( lpExistingFileName );

    PTCHAR pChar = NULL;
    LONG rv;
    TCHAR szTempCopy[MAX_PATH], szTempSourceSpec[MAX_PATH]
        ,szLocalFromFilespec[MAX_PATH],szLocalToFilespec[MAX_PATH];

    _tcscpy( szTempCopy, lpNewFileName );

    // get a pointer to the last backslash in the destination
    // file spec
    pChar = _tcsrchr( szTempCopy, _T( '\\' ) );
    if( !pChar )
    {
        SetLastError( ERROR_INVALID_NAME );
        return FALSE;
    }

    // terminate the string after the backslash
    *++pChar = _T( '\0' );

    // if the destination file spec does not use a network 
    // resource, the WIN32 MoveFileEx can be called since, after 
    // the CopyFileEx, both source and destination 
    // file specs are on the local system 
    if( !bNetworkDestination )
    {
        // generate a temporary file name (actually creates a zero 
        // byte file on the target system)
        if( 0 == GetTempFileName( szTempCopy, _T( "NMF" ), 0, 
            szTempSourceSpec ) )
            return FALSE;

        // copy the source file to the temporary file name
        if( !CopyFileEx( lpExistingFileName, szTempSourceSpec, 
            NULL, NULL, NULL, FALSE ) )
            return FALSE;

        return MoveFileEx( szTempSourceSpec, lpNewFileName, 
            dwFlags );
    }

    // At this point, it is known that the destination file spec
    // uses a network resource

    // generate a temporary file name
    if( 0 == GetTempFileName( szTempCopy, _T( "NMF" ), 0, 
        szTempSourceSpec ) )
        return FALSE;

    // copy the source file to the temporary file name
    if( !CopyFileEx( lpExistingFileName, szTempSourceSpec, NULL, 
        NULL, NULL, FALSE ) )
        return FALSE;

    TCHAR szDestServer[32];

    // determine if newly copied source file name uses a network 
    // resource
    bNetworkSource = UsesNetworkResource( szTempSourceSpec );

    // convert the temporary source file spec to a local valid name
    if( !ConvertNetSpecToLocalValid( szTempSourceSpec, 
        szLocalFromFilespec, sizeof( szLocalFromFilespec ) ) )
        return FALSE;

    // convert the destination file spec to a local valid name
    if( !ConvertNetSpecToLocalValid( lpNewFileName, 
        szLocalToFilespec, sizeof( szLocalToFilespec ) ) )
        return FALSE;

    // extract the server name from the destination filename
    if( !GetServerNameFromFileSpec( lpNewFileName, szDestServer, 
        sizeof( szDestServer ) ) )
        return FALSE;

    HKEY hKey = NULL, hSubKey = NULL;

    // if it is not possible to connect to the remote system's 
    // registry return failure
    if( ERROR_SUCCESS != ( rv = RegConnectRegistry( szDestServer, 
        HKEY_LOCAL_MACHINE, &hKey ) ) )
    {
        SetLastError( rv );
        return FALSE;
    }

    // open the Session Manager key
    if( ERROR_SUCCESS != ( rv = RegOpenKeyEx( hKey, 
        _T( "SYSTEM\\CurrentControlSet\\Control\\Session Manager" )
        , 0, KEY_READ | KEY_WRITE, &hSubKey ) ) )
    {
        RegCloseKey( hKey );
        SetLastError( rv );
        return FALSE;
    }

    // this key is no longer needed since we have a key to the 
    // Session Manager subkey
    RegCloseKey( hKey );
    hKey = NULL;

    DWORD dwBufferSize = 0, dwCurrentLength = 0;
    DWORD dwType;
    TCHAR* lpBuffer = NULL;
    TCHAR* lpInsertionPt = NULL;

    // determine the size of the buffer required by specifying 
    // NULL as the destination buffer
    rv = RegQueryValueEx( hSubKey, 
        _T( "PendingFileRenameOperations" ), 0, &dwType
        , NULL, &dwCurrentLength );

    if( ERROR_SUCCESS == rv )
    {
        // calculate a buffer size large enough to include two 
        // new files, and two nulls
        dwBufferSize = dwCurrentLength + ( MAX_PATH * 2 ) + 2;
        lpBuffer = new TCHAR[dwBufferSize];

        // get the current value
        rv = RegQueryValueEx( hSubKey, 
            _T( "PendingFileRenameOperations" ), 0, &dwType
                , (LPBYTE) lpBuffer, &dwCurrentLength );

        // calculate the actual number of characters (whether 
        // ANSI or UNICODE)
        dwCurrentLength /= sizeof( TCHAR );
        // subtract one since the last null will be replaced by 
        // a file spec
        dwCurrentLength--;
        // set the next file spec insertion point in the 
        // value buffer
        lpInsertionPt = lpBuffer + dwCurrentLength;
    }
    else
    // no value currently exists
    if( ERROR_FILE_NOT_FOUND == rv )
    {
        // allocate a buffer large enough to include two new 
        // files, and three nulls
        dwBufferSize = ( MAX_PATH * 2 ) + 3;
        lpBuffer = new TCHAR[dwBufferSize];

        // set the registry value type
        dwType = REG_MULTI_SZ;
        // set the next file spec insertion point to the 
        // beginning of the buffer
        lpInsertionPt = lpBuffer;
    }

    // if an error other than success or no value occurred, 
    // return failure
    if( ERROR_FILE_NOT_FOUND != rv && ERROR_SUCCESS != rv )
    {
        delete [] lpBuffer;
        RegCloseKey( hSubKey );
        SetLastError( rv );
        return FALSE;
    }

    // copy \??\ preamble
    CopyMemory( lpInsertionPt, szQs, sizeof( TCHAR ) * dwQsize );
    lpInsertionPt += dwQsize;

    // effectively append the from file spec
    CopyMemory( lpInsertionPt, szLocalFromFilespec, 
        sizeof( TCHAR ) * ( _tcslen( szLocalFromFilespec ) + 1 ) );
    lpInsertionPt += _tcslen( szLocalFromFilespec ) + 1;

    // copy \??\ preamble
    CopyMemory( lpInsertionPt, szQs, sizeof( TCHAR ) * dwQsize );
    lpInsertionPt += dwQsize;

    // effectively append the to file spec
    CopyMemory( lpInsertionPt, szLocalToFilespec, 
        sizeof( TCHAR ) * ( _tcslen( szLocalToFilespec ) + 1 ) );
    lpInsertionPt += _tcslen( szLocalToFilespec ) + 1;

    // add the final trailing null (second of two)
    *lpInsertionPt = _T( '\0' );

    // increment buffer length by from and to file spec lengths 
    // plus a total of three nulls 
    dwCurrentLength += ( dwQsize * 2 ) +
        ( _tcslen( szLocalFromFilespec ) + 1 ) + 
        ( _tcslen( szLocalToFilespec ) + 1 ) + 1;

    // set the new value
    rv = RegSetValueEx( hSubKey, 
        _T( "PendingFileRenameOperations" ), 0, dwType
            , (CONST BYTE *) lpBuffer, dwCurrentLength * 
            sizeof( TCHAR ) );

    delete [] lpBuffer;

    RegCloseKey( hSubKey );

    // if an error occurred setting the value, return failure
    if( rv != ERROR_SUCCESS )
    {
        SetLastError( rv );
        return FALSE;
    }

    return TRUE;
}

// CODE below this point has NOT been tested using UNICODE

#include <iostream.h>

// show help info
void Help()
{
    cout << "Moves or deletes a file upon reboot.\n\n";
    cout << "The syntax of this command is:\n\n";
    cout << "NETMFEX SourceFile DestinationFile\n\n";
    cout << "NETMFEX SourceFile delete\n\n";
}

main( int argc, char *argv[], char *envp[] )
{
    BOOL tf;

    // if there are too few arguments, help the user
    if( argc < 3 )
    {
        Help();
        return 1;
    }

    // the user requests help
    if( 0 == _tcscmp( _T( "/?" ), argv[1] ) || 
        0 == _tcscmp( _T( "-?" ), argv[1] ) )
    {
        Help();
        return 1;
    }

    // if the user is deleting a file...
    if( 0 == _stricmp( argv[2], _T("delete") ) )
    {
        tf = NetMoveFileEx( argv[1], NULL, 
                MOVEFILE_DELAY_UNTIL_REBOOT );
    }
    // otherwise the user is moving a file
    else
    {
        tf = NetMoveFileEx( argv[1], argv[2], 
                MOVEFILE_DELAY_UNTIL_REBOOT );
    }

    // if NetMoveFileEx failed, report the error code
    if( !tf )
    {
        cerr << "An error occurred moving file.\nError code: " 
            << GetLastError() << "\n";
        return 2;
    }

    return 0;
}
