#define WIN32
#define STRICT
#define _CONSOLE
#include <windows.h>
#include <initguid.h>
#include <stdio.h>

/* This include file is available from the Internet Client SDK. */
#include <mstask.h>

/* Forwards */
void CreateNewJob(ISchedulingAgent *, LPWSTR, LPWSTR, LPWSTR,
 LPSYSTEMTIME);

/* Predefines */
#define BATCH_NAME (L"WDJ Task Sample")
#define BATCH_APP  (L"notepad.exe")
#define BATCH_PARM (L"")

void main(int argc, char **argv)
{
 CoInitialize(NULL);
 ISchedulingAgent *pSchedulingAgent = NULL;
 CoCreateInstance(CLSID_CSchedulingAgent, NULL, 
 CLSCTX_INPROC_SERVER, IID_ISchedulingAgent, 
 (void **) &pSchedulingAgent);

 if (pSchedulingAgent) {
  if (! lstrcmpiA(argv[1], "/REMOVE")) {
   /* Remove the job from the queue */
   printf("Removing sample job.\n");
   pSchedulingAgent->Delete(BATCH_NAME);
  }
  else {
   /* Get the time in one minute */
   SYSTEMTIME systime;
   GetLocalTime(&systime);
   systime.wMinute++;

   /* Add the job */
   printf("Adding sample job.\n");
   CreateNewJob(pSchedulingAgent, BATCH_NAME, BATCH_APP, BATCH_PARM,
    &systime);

   /* Release the pointer */
   pSchedulingAgent->Release();
  }
 }
 CoUninitialize();
}

void CreateNewJob(ISchedulingAgent * pSchedulingAgent,
 LPWSTR lpwszJobName, LPWSTR lpwszAppName, LPWSTR lpwszParameters,
 LPSYSTEMTIME lptTime)
{
 IUnknown *pUnknown;
 IPersistFile *pPersistFile;
 ITask *pTask;
 ITaskTrigger *pTaskTrigger;
 WORD nTrigger;
 TASK_TRIGGER TaskTrigger;
 
 /* Make a new task */
 pSchedulingAgent->NewWorkItem(lpwszJobName, CLSID_CTask, IID_ITask,
  &pUnknown);

 /* Get ITask interface */
 pUnknown->QueryInterface(IID_ITask, (void **) &pTask);
 
 /* Done with pUnknown */
 pUnknown->Release();
 pUnknown = NULL;
 
 /* Set command name with ITask::SetApplicationName */
 pTask->SetApplicationName(lpwszAppName);
 pTask->SetParameters(lpwszParameters);
 pTask->SetComment(L"Spawn notepad.exe in one minute");

 /* Make sure the task goes away when done */
 pTask->SetFlags(TASK_FLAG_DELETE_WHEN_DONE);

 /* Make a trigger */
 pTask->CreateTrigger(&nTrigger, &pTaskTrigger);

 /* Fill in trigger settings */
 memset(&TaskTrigger, 0, sizeof(TASK_TRIGGER));
 TaskTrigger.cbTriggerSize = sizeof(TASK_TRIGGER);
 TaskTrigger.wBeginYear = lptTime->wYear;
 TaskTrigger.wBeginMonth = lptTime->wMonth;
 TaskTrigger.wBeginDay = lptTime->wDay;
 TaskTrigger.wStartHour = lptTime->wHour;
 TaskTrigger.wStartMinute = lptTime->wMinute;
 TaskTrigger.TriggerType = TASK_TIME_TRIGGER_DAILY;
 TaskTrigger.Type.Daily.DaysInterval = 1;

 /* Apply the newly created trigger to the task */
 pTaskTrigger->SetTrigger(&TaskTrigger);

 /* Make the changes permanent */
 pTask->QueryInterface(IID_IPersistFile, 
  (void **) &pPersistFile);
 pPersistFile->Save(NULL, FALSE);

 /* Free interfaces */
 pTask->Release();
 pTaskTrigger->Release();
 pPersistFile->Release();
} 
