//
// MAR98.CPP
//
// This program demonstrates a problem with Visual C++
// 5.0.  The deque container type that is a part of
// the STL is now part of the standard C++ library.
// This program exercises the the library by using the
// deque as a FIFO, adding one element to the front and
// removing one from the back in a loop.  After a little
// over 500 iterations through the loop, the deque becomes
// corrupted and no longer behaves properly. This program
// detects that by noting that an iterator through the
// container detects more than 10 elements.
//
// Compiling this program using this command line:
//
// cl /GX mar98.cpp
//
// will produce a program that gives this output:
//
// [ 510 lines deleted ]
// [511] 511 510 509 508 507 506 505 504 503 502
// [512] 512 511 510 509 508 507 506 505 504 503 6762624
//
// Too many elements!
//
// Compiling this program using this command line:
//
// cl /DNO_CHECK /GX mar98.cpp
//
// creates a program that attempts to navigate the
// corrupted deque, which leads to the inevitable
// fatal error message box.
//

#include <iostream>
#include <deque>
#include <cassert>

using namespace std;

void main()
{
    deque<int> fifo;
    //
    // First I load up the deque with and initial set of
    // 10 elements.
    //
    for ( int j = -10 ; j != 0 ; j++ )
        fifo.push_front( j );

    //
    // Now I run through a loop exercising the deque
    //
    for ( int i = 0 ; i < 1000 ; i++ ) {
        //
        // Remove one element from the end and add
        // a new element to the beginning
        //
        fifo.pop_back();
        fifo.push_front( i );
        assert( ( fifo.end() - fifo.begin() ) == 10 );
        cout << "[" << i << "] ";
        //
        // Now I print the contents of the queue
        // for verification, using a standard
        // deque iterator
        //
        int counter = 0;
        for ( deque<int>::const_iterator ii = fifo.begin();
              ii != fifo.end();
              ii++ ) {
            cout << *ii << " ";
#if !defined( NO_CHECK )
            if ( ++counter > 10 ) {
                cout << "\n\nToo many elements!\n";
                return;
            }
#endif
        }
        cout << "\n";
    }
}

