/***********/
/* NoCRT.c */
/***********/

/* FOR UNICODE IMPLEMENTATIONS */
/* To implement the UNICODE version of WinMain, do the following:
 *
 *     #ifdef UNICODE
 *     #define WinMain wWinMain
 *     #endif 
 *     #ifdef __cplusplus
 *     extern "C" 
 *     #endif
 *     int WINAPI WinMain(
 *         HINSTANCE hInstance,
 *         HINSTANCE hPrevInstance,
 *         LPTSTR lpCmdLine,
 *         int nCmdShow)
 *     {
 *         ...
 *     }
 */

/* remove runtime libraries */
#pragma comment(linker, "/nodefaultlib")

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

/* wWinMain is currently not defined in winbase.h.
 * When it is, remove this prototype. */
#ifdef UNICODE
int WINAPI wWinMain(
    HINSTANCE hInstance,
    HINSTANCE hPrevInstance,
    LPWSTR lpCmdLine,
    int nShowCmd);
#endif

/* UNICODE subsitiution macros */
#ifdef UNICODE
#define WinMainCRTStartup wWinMainCRTStartup
#define WinMain           wWinMain
#define MyCharNext(s)     ((s)+1)
#else
#define MyCharNext(s)     CharNextA(s) 
#endif

/* helper manifest constants */
#define chQuote ((TCHAR)'\"')
#define chSpace ((TCHAR)' ')
#define chNull  ((TCHAR)'\0')


void WinMainCRTStartup()
{
    /* LOCAL variables */
    STARTUPINFO si;
    HINSTANCE   hInstance = GetModuleHandle(NULL);
    LPTSTR      lpCmdLine = GetCommandLine();
    TCHAR       chDelimiter = 0;
    UINT        uExitCode = (UINT)-1;
    int         nCmdShow = 0;

    /* validate command line exists */
    if ((lpCmdLine!=NULL) && (*lpCmdLine!=chNull))
    {
        /* command line format: 
         *   (optionally quoted) program_name cmd_parms ... */
        /* skip program_name */
        chDelimiter = (TCHAR)
            ((*lpCmdLine == chQuote) ? chQuote : chSpace);
        do
        {
            lpCmdLine = MyCharNext(lpCmdLine);
        }
        while ((*lpCmdLine!=chDelimiter) && (*lpCmdLine!=chNull));

        /* skip delimiter as necessary */
        if (*lpCmdLine == chDelimiter)
            lpCmdLine = MyCharNext(lpCmdLine);

        /* scan for the cmd_parms, skipping leading whitespace */
        while ((*lpCmdLine!=chNull) && (*lpCmdLine<=chSpace))
            lpCmdLine = MyCharNext(lpCmdLine);

        /* get startup info for show window flag */
        si.cb      = sizeof(STARTUPINFO);
        si.dwFlags = 0;
        GetStartupInfo(&si);
        nCmdShow = (si.dwFlags & STARTF_USESHOWWINDOW) ? 
            si.wShowWindow : SW_SHOWDEFAULT;
        /* call WinMain */
        uExitCode = (UINT)
            WinMain(hInstance, NULL, lpCmdLine, nCmdShow);
    }
    /* terminate process */
    ExitProcess(uExitCode);
}

/* End of File */